package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.QueryResultRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultRecordsType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.ReferencesType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBElement;

public class RecordResult<T extends QueryResultRecordType> extends Result<QueryResultRecordsType>
{
  private List<T> records;

  RecordResult(VcloudClient client, QueryResultRecordsType recordsType)
  {
    super(client, recordsType);
    sortRecords_v1_5();
  }

  private void sortRecords_v1_5()
  {
    QueryResultRecordsType recordsType = getResource();

    if (this.records == null) {
      this.records = new ArrayList();
    }
    for (JAXBElement recordType : recordsType.getRecord())
    {
      this.records.add((QueryResultRecordType)recordType.getValue());
    }
  }

  public QueryResultRecordsType getResource()
  {
    return (QueryResultRecordsType)super.getResource();
  }

  public List<T> getRecords()
  {
    if (this.records == null) {
      this.records = new ArrayList();
    }
    return this.records;
  }

  public RecordResult<T> getFirstPage()
    throws VCloudException
  {
    if (hasFirstPage().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getFirstPageReference().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public RecordResult<T> getPreviousPage()
    throws VCloudException
  {
    if (hasPreviousPage().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getPreviousPageReference().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public RecordResult<T> getNextPage()
    throws VCloudException
  {
    if (hasNextPage().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getNextPageReference().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public RecordResult<T> getLastPage()
    throws VCloudException
  {
    if (hasLastPage().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getLastPageReference().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public ReferenceResult getReferenceResult()
    throws VCloudException
  {
    if (hasAlternateReferencesRefResult().booleanValue()) {
      return new ReferenceResult(getVcloudClient(), (ReferencesType)SdkUtil.get(getVcloudClient(), getAlternateReferencesRef().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public RecordResult getRecordResult()
    throws VCloudException
  {
    if (hasAlternateRecordsRefResult().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getAlternateRecordRef().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public RecordResult getIdRecordResult()
    throws VCloudException
  {
    if (hasAlternateIdRecordsRefResult().booleanValue()) {
      return new RecordResult(getVcloudClient(), (QueryResultRecordsType)SdkUtil.get(getVcloudClient(), getAlternateIdRecordRef().getHref(), 200));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}