package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.ErrorType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Response
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private String responseXml;
  private int responseStatusCode;

  String getResponseXml()
  {
    return this.responseXml;
  }

  void setResponseXml(String responseXml)
  {
    this.responseXml = responseXml;
  }

  int getResponseStatusCode()
  {
    return this.responseStatusCode;
  }

  void setResonseStatusCode(int resonseStatucCode)
  {
    this.responseStatusCode = resonseStatucCode;
  }

  public boolean isExpected(int expectedStatusCode)
  {
    if (getResponseStatusCode() != expectedStatusCode) {
      logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.EXPECTED_STATUS_CODE_MSG) + " - " + expectedStatusCode);

      logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.ACTUAL_STATUS_CODE_MSG) + " - " + getResponseStatusCode());

      return false;
    }
    return true;
  }

  public <T> T getResource()
    throws VCloudException
  {
    if ((getResponseXml() == null) || (getResponseXml().isEmpty()))
    {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NO_RESPONSE_RECEIVED_MSG));
    }

    InputStream is = null;
    try {
      is = new ByteArrayInputStream(getResponseXml().getBytes("UTF-8"));
    }
    catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    }
    return JAXBUtil.unmarshallResource(is);
  }

  public void handleUnExpectedResponse()
    throws VCloudException
  {
    if ((getResponseXml() != null) && (!getResponseXml().isEmpty()))
    {
      InputStream is = null;
      try {
        is = new ByteArrayInputStream(getResponseXml().getBytes("UTF-8"));
      }
      catch (UnsupportedEncodingException e) {
        logger.log(Level.SEVERE, e.getMessage());
        throw new VCloudRuntimeException(e);
      }
      ErrorType errorType = (ErrorType)JAXBUtil.unmarshallResource(is);
      throw new VCloudException(errorType);
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NO_ERROR_MSG) + " - " + getResponseStatusCode());
  }
}