package com.vmware.vcloud.sdk;

import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SdkUtil
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private static ResourceBundle resourceBundle;

  public static String getI18nString(SdkMessage sdkMessage)
  {
    try
    {
      return resourceBundle.getString(sdkMessage.name());
    }
    catch (MissingResourceException e)
    {
      logger.log(Level.SEVERE, e.getMessage());
      return e.getMessage();
    }
  }

  public static <T> T get(VcloudClient client, String url, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    return validateResponse(response, statusCode);
  }

  public static <T> T post(VcloudClient client, String url, String requestString, String mediaType, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.post(client, url, requestString, mediaType, "UTF-8");

    return validateResponse(response, statusCode);
  }

  public static <T> T put(VcloudClient client, String url, String requestString, String mediaType, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.put(client, url, requestString, mediaType, "UTF-8");

    return validateResponse(response, statusCode);
  }

  public static <T> T delete(VcloudClient client, String url, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, url);
    return validateResponse(response, statusCode);
  }

  private static <T> T validateResponse(Response response, int statusCode)
    throws VCloudException
  {
    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    return response.getResource();
  }

  static
  {
    if (resourceBundle == null)
      try {
        resourceBundle = ResourceBundle.getBundle("com.vmware.vcloud.sdk.properties.vCloudJavaSDK", Locale.getDefault());
      }
      catch (MissingResourceException e) {
        logger.log(Level.SEVERE, e.getMessage());
        logger.log(Level.INFO, "Loading locale en_US properties");
        resourceBundle = ResourceBundle.getBundle("com.vmware.vcloud.sdk.properties.vCloudJavaSDK", Locale.ENGLISH);
      }
  }
}