package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.ServiceType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Service extends VcloudEntity<ServiceType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  public Service(VcloudClient client, ServiceType serviceResource)
  {
    super(client, serviceResource);
  }

  public static Service getServiceByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new Service(client, (ServiceType)getResourceByReference(client, reference));
  }

  public static Service getServiceById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new Service(client, (ServiceType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.service+xml"));
  }

  public ReferenceResult getApiDefinitionRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/apidefinitions" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }
}