package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.VAppNetworkType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VappNetwork extends VcloudEntity<VAppNetworkType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType vappReference;
  private ReferenceType vappNetworkRepairReference;

  VappNetwork(VcloudClient client, VAppNetworkType vappNetworkType)
  {
    super(client, vappNetworkType);
    sortReferences_v1_5();
  }

  public static VappNetwork getVappNetworkByReference(VcloudClient client, ReferenceType vappNetworkRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + vappNetworkRef.getHref());

    return new VappNetwork(client, (VAppNetworkType)getResourceByReference(client, vappNetworkRef));
  }

  public static VappNetwork getVappNetworkById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new VappNetwork(client, (VAppNetworkType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.network+xml"));
  }

  private void sortReferences_v1_5()
  {
    for (LinkType vappNetworkLink : ((VAppNetworkType)getResource()).getLink())
      if ((vappNetworkLink.getRel().equals("up")) && (vappNetworkLink.getType().equals("application/vnd.vmware.vcloud.vApp+xml")))
      {
        this.vappReference = vappNetworkLink;
      } else if (vappNetworkLink.getRel().equals("repair"))
        this.vappNetworkRepairReference = vappNetworkLink;
  }

  public Task reset()
    throws VCloudException
  {
    if (this.vappNetworkRepairReference != null) {
      return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), this.vappNetworkRepairReference.getHref(), null, null, 202));
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VAPP_NETWORK_NOT_DEPLOYED));
  }

  public ReferenceType getVappReference()
    throws VCloudException
  {
    if (this.vappReference != null)
      return this.vappReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}