package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.IdentifiableResourceType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.ResourceType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public abstract class VcloudResource<T extends ResourceType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private T resourceType;
  private ReferenceType referenceType;
  private VcloudClient client;

  protected VcloudResource(VcloudClient client, T resourceType)
  {
    this.resourceType = resourceType;
    this.client = client;
    setReference(resourceType);
  }

  protected VcloudClient getVcloudClient()
  {
    return this.client;
  }

  public T getResource()
  {
    return this.resourceType;
  }

  public ReferenceType getReference()
  {
    return this.referenceType;
  }

  private void setReference(ResourceType resource)
  {
    this.referenceType = new ReferenceType();
    this.referenceType.setHref(resource.getHref());
    this.referenceType.setType(resource.getType());
    if ((resource instanceof IdentifiableResourceType))
      this.referenceType.setId(((IdentifiableResourceType)resource).getId());
  }

  protected static ResourceType getResourceByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return (ResourceType)SdkUtil.get(client, reference.getHref(), 200);
  }
}