package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.NetworkConfigurationType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.OrgNetworkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

/** @deprecated */
public class AdminOrgNetwork extends VcloudEntity<OrgNetworkType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType orgNetworkReference;
  private ReferenceType adminOrgReference;

  AdminOrgNetwork(VcloudClient client, OrgNetworkType orgNetworkType_v1_5)
  {
    super(client, orgNetworkType_v1_5);
    sortRefs_v1_5();
  }

  public ReferenceType getAdminOrganizationReference()
    throws VCloudException
  {
    if (this.adminOrgReference != null)
      return this.adminOrgReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public ReferenceType getOrgNetworkReference()
    throws VCloudException
  {
    if (this.orgNetworkReference != null)
      return this.orgNetworkReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortRefs_v1_5()
  {
    for (LinkType adminOrgNetworkLink : ((OrgNetworkType)getResource()).getLink()) {
      if ((adminOrgNetworkLink.getRel().equals("up")) && (adminOrgNetworkLink.getType().equals("application/vnd.vmware.admin.organization+xml")))
      {
        this.adminOrgReference = adminOrgNetworkLink;
      }if ((adminOrgNetworkLink.getRel().equals("alternate")) && (adminOrgNetworkLink.getType().equals("application/vnd.vmware.vcloud.orgNetwork+xml")))
      {
        this.orgNetworkReference = adminOrgNetworkLink;
      }
    }
  }

  public NetworkConfigurationType getConfiguration()
  {
    return ((OrgNetworkType)getResource()).getConfiguration();
  }

  public VimObjectRefType getNetworkVimRef()
    throws VCloudException
  {
    for (VCloudExtensionType vcloudExtension : ((OrgNetworkType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        if ((((JAXBElement)jaxbObject).getValue() instanceof VimObjectRefType))
          return (VimObjectRefType)((JAXBElement)jaxbObject).getValue();
      }
    }
    Iterator i$;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public static AdminOrgNetwork getOrgNetworkByReference(VcloudClient client, ReferenceType orgNetworkRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + orgNetworkRef.getHref());

    return new AdminOrgNetwork(client, (OrgNetworkType)getResourceByReference(client, orgNetworkRef));
  }

  public static AdminOrgNetwork getOrgNetworkById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminOrgNetwork(client, (OrgNetworkType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.network+xml"));
  }

  public Task updateOrgNetwork(OrgNetworkType orgNetworkType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String orgNetworkUpdateUrl = getReference().getHref();
    String orgNetworkUpdateRequestXmlString = JAXBUtil.marshal(of.createOrgNetwork(orgNetworkType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + orgNetworkUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), orgNetworkUpdateUrl, orgNetworkUpdateRequestXmlString, "application/vnd.vmware.admin.orgNetwork+xml", 202));
  }

  public Task reset()
    throws VCloudException
  {
    String taskCanelActionUrl = getReference().getHref() + "/action/reset";

    return executeAction(getVcloudClient(), taskCanelActionUrl, null, null, 202);
  }

  public static Task reset(VcloudClient client, ReferenceType adminOrgNetworkReference)
    throws VCloudException
  {
    return executeAction(client, adminOrgNetworkReference.getHref() + "/action/reset", null, null, 202);
  }

  public Task syncSyslogServer()
    throws VCloudException
  {
    String syncActionUrl = getReference().getHref() + "/action/syncSyslogServerSettings";

    return executeAction(getVcloudClient(), syncActionUrl, null, null, 202);
  }

  public static Task syncSyslogServer(VcloudClient client, ReferenceType adminOrgNetworkReference)
    throws VCloudException
  {
    return executeAction(client, adminOrgNetworkReference.getHref() + "/action/syncSyslogServerSettings", null, null, 202);
  }

  public Task delete()
    throws VCloudException
  {
    String orgNetworkUrl = getReference().getHref();
    return deleteOrgNetwork(getVcloudClient(), orgNetworkUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType orgNetworkRef)
    throws VCloudException
  {
    return deleteOrgNetwork(client, orgNetworkRef.getHref());
  }

  private static Task deleteOrgNetwork(VcloudClient client, String orgNetworkUrl)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.delete(client, orgNetworkUrl, 202));
  }

  private static Task executeAction(VcloudClient client, String adminOrgActionUrl, String content, String contentType, int statusCode)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, adminOrgActionUrl, content, contentType, statusCode));
  }
}