package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.RoleType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Role extends VcloudEntity<RoleType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  Role(VcloudClient client, RoleType roleType_v1_5)
  {
    super(client, roleType_v1_5);
  }

  public static Role getRoleByReference(VcloudClient client, ReferenceType roleRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + roleRef.getHref());

    return new Role(client, (RoleType)getResourceByReference(client, roleRef));
  }

  public static Role getRoleById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new Role(client, (RoleType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.role+xml"));
  }

  public Role updateRole(RoleType roleType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String roleUpdateUrl = getReference().getHref();
    String roleUpdateRequestXmlString = JAXBUtil.marshal(of.createRole(roleType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + roleUpdateUrl);

    return new Role(getVcloudClient(), (RoleType)SdkUtil.put(getVcloudClient(), roleUpdateUrl, roleUpdateRequestXmlString, "application/vnd.vmware.admin.role+xml", 200));
  }

  public void delete()
    throws VCloudException
  {
    String roleUrl = getReference().getHref();
    deleteRole(getVcloudClient(), roleUrl);
  }

  public static void delete(VcloudClient client, ReferenceType roleRef)
    throws VCloudException
  {
    deleteRole(client, roleRef.getHref());
  }

  private static void deleteRole(VcloudClient client, String roleUrl)
    throws VCloudException
  {
    SdkUtil.delete(client, roleUrl, 204);
  }
}