package com.vmware.vcloud.sdk.admin.extensions;

import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.VendorServicesType;
import com.vmware.vcloud.api.rest.schema.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.extension.VMWNetworkPoolType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWNetworkPool extends VcloudEntity<VMWNetworkPoolType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  VMWNetworkPool(VcloudClient client, VMWNetworkPoolType vmwNetworkPoolType_v1_5)
  {
    super(client, vmwNetworkPoolType_v1_5);
  }

  public static VMWNetworkPool getVMWNetworkPoolByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new VMWNetworkPool(client, (VMWNetworkPoolType)getResourceByReference(client, reference));
  }

  public static VMWNetworkPool getVMWNetworkPoolById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new VMWNetworkPool(client, (VMWNetworkPoolType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.networkPool+xml"));
  }

  public VMWNetworkPool updateVMWNetworkPool(VMWNetworkPoolType vmwNetworkPoolType)
    throws VCloudException
  {
    ObjectFactory ofFactory = new ObjectFactory();
    String vmwNetworkPoolUpdateUrl = getReference().getHref();
    String vmwNetworkPoolUpdateRequest = JAXBUtil.marshal(ofFactory.createVMWNetworkPool(vmwNetworkPoolType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + vmwNetworkPoolUpdateUrl);

    return new VMWNetworkPool(getVcloudClient(), (VMWNetworkPoolType)SdkUtil.put(getVcloudClient(), vmwNetworkPoolUpdateUrl, vmwNetworkPoolUpdateRequest, "application/vnd.vmware.admin.networkPool+xml", 200));
  }

  public Task delete()
    throws VCloudException
  {
    String vmwNetworkPoolUrl = getReference().getHref();
    return deleteVMWNetworkPool(getVcloudClient(), vmwNetworkPoolUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType vmwNetworkPoolRef)
    throws VCloudException
  {
    return deleteVMWNetworkPool(client, vmwNetworkPoolRef.getHref());
  }

  private static Task deleteVMWNetworkPool(VcloudClient client, String vmwNetworkPoolUrl)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.delete(client, vmwNetworkPoolUrl, 202));
  }

  public VendorServicesType getVMWVendorServices()
    throws VCloudException
  {
    String vendorServicesUrl = getReference().getHref() + "/vendorServices";

    return (VendorServicesType)SdkUtil.get(getVcloudClient(), vendorServicesUrl, 200);
  }
}