package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.extension.AclRuleType;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminAclRule extends VcloudEntity<AclRuleType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminResourceClassActionRef;

  public AdminAclRule(VcloudClient client, AclRuleType aclRuleParams)
  {
    super(client, aclRuleParams);
    sortAdminAclRuleLinkRefs();
  }

  private void sortAdminAclRuleLinkRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((AclRuleType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.admin.resourceClassAction+xml")))
        {
          this.adminResourceClassActionRef = link;
        }
  }

  public static AdminAclRule getAdminAclRuleByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminAclRule(client, (AclRuleType)getResourceByReference(client, reference));
  }

  public static AdminAclRule getAdminAclRuleById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminAclRule(client, (AclRuleType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.aclRule+xml"));
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public ReferenceType getAdminResourceClassActionReference()
    throws VCloudException
  {
    if (this.adminResourceClassActionRef != null)
      return this.adminResourceClassActionRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}