package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.extension.AdminApiDefinitionType;
import com.vmware.vcloud.api.rest.schema.extension.AdminFileDescriptorType;
import com.vmware.vcloud.api.rest.schema.extension.ObjectFactory;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminApiDefinition extends VcloudEntity<AdminApiDefinitionType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminServiceRef;

  AdminApiDefinition(VcloudClient client, AdminApiDefinitionType apiDefinitionParams)
  {
    super(client, apiDefinitionParams);
    sortAdminApiDefinitionRefs();
  }

  private void sortAdminApiDefinitionRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((AdminApiDefinitionType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.admin.service+xml")))
        {
          this.adminServiceRef = link;
        }
  }

  public static AdminApiDefinition getAdminApiDefinitionByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminApiDefinition(client, (AdminApiDefinitionType)getResourceByReference(client, reference));
  }

  public static AdminApiDefinition getAdminApiDefinitionById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminApiDefinition(client, (AdminApiDefinitionType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.apiDefinition+xml"));
  }

  public AdminFileDescriptor createAdminFileDescriptor(AdminFileDescriptorType fileDescParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new ObjectFactory().createFileDescriptor(fileDescParams));

    return new AdminFileDescriptor(getVcloudClient(), (AdminFileDescriptorType)SdkUtil.post(getVcloudClient(), ((AdminApiDefinitionType)getResource()).getHref() + "/files", requestXML, "application/vnd.vmware.admin.fileDescriptor+xml", 201));
  }

  public ReferenceResult getAdminFileDescriptorRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/files" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public ReferenceType getAdminServiceReference()
    throws VCloudException
  {
    if (this.adminServiceRef != null)
      return this.adminServiceRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}