package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.extension.ServiceResourceType;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminServiceResource extends VcloudEntity<ServiceResourceType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminResourceClassRef;

  AdminServiceResource(VcloudClient client, ServiceResourceType serviceResParams)
  {
    super(client, serviceResParams);
    sortAdminServiceResourceLinkRefs();
  }

  private void sortAdminServiceResourceLinkRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((ServiceResourceType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.admin.resourceClass+xml")))
        {
          this.adminResourceClassRef = link;
        }
  }

  public static AdminServiceResource getAdminServiceResourceByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminServiceResource(client, (ServiceResourceType)getResourceByReference(client, reference));
  }

  public static AdminServiceResource getAdminServiceResourceById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminServiceResource(client, (ServiceResourceType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.serviceResource+xml"));
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public ReferenceType getAdminResourceClassReference()
    throws VCloudException
  {
    if (this.adminResourceClassRef != null)
      return this.adminResourceClassRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}