package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.CatalogItemType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class CatalogItem extends VcloudEntity<CatalogItemType>
{
  private static Logger logger = Logger.getLogger(CatalogItem.class.getSimpleName());
  private ReferenceType catalogReference;

  public CatalogItem(VcloudClient client, CatalogItemType catalogItemType)
  {
    super(client, catalogItemType);
    sortReferences();
  }

  public ReferenceType getCatalogReference()
    throws VCloudException
  {
    if (this.catalogReference != null)
      return this.catalogReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getEntityReference()
    throws VCloudException
  {
    if (((CatalogItemType)getResource()).getEntity() != null)
      return ((CatalogItemType)getResource()).getEntity();
    throw new VCloudException("Reference Not Found");
  }

  private void sortReferences()
  {
    for (LinkType catalogItemLink : ((CatalogItemType)getResource()).getLink())
      if ((catalogItemLink.getRel().equals(RelationType.UP)) && (catalogItemLink.getType().equals("application/vnd.vmware.vcloud.catalog+xml")))
      {
        this.catalogReference = catalogItemLink;
      }
  }

  public static CatalogItem getCatalogItemByReference(VcloudClient client, ReferenceType ref)
    throws VCloudException
  {
    return getCatalogItem(client, ref.getHref());
  }

  private static CatalogItem getCatalogItem(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    CatalogItemType catalogItemType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    catalogItemType = (CatalogItemType)response.getResource();
    CatalogItem catalogItem = new CatalogItem(client, catalogItemType);
    return catalogItem;
  }

  public CatalogItem updateCatalogItem(CatalogItemType catalogItemType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String catalogItemUpdateUrl = ((CatalogItemType)getResource()).getHref();
    String catalogItemRequestXmlString = JAXBUtil.marshal(of.createCatalogItem(catalogItemType));

    logger.log(Level.INFO, "CatalogItem URL : " + catalogItemUpdateUrl);
    Response response = RestUtil.put(getVcloudClient(), catalogItemUpdateUrl, catalogItemRequestXmlString, "application/vnd.vmware.vcloud.catalogItem+xml", null);

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    CatalogItemType updatedCatalogItemType = (CatalogItemType)response.getResource();

    CatalogItem catalogItem = new CatalogItem(getVcloudClient(), updatedCatalogItemType);

    return catalogItem;
  }

  public void delete()
    throws VCloudException
  {
    String catalogItemUrl = ((CatalogItemType)getResource()).getHref();
    deleteCatalogItem(getVcloudClient(), catalogItemUrl);
  }

  public static void delete(VcloudClient client, ReferenceType catalogItemRef)
    throws VCloudException
  {
    deleteCatalogItem(client, catalogItemRef.getHref());
  }

  private static void deleteCatalogItem(VcloudClient client, String catalogItemrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, catalogItemrl);

    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }
}