package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.OrgNetworkType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class OrgNetwork extends VcloudEntity<OrgNetworkType>
{
  private static Logger logger = Logger.getLogger(OrgNetwork.class.getSimpleName());
  private ReferenceType orgReference;

  OrgNetwork(VcloudClient client, OrgNetworkType orgNetworkType)
  {
    super(client, orgNetworkType);
    sortReferences();
  }

  public static OrgNetwork getOrgNetworkByReference(VcloudClient client, ReferenceType networkRef)
    throws VCloudException
  {
    String url = networkRef.getHref();
    return getOrgNetwork(client, url);
  }

  private static OrgNetwork getOrgNetwork(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET Org Network " + url);
    Response response = RestUtil.get(client, url);
    OrgNetworkType networkType = null;

    if (response.isExpected(200)) {
      networkType = (OrgNetworkType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    OrgNetwork net = new OrgNetwork(client, networkType);
    return net;
  }

  private void sortReferences()
  {
    for (LinkType orgNetworkLink : ((OrgNetworkType)getResource()).getLink())
      if ((orgNetworkLink.getRel().equals(RelationType.UP)) && (orgNetworkLink.getType().equals("application/vnd.vmware.vcloud.org+xml")))
      {
        this.orgReference = orgNetworkLink;
      }
  }

  public ReferenceType getOrgReference()
    throws VCloudException
  {
    if (this.orgReference != null)
      return this.orgReference;
    throw new VCloudException("Reference Not Found");
  }
}