package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.TasksListType;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class TaskList extends VcloudEntity<TasksListType>
{
  private static Logger logger = Logger.getLogger(TaskList.class.getSimpleName());
  private List<Task> tasks;

  TaskList(VcloudClient client, TasksListType tasksList)
  {
    super(client, tasksList);
    sortTasks();
  }

  public List<Task> getTasks()
  {
    return this.tasks;
  }

  public static TaskList getTaskListByReference(VcloudClient client, ReferenceType taskListLink)
    throws VCloudException
  {
    String url = taskListLink.getHref();
    return getTaskList(client, url);
  }

  private void sortTasks()
  {
    this.tasks = new ArrayList();
    if (((TasksListType)getResource()).getTask() != null) {
      List tasksType = ((TasksListType)getResource()).getTask();
      for (TaskType taskType : tasksType)
        this.tasks.add(new Task(getVcloudClient(), taskType));
    }
  }

  private static TaskList getTaskList(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET Tasks List " + url);
    Response response = RestUtil.get(client, url);
    TasksListType tasksListType = null;

    if (response.isExpected(200)) {
      tasksListType = (TasksListType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    TaskList taskList = new TaskList(client, tasksListType);
    return taskList;
  }
}