package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema.ovf.NetworkSectionNetwork;
import com.vmware.vcloud.api.rest.schema.ovf.NetworkSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.OperatingSystemSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.RASDType;
import com.vmware.vcloud.api.rest.schema.ovf.ResourceType;
import com.vmware.vcloud.api.rest.schema.ovf.SectionType;
import com.vmware.vcloud.api.rest.schema.ovf.StartupSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.VirtualHardwareSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.AbstractVAppType;
import com.vmware.vcloud.api.rest.schema_v1_0.ControlAccessParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.DeployVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.GuestCustomizationSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.LeaseSettingsSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.MediaInsertOrEjectParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConfigSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConnectionSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConnectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.RasdItemsListType;
import com.vmware.vcloud.api.rest.schema_v1_0.RecomposeVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.ScreenTicketType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.UndeployVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppChildrenType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppNetworkConfigurationType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppType;
import com.vmware.vcloud.api.rest.schema_v1_0.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema_v1_0.VmPendingQuestionType;
import com.vmware.vcloud.api.rest.schema_v1_0.VmQuestionAnswerType;
import com.vmware.vcloud.api.rest.schema_v1_0.VmType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class Vapp extends VcloudEntity<AbstractVAppType>
{
  private static Logger logger = Logger.getLogger(Vapp.class.getSimpleName());
  private VirtualHardwareSectionType virtualHardwareSection;
  private OperatingSystemSectionType operatingSystemSection;
  private NetworkConfigSectionType networkConfigSection;
  private NetworkConnectionSectionType networkConnectionSection;
  private NetworkSectionType networkSection;
  private LeaseSettingsSectionType leaseSettingsSection;
  private StartupSectionType startupSection;
  private GuestCustomizationSectionType guestCustomizationSection;
  private List<Vapp> childrenvApps;
  private List<Vapp> childrenVms;
  private VirtualCpu virtualCpu;
  private VirtualMemory virtualMemory;
  private List<VirtualDisk> virtualDisks;
  private List<VirtualNetworkCard> networkCards;
  private List<VirtualMedia> medias;
  private ReferenceType vdcReference;
  private ReferenceType parentVappReference;
  private VimObjectRefType vimObjectRefType;

  public Vapp(VcloudClient client, AbstractVAppType abstractvAppType)
  {
    super(client, abstractvAppType);
    sortOvfSectionsAndReferences();
  }

  public static Vapp getVappByReference(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return getVapp(client, vappRef.getHref());
  }

  public Task updateVapp(AbstractVAppType abstractvAppType)
    throws VCloudException
  {
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String vAppUpdateUrl = ((AbstractVAppType)getResource()).getHref();
    String vAppRequestXmlString = (abstractvAppType instanceof VAppType) ? JAXBUtil.marshal(of.createVApp((VAppType)abstractvAppType)) : JAXBUtil.marshal(of.createVm((VmType)abstractvAppType));

    String vAppMediaType = (abstractvAppType instanceof VAppType) ? "application/vnd.vmware.vcloud.vApp+xml" : "application/vnd.vmware.vcloud.vm+xml";

    logger.log(Level.INFO, "VAPP URL : " + vAppUpdateUrl);
    Response response = RestUtil.put(getVcloudClient(), vAppUpdateUrl, vAppRequestXmlString, vAppMediaType, null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(getVcloudClient(), taskType);

    return task;
  }

  private static Vapp getVapp(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "VAPP URL : " + url);

    Response response = RestUtil.get(client, url);
    AbstractVAppType abstractVAppType = null;

    if (response.isExpected(200)) {
      abstractVAppType = (response.getResource() instanceof VAppType) ? (VAppType)response.getResource() : (VmType)response.getResource();
    }
    else
    {
      response.handleUnExpectedResponse();
    }
    Vapp vapp = (abstractVAppType instanceof VAppType) ? new Vapp(client, (VAppType)abstractVAppType) : new Vapp(client, (VmType)abstractVAppType);

    return vapp;
  }

  public Integer getVappStatus()
  {
    return ((AbstractVAppType)getResource()).getStatus();
  }

  public Boolean isDeployed()
  {
    return ((AbstractVAppType)getResource()).isDeployed();
  }

  public boolean isVm()
  {
    return getResource() instanceof VmType;
  }

  public List<Vapp> getChildrenVapps()
    throws VCloudException
  {
    if (isVm()) {
      throw new VCloudException("This Vapp is a VM, A VM does not contain Vapp's or VM's in it");
    }
    if (this.childrenvApps == null) {
      this.childrenvApps = new ArrayList();
      if (((VAppType)getResource()).getChildren() != null) {
        VAppChildrenType vAppChildrenType = ((VAppType)getResource()).getChildren();

        List vAppTypeList = vAppChildrenType.getVApp();
        for (VAppType vAppType : vAppTypeList) {
          this.childrenvApps.add(new Vapp(getVcloudClient(), vAppType));
        }
      }
    }
    return this.childrenvApps;
  }

  public List<Vapp> getChildrenVms()
    throws VCloudException
  {
    if (isVm()) {
      throw new VCloudException("This Vapp is a VM, A VM does not contain Vapp's or VM's in it");
    }
    if (this.childrenVms == null) {
      this.childrenVms = new ArrayList();
      if (((VAppType)getResource()).getChildren() != null) {
        VAppChildrenType childrenType = ((VAppType)getResource()).getChildren();

        List vmTypeList = childrenType.getVm();
        for (VmType vmType : vmTypeList)
          this.childrenVms.add(new Vapp(getVcloudClient(), vmType));
      }
    }
    return this.childrenVms;
  }

  public LeaseSettingsSectionType getLeaseSettingsSection()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return this.leaseSettingsSection;
  }

  public static LeaseSettingsSectionType getLeaseSettingsSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/leaseSettingsSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (LeaseSettingsSectionType)response.getResource();
  }

  public StartupSectionType getStartUpSection()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return this.startupSection;
  }

  public static StartupSectionType getStartUpSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/startupSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (StartupSectionType)response.getResource();
  }

  public GuestCustomizationSectionType getGuestCustomizationSection()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return this.guestCustomizationSection;
  }

  public static GuestCustomizationSectionType getGuestCustomizationSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/guestCustomizationSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (GuestCustomizationSectionType)response.getResource();
  }

  public NetworkSectionType getNetworkSection()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return this.networkSection;
  }

  public static NetworkSectionType getNetworkSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/networkSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkSectionType)response.getResource();
  }

  public HashMap<String, NetworkSectionNetwork> getNetworksByName()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    HashMap networks = new HashMap();
    if (this.networkSection != null) {
      for (NetworkSectionNetwork network : this.networkSection.getNetwork())
        networks.put(network.getName(), network);
    }
    return networks;
  }

  public NetworkSectionNetwork getNetworkByName(String networkName)
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return (NetworkSectionNetwork)getNetworksByName().get(networkName);
  }

  public Set<String> getNetworkNames()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return getNetworksByName().keySet();
  }

  public Collection<NetworkSectionNetwork> getNetworks()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return getNetworksByName().values();
  }

  public NetworkConfigSectionType getNetworkConfigSection()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return this.networkConfigSection;
  }

  public static NetworkConfigSectionType getNetworkConfigSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/networkConfigSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkConfigSectionType)response.getResource();
  }

  public HashMap<String, VAppNetworkConfigurationType> getVappNetworkConfigurationsByName()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    HashMap networks = new HashMap();
    List netConnections = this.networkConfigSection.getNetworkConfig();

    Iterator it = netConnections.iterator();
    while (it.hasNext()) {
      VAppNetworkConfigurationType conn = (VAppNetworkConfigurationType)it.next();
      networks.put(conn.getNetworkName(), conn);
    }
    return networks;
  }

  public VAppNetworkConfigurationType getVappNetworkConfigurationByName(String vappNetworkConfigurationName)
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return (VAppNetworkConfigurationType)getVappNetworkConfigurationsByName().get(vappNetworkConfigurationName);
  }

  public Set<String> getVappNetworkConfigurationNames()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return getVappNetworkConfigurationsByName().keySet();
  }

  public Collection<VAppNetworkConfigurationType> getVappNetworkConfigurations()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    return getVappNetworkConfigurationsByName().values();
  }

  public VirtualHardwareSectionType getVirtualHardwareSection()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return this.virtualHardwareSection;
  }

  public static VirtualHardwareSectionType getVirtualHardwareSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/virtualHardwareSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (VirtualHardwareSectionType)response.getResource();
  }

  public OperatingSystemSectionType getOperatingSystemSection()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return this.operatingSystemSection;
  }

  public static OperatingSystemSectionType getOperatingSystemSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/operatingSystemSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (OperatingSystemSectionType)response.getResource();
  }

  public Task updateSection(SectionType sectionType)
    throws VCloudException
  {
    com.vmware.vcloud.api.rest.schema.ovf.ObjectFactory ovfObjectFactory = new com.vmware.vcloud.api.rest.schema.ovf.ObjectFactory();
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory vCloudObjectFactory = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String sectionUrl = ((AbstractVAppType)getResource()).getHref();
    String sectionContentType = null;
    String sectionRequestString = null;
    if ((sectionType instanceof VirtualHardwareSectionType)) {
      JAXBElement section = ovfObjectFactory.createVirtualHardwareSection((VirtualHardwareSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/virtualHardwareSection/";

      sectionContentType = "application/vnd.vmware.vcloud.virtualHardwareSection+xml";
    } else if ((sectionType instanceof OperatingSystemSectionType)) {
      JAXBElement section = ovfObjectFactory.createOperatingSystemSection((OperatingSystemSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/operatingSystemSection/";

      sectionContentType = "application/vnd.vmware.vcloud.operatingSystemSection+xml";
    } else if ((sectionType instanceof LeaseSettingsSectionType)) {
      JAXBElement section = vCloudObjectFactory.createLeaseSettingsSection((LeaseSettingsSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/leaseSettingsSection/";

      sectionContentType = "application/vnd.vmware.vcloud.leaseSettingsSection+xml";
    } else if ((sectionType instanceof StartupSectionType)) {
      JAXBElement section = ovfObjectFactory.createStartupSection((StartupSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/startupSection/";
      sectionContentType = "application/vnd.vmware.vcloud.startupSection+xml";
    } else if ((sectionType instanceof NetworkConfigSectionType)) {
      JAXBElement section = vCloudObjectFactory.createNetworkConfigSection((NetworkConfigSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/networkConfigSection/";

      sectionContentType = "application/vnd.vmware.vcloud.networkConfigSection+xml";
    } else if ((sectionType instanceof GuestCustomizationSectionType)) {
      JAXBElement section = vCloudObjectFactory.createGuestCustomizationSection((GuestCustomizationSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/guestCustomizationSection/";

      sectionContentType = "application/vnd.vmware.vcloud.guestCustomizationSection+xml";
    } else if ((sectionType instanceof NetworkConnectionSectionType)) {
      JAXBElement section = vCloudObjectFactory.createNetworkConnectionSection((NetworkConnectionSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/networkConnectionSection/";

      sectionContentType = "application/vnd.vmware.vcloud.networkConnectionSection+xml";
    } else {
      logger.log(Level.INFO, sectionType.getClass().getName() + " Not a valid Section to Update");

      throw new VCloudException("Not a Valid Section to Update");
    }
    Response response = RestUtil.put(getVcloudClient(), sectionUrl, sectionRequestString, sectionContentType, null);

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public ControlAccessParamsType getControlAccess()
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    String url = ((AbstractVAppType)getResource()).getHref() + "/controlAccess/";

    logger.log(Level.INFO, "GET: Vapp Control Access " + url);
    Response response = RestUtil.get(getVcloudClient(), url);
    ControlAccessParamsType controlAccessParamsType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    controlAccessParamsType = (ControlAccessParamsType)response.getResource();

    return controlAccessParamsType;
  }

  public static ControlAccessParamsType getControlAccess(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    String url = vappRef.getHref() + "/controlAccess/";
    logger.log(Level.INFO, "GET: Vapp Control Access " + url);
    Response response = RestUtil.get(client, url);
    ControlAccessParamsType controlAccessParamsType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    controlAccessParamsType = (ControlAccessParamsType)response.getResource();

    return controlAccessParamsType;
  }

  public ControlAccessParamsType updateControlAccess(ControlAccessParamsType controlAccessParams)
    throws VCloudException
  {
    if (isVm())
      throw new VCloudException("Operation not supported on a VM");
    String url = ((AbstractVAppType)getResource()).getHref() + "/action/controlAccess";

    logger.log(Level.INFO, "POST: Vapp Control Access " + url);
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory vcloudObjectFactory = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String requestString = JAXBUtil.marshal(vcloudObjectFactory.createControlAccessParams(controlAccessParams));

    Response response = RestUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.controlAccess+xml", null);

    ControlAccessParamsType controlAccessParamsType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    controlAccessParamsType = (ControlAccessParamsType)response.getResource();

    return controlAccessParamsType;
  }

  public Task insertMedia(MediaInsertOrEjectParamsType mediaInsertOrEjectParamsType)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String url = ((AbstractVAppType)getResource()).getHref() + "/media/action/insertMedia";

    logger.log(Level.INFO, "POST: Vapp Insert Media Access " + url);
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory vcloudObjectFactory = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String requestString = JAXBUtil.marshal(vcloudObjectFactory.createMediaInsertOrEjectParams(mediaInsertOrEjectParamsType));

    Response response = RestUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.mediaInsertOrEjectParams+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public Task ejectMedia(MediaInsertOrEjectParamsType mediaInsertOrEjectParamsType)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String url = ((AbstractVAppType)getResource()).getHref() + "/media/action/ejectMedia";

    logger.log(Level.INFO, "POST: Vapp Eject Media Access " + url);
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory vcloudObjectFactory = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String requestString = JAXBUtil.marshal(vcloudObjectFactory.createMediaInsertOrEjectParams(mediaInsertOrEjectParamsType));

    Response response = RestUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.mediaInsertOrEjectParams+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public VirtualCpu getCpu()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    if ((this.virtualCpu == null) && 
      (this.virtualHardwareSection != null)) {
      List items = this.virtualHardwareSection.getItem();
      if (items != null) {
        for (RASDType item : items) {
          if (item.getResourceType().getValue().equals("3"))
          {
            this.virtualCpu = new VirtualCpu(getVcloudClient(), item);
          }
        }
      }
    }
    if (this.virtualCpu != null) {
      return this.virtualCpu;
    }

    throw new VCloudException("No Virtual CPU");
  }

  public static VirtualCpu getCpu(VcloudClient client, ReferenceType vAppRef)
    throws VCloudException
  {
    String url = vAppRef.getHref() + "/virtualHardwareSection/cpu";

    VirtualCpu virtualCpu = getCpuByUrl(client, url);
    return virtualCpu;
  }

  private static VirtualCpu getCpuByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    RASDType item = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    item = (RASDType)response.getResource();
    return new VirtualCpu(client, item);
  }

  public Task updateCpu(VirtualCpu virtualCpuItem)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String virtualCpuUpdateUrl = ((AbstractVAppType)getResource()).getHref() + "/virtualHardwareSection/cpu";

    String virtulCpuRequestXmlString = JAXBUtil.marshal(of.createItem(virtualCpuItem.getItemResource()));

    logger.log(Level.INFO, "Virtual Cpu URL : " + virtualCpuUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), virtualCpuUpdateUrl, virtulCpuRequestXmlString, "application/vnd.vmware.vcloud.rasdItem+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public VirtualMemory getMemory()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    if ((this.virtualMemory == null) && 
      (this.virtualHardwareSection != null)) {
      List items = this.virtualHardwareSection.getItem();
      if (items != null) {
        for (RASDType item : items) {
          if (item.getResourceType().getValue().equals("4"))
          {
            this.virtualMemory = new VirtualMemory(getVcloudClient(), item);
          }
        }
      }
    }
    if (this.virtualMemory != null) {
      return this.virtualMemory;
    }

    throw new VCloudException("No Virtual Memory");
  }

  public static VirtualMemory getMemory(VcloudClient client, ReferenceType vAppRef)
    throws VCloudException
  {
    String url = vAppRef.getHref() + "/virtualHardwareSection/memory";

    VirtualMemory virtualMemory = getMemoryByUrl(client, url);
    return virtualMemory;
  }

  private static VirtualMemory getMemoryByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    RASDType item = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    item = (RASDType)response.getResource();
    return new VirtualMemory(client, item);
  }

  public Task updateMemory(VirtualMemory virtualMemoryItem)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String virtualMemoryUpdateUrl = ((AbstractVAppType)getResource()).getHref() + "/virtualHardwareSection/memory";

    String virtulMemoryRequestXmlString = JAXBUtil.marshal(of.createItem(virtualMemoryItem.getItemResource()));

    logger.log(Level.INFO, "Virtual Memory URL : " + virtualMemoryUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), virtualMemoryUpdateUrl, virtulMemoryRequestXmlString, "application/vnd.vmware.vcloud.rasdItem+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public List<VirtualDisk> getDisks()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    if (this.virtualDisks == null) {
      this.virtualDisks = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items) {
            if ((item.getResourceType().getValue().equals("17")) || (item.getResourceType().getValue().equals("6")) || (item.getResourceType().getValue().equals("5")))
            {
              this.virtualDisks.add(new VirtualDisk(getVcloudClient(), item));
            }
          }
        }
      }
    }
    return this.virtualDisks;
  }

  public static List<VirtualDisk> getDisks(VcloudClient client, ReferenceType vAppRef)
    throws VCloudException
  {
    String url = vAppRef.getHref() + "/virtualHardwareSection/disks";

    return getDisksByUrl(client, url);
  }

  private static List<VirtualDisk> getDisksByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    RasdItemsListType items = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    items = (RasdItemsListType)response.getResource();

    List hardwareItems = new ArrayList();
    if (items != null) {
      for (RASDType item : items.getItem()) {
        hardwareItems.add(new VirtualDisk(item));
      }
    }
    return hardwareItems;
  }

  public Task updateDisks(List<VirtualDisk> virtualDisks)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String virtualDisksUpdateUrl = ((AbstractVAppType)getResource()).getHref() + "/virtualHardwareSection/disks";

    String virtulDisksRequestXmlString = JAXBUtil.marshal(of.createRasdItemsList(getRASDItemsList(virtualDisks)));

    logger.log(Level.INFO, "Virtual Disks URL : " + virtualDisksUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), virtualDisksUpdateUrl, virtulDisksRequestXmlString, "application/vnd.vmware.vcloud.rasdItemsList+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public List<VirtualNetworkCard> getNetworkCards()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    if (this.networkCards == null) {
      this.networkCards = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items) {
            if (item.getResourceType().getValue().equals("10"))
            {
              this.networkCards.add(new VirtualNetworkCard(getVcloudClient(), item));
            }
          }
        }
      }
    }
    return this.networkCards;
  }

  public static List<VirtualNetworkCard> getNetworkCards(VcloudClient client, ReferenceType vAppRef)
    throws VCloudException
  {
    String url = vAppRef.getHref() + "/virtualHardwareSection/networkCards";

    return getNetworkCardsByUrl(client, url);
  }

  private static List<VirtualNetworkCard> getNetworkCardsByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    RasdItemsListType items = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    items = (RasdItemsListType)response.getResource();

    List hardwareItems = new ArrayList();
    if (items != null) {
      for (RASDType item : items.getItem()) {
        hardwareItems.add(new VirtualNetworkCard(item));
      }
    }
    return hardwareItems;
  }

  public Task updateNetworkCards(List<VirtualNetworkCard> virtualNetworkCards)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    String virtualNetworkCardsUpdateUrl = ((AbstractVAppType)getResource()).getHref() + "/virtualHardwareSection/networkCards";

    String virtulNetworkCardsRequestXmlString = JAXBUtil.marshal(of.createRasdItemsList(getRASDItemsList(virtualNetworkCards)));

    logger.log(Level.INFO, "Virtual NetworkCards URL : " + virtualNetworkCardsUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), virtualNetworkCardsUpdateUrl, virtulNetworkCardsRequestXmlString, "application/vnd.vmware.vcloud.rasdItemsList+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  public List<VirtualMedia> getMedias()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    if (this.medias == null) {
      this.medias = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items) {
            if ((item.getResourceType().getValue().equals("14")) || (item.getResourceType().getValue().equals("15")) || (item.getResourceType().getValue().equals("5")))
            {
              this.medias.add(new VirtualMedia(getVcloudClient(), item));
            }
          }
        }
      }
    }
    return this.medias;
  }

  public static List<VirtualMedia> getMedias(VcloudClient client, ReferenceType vAppRef)
    throws VCloudException
  {
    String url = vAppRef.getHref() + "/virtualHardwareSection/media";

    return getMediasByUrl(client, url);
  }

  private static List<VirtualMedia> getMediasByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    RasdItemsListType items = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    items = (RasdItemsListType)response.getResource();

    List hardwareItems = new ArrayList();
    if (items != null) {
      for (RASDType item : items.getItem()) {
        hardwareItems.add(new VirtualMedia(item));
      }
    }
    return hardwareItems;
  }

  private RasdItemsListType getRASDItemsList(List<? extends HardwareItem> items)
  {
    RasdItemsListType rasdItemsListType = new RasdItemsListType();
    List rasdItems = rasdItemsListType.getItem();
    for (HardwareItem item : items) {
      rasdItems.add(item.getItemResource());
    }
    return rasdItemsListType;
  }

  private void sortOvfSectionsAndReferences()
  {
    List sections = ((AbstractVAppType)getResource()).getSection();

    Iterator it = sections.iterator();
    while (it.hasNext()) {
      JAXBElement sec = (JAXBElement)it.next();
      if ((sec.getValue() instanceof VirtualHardwareSectionType)) {
        this.virtualHardwareSection = ((VirtualHardwareSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof OperatingSystemSectionType)) {
        this.operatingSystemSection = ((OperatingSystemSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof NetworkConfigSectionType)) {
        this.networkConfigSection = ((NetworkConfigSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof NetworkConnectionSectionType)) {
        this.networkConnectionSection = ((NetworkConnectionSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof NetworkSectionType))
        this.networkSection = ((NetworkSectionType)sec.getValue());
      else if ((sec.getValue() instanceof LeaseSettingsSectionType)) {
        this.leaseSettingsSection = ((LeaseSettingsSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof StartupSectionType))
        this.startupSection = ((StartupSectionType)sec.getValue());
      else if ((sec.getValue() instanceof GuestCustomizationSectionType)) {
        this.guestCustomizationSection = ((GuestCustomizationSectionType)sec.getValue());
      }
      logger.log(Level.INFO, ((SectionType)sec.getValue()).getClass().getName());
    }

    for (LinkType vAppLink : ((AbstractVAppType)getResource()).getLink()) {
      if ((vAppLink.getRel().equals(RelationType.UP)) && (vAppLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = vAppLink;
      }if ((vAppLink.getRel().equals(RelationType.UP)) && (vAppLink.getType().equals("application/vnd.vmware.vcloud.vApp+xml")))
      {
        this.parentVappReference = vAppLink;
      }
    }

    for (VCloudExtensionType vcloudExtension : ((AbstractVAppType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        this.vimObjectRefType = ((VimObjectRefType)((JAXBElement)jaxbObject).getValue());
      }
    }
    Iterator i$;
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getParentVappReference()
    throws VCloudException
  {
    if (this.parentVappReference != null)
      return this.parentVappReference;
    throw new VCloudException("Reference Not Found");
  }

  public VimObjectRefType getVimObjectRef()
    throws VCloudException
  {
    if (this.vimObjectRefType != null)
      return this.vimObjectRefType;
    throw new VCloudException("Vim Object Reference Type Not Found");
  }

  public NetworkConnectionSectionType getNetworkConnectionSection()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return this.networkConnectionSection;
  }

  public static NetworkConnectionSectionType getNetworkConnectionSection(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappRef.getHref() + "/networkConnectionSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkConnectionSectionType)response.getResource();
  }

  public HashMap<String, NetworkConnectionType> getNetworkConnectionsByName()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    HashMap networks = new HashMap();
    List netConnections = this.networkConnectionSection.getNetworkConnection();

    Iterator it = netConnections.iterator();
    while (it.hasNext()) {
      NetworkConnectionType conn = (NetworkConnectionType)it.next();
      networks.put(conn.getNetwork(), conn);
    }
    return networks;
  }

  public Set<String> getNetworkConnectionNames()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return getNetworkConnectionsByName().keySet();
  }

  public Collection<NetworkConnectionType> getNetworkConnections()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return getNetworkConnectionsByName().values();
  }

  public HashMap<String, String> getIpAddressesByName()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    HashMap networks = new HashMap();
    List netConnections = this.networkConnectionSection.getNetworkConnection();

    Iterator it = netConnections.iterator();
    while (it.hasNext()) {
      NetworkConnectionType conn = (NetworkConnectionType)it.next();
      networks.put(conn.getNetwork(), conn.getIpAddress());
    }
    return networks;
  }

  public String getIpAddress(String networkName)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return (String)getIpAddressesByName().get(networkName);
  }

  public Collection<String> getIpAddresses()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    return getIpAddressesByName().values();
  }

  public Task recomposeVapp(RecomposeVAppParamsType recomposeVappParamsType)
    throws VCloudException
  {
    if (isVm()) {
      throw new VCloudException("Operation not supported on a VM");
    }
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();

    JAXBElement composeVappParams = of.createRecomposeVAppParams(recomposeVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(composeVappParams);

    String postURL = ((AbstractVAppType)getResource()).getHref() + "/action/recomposeVApp";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.recomposeVAppParams+xml", null);

    TaskType taskType = null;

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    taskType = (TaskType)response.getResource();
    return new Task(getVcloudClient(), taskType);
  }

  public Task deploy(boolean powerOn, int lease)
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/action/deploy";

    String content = createVappDeployParamBody(powerOn, lease);

    return executeVappAction(getVcloudClient(), vappActionUrl, content, "application/vnd.vmware.vcloud.deployVAppParams+xml", null, 202);
  }

  public static Task deploy(VcloudClient client, ReferenceType vappRef, boolean powerOn, int lease)
    throws VCloudException
  {
    String url = vappRef.getHref() + "/action/deploy";

    String content = createVappDeployParamBody(powerOn, lease);

    return executeVappAction(client, url, content, "application/vnd.vmware.vcloud.deployVAppParams+xml", null, 202);
  }

  private static String createVappDeployParamBody(boolean powerOn, int lease)
  {
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    DeployVAppParamsType deployParamsType = new DeployVAppParamsType();

    deployParamsType.setPowerOn(Boolean.valueOf(powerOn));
    deployParamsType.setDeploymentLeaseSeconds(Integer.valueOf(lease));

    JAXBElement deployParams = of.createDeployVAppParams(deployParamsType);

    String content = JAXBUtil.marshal(deployParams);
    return content;
  }

  private static String createVappUnDeployParamBody(boolean saveState)
  {
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    UndeployVAppParamsType undeployVAppParamsType = new UndeployVAppParamsType();
    undeployVAppParamsType.setSaveState(Boolean.valueOf(saveState));
    JAXBElement deployParams = of.createUndeployVAppParams(undeployVAppParamsType);

    String content = JAXBUtil.marshal(deployParams);
    return content;
  }

  public Task undeploy(boolean saveState)
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/action/undeploy";

    String content = createVappUnDeployParamBody(saveState);
    return executeVappAction(getVcloudClient(), vappActionUrl, content, "application/vnd.vmware.vcloud.undeployVAppParams+xml", null, 202);
  }

  public static Task undeploy(VcloudClient client, ReferenceType vappRef, boolean saveState)
    throws VCloudException
  {
    String url = vappRef.getHref() + "/action/undeploy";

    String content = createVappUnDeployParamBody(saveState);
    return executeVappAction(client, url, content, "application/vnd.vmware.vcloud.undeployVAppParams+xml", null, 202);
  }

  public Task powerOn()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/powerOn";

    return executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 202);
  }

  public static Task powerOn(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return executeVappAction(client, vappRef.getHref() + "/power/action/powerOn", null, null, null, 202);
  }

  public Task powerOff()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/powerOff";

    return executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 202);
  }

  public static Task powerOff(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return executeVappAction(client, vappRef.getHref() + "/power/action/powerOff", null, null, null, 202);
  }

  public Task reset()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/reset";

    return executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 202);
  }

  public static Task reset(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return executeVappAction(client, vappRef.getHref() + "/power/action/reset", null, null, null, 202);
  }

  public Task suspend()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/suspend";

    return executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 202);
  }

  public static Task suspend(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return executeVappAction(client, vappRef.getHref() + "/power/action/suspend", null, null, null, 202);
  }

  public Task discardSuspend()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/action/discardSuspendedState";

    return executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 202);
  }

  public static Task discardSuspend(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return executeVappAction(client, vappRef.getHref() + "/action/discardSuspendedState", null, null, null, 202);
  }

  public void shutdown()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/shutdown";

    executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 204);
  }

  public static void shutdown(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    executeVappAction(client, vappRef.getHref() + "/power/action/shutdown", null, null, null, 204);
  }

  public void reboot()
    throws VCloudException
  {
    String vappActionUrl = ((AbstractVAppType)getResource()).getHref() + "/power/action/reboot";

    executeVappAction(getVcloudClient(), vappActionUrl, null, null, null, 204);
  }

  public static void reboot(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    executeVappAction(client, vappRef.getHref() + "/power/action/reboot", null, null, null, 204);
  }

  public static Task delete(VcloudClient client, ReferenceType vappRef)
    throws VCloudException
  {
    return asyncDelete(client, vappRef.getHref());
  }

  public Task delete()
    throws VCloudException
  {
    String vappUrl = ((AbstractVAppType)getResource()).getHref();
    Task task = asyncDelete(getVcloudClient(), vappUrl);
    return task;
  }

  private static Task asyncDelete(VcloudClient client, String anyUrl)
    throws VCloudException
  {
    Task task = null;
    Response response = RestUtil.delete(client, anyUrl);
    TaskType taskType = null;

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    taskType = (TaskType)response.getResource();
    task = new Task(client, taskType);

    return task;
  }

  public VmPendingQuestionType getVmQuestion()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String url = ((AbstractVAppType)getResource()).getHref() + "/question";

    Response response = RestUtil.get(getVcloudClient(), url);
    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    VmPendingQuestionType vmPendingQuestionType = (VmPendingQuestionType)response.getResource();

    return vmPendingQuestionType;
  }

  public static VmPendingQuestionType getVmQuestion(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/question";
    Response response = RestUtil.get(client, url);
    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    VmPendingQuestionType vmPendingQuestionType = (VmPendingQuestionType)response.getResource();

    return vmPendingQuestionType;
  }

  private static String createVmQuestionAnswerBody(int choiceId, int questionId)
  {
    com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory of = new com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory();
    VmQuestionAnswerType vmQuestionAnswerType = new VmQuestionAnswerType();
    vmQuestionAnswerType.setChoiceId(choiceId);
    vmQuestionAnswerType.setQuestionId(questionId);

    JAXBElement vmQuestionAnswer = of.createVmQuestionAnswer(vmQuestionAnswerType);

    return JAXBUtil.marshal(vmQuestionAnswer);
  }

  public void answerVmQuestion(int choiceId, int questionId)
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String url = ((AbstractVAppType)getResource()).getHref() + "/question/action/answer";

    Response response = RestUtil.post(getVcloudClient(), url, createVmQuestionAnswerBody(choiceId, questionId), "application/vnd.vmware.vcloud.vmPendingAnswer+xml", null);

    if (!response.isExpected(204))
    {
      response.handleUnExpectedResponse();
    }
  }

  public static void answerVmQuestion(VcloudClient client, ReferenceType vmRef, int choiceId, int questionId)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/question/action/answer";

    Response response = RestUtil.post(client, url, createVmQuestionAnswerBody(choiceId, questionId), "application/vnd.vmware.vcloud.vmPendingAnswer+xml", null);

    if (!response.isExpected(204))
    {
      response.handleUnExpectedResponse();
    }
  }

  public InputStream getScreen()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String url = ((AbstractVAppType)getResource()).getHref() + "/screen";

    return RestUtil.downloadFile(getVcloudClient(), url);
  }

  public ScreenTicketType acquireTicket()
    throws VCloudException
  {
    if (!isVm())
      throw new VCloudException("Operation not supported on a vApp");
    String vappUrl = ((AbstractVAppType)getResource()).getHref();
    String url = vappUrl + "/screen/action/acquireTicket";
    Response response = RestUtil.post(getVcloudClient(), url, null, null, null);

    ScreenTicketType screenTicketType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    screenTicketType = (ScreenTicketType)response.getResource();

    return screenTicketType;
  }

  private static Task executeVappAction(VcloudClient client, String vappActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.post(client, vappActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }
}