package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema.ovf.EnvelopeType;
import com.vmware.vcloud.api.rest.schema.ovf.NetworkSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.ReferencesType;
import com.vmware.vcloud.api.rest.schema.ovf.SectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.CustomizationSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.FilesListType;
import com.vmware.vcloud.api.rest.schema_v1_0.GuestCustomizationSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.LeaseSettingsSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConfigSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConnectionSectionType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppTemplateChildrenType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppTemplateType;
import com.vmware.vcloud.api.rest.schema_v1_0.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class VappTemplate extends VcloudEntity<VAppTemplateType>
{
  private static Logger logger = Logger.getLogger(VappTemplate.class.getSimpleName());
  private List<VappTemplate> childrenvAppTemplates;
  private HashMap<String, com.vmware.vcloud.api.rest.schema_v1_0.FileType> uploadFileMap;
  private HashMap<String, com.vmware.vcloud.api.rest.schema_v1_0.FileType> uploadedFileMap;
  private NetworkConnectionSectionType networkConnectionSection;
  private NetworkConfigSectionType networkConfigSection;
  private NetworkSectionType networkSection;
  private LeaseSettingsSectionType leaseSettingsSection;
  private CustomizationSectionType customizationSection;
  private GuestCustomizationSectionType guestCustomizationSection;
  private ReferenceType vdcReference;
  private ReferenceType catalogItemReference;
  private ReferenceType parentVappTemplateReference;
  private VimObjectRefType vimObjectRefType;

  public VappTemplate(VcloudClient client, VAppTemplateType vAppTemplateType)
  {
    super(client, vAppTemplateType);
    sortVAppTemplateFiles();
    sortOvfSectionAndReferences();
  }

  public List<VappTemplate> getChildren()
  {
    if (this.childrenvAppTemplates == null) {
      this.childrenvAppTemplates = new ArrayList();
      if (((VAppTemplateType)getResource()).getChildren() != null) {
        VAppTemplateChildrenType vAppTemplChildrenType = ((VAppTemplateType)getResource()).getChildren();

        List vAppTypeList = vAppTemplChildrenType.getVm();

        for (VAppTemplateType vAppTemplType : vAppTypeList) {
          this.childrenvAppTemplates.add(new VappTemplate(getVcloudClient(), vAppTemplType));
        }
      }
    }
    return this.childrenvAppTemplates;
  }

  public static VappTemplate getVappTemplateByReference(VcloudClient client, ReferenceType vappTemplRef)
    throws VCloudException
  {
    return getVappTemplate(client, vappTemplRef.getHref());
  }

  private static VappTemplate getVappTemplate(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    VAppTemplateType vAppTemplateType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    vAppTemplateType = (VAppTemplateType)response.getResource();
    VappTemplate vappTempl = new VappTemplate(client, vAppTemplateType);
    return vappTempl;
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getCatalogItemReference()
    throws VCloudException
  {
    if (this.catalogItemReference != null)
      return this.catalogItemReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getParentVappTemplateReference()
    throws VCloudException
  {
    if (this.parentVappTemplateReference != null)
      return this.parentVappTemplateReference;
    throw new VCloudException("Reference Not Found");
  }

  public VimObjectRefType getVimObjectRef()
    throws VCloudException
  {
    if (this.vimObjectRefType != null)
      return this.vimObjectRefType;
    throw new VCloudException("Vim Object Reference Type Not Found");
  }

  private void sortOvfSectionAndReferences()
  {
    List sections = ((VAppTemplateType)getResource()).getSection();

    Iterator it = sections.iterator();
    while (it.hasNext()) {
      JAXBElement sec = (JAXBElement)it.next();
      if ((sec.getValue() instanceof NetworkConfigSectionType)) {
        this.networkConfigSection = ((NetworkConfigSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof NetworkSectionType))
        this.networkSection = ((NetworkSectionType)sec.getValue());
      else if ((sec.getValue() instanceof NetworkConnectionSectionType)) {
        this.networkConnectionSection = ((NetworkConnectionSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof LeaseSettingsSectionType)) {
        this.leaseSettingsSection = ((LeaseSettingsSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof CustomizationSectionType)) {
        this.customizationSection = ((CustomizationSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof GuestCustomizationSectionType)) {
        this.guestCustomizationSection = ((GuestCustomizationSectionType)sec.getValue());
      }
      logger.log(Level.INFO, ((SectionType)sec.getValue()).getClass().getName());
    }

    for (LinkType vAppTemplateLink : ((VAppTemplateType)getResource()).getLink()) {
      if ((vAppTemplateLink.getRel().equals(RelationType.UP)) && (vAppTemplateLink.getType().equals("application/vnd.vmware.vcloud.vAppTemplate+xml")))
      {
        this.parentVappTemplateReference = vAppTemplateLink;
      }if ((vAppTemplateLink.getRel().equals(RelationType.CATALOG_ITEM)) && (vAppTemplateLink.getType().equals("application/vnd.vmware.vcloud.catalogItem+xml")))
      {
        this.catalogItemReference = vAppTemplateLink;
      }if ((vAppTemplateLink.getRel().equals(RelationType.UP)) && (vAppTemplateLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = vAppTemplateLink;
      }
    }

    for (VCloudExtensionType vcloudExtension : ((VAppTemplateType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        this.vimObjectRefType = ((VimObjectRefType)((JAXBElement)jaxbObject).getValue());
      }
    }
    Iterator i$;
  }

  public Integer getVappTemplateStatus()
  {
    return ((VAppTemplateType)getResource()).getStatus();
  }

  public static EnvelopeType getOvf(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    return getOvf(client, vappTemplateRef.getHref() + "/ovf");
  }

  public EnvelopeType getOvf() throws VCloudException
  {
    return getOvf(getVcloudClient(), ((VAppTemplateType)getResource()).getHref() + "/ovf");
  }

  public static EnvelopeType getOvf(VcloudClient client, String vAppTemplateOvfUrl)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vAppTemplateOvfUrl);

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    EnvelopeType ovfEnvelopeType = (EnvelopeType)response.getResource();

    return ovfEnvelopeType;
  }

  private EnvelopeType downloadOvfEnvelope()
    throws VCloudException
  {
    for (LinkType link : ((VAppTemplateType)getResource()).getLink()) {
      if (link.getRel().equals(RelationType.DOWNLOAD_DEFAULT)) {
        return (EnvelopeType)JAXBUtil.unmarshallResource(RestUtil.downloadFile(getVcloudClient(), link.getHref()));
      }
    }

    throw new VCloudException("Download link not found. Check if Download is disabled.");
  }

  public Task delete()
    throws VCloudException
  {
    String vAppTemplateUrl = ((VAppTemplateType)getResource()).getHref();
    Task task = deletevAppTemplate(getVcloudClient(), vAppTemplateUrl);
    return task;
  }

  public static Task delete(VcloudClient client, ReferenceType vAppTemplateRef)
    throws VCloudException
  {
    return deletevAppTemplate(client, vAppTemplateRef.getHref());
  }

  private static Task deletevAppTemplate(VcloudClient client, String vAppTemplateUrl)
    throws VCloudException
  {
    Task task = null;
    Response response = RestUtil.delete(client, vAppTemplateUrl);
    TaskType taskType = null;

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    taskType = (TaskType)response.getResource();
    task = new Task(client, taskType);

    return task;
  }

  public void disableDownload()
    throws VCloudException
  {
    String vAppTemplateDisableDownloadURL = ((VAppTemplateType)getResource()).getHref() + "/action/disableDownload";

    executeVappTemplateAction(getVcloudClient(), vAppTemplateDisableDownloadURL, null, null, null, 204);
  }

  public static void disableDownload(VcloudClient client, ReferenceType vAppTemplateRef)
    throws VCloudException
  {
    String vAppTemplateEnableDownloadURL = vAppTemplateRef.getHref() + "/action/disableDownload";

    executeVappTemplateAction(client, vAppTemplateEnableDownloadURL, null, null, null, 204);
  }

  public Task enableDownload()
    throws VCloudException
  {
    String vAppTemplateEnableDownloadURL = ((VAppTemplateType)getResource()).getHref() + "/action/enableDownload";

    return executeVappTemplateAction(getVcloudClient(), vAppTemplateEnableDownloadURL, null, null, null, 202);
  }

  public static Task enableDownload(VcloudClient client, ReferenceType vAppTemplateRef)
    throws VCloudException
  {
    String vAppTemplateEnableDownloadURL = vAppTemplateRef.getHref() + "/action/enableDownload";

    return executeVappTemplateAction(client, vAppTemplateEnableDownloadURL, null, null, null, 202);
  }

  private static Task executeVappTemplateAction(VcloudClient client, String vappTemplateActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.post(client, vappTemplateActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }

  public Set<String> getUploadFileNames()
    throws VCloudException
  {
    if (!((VAppTemplateType)getResource()).isOvfDescriptorUploaded().booleanValue()) {
      throw new VCloudException("File names not available : OVF Descriptor is Not yet uploaded");
    }

    return this.uploadFileMap.keySet();
  }

  public Set<String> getUploadedFileNames()
    throws VCloudException
  {
    return this.uploadedFileMap.keySet();
  }

  public void uploadOVFFile(InputStream is, long size)
    throws VCloudException
  {
    if (((VAppTemplateType)getResource()).isOvfDescriptorUploaded().booleanValue()) {
      throw new VCloudException("OVF Descriptor is already uploaded");
    }
    if (this.uploadFileMap.size() != 1) {
      throw new VCloudException("Invalid file(s) map size: " + this.uploadFileMap.size());
    }

    Set keys = this.uploadFileMap.keySet();
    String fileName = (String)keys.iterator().next();
    com.vmware.vcloud.api.rest.schema_v1_0.FileType file = (com.vmware.vcloud.api.rest.schema_v1_0.FileType)this.uploadFileMap.get(fileName);

    RestUtil.uploadFile(getVcloudClient(), file, is, size, 0L, 0L);
  }

  public void uploadFile(String fileName, InputStream is, long size)
    throws VCloudException
  {
    if (!((VAppTemplateType)getResource()).isOvfDescriptorUploaded().booleanValue()) {
      throw new VCloudException("Invalid State: OVF Descriptor is Not yet uploaded");
    }

    if (this.uploadFileMap.isEmpty()) {
      throw new VCloudException("No file(s) to upload ");
    }

    com.vmware.vcloud.api.rest.schema_v1_0.FileType file = (com.vmware.vcloud.api.rest.schema_v1_0.FileType)this.uploadFileMap.get(fileName);
    if (file != null)
    {
      RestUtil.uploadFile(getVcloudClient(), file, is, size, 0L, 0L);
    }
    else throw new VCloudException("File Name Not found");
  }

  public void uploadFile(String fileName, InputStream is, long startByte, long endByte)
    throws VCloudException
  {
    if (!((VAppTemplateType)getResource()).isOvfDescriptorUploaded().booleanValue()) {
      throw new VCloudException("Invalid State: OVF Descriptor is Not yet uploaded");
    }

    if (this.uploadFileMap.isEmpty()) {
      throw new VCloudException("No file(s) to upload ");
    }

    com.vmware.vcloud.api.rest.schema_v1_0.FileType file = (com.vmware.vcloud.api.rest.schema_v1_0.FileType)this.uploadFileMap.get(fileName);
    if (file != null) {
      RestUtil.uploadFile(getVcloudClient(), file, is, file.getSize().longValue(), startByte, endByte);
    }
    else
      throw new VCloudException("File Name Not found");
  }

  public InputStream downloadOVFFile()
    throws VCloudException
  {
    for (LinkType link : ((VAppTemplateType)getResource()).getLink()) {
      if (link.getRel().equals(RelationType.DOWNLOAD_DEFAULT)) {
        return RestUtil.downloadFile(getVcloudClient(), link.getHref());
      }
    }

    throw new VCloudException("Download link not found. Check if Download is disabled.");
  }

  public String getOVFDownloadURL()
    throws VCloudException
  {
    for (LinkType link : ((VAppTemplateType)getResource()).getLink()) {
      if (link.getRel().equals(RelationType.DOWNLOAD_DEFAULT))
        return link.getHref();
    }
    throw new VCloudException("Download link not found. Check if Download is disabled.");
  }

  public HashMap<String, com.vmware.vcloud.api.rest.schema.ovf.FileType> getDownloadFileNames()
    throws VCloudException
  {
    HashMap downloadFileMap = new HashMap();

    ReferencesType fileReferences = downloadOvfEnvelope().getReferences();
    if (fileReferences != null) {
      List files = fileReferences.getFile();

      for (com.vmware.vcloud.api.rest.schema.ovf.FileType file : files) {
        if (!downloadFileMap.containsKey(file.getHref()))
          downloadFileMap.put(file.getHref(), file);
      }
    }
    return downloadFileMap;
  }

  public InputStream downloadFile(String fileName)
    throws VCloudException
  {
    for (LinkType link : ((VAppTemplateType)getResource()).getLink()) {
      if (link.getRel().equals(RelationType.DOWNLOAD_DEFAULT)) {
        HashMap downloadFiles = getDownloadFileNames();
        if (!downloadFiles.containsKey(fileName)) {
          throw new VCloudException("File name not found, Give a valid file name for download");
        }
        return RestUtil.downloadFile(getVcloudClient(), link.getHref().replaceAll("descriptor.ovf", fileName));
      }
    }

    throw new VCloudException("Download link not found. Check if Download is disabled.");
  }

  public HashMap<String, com.vmware.vcloud.api.rest.schema_v1_0.FileType> monitorUpload()
    throws VCloudException
  {
    HashMap filesStatusMap = new HashMap();
    VappTemplate vappTemplate = getVappTemplate(getVcloudClient(), ((VAppTemplateType)getResource()).getHref());

    if (((VAppTemplateType)getResource()).getFiles() != null) {
      FilesListType filesType = ((VAppTemplateType)vappTemplate.getResource()).getFiles();
      List files = filesType.getFile();
      for (com.vmware.vcloud.api.rest.schema_v1_0.FileType file : files) {
        filesStatusMap.put(file.getName(), file);
      }
    }
    return filesStatusMap;
  }

  public Task updateVappTemplate(VAppTemplateType vappTemplateType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String vappTemplateUpdateUrl = ((VAppTemplateType)getResource()).getHref();
    String vappTemplateRequestXmlString = JAXBUtil.marshal(of.createVAppTemplate(vappTemplateType));

    logger.log(Level.INFO, "VAPPTEMPLATE URL : " + vappTemplateUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), vappTemplateUpdateUrl, vappTemplateRequestXmlString, "application/vnd.vmware.vcloud.vAppTemplate+xml", null);

    TaskType taskType = null;

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    taskType = (TaskType)response.getResource();
    Task task = new Task(getVcloudClient(), taskType);

    return task;
  }

  public NetworkSectionType getNetworkSection()
  {
    return this.networkSection;
  }

  public static NetworkSectionType getNetworkSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/networkSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkSectionType)response.getResource();
  }

  public NetworkConfigSectionType getNetworkConfigSection()
  {
    return this.networkConfigSection;
  }

  public static NetworkConfigSectionType getNetworkConfigSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/networkConfigSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkConfigSectionType)response.getResource();
  }

  public LeaseSettingsSectionType getLeaseSettingsSection()
  {
    return this.leaseSettingsSection;
  }

  public static LeaseSettingsSectionType getLeaseSettingsSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/leaseSettingsSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (LeaseSettingsSectionType)response.getResource();
  }

  public NetworkConnectionSectionType getNetworkConnectionSection()
  {
    return this.networkConnectionSection;
  }

  public static NetworkConnectionSectionType getNetworkConnectionSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/networkConnectionSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (NetworkConnectionSectionType)response.getResource();
  }

  public GuestCustomizationSectionType getGuestCustomizationSection()
  {
    return this.guestCustomizationSection;
  }

  public static GuestCustomizationSectionType getGuestCustomizationSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/guestCustomizationSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (GuestCustomizationSectionType)response.getResource();
  }

  public CustomizationSectionType getCustomizationSection()
  {
    return this.customizationSection;
  }

  public static CustomizationSectionType getCustomizationSection(VcloudClient client, ReferenceType vappTemplateRef)
    throws VCloudException
  {
    Response response = RestUtil.get(client, vappTemplateRef.getHref() + "/customizationSection/");

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    return (CustomizationSectionType)response.getResource();
  }

  public Task updateSection(SectionType sectionType)
    throws VCloudException
  {
    ObjectFactory vCloudObjectFactory = new ObjectFactory();
    String sectionUrl = ((VAppTemplateType)getResource()).getHref();
    String sectionContentType = null;
    String sectionRequestString = null;
    if ((sectionType instanceof LeaseSettingsSectionType)) {
      JAXBElement section = vCloudObjectFactory.createLeaseSettingsSection((LeaseSettingsSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/leaseSettingsSection/";

      sectionContentType = "application/vnd.vmware.vcloud.leaseSettingsSection+xml";
    } else if ((sectionType instanceof GuestCustomizationSectionType)) {
      JAXBElement section = vCloudObjectFactory.createGuestCustomizationSection((GuestCustomizationSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/guestCustomizationSection/";

      sectionContentType = "application/vnd.vmware.vcloud.guestCustomizationSection+xml";
    } else if ((sectionType instanceof CustomizationSectionType)) {
      JAXBElement section = vCloudObjectFactory.createCustomizationSection((CustomizationSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/customizationSection/";
      sectionContentType = "application/vnd.vmware.vcloud.customizationSection+xml";
    } else {
      logger.log(Level.INFO, sectionType.getClass().getName() + " Not a valid Section to Update");

      throw new VCloudException("Not a Valid Section to Update");
    }
    Response response = RestUtil.put(getVcloudClient(), sectionUrl, sectionRequestString, sectionContentType, null);

    if (!response.isExpected(202))
    {
      response.handleUnExpectedResponse();
    }
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(getVcloudClient(), taskType);
    return task;
  }

  private void sortVAppTemplateFiles()
  {
    this.uploadFileMap = new HashMap();
    this.uploadedFileMap = new HashMap();
    FilesListType fileList = ((VAppTemplateType)getResource()).getFiles();
    if (fileList == null) {
      return;
    }
    List files = fileList.getFile();

    Iterator fileIt = files.iterator();

    while (fileIt.hasNext()) {
      com.vmware.vcloud.api.rest.schema_v1_0.FileType file = (com.vmware.vcloud.api.rest.schema_v1_0.FileType)fileIt.next();
      String fileName = file.getName();
      if (file.getSize().compareTo(file.getBytesTransferred()) == 0)
        this.uploadedFileMap.put(fileName, file);
      else
        this.uploadFileMap.put(fileName, file);
    }
  }
}