package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkConfigurationType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.OrgNetworkType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminOrgNetwork extends VcloudEntity<OrgNetworkType>
{
  private static Logger logger = Logger.getLogger(AdminOrgNetwork.class.getSimpleName());
  private ReferenceType orgNetworkReference;
  private ReferenceType adminOrgReference;

  AdminOrgNetwork(VcloudClient client, OrgNetworkType orgNetworkType)
  {
    super(client, orgNetworkType);
    sortRefs();
  }

  public ReferenceType getAdminOrganizationReference()
    throws VCloudException
  {
    if (this.adminOrgReference != null)
      return this.adminOrgReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getOrgNetworkReference()
    throws VCloudException
  {
    if (this.orgNetworkReference != null)
      return this.orgNetworkReference;
    throw new VCloudException("Reference Not Found");
  }

  private void sortRefs()
  {
    for (LinkType adminOrgNetworkLink : ((OrgNetworkType)getResource()).getLink()) {
      if ((adminOrgNetworkLink.getRel().equals(RelationType.UP)) && (adminOrgNetworkLink.getType().equals("application/vnd.vmware.admin.organization+xml")))
      {
        this.adminOrgReference = adminOrgNetworkLink;
      }if ((adminOrgNetworkLink.getRel().equals(RelationType.ALTERNATE)) && (adminOrgNetworkLink.getType().equals("application/vnd.vmware.vcloud.network+xml")))
      {
        this.orgNetworkReference = adminOrgNetworkLink;
      }
    }
  }

  public NetworkConfigurationType getConfiguration()
  {
    return ((OrgNetworkType)getResource()).getConfiguration();
  }

  public static AdminOrgNetwork getOrgNetworkByReference(VcloudClient client, ReferenceType orgNetworkRef)
    throws VCloudException
  {
    return get(client, orgNetworkRef.getHref());
  }

  private static AdminOrgNetwork get(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: OrgNetwork " + url);
    Response response = RestUtil.get(client, url);
    OrgNetworkType orgNetworkType = null;
    if (response.isExpected(200))
      orgNetworkType = (OrgNetworkType)response.getResource();
    else
      response.handleUnExpectedResponse();
    AdminOrgNetwork orgNetwork = new AdminOrgNetwork(client, orgNetworkType);
    return orgNetwork;
  }

  public Task updateOrgNetwork(OrgNetworkType orgNetworkType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String orgNetworkUpdateUrl = ((OrgNetworkType)getResource()).getHref();
    String orgNetworkUpdateRequestXmlString = JAXBUtil.marshal(of.createOrgNetwork(orgNetworkType));

    logger.log(Level.INFO, "Update OrgNetwork URL : " + orgNetworkUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), orgNetworkUpdateUrl, orgNetworkUpdateRequestXmlString, "application/vnd.vmware.admin.network+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202))
      taskType = (TaskType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new Task(getVcloudClient(), taskType);
  }

  public Task delete()
    throws VCloudException
  {
    String orgNetworkUrl = ((OrgNetworkType)getResource()).getHref();
    return deleteOrgNetwork(getVcloudClient(), orgNetworkUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType orgNetworkRef)
    throws VCloudException
  {
    return deleteOrgNetwork(client, orgNetworkRef.getHref());
  }

  private static Task deleteOrgNetwork(VcloudClient client, String orgNetworkUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, orgNetworkUrl);
    TaskType taskType = null;
    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(client, taskType);

    return task;
  }
}