package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.AdminOrgType;
import com.vmware.vcloud.api.rest.schema_v1_0.AdminVdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.CatalogType;
import com.vmware.vcloud.api.rest.schema_v1_0.CatalogsListType;
import com.vmware.vcloud.api.rest.schema_v1_0.GroupType;
import com.vmware.vcloud.api.rest.schema_v1_0.GroupsListType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.OrgNetworkType;
import com.vmware.vcloud.api.rest.schema_v1_0.OrgSettingsType;
import com.vmware.vcloud.api.rest.schema_v1_0.OrganizationReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.UserType;
import com.vmware.vcloud.api.rest.schema_v1_0.UsersListType;
import com.vmware.vcloud.api.rest.schema_v1_0.VdcsType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class AdminOrganization extends VcloudEntity<AdminOrgType>
{
  private static Logger logger = Logger.getLogger(AdminOrganization.class.getSimpleName());
  private HashMap<String, ReferenceType> userRefsByName;
  private HashMap<String, ReferenceType> catalogRefsByName;
  private HashMap<String, ReferenceType> groupRefsByName;
  private HashMap<String, ReferenceType> adminVdcRefsByName;
  private HashMap<String, ReferenceType> orgNetworkRefsByName;
  private ReferenceType orgReference;
  private ReferenceType tasksListRef;

  AdminOrganization(VcloudClient client, AdminOrgType adminOrgType)
  {
    super(client, adminOrgType);
    sortAdminOrgRefs();
  }

  public static AdminOrganization getAdminOrgByReference(VcloudClient client, OrganizationReferenceType orgRef)
    throws VCloudException
  {
    return getAdminOrg(client, orgRef.getHref());
  }

  private static AdminOrganization getAdminOrg(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: AdminOrganization  " + url);
    Response response = RestUtil.get(client, url);
    AdminOrgType adminOrgType = null;
    if (response.isExpected(200))
      adminOrgType = (AdminOrgType)response.getResource();
    else
      response.handleUnExpectedResponse();
    AdminOrganization adminOrg = new AdminOrganization(client, adminOrgType);
    return adminOrg;
  }

  public OrgSettingsType getSettings()
  {
    return ((AdminOrgType)getResource()).getSettings();
  }

  public AdminCatalog createCatalog(CatalogType catalogType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement catalogParams = of.createCatalog(catalogType);
    String requestBodyXml = JAXBUtil.marshal(catalogParams);
    String postURL = ((AdminOrgType)getResource()).getHref() + "/catalogs";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.catalog+xml", null);

    CatalogType addedAdminCatalogType = null;
    if (response.isExpected(201))
      addedAdminCatalogType = (CatalogType)response.getResource();
    else
      response.handleUnExpectedResponse();
    return new AdminCatalog(getVcloudClient(), addedAdminCatalogType);
  }

  public User createUser(UserType userType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement userParams = of.createUser(userType);
    String requestBodyXml = JAXBUtil.marshal(userParams);
    String postURL = ((AdminOrgType)getResource()).getHref() + "/users";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.user+xml", null);

    UserType addedUserType = null;
    if (response.isExpected(201))
      addedUserType = (UserType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new User(getVcloudClient(), addedUserType);
  }

  public Group importGroup(GroupType groupType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement groupParams = of.createGroup(groupType);
    String requestBodyXml = JAXBUtil.marshal(groupParams);
    String postURL = ((AdminOrgType)getResource()).getHref() + "/groups";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.group+xml", null);

    GroupType addedGroupType = null;
    if (response.isExpected(201))
      addedGroupType = (GroupType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new Group(getVcloudClient(), addedGroupType);
  }

  public AdminVdc createAdminVdc(AdminVdcType adminVdcType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement adminVdcParams = of.createAdminVdc(adminVdcType);

    String requestBodyXml = JAXBUtil.marshal(adminVdcParams);
    String postURL = ((AdminOrgType)getResource()).getHref() + "/vdcs";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.vdc+xml", null);

    AdminVdcType addedAdminVdcType = null;
    if (response.isExpected(201))
      addedAdminVdcType = (AdminVdcType)response.getResource();
    else
      response.handleUnExpectedResponse();
    return new AdminVdc(getVcloudClient(), addedAdminVdcType);
  }

  public AdminOrgNetwork createOrgNetwork(OrgNetworkType orgNetworkType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement orgNetworkParams = of.createOrgNetwork(orgNetworkType);

    String requestBodyXml = JAXBUtil.marshal(orgNetworkParams);
    String postURL = ((AdminOrgType)getResource()).getHref() + "/networks";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.network+xml", null);

    OrgNetworkType addedOrgNetworkType = null;
    if (response.isExpected(201))
      addedOrgNetworkType = (OrgNetworkType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new AdminOrgNetwork(getVcloudClient(), addedOrgNetworkType);
  }

  public ReferenceType getOrgReference()
    throws VCloudException
  {
    if (this.orgReference != null)
      return this.orgReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getTaskListReference()
    throws VCloudException
  {
    if (this.tasksListRef != null)
      return this.tasksListRef;
    throw new VCloudException("Reference Not Found");
  }

  private void sortAdminOrgRefs()
  {
    logger.log(Level.INFO, "SortAdminOrgRefs  ");

    this.userRefsByName = new HashMap();
    this.groupRefsByName = new HashMap();
    this.catalogRefsByName = new HashMap();
    this.adminVdcRefsByName = new HashMap();
    this.orgNetworkRefsByName = new HashMap();

    if (((AdminOrgType)getResource()).getUsers() != null) {
      for (ReferenceType userRef : ((AdminOrgType)getResource()).getUsers().getUserReference())
      {
        this.userRefsByName.put(userRef.getName(), userRef);
      }
    }
    if (((AdminOrgType)getResource()).getGroups() != null) {
      for (ReferenceType groupRef : ((AdminOrgType)getResource()).getGroups().getGroupReference())
      {
        this.groupRefsByName.put(groupRef.getName(), groupRef);
      }
    }
    if (((AdminOrgType)getResource()).getCatalogs() != null) {
      for (ReferenceType catalogRef : ((AdminOrgType)getResource()).getCatalogs().getCatalogReference())
      {
        this.catalogRefsByName.put(catalogRef.getName(), catalogRef);
      }
    }
    if (((AdminOrgType)getResource()).getVdcs() != null) {
      for (ReferenceType vdcRef : ((AdminOrgType)getResource()).getVdcs().getVdc())
      {
        this.adminVdcRefsByName.put(vdcRef.getName(), vdcRef);
      }
    }
    if (((AdminOrgType)getResource()).getNetworks() != null) {
      for (ReferenceType networkRef : ((AdminOrgType)getResource()).getNetworks().getNetwork())
      {
        this.orgNetworkRefsByName.put(networkRef.getName(), networkRef);
      }
    }
    for (LinkType adminOrgLink : ((AdminOrgType)getResource()).getLink()) {
      if ((adminOrgLink.getRel().equals(RelationType.ALTERNATE)) && (adminOrgLink.getType().equals("application/vnd.vmware.vcloud.org+xml")))
      {
        this.orgReference = adminOrgLink;
      }if ((adminOrgLink.getRel().equals(RelationType.DOWN)) && (adminOrgLink.getType().equals("application/vnd.vmware.vcloud.tasksList+xml")))
      {
        this.tasksListRef = adminOrgLink;
      }
    }
  }

  public AdminOrganization updateAdminOrg(AdminOrgType adminOrgType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String adminOrgUpdateUrl = ((AdminOrgType)getResource()).getHref();
    String adminOrgUpdateRequestXmlString = JAXBUtil.marshal(of.createAdminOrg(adminOrgType));

    logger.log(Level.INFO, "Update ADMIN ORG URL : " + adminOrgUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), adminOrgUpdateUrl, adminOrgUpdateRequestXmlString, "application/vnd.vmware.admin.organization+xml", null);

    AdminOrgType updatedAdminOrgType = null;

    if (response.isExpected(200))
      updatedAdminOrgType = (AdminOrgType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new AdminOrganization(getVcloudClient(), updatedAdminOrgType);
  }

  public void delete()
    throws VCloudException
  {
    String adminOrgUrl = ((AdminOrgType)getResource()).getHref();
    deleteAdminOrg(getVcloudClient(), adminOrgUrl);
  }

  public static void delete(VcloudClient client, ReferenceType adminOrgRef)
    throws VCloudException
  {
    deleteAdminOrg(client, adminOrgRef.getHref());
  }

  private static void deleteAdminOrg(VcloudClient client, String adminOrgUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, adminOrgUrl);
    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = ((AdminOrgType)getResource()).getHref() + "/action/enable";

    executeOrgAction(getVcloudClient(), enableURL, null, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType orgRef)
    throws VCloudException
  {
    String enableURL = orgRef.getHref() + "/action/enable";

    executeOrgAction(client, enableURL, null, null, null, 204);
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = ((AdminOrgType)getResource()).getHref() + "/action/disable";

    executeOrgAction(getVcloudClient(), disableUrl, null, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType adminOrgRef)
    throws VCloudException
  {
    String enableURL = adminOrgRef.getHref() + "/action/disable";

    executeOrgAction(client, enableURL, null, null, null, 204);
  }

  private static Task executeOrgAction(VcloudClient client, String orgActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    logger.log(Level.INFO, "POST: Org Action URL " + orgActionUrl);
    Response response = RestUtil.post(client, orgActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }

  public HashMap<String, ReferenceType> getUserRefsByName()
  {
    return this.userRefsByName;
  }

  public Collection<ReferenceType> getUserRefs()
  {
    return this.userRefsByName.values();
  }

  public ReferenceType getUserRefByName(String name)
  {
    return (ReferenceType)this.userRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getGroupRefsByName()
  {
    return this.groupRefsByName;
  }

  public Collection<ReferenceType> getGroupRefs()
  {
    return this.groupRefsByName.values();
  }

  public ReferenceType getGroupRefByName(String name)
  {
    return (ReferenceType)this.groupRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getCatalogRefsByName()
  {
    return this.catalogRefsByName;
  }

  public Collection<ReferenceType> getCatalogRefs()
  {
    return this.catalogRefsByName.values();
  }

  public ReferenceType getCatalogRefByName(String name)
  {
    return (ReferenceType)this.catalogRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getAdminVdcRefsByName()
  {
    return this.adminVdcRefsByName;
  }

  public Collection<ReferenceType> getAdminVdcRefs()
  {
    return this.adminVdcRefsByName.values();
  }

  public ReferenceType getAdminVdcRefByName(String name)
  {
    return (ReferenceType)this.adminVdcRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getAdminOrgNetworkRefsByName()
  {
    return this.orgNetworkRefsByName;
  }

  public Collection<ReferenceType> getAdminOrgNetworkRefs()
  {
    return this.orgNetworkRefsByName.values();
  }

  public ReferenceType getAdminOrgNetworkRefByName(String name)
  {
    return (ReferenceType)this.orgNetworkRefsByName.get(name);
  }
}