package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.GroupType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.UsersListType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Group extends VcloudEntity<GroupType>
{
  private static Logger logger = Logger.getLogger(Group.class.getSimpleName());
  private HashMap<String, ReferenceType> userRefsByName;

  Group(VcloudClient client, GroupType groupType)
  {
    super(client, groupType);
    sortRefs();
  }

  public static Group getGroupByReference(VcloudClient client, ReferenceType groupRef)
    throws VCloudException
  {
    return get(client, groupRef.getHref());
  }

  private static Group get(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: Group " + url);
    Response response = RestUtil.get(client, url);
    GroupType groupType = null;
    if (response.isExpected(200))
      groupType = (GroupType)response.getResource();
    else
      response.handleUnExpectedResponse();
    Group group = new Group(client, groupType);
    return group;
  }

  public Group updateGroup(GroupType groupType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String groupUpdateUrl = ((GroupType)getResource()).getHref();
    String groupUpdateRequestXmlString = JAXBUtil.marshal(of.createGroup(groupType));

    logger.log(Level.INFO, "Update Group URL : " + groupUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), groupUpdateUrl, groupUpdateRequestXmlString, "application/vnd.vmware.admin.group+xml", null);

    GroupType updatedGroupType = null;
    if (response.isExpected(200))
      updatedGroupType = (GroupType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new Group(getVcloudClient(), updatedGroupType);
  }

  public void delete()
    throws VCloudException
  {
    String groupUrl = ((GroupType)getResource()).getHref();
    deleteGroup(getVcloudClient(), groupUrl);
  }

  public static void delete(VcloudClient client, ReferenceType groupRef)
    throws VCloudException
  {
    deleteGroup(client, groupRef.getHref());
  }

  private static void deleteGroup(VcloudClient client, String groupUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, groupUrl);
    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }

  private void sortRefs()
  {
    this.userRefsByName = new HashMap();
    if (((GroupType)getResource()).getUsersList() != null)
      for (ReferenceType userRef : ((GroupType)getResource()).getUsersList().getUserReference())
      {
        this.userRefsByName.put(userRef.getName(), userRef);
      }
  }

  public ReferenceType getRoleReference()
  {
    return ((GroupType)getResource()).getRole();
  }

  public HashMap<String, ReferenceType> getUserRefsByName()
  {
    return this.userRefsByName;
  }

  public Collection<ReferenceType> getUserRefs()
  {
    return this.userRefsByName.values();
  }

  public ReferenceType getUserRefByName(String userName)
  {
    return (ReferenceType)this.userRefsByName.get(userName);
  }
}