package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkPoolReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ProviderVdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.VdcReferencesType;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ProviderVdc extends VcloudEntity<ProviderVdcType>
{
  private static Logger logger = Logger.getLogger(ProviderVdc.class.getSimpleName());
  private HashMap<String, ReferenceType> externalNetworkRefsByName;
  private HashMap<String, ReferenceType> networkPoolRefsByName;

  ProviderVdc(VcloudClient client, ProviderVdcType providerVdcType)
  {
    super(client, providerVdcType);
    sortRefs();
  }

  public static ProviderVdc getProviderVdcByReference(VcloudClient client, ReferenceType providerVdcRef)
    throws VCloudException
  {
    return get(client, providerVdcRef.getHref());
  }

  private static ProviderVdc get(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: ProviderVdc " + url);
    Response response = RestUtil.get(client, url);
    ProviderVdcType providerVdcType = null;
    if (response.isExpected(200))
      providerVdcType = (ProviderVdcType)response.getResource();
    else
      response.handleUnExpectedResponse();
    ProviderVdc providerVdc = new ProviderVdc(client, providerVdcType);
    return providerVdc;
  }

  private void sortRefs()
  {
    this.externalNetworkRefsByName = new HashMap();
    this.networkPoolRefsByName = new HashMap();

    if (((ProviderVdcType)getResource()).getAvailableNetworks() != null) {
      for (ReferenceType externalNetworkRef : ((ProviderVdcType)getResource()).getAvailableNetworks().getNetwork())
      {
        this.externalNetworkRefsByName.put(externalNetworkRef.getName(), externalNetworkRef);
      }
    }

    if (((ProviderVdcType)getResource()).getNetworkPoolReferences() != null)
      for (ReferenceType networkPoolRef : ((ProviderVdcType)getResource()).getNetworkPoolReferences().getNetworkPoolReference())
      {
        this.networkPoolRefsByName.put(networkPoolRef.getName(), networkPoolRef);
      }
  }

  public static HashMap<String, ReferenceType> getAdminVdcRefsByName(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    String url = vmwProviderVdcRef.getHref() + "/vdcReferences";

    return getAdminVdcRefs(client, url);
  }

  public HashMap<String, ReferenceType> getAdminVdcRefsByName()
    throws VCloudException
  {
    String url = ((ProviderVdcType)getResource()).getHref() + "/vdcReferences";

    return getAdminVdcRefs(getVcloudClient(), url);
  }

  private static HashMap<String, ReferenceType> getAdminVdcRefs(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET Admin Vdc References " + url);
    Response response = RestUtil.get(client, url);
    VdcReferencesType vdcReferencesType = null;

    if (response.isExpected(200))
      vdcReferencesType = (VdcReferencesType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    HashMap adminVdcRefs = new HashMap();
    for (ReferenceType vdcReference : vdcReferencesType.getVdcReference())
      adminVdcRefs.put(vdcReference.getName(), vdcReference);
    return adminVdcRefs;
  }

  public HashMap<String, ReferenceType> getExternalNetworkRefsByName()
  {
    return this.externalNetworkRefsByName;
  }

  public Collection<ReferenceType> getExternalNetworkRefs()
  {
    return this.externalNetworkRefsByName.values();
  }

  public ReferenceType getExternalNetworkRefByName(String externalNetworkName)
  {
    return (ReferenceType)this.externalNetworkRefsByName.get(externalNetworkName);
  }

  public HashMap<String, ReferenceType> getVMWNetworkPoolRefsByName()
  {
    return this.networkPoolRefsByName;
  }

  public Collection<ReferenceType> getVMWNetworkPoolRefs()
  {
    return this.networkPoolRefsByName.values();
  }

  public ReferenceType getVMWNetworkPoolRefByName(String networkPoolName)
  {
    return (ReferenceType)this.networkPoolRefsByName.get(networkPoolName);
  }
}