package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.GroupsListType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.UserType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class User extends VcloudEntity<UserType>
{
  private static Logger logger = Logger.getLogger(User.class.getSimpleName());
  private HashMap<String, ReferenceType> groupRefsByName;

  User(VcloudClient client, UserType userType)
  {
    super(client, userType);
    sortReferences();
  }

  private void sortReferences()
  {
    this.groupRefsByName = new HashMap();

    GroupsListType groupList = ((UserType)getResource()).getGroupReferences();
    if (groupList != null)
      for (ReferenceType groupRef : groupList.getGroupReference())
        this.groupRefsByName.put(groupRef.getName(), groupRef);
  }

  public static User getUserByReference(VcloudClient client, ReferenceType userRef)
    throws VCloudException
  {
    return get(client, userRef.getHref());
  }

  private static User get(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: User " + url);

    Response response = RestUtil.get(client, url);
    UserType userType = null;
    if (response.isExpected(200))
      userType = (UserType)response.getResource();
    else
      response.handleUnExpectedResponse();
    User user = new User(client, userType);
    return user;
  }

  public User updateUser(UserType userType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String userUpdateUrl = ((UserType)getResource()).getHref();
    String userUpdateRequestXmlString = JAXBUtil.marshal(of.createUser(userType));

    logger.log(Level.INFO, "Update User URL : " + userUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), userUpdateUrl, userUpdateRequestXmlString, "application/vnd.vmware.admin.user+xml", null);

    UserType updatedUserType = null;
    if (response.isExpected(200))
      updatedUserType = (UserType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new User(getVcloudClient(), updatedUserType);
  }

  public void delete()
    throws VCloudException
  {
    String userUrl = ((UserType)getResource()).getHref();
    deleteUser(getVcloudClient(), userUrl);
  }

  public static void delete(VcloudClient client, ReferenceType userRef)
    throws VCloudException
  {
    deleteUser(client, userRef.getHref());
  }

  private static void deleteUser(VcloudClient client, String userUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, userUrl);
    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }

  public ReferenceType getRoleReference()
    throws VCloudException
  {
    return ((UserType)getResource()).getRole();
  }

  public HashMap<String, ReferenceType> getGroupRefsByName()
  {
    return this.groupRefsByName;
  }

  public ReferenceType getGroupRefByName(String groupName)
  {
    return (ReferenceType)this.groupRefsByName.get(groupName);
  }

  public Collection<ReferenceType> getGroupRefs()
  {
    return this.groupRefsByName.values();
  }
}