package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.AdminOrgType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.OrganizationReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.OrganizationReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ProviderVdcReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RightReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.RoleReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.RoleType;
import com.vmware.vcloud.api.rest.schema_v1_0.VCloudType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import java.util.Collection;
import java.util.HashMap;
import javax.xml.bind.JAXBElement;

public class VcloudAdmin
{
  private VCloudType vcloudResource;
  private VcloudClient client;
  private HashMap<String, OrganizationReferenceType> adminOrganizationRefsByName;
  private HashMap<String, ReferenceType> providerVdcRefsByName;
  private HashMap<String, ReferenceType> rightRefsByName;
  private HashMap<String, ReferenceType> roleRefsByName;
  private HashMap<String, ReferenceType> externalNetworkRefsByName;

  public VcloudAdmin(VcloudClient client)
    throws VCloudException
  {
    this.client = client;

    Response response = RestUtil.get(client, client.getVcloudApiURL() + "/admin/");

    VCloudType vCloudType = null;
    if (response.isExpected(200))
      vCloudType = (VCloudType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    this.vcloudResource = vCloudType;

    sortAdminRefs();
  }

  public VCloudType getResource()
  {
    return this.vcloudResource;
  }

  private void sortAdminRefs()
  {
    this.adminOrganizationRefsByName = new HashMap();
    this.providerVdcRefsByName = new HashMap();
    this.rightRefsByName = new HashMap();
    this.roleRefsByName = new HashMap();
    this.externalNetworkRefsByName = new HashMap();

    if (getResource().getOrganizationReferences() != null) {
      for (OrganizationReferenceType orgRef : this.vcloudResource.getOrganizationReferences().getOrganizationReference())
      {
        this.adminOrganizationRefsByName.put(orgRef.getName(), orgRef);
      }
    }
    if (getResource().getProviderVdcReferences() != null) {
      for (ReferenceType providerVdcRef : this.vcloudResource.getProviderVdcReferences().getProviderVdcReference())
      {
        this.providerVdcRefsByName.put(providerVdcRef.getName(), providerVdcRef);
      }
    }

    if (getResource().getRightReferences() != null) {
      for (ReferenceType rightRef : this.vcloudResource.getRightReferences().getRightReference())
      {
        this.rightRefsByName.put(rightRef.getName(), rightRef);
      }
    }
    if (getResource().getRoleReferences() != null) {
      for (ReferenceType roleRef : this.vcloudResource.getRoleReferences().getRoleReference())
      {
        this.roleRefsByName.put(roleRef.getName(), roleRef);
      }
    }
    if (getResource().getNetworks() != null)
      for (ReferenceType networkRef : this.vcloudResource.getNetworks().getNetwork())
      {
        this.externalNetworkRefsByName.put(networkRef.getName(), networkRef);
      }
  }

  public AdminOrganization getSystemAdminOrg()
    throws VCloudException
  {
    OrganizationReferenceType systemAdminRef = new OrganizationReferenceType();
    systemAdminRef.setHref(this.client.getVcloudApiURL() + "/admin" + "/org/" + "1");

    return AdminOrganization.getAdminOrgByReference(this.client, systemAdminRef);
  }

  public Role createRole(RoleType roleType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement roleParams = of.createRole(roleType);
    String requestBodyXml = JAXBUtil.marshal(roleParams);
    String postURL = this.vcloudResource.getHref() + "roles";

    Response response = RestUtil.post(this.client, postURL, requestBodyXml, "application/vnd.vmware.admin.role+xml", null);

    RoleType addedRoleType = null;
    if (response.isExpected(201))
      addedRoleType = (RoleType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    Role role = new Role(this.client, addedRoleType);
    return role;
  }

  public AdminOrganization createAdminOrg(AdminOrgType adminOrgType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement adminOrgParams = of.createAdminOrg(adminOrgType);

    String requestBodyXml = JAXBUtil.marshal(adminOrgParams);
    String postURL = this.vcloudResource.getHref() + "orgs";

    Response response = RestUtil.post(this.client, postURL, requestBodyXml, "application/vnd.vmware.admin.organization+xml", null);

    AdminOrgType addedAdminOrgType = null;
    if (response.isExpected(201))
      addedAdminOrgType = (AdminOrgType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    AdminOrganization adminOrganization = new AdminOrganization(this.client, addedAdminOrgType);

    return adminOrganization;
  }

  public HashMap<String, OrganizationReferenceType> getAdminOrgRefsByName()
  {
    return this.adminOrganizationRefsByName;
  }

  public Collection<OrganizationReferenceType> getAdminOrgRefs()
  {
    return this.adminOrganizationRefsByName.values();
  }

  public OrganizationReferenceType getAdminOrgRefByName(String name)
  {
    return (OrganizationReferenceType)this.adminOrganizationRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getProviderVdcRefsByName()
  {
    return this.providerVdcRefsByName;
  }

  public Collection<ReferenceType> getProviderVdcRefs()
  {
    return this.providerVdcRefsByName.values();
  }

  public ReferenceType getProviderVdcRefByName(String name)
  {
    return (ReferenceType)this.providerVdcRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getRightRefsByName()
  {
    return this.rightRefsByName;
  }

  public Collection<ReferenceType> getRightRefs()
  {
    return this.rightRefsByName.values();
  }

  public ReferenceType getRightRefByName(String name)
  {
    return (ReferenceType)this.rightRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getRoleRefsByName()
  {
    return this.roleRefsByName;
  }

  public Collection<ReferenceType> getRoleRefs()
  {
    return this.roleRefsByName.values();
  }

  public ReferenceType getRoleRefByName(String name)
  {
    return (ReferenceType)this.roleRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getExternalNetworkRefsByName()
  {
    return this.externalNetworkRefsByName;
  }

  public Collection<ReferenceType> getExternalNetworkRefs()
  {
    return this.externalNetworkRefsByName.values();
  }

  public ReferenceType getExternalNetworkRefByName(String name)
  {
    return (ReferenceType)this.externalNetworkRefsByName.get(name);
  }
}