package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.LicensingReportSampleType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.LicensingReportType;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudResource;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.datatype.XMLGregorianCalendar;

public class LicenseReport extends VcloudResource<LicensingReportType>
{
  private static Logger logger = Logger.getLogger(LicenseReport.class.getSimpleName());
  private HashMap<XMLGregorianCalendar, LicensingReportSampleType> samples;

  LicenseReport(VcloudClient client, LicensingReportType licensingReportType)
  {
    super(client, licensingReportType);
    sortLicenseReportValues();
  }

  public static LicenseReport getLicenseReportByReference(VcloudClient client, ReferenceType referenceType)
    throws VCloudException
  {
    return getLicenseReport(client, referenceType.getHref());
  }

  private static LicenseReport getLicenseReport(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: LicenseReport " + url);
    Response response = RestUtil.get(client, url);
    LicensingReportType licenseReportType = null;

    if (response.isExpected(200))
      licenseReportType = (LicensingReportType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new LicenseReport(client, licenseReportType);
  }

  private void sortLicenseReportValues()
  {
    this.samples = new HashMap();
    for (LicensingReportSampleType sample : ((LicensingReportType)getResource()).getSample())
      this.samples.put(sample.getObservationDate(), sample);
  }

  public HashMap<XMLGregorianCalendar, LicensingReportSampleType> getSamplesByDate()
  {
    return this.samples;
  }

  public XMLGregorianCalendar getReportDate()
  {
    return ((LicensingReportType)getResource()).getReportDate();
  }

  public String getProductSerialNumber()
  {
    return ((LicensingReportType)getResource()).getProductSerialNumber();
  }
}