package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.HostType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.PrepareHostParamsType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWHost extends VcloudEntity<HostType>
{
  private static Logger logger = Logger.getLogger(VMWHost.class.getSimpleName());

  VMWHost(VcloudClient client, HostType hostType)
  {
    super(client, hostType);
  }

  public static VMWHost getVMWHostByReference(VcloudClient client, ReferenceType referenceType)
    throws VCloudException
  {
    return getVMWHost(client, referenceType.getHref());
  }

  private static VMWHost getVMWHost(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VMWHost " + url);
    Response response = RestUtil.get(client, url);
    HostType vmwHostType = null;

    if (response.isExpected(200))
      vmwHostType = (HostType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    VMWHost vmwHostHelper = new VMWHost(client, vmwHostType);
    return vmwHostHelper;
  }

  public void disableHost()
    throws VCloudException
  {
    String disableUrl = ((HostType)getResource()).getHref() + "/action/disable";

    executeHostAction(getVcloudClient(), disableUrl, null, null, null, 204);
  }

  public static void disableHost(VcloudClient client, ReferenceType vmwHostRef)
    throws VCloudException
  {
    String disableUrl = vmwHostRef.getHref() + "/action/disable";

    executeHostAction(client, disableUrl, null, null, null, 204);
  }

  public void enableHost()
    throws VCloudException
  {
    String enableURL = ((HostType)getResource()).getHref() + "/action/enable";

    executeHostAction(getVcloudClient(), enableURL, null, null, null, 204);
  }

  public static void enableHost(VcloudClient client, ReferenceType vmwHostRef)
    throws VCloudException
  {
    String enableURL = vmwHostRef.getHref() + "/action/enable";

    executeHostAction(client, enableURL, null, null, null, 204);
  }

  public Task unPrepareHost()
    throws VCloudException
  {
    String url = ((HostType)getResource()).getHref() + "/action/unprepare";

    return executeHostAction(getVcloudClient(), url, null, null, null, 202);
  }

  public static Task unPrepareHost(VcloudClient client, ReferenceType hostRef)
    throws VCloudException
  {
    String url = hostRef.getHref() + "/action/unprepare";

    return executeHostAction(client, url, null, null, null, 202);
  }

  public Task prepareHost(String username, String password)
    throws VCloudException
  {
    PrepareHostParamsType prepareHostParamsType = new PrepareHostParamsType();
    prepareHostParamsType.setPassword(password);
    prepareHostParamsType.setUsername(username);

    ObjectFactory objectFactory = new ObjectFactory();
    String requestXmlString = JAXBUtil.marshal(objectFactory.createPrepareHostParams(prepareHostParamsType));

    String url = ((HostType)getResource()).getHref() + "/action/prepare";

    return executeHostAction(getVcloudClient(), url, requestXmlString, "application/vnd.vmware.admin.prepareHostParams+xml", null, 202);
  }

  public static Task prepareHost(VcloudClient client, ReferenceType hostRef, String username, String password)
    throws VCloudException
  {
    PrepareHostParamsType prepareHostParamsType = new PrepareHostParamsType();
    prepareHostParamsType.setPassword(password);
    prepareHostParamsType.setUsername(username);
    String url = hostRef.getHref() + "/action/prepare";

    ObjectFactory objectFactory = new ObjectFactory();
    String requestXmlString = JAXBUtil.marshal(objectFactory.createPrepareHostParams(prepareHostParamsType));

    return executeHostAction(client, url, requestXmlString, "application/vnd.vmware.admin.prepareHostParams+xml", null, 202);
  }

  public Task repairHost()
    throws VCloudException
  {
    String url = ((HostType)getResource()).getHref() + "/action/repair";

    return executeHostAction(getVcloudClient(), url, null, null, null, 202);
  }

  public static Task repairHost(VcloudClient client, ReferenceType hostRef)
    throws VCloudException
  {
    return executeHostAction(client, hostRef.getHref() + "/action/repair", null, null, null, 202);
  }

  public Task upgradeHost()
    throws VCloudException
  {
    String url = ((HostType)getResource()).getHref() + "/action/upgrade";

    return executeHostAction(getVcloudClient(), url, null, null, null, 202);
  }

  public static Task upgradeHost(VcloudClient client, ReferenceType hostRef)
    throws VCloudException
  {
    return executeHostAction(client, hostRef.getHref() + "/action/upgrade", null, null, null, 202);
  }

  private static Task executeHostAction(VcloudClient client, String vmwHostActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    logger.log(Level.INFO, "POST: HOST Action URL " + vmwHostActionUrl);
    Response response = RestUtil.post(client, vmwHostActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }
}