package devmgr.versioned.symbol;

import devmgr.versioned.jrpc.RPCClient;
import devmgr.versioned.jrpc.RPCError;
import devmgr.versioned.jrpc.XDRint;
import devmgr.versioned.jrpc.XDRlong;
import devmgr.versioned.jrpc.XDRstring;
import devmgr.versioned.jrpc.XDRvoid;
import java.io.IOException;
import java.net.InetAddress;

public class SYMbolAPIClientV1 extends RPCClient
  implements SYMbolAPIConstants
{
  public static final int PROGRAM_NUMBER = 1398361410;
  public static final int PROGRAM_VERSION = 1;
  public static final int PROC_discoverControllers = 1;
  public static final int PROC_bindToController = 2;
  public static final int PROC_assignVolumeGroupOwnership = 3;
  public static final int PROC_assignDrivesAsHotSpares = 4;
  public static final int PROC_assignSpecificDrivesAsHotSpares = 5;
  public static final int PROC_getVolumeCandidates = 6;
  public static final int PROC_createVolume = 7;
  public static final int PROC_deassignDrivesAsHotSpares = 8;
  public static final int PROC_deleteVolume = 9;
  public static final int PROC_setControllerToFailed = 10;
  public static final int PROC_setDriveToFailed = 11;
  public static final int PROC_startVolumeFormat = 12;
  public static final int PROC_initializeDrive = 13;
  public static final int PROC_loadControllerFirmware = 14;
  public static final int PROC_loadControllerNVSRAM = 15;
  public static final int PROC_resetMel = 16;
  public static final int PROC_setVolumeGroupToOffline = 17;
  public static final int PROC_setVolumeGroupToOnline = 18;
  public static final int PROC_startDriveReconstruction = 19;
  public static final int PROC_startVolumeGroupDefrag = 20;
  public static final int PROC_startVolumeGroupExpansion = 21;
  public static final int PROC_startVolumeRAIDMigration = 22;
  public static final int PROC_startVolumeSegmentSizing = 23;
  public static final int PROC_setControllerToPassive = 24;
  public static final int PROC_setControllerToActive = 25;
  public static final int PROC_setSACacheParams = 26;
  public static final int PROC_setSAUserLabel = 27;
  public static final int PROC_setControllerTime = 28;
  public static final int PROC_setVolumeCacheParams = 29;
  public static final int PROC_setVolumeParams = 30;
  public static final int PROC_setVolumeUserLabel = 31;
  public static final int PROC_startSAIdentification = 32;
  public static final int PROC_startDriveIdentification = 33;
  public static final int PROC_stopIdentification = 34;
  public static final int PROC_setHostInterfaceParams = 35;
  public static final int PROC_setControllerToOptimal = 36;
  public static final int PROC_setDriveToOptimal = 37;
  public static final int PROC_forceVolumeGroupToOptimal = 38;
  public static final int PROC_getControllerHostInterfaces = 39;
  public static final int PROC_getObjectGraph = 40;
  public static final int PROC_getVolumeActionProgress = 41;
  public static final int PROC_getRecoveryFailureList = 42;
  public static final int PROC_getSAData = 43;
  public static final int PROC_getVolumePerformance = 44;
  public static final int PROC_setSATrayPositions = 45;
  public static final int PROC_setVolumeMediaScanParams = 46;
  public static final int PROC_setSAMediaScanPeriod = 47;
  public static final int PROC_getChangeState = 48;
  public static final int PROC_clearSAConfiguration = 49;
  public static final int PROC_autoSAConfiguration = 50;
  public static final int PROC_getMelExtent = 51;
  public static final int PROC_getMelEntries = 52;
  public static final int PROC_getCriticalMelEntries = 53;
  public static final int PROC_getControllerNVSRAM = 54;
  public static final int PROC_setControllerNVSRAM = 55;
  public static final int PROC_setSAPassword = 56;
  public static final int PROC_pingController = 57;
  public static final int PROC_startVolumeParityCheck = 58;
  public static final int PROC_getParityCheckProgress = 59;
  public static final int PROC_getLUNMappings = 61;
  public static final int PROC_createSAPortGroup = 62;
  public static final int PROC_deleteSAPortGroup = 63;
  public static final int PROC_moveSAPort = 64;
  public static final int PROC_getSAPort = 65;
  public static final int PROC_createHost = 66;
  public static final int PROC_createCluster = 67;
  public static final int PROC_deleteCluster = 68;
  public static final int PROC_renameCluster = 69;
  public static final int PROC_deleteHost = 70;
  public static final int PROC_renameHost = 71;
  public static final int PROC_moveHost = 72;
  public static final int PROC_createHostPort = 73;
  public static final int PROC_deleteHostPort = 74;
  public static final int PROC_renameHostPort = 75;
  public static final int PROC_moveHostPort = 76;
  public static final int PROC_createLUNMapping = 77;
  public static final int PROC_deleteLUNMapping = 78;
  public static final int PROC_getUnlabeledHostPorts = 79;
  public static final int PROC_setHostPortType = 80;
  public static final int PROC_moveLUNMapping = 81;
  public static final int PROC_enableFeature = 82;
  public static final int PROC_disableFeature = 83;
  public static final int PROC_stateCapture = 84;
  public static final int PROC_loadDriveFirmware = 85;
  public static final int PROC_loadESMFirmware = 86;
  public static final int PROC_getHostSpecificNVSRAM = 87;
  public static final int PROC_setHostSpecificNVSRAM = 88;
  public static final int PROC_setBatteryParams = 89;
  public static final int PROC_assignVolumeOwnership = 90;
  public static final int PROC_issueRuntimeDiagnostics = 91;
  public static final int PROC_resetController = 92;
  public static final int PROC_quiesceController = 93;
  public static final int PROC_unquiesceController = 94;
  public static final int PROC_startVolumeExpansion = 95;
  public static final int PROC_createSnapshot = 96;
  public static final int PROC_disableSnapshot = 97;
  public static final int PROC_recreateSnapshot = 98;
  public static final int PROC_setSnapshotParams = 99;
  public static final int PROC_getRepositoryUtilization = 100;
  public static final int PROC_calculateDVECapacity = 101;
  public static final int PROC_getReadLinkStatus = 102;
  public static final int PROC_setRLSBaseline = 103;
  public static final int PROC_getMetadataVolumeCapacity = 104;
  public static final int PROC_createMetadataVolume = 105;
  public static final int PROC_activateMirroring = 106;
  public static final int PROC_deactivateMirroring = 107;
  public static final int PROC_changeSynchronizationPriority = 108;
  public static final int PROC_getVolumeListForMirroring = 109;
  public static final int PROC_createMirror = 110;
  public static final int PROC_roleChange = 111;
  public static final int PROC_removeMirror = 112;
  public static final int PROC_startSyncMirror = 113;
  public static final int PROC_startChannelIdentification = 114;
  public static final int PROC_startTrayIdentification = 115;
  public static final int PROC_getDacstoreIncompatibleVolumes = 116;
  public static final int PROC_getControllerTime = 117;
  public static final int PROC_establishVolumeCopy = 118;
  public static final int PROC_removeVolumeCopy = 119;
  public static final int PROC_setVolumeCopyParams = 120;
  public static final int PROC_startVolumeCopy = 121;
  public static final int PROC_stopVolumeCopy = 122;
  public static final int PROC_getVolumeCopyTargetCandidates = 123;
  public static final int PROC_getVolumeCopySourceCandidates = 124;
  public static final int PROC_setNetworkParameters = 125;
  public static final int PROC_setRloginCapability = 126;
  public static final int PROC_setVolXferAlertDelayPeriod = 127;
  public static final int PROC_getPersistentRegistrations = 128;
  public static final int PROC_getPersistentRegistrationsForVolume = 129;
  public static final int PROC_clearPersistentRegistrations = 130;
  public static final int PROC_changeMirrorWriteMode = 131;
  public static final int PROC_suspendMirror = 132;
  public static final int PROC_resumeMirror = 133;
  public static final int PROC_setAutoResync = 134;
  public static final int PROC_activateStagedControllerFirmware = 135;
  public static final int PROC_invalidateStagedControllerFirmware = 136;
  public static final int PROC_getDriveLogData = 137;
  public static final int PROC_getAutoConfigCandidates = 138;
  public static final int PROC_createAutoConfig = 139;
  public static final int PROC_autoAssignHotSpares = 140;
  public static final int PROC_getHotSpareCoverage = 141;
  public static final int PROC_getHotSpareCandidates = 142;
  public static final int PROC_readUnreadableSectorDatabase = 143;
  public static final int PROC_clearUnreadableSectors = 144;
  public static final int PROC_communicationCheck = 145;
  public static final int PROC_startDriveFirmwareDownload = 146;
  public static final int PROC_sendDriveFirmware = 147;
  public static final int PROC_updateDriveFirmware = 148;
  public static final int PROC_stopDriveFirmwareDownload = 149;
  public static final int PROC_getDriveFirmwareDownloadProgress = 150;
  public static final int PROC_recoverVolume = 151;
  public static final int PROC_setDriveChannelState = 152;
  public static final int PROC_getDriveChannelStatistics = 153;
  public static final int PROC_clearDriveChannelStatistics = 154;
  public static final int PROC_suspendConsistencyGroup = 155;
  public static final int PROC_resumeConsistencyGroup = 156;
  public static final int PROC_setControllerServiceMode = 157;
  public static final int PROC_scanVolume = 158;
  public static final int PROC_updateVolumeAttribute = 159;
  public static final int PROC_writeMgmtClientRecords = 160;
  public static final int PROC_readMgmtClientRecords = 161;
  public static final int PROC_deleteMgmtClientRecords = 162;
  public static final int PROC_suspendMirrorList = 163;
  public static final int PROC_resumeMirrorList = 164;
  public static final int PROC_getPRegistrations = 165;
  public static final int PROC_updateTray = 166;
  public static final int PROC_setAlarm = 167;
  public static final int PROC_issueDiscreteLinesTest = 168;
  public static final int PROC_requestToRemove = 169;
  public static final int PROC_setSimplexMode = 170;
  public static final int PROC_getPRegistrationsForVolume = 171;
  public static final int PROC_requestRemoval = 172;
  public static final int PROC_loadEsmFirmwareOnMismatch = 173;
  public static final int PROC_getSocErrorStatistics = 174;
  public static final int PROC_clearSocErrorStatistics = 175;
  public static final int PROC_getDdcLog = 176;
  public static final int PROC_clearDdcNeedsAttention = 177;
  public static final int PROC_powerDownArray = 178;

  public SYMbolAPIClientV1(InetAddress paramInetAddress, int paramInt, boolean paramBoolean)
    throws RPCError, IOException
  {
    super(paramInetAddress, 1398361410, 1, paramInt, paramBoolean);
  }

  public DiscoveryResponse discoverControllers()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    DiscoveryResponse localDiscoveryResponse = new DiscoveryResponse();
    call(1, localXDRvoid, localDiscoveryResponse);
    return localDiscoveryResponse;
  }

  public ReturnCode bindToController(ControllerDescriptor paramControllerDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(2, paramControllerDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode assignVolumeGroupOwnership(VolumeGroupOwnershipUpdateDescriptor paramVolumeGroupOwnershipUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(3, paramVolumeGroupOwnershipUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode assignDrivesAsHotSpares(int paramInt)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    localXDRint.setValue(paramInt);
    ReturnCode localReturnCode = new ReturnCode();
    call(4, localXDRint, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode assignSpecificDrivesAsHotSpares(DriveRefList paramDriveRefList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(5, paramDriveRefList, localReturnCode);
    return localReturnCode;
  }

  public VolumeCandidateList getVolumeCandidates(VolumeCandidateRequest paramVolumeCandidateRequest)
    throws RPCError, IOException
  {
    VolumeCandidateList localVolumeCandidateList = new VolumeCandidateList();
    call(6, paramVolumeCandidateRequest, localVolumeCandidateList);
    return localVolumeCandidateList;
  }

  public ReturnCode createVolume(VolumeCreationDescriptor paramVolumeCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(7, paramVolumeCreationDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode deassignDrivesAsHotSpares(DriveRefList paramDriveRefList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(8, paramDriveRefList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode deleteVolume(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(9, paramAbstractVolRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerToFailed(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(10, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setDriveToFailed(DriveRef paramDriveRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(11, paramDriveRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeFormat(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(12, paramAbstractVolRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode initializeDrive(DriveRef paramDriveRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(13, paramDriveRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode loadControllerFirmware(FirmwareUpdateDescriptor paramFirmwareUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(14, paramFirmwareUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode loadControllerNVSRAM(FirmwareUpdateDescriptor paramFirmwareUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(15, paramFirmwareUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode resetMel()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(16, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeGroupToOffline(VolumeGroupRef paramVolumeGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(17, paramVolumeGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeGroupToOnline(VolumeGroupRef paramVolumeGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(18, paramVolumeGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startDriveReconstruction(DriveRef paramDriveRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(19, paramDriveRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeGroupDefrag(VolumeGroupRef paramVolumeGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(20, paramVolumeGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeGroupExpansion(VolumeGroupExpansionDescriptor paramVolumeGroupExpansionDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(21, paramVolumeGroupExpansionDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeRAIDMigration(VolumeRAIDMigrationDescriptor paramVolumeRAIDMigrationDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(22, paramVolumeRAIDMigrationDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeSegmentSizing(VolumeSegmentSizingDescriptor paramVolumeSegmentSizingDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(23, paramVolumeSegmentSizingDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerToPassive(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(24, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerToActive(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(25, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSACacheParams(SACacheParamsUpdateDescriptor paramSACacheParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(26, paramSACacheParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSAUserLabel(UserAssignedLabel paramUserAssignedLabel)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(27, paramUserAssignedLabel, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerTime(long paramLong)
    throws RPCError, IOException
  {
    XDRlong localXDRlong = new XDRlong();
    localXDRlong.setValue(paramLong);
    ReturnCode localReturnCode = new ReturnCode();
    call(28, localXDRlong, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeCacheParams(VolumeCacheParamsUpdateDescriptor paramVolumeCacheParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(29, paramVolumeCacheParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeParams(VolumeParamsUpdateDescriptor paramVolumeParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(30, paramVolumeParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeUserLabel(VolumeLabelUpdateDescriptor paramVolumeLabelUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(31, paramVolumeLabelUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startSAIdentification()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(32, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startDriveIdentification(DriveRefList paramDriveRefList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(33, paramDriveRefList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode stopIdentification()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(34, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setHostInterfaceParams(InterfaceUpdateDescriptor paramInterfaceUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(35, paramInterfaceUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerToOptimal(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(36, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setDriveToOptimal(DriveRef paramDriveRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(37, paramDriveRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode forceVolumeGroupToOptimal(VolumeGroupRef paramVolumeGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(38, paramVolumeGroupRef, localReturnCode);
    return localReturnCode;
  }

  public IOInterfaceTypeDataList getControllerHostInterfaces()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    IOInterfaceTypeDataList localIOInterfaceTypeDataList = new IOInterfaceTypeDataList();
    call(39, localXDRvoid, localIOInterfaceTypeDataList);
    return localIOInterfaceTypeDataList;
  }

  public ObjectBundle getObjectGraph()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ObjectBundle localObjectBundle = new ObjectBundle();
    call(40, localXDRvoid, localObjectBundle);
    return localObjectBundle;
  }

  public VolumeActionProgress getVolumeActionProgress(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    VolumeActionProgress localVolumeActionProgress = new VolumeActionProgress();
    call(41, paramAbstractVolRef, localVolumeActionProgress);
    return localVolumeActionProgress;
  }

  public FailureTypeList getRecoveryFailureList()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    FailureTypeList localFailureTypeList = new FailureTypeList();
    call(42, localXDRvoid, localFailureTypeList);
    return localFailureTypeList;
  }

  public SAData getSAData()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    SAData localSAData = new SAData();
    call(43, localXDRvoid, localSAData);
    return localSAData;
  }

  public VolumePerformanceList getVolumePerformance(AbstractVolRefList paramAbstractVolRefList)
    throws RPCError, IOException
  {
    VolumePerformanceList localVolumePerformanceList = new VolumePerformanceList();
    call(44, paramAbstractVolRefList, localVolumePerformanceList);
    return localVolumePerformanceList;
  }

  public ReturnCode setSATrayPositions(TrayPositionList paramTrayPositionList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(45, paramTrayPositionList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeMediaScanParams(VolumeMediaScanParamsUpdateDescriptor paramVolumeMediaScanParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(46, paramVolumeMediaScanParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSAMediaScanPeriod(int paramInt)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    localXDRint.setValue(paramInt);
    ReturnCode localReturnCode = new ReturnCode();
    call(47, localXDRint, localReturnCode);
    return localReturnCode;
  }

  public ChangeState getChangeState(ChangeQueryDescriptor paramChangeQueryDescriptor)
    throws RPCError, IOException
  {
    ChangeState localChangeState = new ChangeState();
    call(48, paramChangeQueryDescriptor, localChangeState);
    return localChangeState;
  }

  public ReturnCode clearSAConfiguration()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(49, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode autoSAConfiguration()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(50, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public MelExtent getMelExtent()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    MelExtent localMelExtent = new MelExtent();
    call(51, localXDRvoid, localMelExtent);
    return localMelExtent;
  }

  public MelEntryList getMelEntries(MelExtent paramMelExtent)
    throws RPCError, IOException
  {
    MelEntryList localMelEntryList = new MelEntryList();
    call(52, paramMelExtent, localMelEntryList);
    return localMelEntryList;
  }

  public MelEntryList getCriticalMelEntries(MelExtent paramMelExtent)
    throws RPCError, IOException
  {
    MelEntryList localMelEntryList = new MelEntryList();
    call(53, paramMelExtent, localMelEntryList);
    return localMelEntryList;
  }

  public NVSRAMRegionList getControllerNVSRAM(NVSRAMRegionId paramNVSRAMRegionId)
    throws RPCError, IOException
  {
    NVSRAMRegionList localNVSRAMRegionList = new NVSRAMRegionList();
    call(54, paramNVSRAMRegionId, localNVSRAMRegionList);
    return localNVSRAMRegionList;
  }

  public ReturnCode setControllerNVSRAM(NVSRAMUpdateDescriptor paramNVSRAMUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(55, paramNVSRAMUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSAPassword(EncryptedPassword paramEncryptedPassword)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(56, paramEncryptedPassword, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode pingController()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(57, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeParityCheck(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(58, paramAbstractVolRef, localReturnCode);
    return localReturnCode;
  }

  public int getParityCheckProgress(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    call(59, paramAbstractVolRef, localXDRint);
    return localXDRint.getValue();
  }

  public LUNMappings getLUNMappings(SYMbolRef paramSYMbolRef)
    throws RPCError, IOException
  {
    LUNMappings localLUNMappings = new LUNMappings();
    call(61, paramSYMbolRef, localLUNMappings);
    return localLUNMappings;
  }

  public ReturnCodeWithRef createSAPortGroup(UserAssignedLabel paramUserAssignedLabel)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(62, paramUserAssignedLabel, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCode deleteSAPortGroup(SAPortGroupRef paramSAPortGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(63, paramSAPortGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode moveSAPort(SYMbolRefPair paramSYMbolRefPair)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(64, paramSYMbolRefPair, localReturnCode);
    return localReturnCode;
  }

  public ReturnCodeWithRef getSAPort(SAPortGetDescriptor paramSAPortGetDescriptor)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(65, paramSAPortGetDescriptor, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCodeWithRef createHost(HostCreationDescriptor paramHostCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(66, paramHostCreationDescriptor, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCodeWithRef createCluster(UserAssignedLabel paramUserAssignedLabel)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(67, paramUserAssignedLabel, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCode deleteCluster(ClusterRef paramClusterRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(68, paramClusterRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode renameCluster(SYMbolRefWithUAL paramSYMbolRefWithUAL)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(69, paramSYMbolRefWithUAL, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode deleteHost(HostRef paramHostRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(70, paramHostRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode renameHost(SYMbolRefWithUAL paramSYMbolRefWithUAL)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(71, paramSYMbolRefWithUAL, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode moveHost(SYMbolRefPair paramSYMbolRefPair)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(72, paramSYMbolRefPair, localReturnCode);
    return localReturnCode;
  }

  public ReturnCodeWithRef createHostPort(HostPortCreationDescriptor paramHostPortCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(73, paramHostPortCreationDescriptor, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCode deleteHostPort(HostPortRef paramHostPortRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(74, paramHostPortRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode renameHostPort(HostPortRenameDescriptor paramHostPortRenameDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(75, paramHostPortRenameDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode moveHostPort(SYMbolRefPair paramSYMbolRefPair)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(76, paramSYMbolRefPair, localReturnCode);
    return localReturnCode;
  }

  public ReturnCodeWithRef createLUNMapping(LUNMappingCreationDescriptor paramLUNMappingCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(77, paramLUNMappingCreationDescriptor, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCode deleteLUNMapping(LUNMappingRef paramLUNMappingRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(78, paramLUNMappingRef, localReturnCode);
    return localReturnCode;
  }

  public UnlabeledHostPorts getUnlabeledHostPorts()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    UnlabeledHostPorts localUnlabeledHostPorts = new UnlabeledHostPorts();
    call(79, localXDRvoid, localUnlabeledHostPorts);
    return localUnlabeledHostPorts;
  }

  public ReturnCode setHostPortType(HostPortTypeUpdateDescriptor paramHostPortTypeUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(80, paramHostPortTypeUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode moveLUNMapping(LUNMappingUpdateDescriptor paramLUNMappingUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(81, paramLUNMappingUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode enableFeature(FeatureKey paramFeatureKey)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(82, paramFeatureKey, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode disableFeature(Capability paramCapability)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(83, paramCapability, localReturnCode);
    return localReturnCode;
  }

  public ReturnCodeWithOpaqueData stateCapture(String paramString)
    throws RPCError, IOException
  {
    XDRstring localXDRstring = new XDRstring();
    localXDRstring.setValue(paramString);
    ReturnCodeWithOpaqueData localReturnCodeWithOpaqueData = new ReturnCodeWithOpaqueData();
    call(84, localXDRstring, localReturnCodeWithOpaqueData);
    return localReturnCodeWithOpaqueData;
  }

  public ReturnCode loadDriveFirmware(DriveFirmwareUpdateDescriptor paramDriveFirmwareUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(85, paramDriveFirmwareUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode loadESMFirmware(ESMFirmwareUpdateDescriptor paramESMFirmwareUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(86, paramESMFirmwareUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public NVSRAMRegionList getHostSpecificNVSRAM(NVSRAMRegionIdAndHostIndex paramNVSRAMRegionIdAndHostIndex)
    throws RPCError, IOException
  {
    NVSRAMRegionList localNVSRAMRegionList = new NVSRAMRegionList();
    call(87, paramNVSRAMRegionIdAndHostIndex, localNVSRAMRegionList);
    return localNVSRAMRegionList;
  }

  public ReturnCode setHostSpecificNVSRAM(HostSpecificNVSRAMUpdateDescriptor paramHostSpecificNVSRAMUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(88, paramHostSpecificNVSRAMUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setBatteryParams(BatteryParamsUpdateDescriptor paramBatteryParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(89, paramBatteryParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode assignVolumeOwnership(VolumeOwnershipUpdateDescriptor paramVolumeOwnershipUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(90, paramVolumeOwnershipUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public RuntimeDiagResults issueRuntimeDiagnostics(RuntimeDiagDescriptor paramRuntimeDiagDescriptor)
    throws RPCError, IOException
  {
    RuntimeDiagResults localRuntimeDiagResults = new RuntimeDiagResults();
    call(91, paramRuntimeDiagDescriptor, localRuntimeDiagResults);
    return localRuntimeDiagResults;
  }

  public ReturnCode resetController(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(92, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode quiesceController(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(93, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode unquiesceController(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(94, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeExpansion(VolumeExpansionDescriptor paramVolumeExpansionDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(95, paramVolumeExpansionDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode createSnapshot(SnapshotCreationDescriptor paramSnapshotCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(96, paramSnapshotCreationDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode disableSnapshot(SnapshotRef paramSnapshotRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(97, paramSnapshotRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode recreateSnapshot(SnapshotParamsUpdateDescriptor paramSnapshotParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(98, paramSnapshotParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSnapshotParams(SnapshotParamsUpdateDescriptor paramSnapshotParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(99, paramSnapshotParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public RepositoryUtilizationList getRepositoryUtilization(SnapshotRefList paramSnapshotRefList)
    throws RPCError, IOException
  {
    RepositoryUtilizationList localRepositoryUtilizationList = new RepositoryUtilizationList();
    call(100, paramSnapshotRefList, localRepositoryUtilizationList);
    return localRepositoryUtilizationList;
  }

  public long calculateDVECapacity(VolumeExpansionDescriptor paramVolumeExpansionDescriptor)
    throws RPCError, IOException
  {
    XDRlong localXDRlong = new XDRlong();
    call(101, paramVolumeExpansionDescriptor, localXDRlong);
    return localXDRlong.getValue();
  }

  public RLSResults getReadLinkStatus(RLSCommandDescriptor paramRLSCommandDescriptor)
    throws RPCError, IOException
  {
    RLSResults localRLSResults = new RLSResults();
    call(102, paramRLSCommandDescriptor, localRLSResults);
    return localRLSResults;
  }

  public ReturnCode setRLSBaseline()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(103, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public int getMetadataVolumeCapacity(MetadataVolType paramMetadataVolType)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    call(104, paramMetadataVolType, localXDRint);
    return localXDRint.getValue();
  }

  public ReturnCode createMetadataVolume(MetadataVolCreationDescriptor paramMetadataVolCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(105, paramMetadataVolCreationDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode activateMirroring()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(106, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode deactivateMirroring()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(107, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode changeSynchronizationPriority(MirrorSyncPriorityDescriptor paramMirrorSyncPriorityDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(108, paramMirrorSyncPriorityDescriptor, localReturnCode);
    return localReturnCode;
  }

  public MirrorVolumeCandidateList getVolumeListForMirroring(MirrorCandidateDescriptor paramMirrorCandidateDescriptor)
    throws RPCError, IOException
  {
    MirrorVolumeCandidateList localMirrorVolumeCandidateList = new MirrorVolumeCandidateList();
    call(109, paramMirrorCandidateDescriptor, localMirrorVolumeCandidateList);
    return localMirrorVolumeCandidateList;
  }

  public ReturnCode createMirror(MirrorCreationDescriptor paramMirrorCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(110, paramMirrorCreationDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode roleChange(MirrorRoleChangeDescriptor paramMirrorRoleChangeDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(111, paramMirrorRoleChangeDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode removeMirror(MirrorProxyRef paramMirrorProxyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(112, paramMirrorProxyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startSyncMirror(MirrorProxyRef paramMirrorProxyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(113, paramMirrorProxyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startChannelIdentification(int paramInt)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    localXDRint.setValue(paramInt);
    ReturnCode localReturnCode = new ReturnCode();
    call(114, localXDRint, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startTrayIdentification(TrayRefList paramTrayRefList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(115, paramTrayRefList, localReturnCode);
    return localReturnCode;
  }

  public AbstractVolRefList getDacstoreIncompatibleVolumes(long paramLong)
    throws RPCError, IOException
  {
    XDRlong localXDRlong = new XDRlong();
    localXDRlong.setValue(paramLong);
    AbstractVolRefList localAbstractVolRefList = new AbstractVolRefList();
    call(116, localXDRlong, localAbstractVolRefList);
    return localAbstractVolRefList;
  }

  public ControllerTime getControllerTime()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ControllerTime localControllerTime = new ControllerTime();
    call(117, localXDRvoid, localControllerTime);
    return localControllerTime;
  }

  public ReturnCodeWithRef establishVolumeCopy(VolumeCopyCreationDescriptor paramVolumeCopyCreationDescriptor)
    throws RPCError, IOException
  {
    ReturnCodeWithRef localReturnCodeWithRef = new ReturnCodeWithRef();
    call(118, paramVolumeCopyCreationDescriptor, localReturnCodeWithRef);
    return localReturnCodeWithRef;
  }

  public ReturnCode removeVolumeCopy(VolumeCopyRef paramVolumeCopyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(119, paramVolumeCopyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolumeCopyParams(VolumeCopyParamsUpdateDescriptor paramVolumeCopyParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(120, paramVolumeCopyParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode startVolumeCopy(VolumeCopyRef paramVolumeCopyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(121, paramVolumeCopyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode stopVolumeCopy(VolumeCopyRef paramVolumeCopyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(122, paramVolumeCopyRef, localReturnCode);
    return localReturnCode;
  }

  public VolumeCopyCandidateList getVolumeCopyTargetCandidates(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    VolumeCopyCandidateList localVolumeCopyCandidateList = new VolumeCopyCandidateList();
    call(123, paramAbstractVolRef, localVolumeCopyCandidateList);
    return localVolumeCopyCandidateList;
  }

  public VolumeCopyCandidateList getVolumeCopySourceCandidates()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    VolumeCopyCandidateList localVolumeCopyCandidateList = new VolumeCopyCandidateList();
    call(124, localXDRvoid, localVolumeCopyCandidateList);
    return localVolumeCopyCandidateList;
  }

  public ReturnCode setNetworkParameters(EthernetParamsUpdateDescriptor paramEthernetParamsUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(125, paramEthernetParamsUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setRloginCapability(RloginUpdateDescriptor paramRloginUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(126, paramRloginUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setVolXferAlertDelayPeriod(int paramInt)
    throws RPCError, IOException
  {
    XDRint localXDRint = new XDRint();
    localXDRint.setValue(paramInt);
    ReturnCode localReturnCode = new ReturnCode();
    call(127, localXDRint, localReturnCode);
    return localReturnCode;
  }

  public PersistentRegistrationList getPersistentRegistrations()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    PersistentRegistrationList localPersistentRegistrationList = new PersistentRegistrationList();
    call(128, localXDRvoid, localPersistentRegistrationList);
    return localPersistentRegistrationList;
  }

  public PersistentRegistrationList getPersistentRegistrationsForVolume(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    PersistentRegistrationList localPersistentRegistrationList = new PersistentRegistrationList();
    call(129, paramAbstractVolRef, localPersistentRegistrationList);
    return localPersistentRegistrationList;
  }

  public ReturnCode clearPersistentRegistrations(AbstractVolRefList paramAbstractVolRefList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(130, paramAbstractVolRefList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode changeMirrorWriteMode(MirrorWriteModeDescriptor paramMirrorWriteModeDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(131, paramMirrorWriteModeDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode suspendMirror(MirrorProxyRef paramMirrorProxyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(132, paramMirrorProxyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode resumeMirror(MirrorProxyRef paramMirrorProxyRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(133, paramMirrorProxyRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setAutoResync(MirrorAutoResyncDescriptor paramMirrorAutoResyncDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(134, paramMirrorAutoResyncDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode activateStagedControllerFirmware()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(135, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode invalidateStagedControllerFirmware()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(136, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public DriveLogData getDriveLogData(DriveRef paramDriveRef)
    throws RPCError, IOException
  {
    DriveLogData localDriveLogData = new DriveLogData();
    call(137, paramDriveRef, localDriveLogData);
    return localDriveLogData;
  }

  public AutoConfigCandidateList getAutoConfigCandidates(AutoConfigTemplateList paramAutoConfigTemplateList)
    throws RPCError, IOException
  {
    AutoConfigCandidateList localAutoConfigCandidateList = new AutoConfigCandidateList();
    call(138, paramAutoConfigTemplateList, localAutoConfigCandidateList);
    return localAutoConfigCandidateList;
  }

  public ReturnCode createAutoConfig(AutoConfigCandidateList paramAutoConfigCandidateList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(139, paramAutoConfigCandidateList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode autoAssignHotSpares()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(140, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public HotSpareCoverageList getHotSpareCoverage()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    HotSpareCoverageList localHotSpareCoverageList = new HotSpareCoverageList();
    call(141, localXDRvoid, localHotSpareCoverageList);
    return localHotSpareCoverageList;
  }

  public HotSpareCandidateList getHotSpareCandidates(PhysicalDriveType paramPhysicalDriveType)
    throws RPCError, IOException
  {
    HotSpareCandidateList localHotSpareCandidateList = new HotSpareCandidateList();
    call(142, paramPhysicalDriveType, localHotSpareCandidateList);
    return localHotSpareCandidateList;
  }

  public UnreadableSectorTableInfo readUnreadableSectorDatabase()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    UnreadableSectorTableInfo localUnreadableSectorTableInfo = new UnreadableSectorTableInfo();
    call(143, localXDRvoid, localUnreadableSectorTableInfo);
    return localUnreadableSectorTableInfo;
  }

  public ReturnCode clearUnreadableSectors(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(144, paramAbstractVolRef, localReturnCode);
    return localReturnCode;
  }

  public RemoteCommunicationCheckSampleList communicationCheck(RemoteCommunicationCheckDescriptor paramRemoteCommunicationCheckDescriptor)
    throws RPCError, IOException
  {
    RemoteCommunicationCheckSampleList localRemoteCommunicationCheckSampleList = new RemoteCommunicationCheckSampleList();
    call(145, paramRemoteCommunicationCheckDescriptor, localRemoteCommunicationCheckSampleList);
    return localRemoteCommunicationCheckSampleList;
  }

  public DriveFirmwareStatus startDriveFirmwareDownload(DriveFirmwareDownload paramDriveFirmwareDownload)
    throws RPCError, IOException
  {
    DriveFirmwareStatus localDriveFirmwareStatus = new DriveFirmwareStatus();
    call(146, paramDriveFirmwareDownload, localDriveFirmwareStatus);
    return localDriveFirmwareStatus;
  }

  public ReturnCode sendDriveFirmware(DriveFirmwareChunkDescriptor paramDriveFirmwareChunkDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(147, paramDriveFirmwareChunkDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode updateDriveFirmware()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(148, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode stopDriveFirmwareDownload()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(149, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public DriveFirmwareStatus getDriveFirmwareDownloadProgress()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    DriveFirmwareStatus localDriveFirmwareStatus = new DriveFirmwareStatus();
    call(150, localXDRvoid, localDriveFirmwareStatus);
    return localDriveFirmwareStatus;
  }

  public ReturnCode recoverVolume(RecoverVolumeDescriptor paramRecoverVolumeDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(151, paramRecoverVolumeDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setDriveChannelState(DriveChannelStateDescriptor paramDriveChannelStateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(152, paramDriveChannelStateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public DriveChannelList getDriveChannelStatistics()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    DriveChannelList localDriveChannelList = new DriveChannelList();
    call(153, localXDRvoid, localDriveChannelList);
    return localDriveChannelList;
  }

  public ReturnCode clearDriveChannelStatistics()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(154, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode suspendConsistencyGroup(ConsistencyGroupRef paramConsistencyGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(155, paramConsistencyGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode resumeConsistencyGroup(ConsistencyGroupRef paramConsistencyGroupRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(156, paramConsistencyGroupRef, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setControllerServiceMode(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(157, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public ParityScanResponse scanVolume(VolumeScanDescriptor paramVolumeScanDescriptor)
    throws RPCError, IOException
  {
    ParityScanResponse localParityScanResponse = new ParityScanResponse();
    call(158, paramVolumeScanDescriptor, localParityScanResponse);
    return localParityScanResponse;
  }

  public ReturnCode updateVolumeAttribute(VolumeAttributeUpdateDescriptor paramVolumeAttributeUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(159, paramVolumeAttributeUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode writeMgmtClientRecords(MgmtClientRecordWriteDescriptor paramMgmtClientRecordWriteDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(160, paramMgmtClientRecordWriteDescriptor, localReturnCode);
    return localReturnCode;
  }

  public MgmtClientRecordReadResult readMgmtClientRecords(MgmtClientRecordReadDescriptor paramMgmtClientRecordReadDescriptor)
    throws RPCError, IOException
  {
    MgmtClientRecordReadResult localMgmtClientRecordReadResult = new MgmtClientRecordReadResult();
    call(161, paramMgmtClientRecordReadDescriptor, localMgmtClientRecordReadResult);
    return localMgmtClientRecordReadResult;
  }

  public ReturnCode deleteMgmtClientRecords(MgmtClientRecordDeleteDescriptor paramMgmtClientRecordDeleteDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(162, paramMgmtClientRecordDeleteDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode suspendMirrorList(MirrorVolumeList paramMirrorVolumeList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(163, paramMirrorVolumeList, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode resumeMirrorList(MirrorVolumeList paramMirrorVolumeList)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(164, paramMirrorVolumeList, localReturnCode);
    return localReturnCode;
  }

  public PRegistrationList getPRegistrations()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    PRegistrationList localPRegistrationList = new PRegistrationList();
    call(165, localXDRvoid, localPRegistrationList);
    return localPRegistrationList;
  }

  public ReturnCode updateTray(TrayUpdateDescriptor paramTrayUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(166, paramTrayUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setAlarm(AlarmUpdateDescriptor paramAlarmUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(167, paramAlarmUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode issueDiscreteLinesTest()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(168, localXDRvoid, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode requestToRemove(RTRUpdateDescriptor paramRTRUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(169, paramRTRUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode setSimplexMode(ControllerRedundancyMode paramControllerRedundancyMode)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(170, paramControllerRedundancyMode, localReturnCode);
    return localReturnCode;
  }

  public PRegistrationList getPRegistrationsForVolume(AbstractVolRef paramAbstractVolRef)
    throws RPCError, IOException
  {
    PRegistrationList localPRegistrationList = new PRegistrationList();
    call(171, paramAbstractVolRef, localPRegistrationList);
    return localPRegistrationList;
  }

  public ReturnCode requestRemoval(RTRUpdateDescription paramRTRUpdateDescription)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(172, paramRTRUpdateDescription, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode loadEsmFirmwareOnMismatch(ESMFirmwareUpdateDescriptor paramESMFirmwareUpdateDescriptor)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(173, paramESMFirmwareUpdateDescriptor, localReturnCode);
    return localReturnCode;
  }

  public SocStatisticsReturned getSocErrorStatistics(SocCommandDescriptor paramSocCommandDescriptor)
    throws RPCError, IOException
  {
    SocStatisticsReturned localSocStatisticsReturned = new SocStatisticsReturned();
    call(174, paramSocCommandDescriptor, localSocStatisticsReturned);
    return localSocStatisticsReturned;
  }

  public ReturnCode clearSocErrorStatistics(ControllerRef paramControllerRef)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(175, paramControllerRef, localReturnCode);
    return localReturnCode;
  }

  public DdcLogDataReturned getDdcLog(DdcExtent paramDdcExtent)
    throws RPCError, IOException
  {
    DdcLogDataReturned localDdcLogDataReturned = new DdcLogDataReturned();
    call(176, paramDdcExtent, localDdcLogDataReturned);
    return localDdcLogDataReturned;
  }

  public ReturnCode clearDdcNeedsAttention(DdcTag paramDdcTag)
    throws RPCError, IOException
  {
    ReturnCode localReturnCode = new ReturnCode();
    call(177, paramDdcTag, localReturnCode);
    return localReturnCode;
  }

  public ReturnCode powerDownArray()
    throws RPCError, IOException
  {
    XDRvoid localXDRvoid = new XDRvoid();
    ReturnCode localReturnCode = new ReturnCode();
    call(178, localXDRvoid, localReturnCode);
    return localReturnCode;
  }
}