package org.eclipse.jdt.internal.compiler;

import java.util.Arrays;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;

public class ClassFilePool
{
  public static final int POOL_SIZE = 7;
  ClassFile[] classFiles = new ClassFile[7];

  public static ClassFilePool newInstance()
  {
    return new ClassFilePool();
  }

  public void release(ClassFile classFile) {
    for (int i = 0; i < 7; i++) {
      ClassFile currentClassFile = this.classFiles[i];
      if (currentClassFile == classFile) {
        classFile.isShared = false;
        return;
      }
    }
  }

  public ClassFile acquire(SourceTypeBinding typeBinding) {
    for (int i = 0; i < 7; i++) {
      ClassFile classFile = this.classFiles[i];
      if (classFile == null) {
        ClassFile newClassFile = new ClassFile(typeBinding);
        this.classFiles[i] = newClassFile;
        newClassFile.isShared = true;
        return newClassFile;
      }
      if (!classFile.isShared)
      {
        classFile.reset(typeBinding);
        classFile.isShared = true;
        return classFile;
      }
    }
    return new ClassFile(typeBinding);
  }
  public void reset() {
    Arrays.fill(this.classFiles, null);
  }
}