package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.impl.DoubleConstant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.util.FloatUtil;

public class DoubleLiteral extends NumberLiteral
{
  double value;

  public DoubleLiteral(char[] token, int s, int e)
  {
    super(token, s, e);
  }

  public void computeConstant() {
    try {
      computedValue = Double.valueOf(String.valueOf(this.source));
    }
    catch (NumberFormatException localNumberFormatException1)
    {
      try
      {
        Double computedValue;
        double v = FloatUtil.valueOfHexDoubleLiteral(this.source);
        if (v == (1.0D / 0.0D))
        {
          return;
        }
        if (Double.isNaN(v))
        {
          return;
        }
        this.value = v;
        this.constant = DoubleConstant.fromValue(v);
      }
      catch (NumberFormatException localNumberFormatException2)
      {
      }
      return;
    }
    Double computedValue;
    double doubleValue = computedValue.doubleValue();
    if (doubleValue > 1.7976931348623157E+308D)
    {
      return;
    }
    if (doubleValue < 4.9E-324D)
    {
      boolean isHexaDecimal = false;
      for (int i = 0; i < this.source.length; i++) {
        switch (this.source[i]) {
        case '.':
        case '0':
          break;
        case 'X':
        case 'x':
          isHexaDecimal = true;
          break;
        case 'D':
        case 'E':
        case 'F':
        case 'd':
        case 'e':
        case 'f':
          if (!isHexaDecimal) break label236;
          return;
        case 'P':
        case 'p':
          break;
        default:
          return;
        }
      }
    }
    label236: this.value = doubleValue;
    this.constant = DoubleConstant.fromValue(this.value);
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired)
  {
    int pc = codeStream.position;
    if (valueRequired) {
      codeStream.generateConstant(this.constant, this.implicitConversion);
    }
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  public TypeBinding literalType(BlockScope scope) {
    return TypeBinding.DOUBLE;
  }
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}