package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class EmptyStatement extends Statement
{
  public EmptyStatement(int startPosition, int endPosition)
  {
    this.sourceStart = startPosition;
    this.sourceEnd = endPosition;
  }

  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    return flowInfo;
  }

  public boolean complainIfUnreachable(FlowInfo flowInfo, BlockScope scope, boolean didAlreadyComplain)
  {
    if (scope.compilerOptions().complianceLevel < 3145728L) {
      return false;
    }
    return super.complainIfUnreachable(flowInfo, scope, didAlreadyComplain);
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream)
  {
  }

  public StringBuffer printStatement(int tab, StringBuffer output) {
    return printIndent(tab, output).append(';');
  }

  public void resolve(BlockScope scope) {
    if ((this.bits & 0x1) == 0)
      scope.problemReporter().superfluousSemicolon(this.sourceStart, this.sourceEnd);
    else
      scope.problemReporter().emptyControlFlowStatement(this.sourceStart, this.sourceEnd);
  }

  public void traverse(ASTVisitor visitor, BlockScope scope)
  {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}