package org.eclipse.jdt.internal.compiler.ast;

import java.util.ArrayList;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.NullInfoRegistry;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.problem.ShouldNotImplement;
import org.eclipse.jdt.internal.compiler.util.Messages;

public abstract class Expression extends Statement
{
  public Constant constant;
  public int statementEnd = -1;
  public int implicitConversion;
  public TypeBinding resolvedType;

  public static final boolean isConstantValueRepresentable(Constant constant, int constantTypeID, int targetTypeID)
  {
    if (targetTypeID == constantTypeID)
      return true;
    switch (targetTypeID) {
    case 2:
      switch (constantTypeID) {
      case 2:
        return true;
      case 8:
        return constant.doubleValue() == constant.charValue();
      case 9:
        return constant.floatValue() == constant.charValue();
      case 10:
        return constant.intValue() == constant.charValue();
      case 4:
        return constant.shortValue() == constant.charValue();
      case 3:
        return constant.byteValue() == constant.charValue();
      case 7:
        return constant.longValue() == constant.charValue();
      case 5:
      case 6: } return false;
    case 9:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.floatValue();
      case 8:
        return constant.doubleValue() == constant.floatValue();
      case 9:
        return true;
      case 10:
        return constant.intValue() == constant.floatValue();
      case 4:
        return constant.shortValue() == constant.floatValue();
      case 3:
        return constant.byteValue() == constant.floatValue();
      case 7:
        return (float)constant.longValue() == constant.floatValue();
      case 5:
      case 6: } return false;
    case 8:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.doubleValue();
      case 8:
        return true;
      case 9:
        return constant.floatValue() == constant.doubleValue();
      case 10:
        return constant.intValue() == constant.doubleValue();
      case 4:
        return constant.shortValue() == constant.doubleValue();
      case 3:
        return constant.byteValue() == constant.doubleValue();
      case 7:
        return constant.longValue() == constant.doubleValue();
      case 5:
      case 6: } return false;
    case 3:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.byteValue();
      case 8:
        return constant.doubleValue() == constant.byteValue();
      case 9:
        return constant.floatValue() == constant.byteValue();
      case 10:
        return constant.intValue() == constant.byteValue();
      case 4:
        return constant.shortValue() == constant.byteValue();
      case 3:
        return true;
      case 7:
        return constant.longValue() == constant.byteValue();
      case 5:
      case 6: } return false;
    case 4:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.shortValue();
      case 8:
        return constant.doubleValue() == constant.shortValue();
      case 9:
        return constant.floatValue() == constant.shortValue();
      case 10:
        return constant.intValue() == constant.shortValue();
      case 4:
        return true;
      case 3:
        return constant.byteValue() == constant.shortValue();
      case 7:
        return constant.longValue() == constant.shortValue();
      case 5:
      case 6: } return false;
    case 10:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.intValue();
      case 8:
        return constant.doubleValue() == constant.intValue();
      case 9:
        return constant.floatValue() == constant.intValue();
      case 10:
        return true;
      case 4:
        return constant.shortValue() == constant.intValue();
      case 3:
        return constant.byteValue() == constant.intValue();
      case 7:
        return constant.longValue() == constant.intValue();
      case 5:
      case 6: } return false;
    case 7:
      switch (constantTypeID) {
      case 2:
        return constant.charValue() == constant.longValue();
      case 8:
        return constant.doubleValue() == constant.longValue();
      case 9:
        return constant.floatValue() == (float)constant.longValue();
      case 10:
        return constant.intValue() == constant.longValue();
      case 4:
        return constant.shortValue() == constant.longValue();
      case 3:
        return constant.byteValue() == constant.longValue();
      case 7:
        return true;
      case 5:
      case 6: } return false;
    case 5:
    case 6:
    }
    return false;
  }

  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo)
  {
    return flowInfo;
  }

  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, boolean valueRequired)
  {
    return analyseCode(currentScope, flowContext, flowInfo);
  }

  public final boolean checkCastTypesCompatibility(Scope scope, TypeBinding castType, TypeBinding expressionType, Expression expression)
  {
    if ((castType == null) || (expressionType == null)) return true;

    boolean use15specifics = scope.compilerOptions().sourceLevel >= 3211264L;
    if (castType.isBaseType()) {
      if (expressionType.isBaseType()) {
        if (expressionType == castType) {
          if (expression != null) {
            this.constant = expression.constant;
          }
          tagAsUnnecessaryCast(scope, castType);
          return true;
        }
        boolean necessary = false;
        if ((expressionType.isCompatibleWith(castType)) || 
          ((necessary = BaseTypeBinding.isNarrowing(castType.id, expressionType.id)))) {
          if (expression != null) {
            expression.implicitConversion = ((castType.id << 4) + expressionType.id);
            if (expression.constant != Constant.NotAConstant) {
              this.constant = expression.constant.castTo(expression.implicitConversion);
            }
          }
          if (!necessary) tagAsUnnecessaryCast(scope, castType);
          return true;
        }
      }
      else if ((use15specifics) && 
        (scope.environment().computeBoxingType(expressionType).isCompatibleWith(castType))) {
        tagAsUnnecessaryCast(scope, castType);
        return true;
      }
      return false;
    }if ((use15specifics) && 
      (expressionType.isBaseType()) && 
      (scope.environment().computeBoxingType(expressionType).isCompatibleWith(castType))) {
      tagAsUnnecessaryCast(scope, castType);
      return true;
    }

    switch (expressionType.kind())
    {
    case 132:
      if (expressionType == TypeBinding.NULL) {
        tagAsUnnecessaryCast(scope, castType);
        return true;
      }
      return false;
    case 68:
      if (castType == expressionType) {
        tagAsUnnecessaryCast(scope, castType);
        return true;
      }
      switch (castType.kind())
      {
      case 68:
        TypeBinding castElementType = ((ArrayBinding)castType).elementsType();
        TypeBinding exprElementType = ((ArrayBinding)expressionType).elementsType();
        if ((exprElementType.isBaseType()) || (castElementType.isBaseType())) {
          if (castElementType == exprElementType) {
            tagAsNeedCheckCast();
            return true;
          }
          return false;
        }

        return checkCastTypesCompatibility(scope, castElementType, exprElementType, expression);
      case 4100:
        TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
        if (match == null) {
          checkUnsafeCast(scope, castType, expressionType, null, true);
        }

        return checkCastTypesCompatibility(scope, ((TypeVariableBinding)castType).upperBound(), expressionType, expression);
      }

      switch (castType.id) {
      case 36:
      case 37:
        tagAsNeedCheckCast();
        return true;
      case 1:
        tagAsUnnecessaryCast(scope, castType);
        return true;
      }
      return false;
    case 4100:
      TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
      if (match != null) {
        return checkUnsafeCast(scope, castType, expressionType, match, false);
      }

      return checkCastTypesCompatibility(scope, castType, ((TypeVariableBinding)expressionType).upperBound(), expression);
    case 516:
      TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
      if (match != null) {
        return checkUnsafeCast(scope, castType, expressionType, match, false);
      }

      return checkCastTypesCompatibility(scope, castType, ((WildcardBinding)expressionType).bound, expression);
    }

    if (expressionType.isInterface()) {
      switch (castType.kind())
      {
      case 68:
        switch (expressionType.id) {
        case 36:
        case 37:
          tagAsNeedCheckCast();
          return true;
        }
        return false;
      case 4100:
        TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
        if (match == null) {
          checkUnsafeCast(scope, castType, expressionType, null, true);
        }

        return checkCastTypesCompatibility(scope, ((TypeVariableBinding)castType).upperBound(), expressionType, expression);
      }

      if (castType.isInterface())
      {
        ReferenceBinding interfaceType = (ReferenceBinding)expressionType;
        TypeBinding match = interfaceType.findSuperTypeWithSameErasure(castType);
        if (match != null) {
          return checkUnsafeCast(scope, castType, interfaceType, match, false);
        }
        tagAsNeedCheckCast();
        match = castType.findSuperTypeWithSameErasure(interfaceType);
        if (match != null) {
          return checkUnsafeCast(scope, castType, interfaceType, match, true);
        }
        if (use15specifics) {
          checkUnsafeCast(scope, castType, expressionType, null, true);

          if (interfaceType.hasIncompatibleSuperType((ReferenceBinding)castType))
            return false;
        }
        else {
          MethodBinding[] castTypeMethods = getAllInheritedMethods((ReferenceBinding)castType);
          MethodBinding[] expressionTypeMethods = getAllInheritedMethods((ReferenceBinding)expressionType);
          int exprMethodsLength = expressionTypeMethods.length;
          int i = 0; for (int castMethodsLength = castTypeMethods.length; i < castMethodsLength; i++) {
            for (int j = 0; j < exprMethodsLength; j++) {
              if ((castTypeMethods[i].returnType != expressionTypeMethods[j].returnType) && 
                (CharOperation.equals(castTypeMethods[i].selector, expressionTypeMethods[j].selector)) && 
                (castTypeMethods[i].areParametersEqual(expressionTypeMethods[j]))) {
                return false;
              }
            }
          }
        }

        return true;
      }

      if (castType.id == 1) {
        tagAsUnnecessaryCast(scope, castType);
        return true;
      }

      tagAsNeedCheckCast();
      TypeBinding match = castType.findSuperTypeWithSameErasure(expressionType);
      if (match != null) {
        return checkUnsafeCast(scope, castType, expressionType, match, true);
      }
      if (((ReferenceBinding)castType).isFinal())
      {
        return false;
      }
      if (use15specifics) {
        checkUnsafeCast(scope, castType, expressionType, null, true);

        if (((ReferenceBinding)castType).hasIncompatibleSuperType((ReferenceBinding)expressionType)) {
          return false;
        }
      }
      return true;
    }

    switch (castType.kind())
    {
    case 68:
      if (expressionType.id == 1) {
        if (use15specifics) checkUnsafeCast(scope, castType, expressionType, expressionType, true);
        tagAsNeedCheckCast();
        return true;
      }
      return false;
    case 4100:
      TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
      if (match == null) {
        checkUnsafeCast(scope, castType, expressionType, match, true);
      }

      return checkCastTypesCompatibility(scope, ((TypeVariableBinding)castType).upperBound(), expressionType, expression);
    }

    if (castType.isInterface())
    {
      ReferenceBinding refExprType = (ReferenceBinding)expressionType;
      TypeBinding match = refExprType.findSuperTypeWithSameErasure(castType);
      if (match != null) {
        return checkUnsafeCast(scope, castType, expressionType, match, false);
      }

      if (refExprType.isFinal()) {
        return false;
      }
      tagAsNeedCheckCast();
      match = castType.findSuperTypeWithSameErasure(expressionType);
      if (match != null) {
        return checkUnsafeCast(scope, castType, expressionType, match, true);
      }
      if (use15specifics) {
        checkUnsafeCast(scope, castType, expressionType, null, true);

        if (refExprType.hasIncompatibleSuperType((ReferenceBinding)castType))
          return false;
      }
      return true;
    }

    TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
    if (match != null) {
      if ((expression != null) && (castType.id == 11)) this.constant = expression.constant;
      return checkUnsafeCast(scope, castType, expressionType, match, false);
    }
    match = castType.findSuperTypeWithSameErasure(expressionType);
    if (match != null) {
      tagAsNeedCheckCast();
      return checkUnsafeCast(scope, castType, expressionType, match, true);
    }
    return false;
  }

  public void checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo)
  {
    LocalVariableBinding local = localVariableBinding();
    if ((local != null) && 
      ((local.type.tagBits & 0x2) == 0L)) {
      if ((this.bits & 0x20000) == 0) {
        flowContext.recordUsingNullReference(scope, local, this, 
          3, flowInfo);
      }
      flowInfo.markAsComparedEqualToNonNull(local);

      if (flowContext.initsOnFinally != null)
        flowContext.initsOnFinally.markAsComparedEqualToNonNull(local);
    }
  }

  public boolean checkUnsafeCast(Scope scope, TypeBinding castType, TypeBinding expressionType, TypeBinding match, boolean isNarrowing)
  {
    if (match == castType) {
      if (!isNarrowing) tagAsUnnecessaryCast(scope, castType);
      return true;
    }
    if ((match != null) && (
      (castType.isBoundParameterizedType()) || 
      (expressionType.isBoundParameterizedType())))
    {
      if (match.isProvablyDistinctFrom(isNarrowing ? expressionType : castType, 0)) {
        return false;
      }
    }
    if (!isNarrowing) tagAsUnnecessaryCast(scope, castType);
    return true;
  }

  public void computeConversion(Scope scope, TypeBinding runtimeType, TypeBinding compileTimeType)
  {
    if ((runtimeType == null) || (compileTimeType == null))
      return;
    if (this.implicitConversion != 0) return;

    if ((runtimeType != TypeBinding.NULL) && (runtimeType.isBaseType())) {
      if (!compileTimeType.isBaseType()) {
        TypeBinding unboxedType = scope.environment().computeBoxingType(compileTimeType);
        this.implicitConversion = 1024;
        scope.problemReporter().autoboxing(this, compileTimeType, runtimeType);
        compileTimeType = unboxedType;
      }
    } else { if ((compileTimeType != TypeBinding.NULL) && (compileTimeType.isBaseType())) {
        TypeBinding boxedType = scope.environment().computeBoxingType(runtimeType);
        if (boxedType == runtimeType)
          boxedType = compileTimeType;
        this.implicitConversion = (0x200 | (boxedType.id << 4) + compileTimeType.id);
        scope.problemReporter().autoboxing(this, compileTimeType, scope.environment().computeBoxingType(boxedType));
        return;
      }if ((this.constant != Constant.NotAConstant) && (this.constant.typeID() != 11)) {
        this.implicitConversion = 512;
        return;
      }
    }
    int compileTimeTypeID;
    if ((compileTimeTypeID = compileTimeType.id) == 2147483647)
      compileTimeTypeID = compileTimeType.erasure().id == 11 ? 11 : 1;
    int runtimeTypeID;
    switch (runtimeTypeID = runtimeType.id) {
    case 2:
    case 3:
    case 4:
      this.implicitConversion |= 160 + compileTimeTypeID;
      break;
    case 5:
    case 7:
    case 8:
    case 9:
    case 10:
    case 11:
      this.implicitConversion |= (runtimeTypeID << 4) + compileTimeTypeID;
    case 6:
    }
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream)
  {
    if ((this.bits & 0x80000000) == 0) {
      return;
    }
    generateCode(currentScope, codeStream, false);
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired)
  {
    if (this.constant != Constant.NotAConstant)
    {
      int pc = codeStream.position;
      codeStream.generateConstant(this.constant, this.implicitConversion);
      codeStream.recordPositionsFrom(pc, this.sourceStart);
    }
    else {
      throw new ShouldNotImplement(Messages.ast_missingCode);
    }
  }

  public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired)
  {
    Constant cst = optimizedBooleanConstant();
    generateCode(currentScope, codeStream, (valueRequired) && (cst == Constant.NotAConstant));
    if ((cst != Constant.NotAConstant) && (cst.typeID() == 5)) {
      int pc = codeStream.position;
      if (cst.booleanValue())
      {
        if ((valueRequired) && 
          (falseLabel == null))
        {
          if (trueLabel != null) {
            codeStream.goto_(trueLabel);
          }
        }

      }
      else if ((valueRequired) && 
        (falseLabel != null))
      {
        if (trueLabel == null) {
          codeStream.goto_(falseLabel);
        }

      }

      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    }

    int position = codeStream.position;
    if (valueRequired) {
      if (falseLabel == null) {
        if (trueLabel != null)
        {
          codeStream.ifne(trueLabel);
        }
      }
      else if (trueLabel == null)
      {
        codeStream.ifeq(falseLabel);
      }

    }

    codeStream.updateLastRecordedEndPC(currentScope, position);
  }

  public void generateOptimizedStringConcatenation(BlockScope blockScope, CodeStream codeStream, int typeID)
  {
    if ((typeID == 11) && (this.constant != Constant.NotAConstant) && (this.constant.stringValue().length() == 0)) {
      return;
    }
    generateCode(blockScope, codeStream, true);
    codeStream.invokeStringConcatenationAppendForType(typeID);
  }

  public void generateOptimizedStringConcatenationCreation(BlockScope blockScope, CodeStream codeStream, int typeID)
  {
    codeStream.newStringContatenation();
    codeStream.dup();
    switch (typeID)
    {
    case 0:
    case 1:
      codeStream.invokeStringConcatenationDefaultConstructor();
      generateCode(blockScope, codeStream, true);
      codeStream.invokeStringConcatenationAppendForType(1);
      return;
    case 11:
    case 12:
      if (this.constant != Constant.NotAConstant) {
        String stringValue = this.constant.stringValue();
        if (stringValue.length() == 0) {
          codeStream.invokeStringConcatenationDefaultConstructor();
          return;
        }
        codeStream.ldc(stringValue);
      }
      else {
        generateCode(blockScope, codeStream, true);
        codeStream.invokeStringValueOf(1);
      }
      break;
    default:
      generateCode(blockScope, codeStream, true);
      codeStream.invokeStringValueOf(typeID);
    }
    codeStream.invokeStringConcatenationStringConstructor();
  }

  private MethodBinding[] getAllInheritedMethods(ReferenceBinding binding) {
    ArrayList collector = new ArrayList();
    getAllInheritedMethods0(binding, collector);
    return (MethodBinding[])collector.toArray(new MethodBinding[collector.size()]);
  }

  private void getAllInheritedMethods0(ReferenceBinding binding, ArrayList collector) {
    if (!binding.isInterface()) return;
    MethodBinding[] methodBindings = binding.methods();
    int i = 0; for (int max = methodBindings.length; i < max; i++) {
      collector.add(methodBindings[i]);
    }
    ReferenceBinding[] superInterfaces = binding.superInterfaces();
    int i = 0; for (int max = superInterfaces.length; i < max; i++)
      getAllInheritedMethods0(superInterfaces[i], collector);
  }

  public boolean isCompactableOperation()
  {
    return false;
  }

  public boolean isConstantValueOfTypeAssignableToType(TypeBinding constantType, TypeBinding targetType)
  {
    if (this.constant == Constant.NotAConstant)
      return false;
    if (constantType == targetType)
      return true;
    if ((constantType.isBaseType()) && (targetType.isBaseType()))
    {
      if (((constantType == TypeBinding.INT) || 
        (BaseTypeBinding.isWidening(10, constantType.id))) && 
        (BaseTypeBinding.isNarrowing(targetType.id, 10)))
      {
        return isConstantValueRepresentable(this.constant, constantType.id, targetType.id);
      }
    }
    return false;
  }

  public boolean isTypeReference() {
    return false;
  }

  public LocalVariableBinding localVariableBinding()
  {
    return null;
  }

  public void markAsNonNull()
  {
    this.bits |= 131072;
  }

  public int nullStatus(FlowInfo flowInfo)
  {
    if ((this.constant != null) && (this.constant != Constant.NotAConstant)) {
      return -1;
    }
    LocalVariableBinding local = localVariableBinding();
    if (local != null) {
      if (flowInfo.isDefinitelyNull(local))
        return 1;
      if (flowInfo.isDefinitelyNonNull(local))
        return -1;
      return 0;
    }
    return -1;
  }

  public Constant optimizedBooleanConstant()
  {
    return this.constant;
  }

  public TypeBinding postConversionType(Scope scope)
  {
    TypeBinding convertedType = this.resolvedType;
    int runtimeType = (this.implicitConversion & 0xFF) >> 4;
    switch (runtimeType) {
    case 5:
      convertedType = TypeBinding.BOOLEAN;
      break;
    case 3:
      convertedType = TypeBinding.BYTE;
      break;
    case 4:
      convertedType = TypeBinding.SHORT;
      break;
    case 2:
      convertedType = TypeBinding.CHAR;
      break;
    case 10:
      convertedType = TypeBinding.INT;
      break;
    case 9:
      convertedType = TypeBinding.FLOAT;
      break;
    case 7:
      convertedType = TypeBinding.LONG;
      break;
    case 8:
      convertedType = TypeBinding.DOUBLE;
    case 6:
    }

    if ((this.implicitConversion & 0x200) != 0) {
      convertedType = scope.environment().computeBoxingType(convertedType);
    }
    return convertedType;
  }

  public StringBuffer print(int indent, StringBuffer output) {
    printIndent(indent, output);
    return printExpression(indent, output);
  }

  public abstract StringBuffer printExpression(int paramInt, StringBuffer paramStringBuffer);

  public StringBuffer printStatement(int indent, StringBuffer output) {
    return print(indent, output).append(";");
  }

  public void resolve(BlockScope scope)
  {
    resolveType(scope);
  }

  public TypeBinding resolveType(BlockScope scope)
  {
    return null;
  }

  public TypeBinding resolveType(ClassScope scope)
  {
    return null;
  }

  public TypeBinding resolveTypeExpecting(BlockScope scope, TypeBinding expectedType)
  {
    setExpectedType(expectedType);
    TypeBinding expressionType = resolveType(scope);
    if (expressionType == null) return null;
    if (expressionType == expectedType) return expressionType;

    if (!expressionType.isCompatibleWith(expectedType)) {
      if (scope.isBoxingCompatibleWith(expressionType, expectedType)) {
        computeConversion(scope, expectedType, expressionType);
      } else {
        scope.problemReporter().typeMismatchError(expressionType, expectedType, this);
        return null;
      }
    }
    return expressionType;
  }

  public Object reusableJSRTarget()
  {
    if (this.constant != Constant.NotAConstant)
      return this.constant;
    return null;
  }

  public void setExpectedType(TypeBinding expectedType)
  {
  }

  public void tagAsNeedCheckCast()
  {
  }

  public void tagAsUnnecessaryCast(Scope scope, TypeBinding castType)
  {
  }

  public Expression toTypeReference()
  {
    return this;
  }

  public void traverse(ASTVisitor visitor, BlockScope scope)
  {
  }

  public void traverse(ASTVisitor visitor, ClassScope scope)
  {
  }
}