package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.impl.DoubleConstant;
import org.eclipse.jdt.internal.compiler.impl.IntConstant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class IntLiteral extends NumberLiteral
{
  public int value;
  public static final IntLiteral One = new IntLiteral(new char[] { '1' }, 0, 0, 1);

  static final Constant FORMAT_ERROR = DoubleConstant.fromValue((1.0D / 0.0D));

  public IntLiteral(char[] token, int s, int e) { super(token, s, e); }

  public IntLiteral(char[] token, int s, int e, int value) {
    this(token, s, e);
    this.value = value;
  }

  public IntLiteral(int intValue)
  {
    super(null, 0, 0);
    this.constant = IntConstant.fromValue(intValue);
    this.value = intValue;
  }

  public void computeConstant()
  {
    long MAX = 2147483647L;
    if (this == One) { this.constant = IntConstant.fromValue(1); return;
    }
    int length = this.source.length;
    long computedValue = 0L;
    if (this.source[0] == '0') {
      MAX = 4294967295L;
      if (length == 1) { this.constant = IntConstant.fromValue(0);
        return;
      }
      int radix;
      int shift;
      int j;
      int radix;
      if ((this.source[1] == 'x') || (this.source[1] == 'X')) {
        int shift = 4; int j = 2; radix = 16;
      } else {
        shift = 3; j = 1; radix = 8;
      }while (this.source[j] == '0') {
        j++;
        if (j == length)
        {
          this.constant = IntConstant.fromValue(this.value = (int)computedValue);
          return;
        }
      }
      while (j < length)
      {
        int digitValue;
        if ((digitValue = ScannerHelper.digit(this.source[(j++)], radix)) < 0) {
          this.constant = FORMAT_ERROR; return;
        }computedValue = computedValue << shift | digitValue;
        if (computedValue > MAX) return; 
      }
    }
    else { for (int i = 0; i < length; i++)
      {
        int digitValue;
        if ((digitValue = ScannerHelper.digit(this.source[i], 10)) < 0) {
          this.constant = FORMAT_ERROR; return;
        }computedValue = 10L * computedValue + digitValue;
        if (computedValue > MAX) return; 
      }
    }
    this.constant = IntConstant.fromValue(this.value = (int)computedValue);
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired)
  {
    int pc = codeStream.position;
    if (valueRequired) {
      codeStream.generateConstant(this.constant, this.implicitConversion);
    }
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  public TypeBinding literalType(BlockScope scope) {
    return TypeBinding.INT;
  }

  public final boolean mayRepresentMIN_VALUE()
  {
    return (this.source.length == 10) && 
      (this.source[0] == '2') && 
      (this.source[1] == '1') && 
      (this.source[2] == '4') && 
      (this.source[3] == '7') && 
      (this.source[4] == '4') && 
      (this.source[5] == '8') && 
      (this.source[6] == '3') && 
      (this.source[7] == '6') && 
      (this.source[8] == '4') && 
      (this.source[9] == '8') && 
      ((this.bits & 0x1FE00000) >> 21 == 0);
  }

  public TypeBinding resolveType(BlockScope scope)
  {
    TypeBinding tb = super.resolveType(scope);
    if (this.constant == FORMAT_ERROR) {
      this.constant = Constant.NotAConstant;
      scope.problemReporter().constantOutOfFormat(this);
      this.resolvedType = null;
      return null;
    }
    return tb;
  }

  public StringBuffer printExpression(int indent, StringBuffer output) {
    if (this.source == null)
    {
      return output.append(String.valueOf(this.value));
    }
    return super.printExpression(indent, output);
  }

  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}