package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class JavadocImplicitTypeReference extends TypeReference
{
  public char[] token;

  public JavadocImplicitTypeReference(char[] name, int pos)
  {
    this.token = name;
    this.sourceStart = pos;
    this.sourceEnd = pos;
  }

  public TypeReference copyDims(int dim)
  {
    return null;
  }

  protected TypeBinding getTypeBinding(Scope scope)
  {
    this.constant = Constant.NotAConstant;
    return this.resolvedType = scope.enclosingSourceType();
  }

  public char[] getLastToken() {
    return this.token;
  }

  public char[][] getTypeName()
  {
    if (this.token != null) {
      char[][] tokens = { this.token };
      return tokens;
    }
    return null;
  }
  public boolean isThis() {
    return true;
  }

  private TypeBinding internalResolveType(Scope scope)
  {
    this.constant = Constant.NotAConstant;
    if (this.resolvedType != null) {
      return this.resolvedType.isValidBinding() ? this.resolvedType : null;
    }
    this.resolvedType = scope.enclosingSourceType();
    if (this.resolvedType == null)
      return null;
    if (!this.resolvedType.isValidBinding()) {
      reportInvalidType(scope);
      return null;
    }
    if (isTypeUseDeprecated(this.resolvedType, scope))
      reportDeprecatedType(this.resolvedType, scope);
    return this.resolvedType;
  }

  protected void reportInvalidType(Scope scope) {
    scope.problemReporter().javadocInvalidType(this, this.resolvedType, scope.getDeclarationModifiers());
  }
  protected void reportDeprecatedType(TypeBinding type, Scope scope) {
    scope.problemReporter().javadocDeprecatedType(type, this, scope.getDeclarationModifiers());
  }

  public TypeBinding resolveType(BlockScope blockScope, boolean checkBounds) {
    return internalResolveType(blockScope);
  }

  public TypeBinding resolveType(ClassScope classScope) {
    return internalResolveType(classScope);
  }

  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }

  public void traverse(ASTVisitor visitor, ClassScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }

  public StringBuffer printExpression(int indent, StringBuffer output) {
    return new StringBuffer();
  }
}