package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class SingleTypeReference extends TypeReference
{
  public char[] token;

  public SingleTypeReference(char[] source, long pos)
  {
    this.token = source;
    this.sourceStart = ((int)(pos >>> 32));
    this.sourceEnd = ((int)(pos & 0xFFFFFFFF));
  }

  public TypeReference copyDims(int dim)
  {
    return new ArrayTypeReference(this.token, dim, (this.sourceStart << 32) + this.sourceEnd);
  }

  public char[] getLastToken() {
    return this.token;
  }
  protected TypeBinding getTypeBinding(Scope scope) {
    if (this.resolvedType != null) {
      return this.resolvedType;
    }
    this.resolvedType = scope.getType(this.token);

    if ((scope.kind == 3) && (this.resolvedType.isValidBinding()) && 
      (((ClassScope)scope).detectHierarchyCycle(this.resolvedType, this)))
      return null;
    return this.resolvedType;
  }

  public char[][] getTypeName() {
    return new char[][] { this.token };
  }

  public StringBuffer printExpression(int indent, StringBuffer output)
  {
    return output.append(this.token);
  }

  public TypeBinding resolveTypeEnclosing(BlockScope scope, ReferenceBinding enclosingType)
  {
    TypeBinding memberType = scope.getMemberType(this.token, enclosingType);
    if (!memberType.isValidBinding()) {
      this.resolvedType = memberType;
      scope.problemReporter().invalidEnclosingType(this, memberType, enclosingType);
      return null;
    }
    if (isTypeUseDeprecated(memberType, scope))
      scope.problemReporter().deprecatedType(memberType, this);
    memberType = scope.environment().convertToRawType(memberType);
    if ((memberType.isRawType()) && 
      ((this.bits & 0x40000000) == 0) && 
      (scope.compilerOptions().getSeverity(35184372088832L) != -1)) {
      scope.problemReporter().rawTypeReference(this, memberType);
    }
    return this.resolvedType = memberType;
  }

  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }

  public void traverse(ASTVisitor visitor, ClassScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}