package org.eclipse.jdt.internal.compiler.batch;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;
import org.eclipse.jdt.core.compiler.CategorizedProblem;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.AbstractAnnotationProcessorManager;
import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.ClassFilePool;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.Compiler;
import org.eclipse.jdt.internal.compiler.ICompilerRequestor;
import org.eclipse.jdt.internal.compiler.IErrorHandlingPolicy;
import org.eclipse.jdt.internal.compiler.IProblemFactory;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.env.AccessRule;
import org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.problem.ProblemSeverities;
import org.eclipse.jdt.internal.compiler.util.GenericXMLWriter;
import org.eclipse.jdt.internal.compiler.util.HashtableOfInt;
import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
import org.eclipse.jdt.internal.compiler.util.Messages;
import org.eclipse.jdt.internal.compiler.util.SuffixConstants;
import org.eclipse.jdt.internal.compiler.util.Util;

public class Main
  implements ProblemSeverities, SuffixConstants
{
  public static final String bundleName = "org.eclipse.jdt.internal.compiler.batch.messages";
  public static final int DEFAULT_SIZE_CLASSPATH = 4;
  public static final String NONE = "none";
  public Compiler batchCompiler;
  public ResourceBundle bundle;
  protected FileSystem.Classpath[] checkedClasspaths;
  public Locale compilerLocale;
  public CompilerOptions compilerOptions;
  public String destinationPath;
  public String[] destinationPaths;
  private boolean didSpecifySource;
  private boolean didSpecifyTarget;
  public String[] encodings;
  public int exportedClassFilesCounter;
  public String[] filenames;
  public String[] classNames;
  public int globalErrorsCount;
  public int globalProblemsCount;
  public int globalTasksCount;
  public int globalWarningsCount;
  private File javaHomeCache;
  private boolean javaHomeChecked = false;
  public long lineCount;
  public String log;
  public Logger logger;
  public int maxProblems;
  public boolean noWarn = false;
  public Map options;
  protected PrintWriter out;
  public boolean proceed = true;
  public boolean proceedOnError = false;
  public boolean produceRefInfo = false;
  public int repetitions;
  public boolean showProgress = false;
  public long startTime;
  public boolean systemExitWhenFinished = true;
  public long[] times;
  public int timesCounter;
  public boolean timing = false;

  public boolean verbose = false;
  private String[] expandedCommandLine;
  private PrintWriter err;
  ArrayList extraProblems;
  static Class class$0;

  public static boolean compile(String commandLine)
  {
    return compile(commandLine, new PrintWriter(System.out), new PrintWriter(System.err));
  }

  public static boolean compile(String commandLine, PrintWriter outWriter, PrintWriter errWriter)
  {
    return new Main(outWriter, errWriter, false).compile(tokenize(commandLine));
  }
  public static File[][] getLibrariesFiles(File[] files) {
    FilenameFilter filter = new FilenameFilter() {
      public boolean accept(File dir, String name) {
        String lowerCaseName = name.toLowerCase();
        if ((lowerCaseName.endsWith(".jar")) || (lowerCaseName.endsWith(".zip"))) {
          return true;
        }
        return false;
      }
    };
    int filesLength = files.length;
    File[][] result = new File[filesLength][];
    for (int i = 0; i < filesLength; i++) {
      File currentFile = files[i];
      if ((currentFile.exists()) && (currentFile.isDirectory())) {
        result[i] = currentFile.listFiles(filter);
      }
    }
    return result;
  }
  public static void main(String[] argv) {
    new Main(new PrintWriter(System.out), new PrintWriter(System.err), true).compile(argv);
  }

  public static String[] tokenize(String commandLine) {
    int count = 0;
    String[] arguments = new String[10];
    StringTokenizer tokenizer = new StringTokenizer(commandLine, " \"", true);
    String token = Util.EMPTY_STRING;
    boolean insideQuotes = false;
    boolean startNewToken = true;

    while (tokenizer.hasMoreTokens()) {
      token = tokenizer.nextToken();

      if (token.equals(" ")) {
        if (insideQuotes)
        {
          int tmp59_58 = (count - 1);
          String[] tmp59_55 = arguments; tmp59_55[tmp59_58] = (tmp59_55[tmp59_58] + token);
          startNewToken = false;
        } else {
          startNewToken = true;
        }
      } else if (token.equals("\"")) {
        if ((!insideQuotes) && (startNewToken)) {
          if (count == arguments.length)
            System.arraycopy(arguments, 0, arguments = new String[count * 2], 0, count);
          arguments[(count++)] = Util.EMPTY_STRING;
        }
        insideQuotes = !insideQuotes;
        startNewToken = false;
      } else {
        if (insideQuotes)
        {
          int tmp170_169 = (count - 1);
          String[] tmp170_166 = arguments; tmp170_166[tmp170_169] = (tmp170_166[tmp170_169] + token);
        }
        else if ((token.length() > 0) && (!startNewToken))
        {
          int tmp212_211 = (count - 1);
          String[] tmp212_208 = arguments; tmp212_208[tmp212_211] = (tmp212_208[tmp212_211] + token);
        } else {
          if (count == arguments.length)
            System.arraycopy(arguments, 0, arguments = new String[count * 2], 0, count);
          String trimmedToken = token.trim();
          if (trimmedToken.length() != 0) {
            arguments[(count++)] = trimmedToken;
          }
        }

        startNewToken = false;
      }
    }
    System.arraycopy(arguments, 0, arguments = new String[count], 0, count);
    return arguments;
  }

  public Main(PrintWriter outWriter, PrintWriter errWriter, boolean systemExitWhenFinished)
  {
    this(outWriter, errWriter, systemExitWhenFinished, null);
  }

  public Main(PrintWriter outWriter, PrintWriter errWriter, boolean systemExitWhenFinished, Map customDefaultOptions) {
    initialize(outWriter, errWriter, systemExitWhenFinished, customDefaultOptions);
    relocalize();
  }
  public void addExtraProblems(CategorizedProblem problem) {
    if (this.extraProblems == null) {
      this.extraProblems = new ArrayList();
    }
    this.extraProblems.add(problem);
  }

  protected void addNewEntry(ArrayList paths, String currentClasspathName, ArrayList currentRuleSpecs, String customEncoding, String destPath, boolean isSourceOnly, boolean rejectDestinationPathOnJars)
    throws InvalidInputException
  {
    int rulesSpecsSize = currentRuleSpecs.size();
    AccessRuleSet accessRuleSet = null;
    if (rulesSpecsSize != 0) {
      AccessRule[] accessRules = new AccessRule[currentRuleSpecs.size()];
      boolean rulesOK = true;
      Iterator i = currentRuleSpecs.iterator();
      int j = 0;
      while (i.hasNext()) {
        String ruleSpec = (String)i.next();
        char key = ruleSpec.charAt(0);
        String pattern = ruleSpec.substring(1);
        if (pattern.length() > 0)
          switch (key) {
          case '+':
            accessRules[(j++)] = new AccessRule(
              pattern.toCharArray(), 0);
            break;
          case '~':
            accessRules[(j++)] = new AccessRule(
              pattern.toCharArray(), 
              16777496);
            break;
          case '-':
            accessRules[(j++)] = new AccessRule(
              pattern.toCharArray(), 
              16777523);
            break;
          case '?':
            accessRules[(j++)] = new AccessRule(
              pattern.toCharArray(), 
              16777523, true);
            break;
          default:
            rulesOK = false; break;
          }
        else {
          rulesOK = false;
        }
      }
      if (rulesOK) {
        String[] templates = new String[4];
        templates[0] = 
          bind("template.restrictedAccess.type", 
          new String[] { "{0}", currentClasspathName });
        templates[1] = 
          bind("template.restrictedAccess.constructor", 
          new String[] { "{0}", currentClasspathName });
        templates[2] = 
          bind("template.restrictedAccess.method", 
          new String[] { "{0}", "{1}", currentClasspathName });
        templates[3] = 
          bind("template.restrictedAccess.field", 
          new String[] { "{0}", "{1}", currentClasspathName });
        accessRuleSet = new AccessRuleSet(accessRules, templates);
      } else {
        if (currentClasspathName.length() != 0)
        {
          this.logger.logIncorrectClasspath(currentClasspathName);
        }
        return;
      }
    }
    if ("none".equals(destPath)) {
      destPath = "none";
    }
    if ((rejectDestinationPathOnJars) && (destPath != null) && (
      (currentClasspathName.endsWith(".jar")) || 
      (currentClasspathName.endsWith(".zip")))) {
      throw new InvalidInputException(
        bind("configure.unexpectedDestinationPathEntryFile", 
        currentClasspathName));
    }
    FileSystem.Classpath currentClasspath = FileSystem.getClasspath(
      currentClasspathName, 
      customEncoding, 
      isSourceOnly, 
      accessRuleSet, 
      destPath);
    if (currentClasspath != null)
      paths.add(currentClasspath);
    else if (currentClasspathName.length() != 0)
    {
      this.logger.logIncorrectClasspath(currentClasspathName);
    }
  }

  public String bind(String id)
  {
    return bind(id, null);
  }

  public String bind(String id, String binding)
  {
    return bind(id, new String[] { binding });
  }

  public String bind(String id, String binding1, String binding2)
  {
    return bind(id, new String[] { binding1, binding2 });
  }

  public String bind(String id, String[] arguments)
  {
    if (id == null)
      return "No message available";
    String message = null;
    try {
      message = this.bundle.getString(id);
    }
    catch (MissingResourceException localMissingResourceException)
    {
      return "Missing message: " + id + " in: " + "org.eclipse.jdt.internal.compiler.batch.messages";
    }
    return MessageFormat.format(message, arguments);
  }

  private boolean checkVMVersion(long minimalSupportedVersion)
  {
    String classFileVersion = System.getProperty("java.class.version");
    if (classFileVersion == null)
    {
      return false;
    }
    int index = classFileVersion.indexOf('.');
    if (index == -1)
    {
      return false;
    }
    try
    {
      majorVersion = Integer.parseInt(classFileVersion.substring(0, index));
    }
    catch (NumberFormatException localNumberFormatException)
    {
      int majorVersion;
      return false;
    }
    int majorVersion;
    switch (majorVersion) {
    case 45:
      return 2949123L >= minimalSupportedVersion;
    case 46:
      return 3014656L >= minimalSupportedVersion;
    case 47:
      return 3080192L >= minimalSupportedVersion;
    case 48:
      return 3145728L >= minimalSupportedVersion;
    case 49:
      return 3211264L >= minimalSupportedVersion;
    case 50:
      return 3276800L >= minimalSupportedVersion;
    case 51:
      return 3342336L >= minimalSupportedVersion;
    }

    return false;
  }

  public boolean compile(String[] argv)
  {
    try
    {
      configure(argv);
      if (this.proceed)
      {
        if (this.showProgress) this.logger.compiling();
        for (int i = 0; i < this.repetitions; i++) {
          this.globalProblemsCount = 0;
          this.globalErrorsCount = 0;
          this.globalWarningsCount = 0;
          this.globalTasksCount = 0;
          this.lineCount = 0L;
          this.exportedClassFilesCounter = 0;

          if (this.repetitions > 1) {
            this.logger.flush();
            this.logger.logRepetition(i, this.repetitions);
          }

          performCompilation();
        }
        if (this.times != null) {
          this.logger.logAverage(this.times, this.lineCount);
        }
        if (this.showProgress) this.logger.printNewLine();
      }
      if (this.systemExitWhenFinished) {
        this.logger.flush();
        this.logger.close();
        System.exit(this.globalErrorsCount > 0 ? -1 : 0);
      }
    } catch (InvalidInputException e) {
      this.logger.logException(e);
      if (this.systemExitWhenFinished) {
        this.logger.flush();
        this.logger.close();
        System.exit(-1);
      }
      return false; } catch (RuntimeException e) {
      while (true) {
        this.logger.logException(e);
        if (this.systemExitWhenFinished) {
          this.logger.flush();
          this.logger.close();
          System.exit(-1);
        }
      }
    } finally {
      this.logger.flush();
      this.logger.close();
    }
    this.logger.flush();
    this.logger.close();

    if (this.globalErrorsCount == 0)
      return true;
    return false;
  }

  protected void handleWarningToken(String token, boolean isEnabling, boolean useEnableJavadoc)
    throws InvalidInputException
  {
    if (token.equals("constructorName")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.methodWithConstructorName", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("pkgDefaultMethod")) || (token.equals("packageDefaultMethod"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("maskedCatchBlock")) || (token.equals("maskedCatchBlocks"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("deprecation")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecation", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecationInDeprecatedCode", 
        "disabled");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecationWhenOverridingDeprecatedMethod", 
        "disabled");
    } else if (token.equals("allDeprecation")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecation", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecationInDeprecatedCode", 
        isEnabling ? "enabled" : "disabled");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.deprecationWhenOverridingDeprecatedMethod", 
        isEnabling ? "enabled" : "disabled");
    } else if ((token.equals("unusedLocal")) || (token.equals("unusedLocals"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedLocal", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("unusedArgument")) || (token.equals("unusedArguments"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedParameter", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("unusedImport")) || (token.equals("unusedImports"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedImport", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("unusedPrivate")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedPrivateMember", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("unusedLabel")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedLabel", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("localHiding")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.localVariableHiding", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("fieldHiding")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.fieldHiding", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("specialParamHiding")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.specialParameterHidingField", 
        isEnabling ? "enabled" : "disabled");
    } else if (token.equals("conditionAssign")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("syntheticAccess")) || 
      (token.equals("synthetic-access"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("nls")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("staticReceiver")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.staticAccessReceiver", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("indirectStatic")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.indirectStaticAccess", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("noEffectAssign")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.noEffectAssignment", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("intfNonInherited")) || (token.equals("interfaceNonInherited"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("charConcat")) || (token.equals("noImplicitStringConversion"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("semicolon")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.emptyStatement", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("serial")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.missingSerialVersion", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("emptyBlock")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("uselessTypeCheck")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("unchecked")) || (token.equals("unsafe"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("raw")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.rawTypeReference", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("finalBound")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.finalParameterBound", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("suppress")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.suppressWarnings", 
        isEnabling ? "enabled" : "disabled");
    } else if (token.equals("warningToken")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unhandledWarningToken", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("unnecessaryElse")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unnecessaryElse", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("javadoc")) {
      if (!useEnableJavadoc) {
        this.options.put(
          "org.eclipse.jdt.core.compiler.doc.comment.support", 
          isEnabling ? "enabled" : "disabled");
      }

      if (isEnabling) {
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadoc", 
          "warning");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTags", 
          "enabled");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsDeprecatedRef", 
          "disabled");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsNotVisibleRef", 
          "disabled");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsVisibility", 
          "private");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.missingJavadocTags", 
          "warning");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.missingJavadocTagsVisibility", 
          "private");
      }
    } else if (token.equals("allJavadoc")) {
      if (!useEnableJavadoc) {
        this.options.put(
          "org.eclipse.jdt.core.compiler.doc.comment.support", 
          isEnabling ? "enabled" : "disabled");
      }

      if (isEnabling) {
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadoc", 
          "warning");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTags", 
          "enabled");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsVisibility", 
          "private");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.missingJavadocTags", 
          "warning");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.missingJavadocTagsVisibility", 
          "private");
        this.options.put(
          "org.eclipse.jdt.core.compiler.problem.missingJavadocComments", 
          "warning");
      }
    } else if (token.startsWith("tasks")) {
      String taskTags = Util.EMPTY_STRING;
      int start = token.indexOf('(');
      int end = token.indexOf(')');
      if ((start >= 0) && (end >= 0) && (start < end)) {
        taskTags = token.substring(start + 1, end).trim();
        taskTags = taskTags.replace('|', ',');
      }
      if (taskTags.length() == 0) {
        throw new InvalidInputException(bind("configure.invalidTaskTag", token));
      }
      this.options.put(
        "org.eclipse.jdt.core.compiler.taskTags", 
        isEnabling ? taskTags : Util.EMPTY_STRING);
    } else if (token.equals("assertIdentifier")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.assertIdentifier", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("enumIdentifier")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.enumIdentifier", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("finally")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("unusedThrown")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("unqualifiedField")) || 
      (token.equals("unqualified-field-access"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("typeHiding")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.typeParameterHiding", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("varargsCast")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("null")) {
      if (isEnabling) {
        this.options.put("org.eclipse.jdt.core.compiler.problem.nullReference", 
          "warning");
        this.options.put("org.eclipse.jdt.core.compiler.problem.potentialNullReference", 
          "warning");
        this.options.put("org.eclipse.jdt.core.compiler.problem.redundantNullCheck", 
          "warning");
      } else {
        this.options.put("org.eclipse.jdt.core.compiler.problem.nullReference", 
          "ignore");
        this.options.put("org.eclipse.jdt.core.compiler.problem.potentialNullReference", 
          "ignore");
        this.options.put("org.eclipse.jdt.core.compiler.problem.redundantNullCheck", 
          "ignore");
      }
    } else if (token.equals("nullDereference")) {
      if (isEnabling) {
        this.options.put("org.eclipse.jdt.core.compiler.problem.nullReference", 
          "warning");
      } else {
        this.options.put("org.eclipse.jdt.core.compiler.problem.nullReference", 
          "ignore");
        this.options.put("org.eclipse.jdt.core.compiler.problem.potentialNullReference", 
          "ignore");
        this.options.put("org.eclipse.jdt.core.compiler.problem.redundantNullCheck", 
          "ignore");
      }
    } else if (token.equals("boxing")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.autoboxing", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("over-ann")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("dep-ann")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("intfAnnotation")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.annotationSuperInterface", 
        isEnabling ? "warning" : "ignore");
    } else if ((token.equals("enumSwitch")) || 
      (token.equals("incomplete-switch"))) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("hiding")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.localVariableHiding", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.fieldHiding", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.typeParameterHiding", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("static-access")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.staticAccessReceiver", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.indirectStaticAccess", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("unused")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedLocal", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedParameter", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedImport", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedPrivateMember", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException", 
        isEnabling ? "warning" : "ignore");
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.unusedLabel", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("paramAssign")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.parameterAssignment", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("discouraged")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.discouragedReference", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("forbidden")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.forbiddenReference", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("fallthrough")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.fallthroughCase", 
        isEnabling ? "warning" : "ignore");
    } else if (token.equals("super")) {
      this.options.put(
        "org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation", 
        isEnabling ? "warning" : "ignore");
    } else {
      throw new InvalidInputException(bind("configure.invalidWarning", token));
    }
  }

  protected ArrayList handleBootclasspath(ArrayList bootclasspaths, String customEncoding)
    throws InvalidInputException
  {
    int bootclasspathsSize = bootclasspaths == null ? 0 : bootclasspaths.size();
    if (bootclasspathsSize != 0) {
      String[] paths = new String[bootclasspathsSize];
      bootclasspaths.toArray(paths);
      bootclasspaths.clear();
      for (int i = 0; i < bootclasspathsSize; i++)
        processPathEntries(4, bootclasspaths, 
          paths[i], customEncoding, false, true);
    }
    else {
      bootclasspaths = new ArrayList(4);

      String javaversion = System.getProperty("java.version");
      if ((javaversion != null) && (javaversion.equalsIgnoreCase("1.1.8"))) {
        this.logger.logWrongJDK();
        this.proceed = false;
        return null;
      }

      String bootclasspathProperty = System.getProperty("sun.boot.class.path");
      if ((bootclasspathProperty == null) || (bootclasspathProperty.length() == 0))
      {
        bootclasspathProperty = System.getProperty("vm.boot.class.path");
        if ((bootclasspathProperty == null) || (bootclasspathProperty.length() == 0))
        {
          bootclasspathProperty = System.getProperty("org.apache.harmony.boot.class.path");
        }
      }
      if ((bootclasspathProperty != null) && (bootclasspathProperty.length() != 0)) {
        StringTokenizer tokenizer = new StringTokenizer(bootclasspathProperty, File.pathSeparator);

        while (tokenizer.hasMoreTokens()) {
          String token = tokenizer.nextToken();
          FileSystem.Classpath currentClasspath = 
            FileSystem.getClasspath(token, customEncoding, null);
          if (currentClasspath != null)
            bootclasspaths.add(currentClasspath);
        }
      }
      else
      {
        File javaHome = getJavaHome();
        if (javaHome != null) {
          File[] directoriesToCheck = (File[])null;
          if (System.getProperty("os.name").startsWith("Mac")) {
            directoriesToCheck = new File[] { 
              new File(javaHome, "../Classes") };
          }
          else
          {
            directoriesToCheck = new File[] { 
              new File(javaHome, "lib") };
          }

          File[][] systemLibrariesJars = getLibrariesFiles(directoriesToCheck);
          if (systemLibrariesJars != null) {
            int i = 0; for (int max = systemLibrariesJars.length; i < max; i++) {
              File[] current = systemLibrariesJars[i];
              if (current != null) {
                int j = 0; for (int max2 = current.length; j < max2; j++) {
                  FileSystem.Classpath classpath = 
                    FileSystem.getClasspath(current[j].getAbsolutePath(), 
                    null, false, null, null);
                  if (classpath != null) {
                    bootclasspaths.add(classpath);
                  }
                }
              }
            }
          }
        }
      }
    }
    return bootclasspaths;
  }

  protected ArrayList handleClasspath(ArrayList classpaths, String customEncoding)
    throws InvalidInputException
  {
    int classpathsSize = classpaths == null ? 0 : classpaths.size();
    if (classpathsSize != 0) {
      String[] paths = new String[classpathsSize];
      classpaths.toArray(paths);
      classpaths.clear();
      for (int i = 0; i < classpathsSize; i++)
        processPathEntries(4, classpaths, paths[i], 
          customEncoding, false, true);
    }
    else
    {
      classpaths = new ArrayList(4);
      String classProp = System.getProperty("java.class.path");
      if ((classProp == null) || (classProp.length() == 0)) {
        this.logger.logNoClasspath();
        classpaths.add(FileSystem.getClasspath(System.getProperty("user.dir"), customEncoding, null));
      } else {
        StringTokenizer tokenizer = new StringTokenizer(classProp, File.pathSeparator);

        while (tokenizer.hasMoreTokens()) {
          String token = tokenizer.nextToken();
          FileSystem.Classpath currentClasspath = 
            FileSystem.getClasspath(token, customEncoding, null);
          if (currentClasspath != null)
            classpaths.add(currentClasspath);
          else if (token.length() != 0) {
            this.logger.logIncorrectClasspath(token);
          }
        }
      }
    }
    return classpaths;
  }

  protected ArrayList handleExtdirs(ArrayList extdirsClasspaths)
  {
    File javaHome = getJavaHome();

    if (extdirsClasspaths == null) {
      extdirsClasspaths = new ArrayList(4);
      String extdirsStr = System.getProperty("java.ext.dirs");
      if (extdirsStr == null) {
        extdirsClasspaths.add(javaHome.getAbsolutePath() + "/lib/ext");
      } else {
        StringTokenizer tokenizer = new StringTokenizer(extdirsStr, File.pathSeparator);
        while (tokenizer.hasMoreTokens()) {
          extdirsClasspaths.add(tokenizer.nextToken());
        }

      }

    }

    if (extdirsClasspaths.size() != 0) {
      File[] directoriesToCheck = new File[extdirsClasspaths.size()];
      for (int i = 0; i < directoriesToCheck.length; i++)
        directoriesToCheck[i] = new File((String)extdirsClasspaths.get(i));
      extdirsClasspaths.clear();
      File[][] extdirsJars = getLibrariesFiles(directoriesToCheck);
      if (extdirsJars != null) {
        int i = 0; for (int max = extdirsJars.length; i < max; i++) {
          File[] current = extdirsJars[i];
          if (current != null) {
            int j = 0; for (int max2 = current.length; j < max2; j++) {
              FileSystem.Classpath classpath = 
                FileSystem.getClasspath(
                current[j].getAbsolutePath(), 
                null, null);
              if (classpath != null)
                extdirsClasspaths.add(classpath);
            }
          }
          else if (directoriesToCheck[i].isFile()) {
            this.logger.logIncorrectExtDirsEntry(directoriesToCheck[i].getAbsolutePath());
          }
        }
      }
    }

    return extdirsClasspaths;
  }

  protected ArrayList handleEndorseddirs(ArrayList endorsedDirClasspaths)
  {
    File javaHome = getJavaHome();

    if (endorsedDirClasspaths == null) {
      endorsedDirClasspaths = new ArrayList(4);
      String endorsedDirsStr = System.getProperty("java.endorsed.dirs");
      if (endorsedDirsStr == null) {
        if (javaHome != null)
          endorsedDirClasspaths.add(javaHome.getAbsolutePath() + "/lib/endorsed");
      }
      else {
        StringTokenizer tokenizer = new StringTokenizer(endorsedDirsStr, File.pathSeparator);
        while (tokenizer.hasMoreTokens()) {
          endorsedDirClasspaths.add(tokenizer.nextToken());
        }

      }

    }

    if (endorsedDirClasspaths.size() != 0) {
      File[] directoriesToCheck = new File[endorsedDirClasspaths.size()];
      for (int i = 0; i < directoriesToCheck.length; i++)
        directoriesToCheck[i] = new File((String)endorsedDirClasspaths.get(i));
      endorsedDirClasspaths.clear();
      File[][] endorsedDirsJars = getLibrariesFiles(directoriesToCheck);
      if (endorsedDirsJars != null) {
        int i = 0; for (int max = endorsedDirsJars.length; i < max; i++) {
          File[] current = endorsedDirsJars[i];
          if (current != null) {
            int j = 0; for (int max2 = current.length; j < max2; j++) {
              FileSystem.Classpath classpath = 
                FileSystem.getClasspath(
                current[j].getAbsolutePath(), 
                null, null);
              if (classpath != null)
                endorsedDirClasspaths.add(classpath);
            }
          }
          else if (directoriesToCheck[i].isFile()) {
            this.logger.logIncorrectEndorsedDirsEntry(directoriesToCheck[i].getAbsolutePath());
          }
        }
      }
    }
    return endorsedDirClasspaths;
  }

  public void configure(String[] argv)
    throws InvalidInputException
  {
    if ((argv == null) || (argv.length == 0)) {
      printUsage();
      return;
    }

    ArrayList bootclasspaths = new ArrayList(4);
    String sourcepathClasspathArg = null;
    ArrayList sourcepathClasspaths = new ArrayList(4);
    ArrayList classpaths = new ArrayList(4);
    ArrayList extdirsClasspaths = null;
    ArrayList endorsedDirClasspaths = null;

    int index = -1;
    int filesCount = 0;
    int classCount = 0;
    int argCount = argv.length;
    int mode = 0;
    this.repetitions = 0;
    boolean printUsageRequired = false;
    String usageSection = null;
    boolean printVersionRequired = false;

    boolean didSpecifyDefaultEncoding = false;
    boolean didSpecifyDeprecation = false;
    boolean didSpecifyWarnings = false;
    boolean useEnableJavadoc = false;
    boolean didSpecifyCompliance = false;
    boolean didSpecifyDisabledAnnotationProcessing = false;

    String customEncoding = null;
    String customDestinationPath = null;
    String currentSourceDirectory = null;
    String currentArg = Util.EMPTY_STRING;

    boolean needExpansion = false;
    for (int i = 0; i < argCount; i++) {
      if (argv[i].startsWith("@")) {
        needExpansion = true;
        break;
      }
    }

    String[] newCommandLineArgs = (String[])null;
    if (needExpansion) {
      newCommandLineArgs = new String[argCount];
      index = 0;
      for (int i = 0; i < argCount; i++) {
        String[] newArgs = (String[])null;
        String arg = argv[i].trim();
        if (arg.startsWith("@")) {
          try {
            LineNumberReader reader = new LineNumberReader(new StringReader(new String(Util.getFileCharContent(new File(arg.substring(1)), null))));
            StringBuffer buffer = new StringBuffer();
            String line;
            while ((line = reader.readLine()) != null) {
              String line = line.trim();
              if (!line.startsWith("#")) {
                buffer.append(line).append(" ");
              }
            }
            newArgs = tokenize(buffer.toString());
          } catch (IOException localIOException) {
            throw new InvalidInputException(
              bind("configure.invalidexpansionargumentname", arg));
          }
        }
        if (newArgs != null) {
          int newCommandLineArgsLength = newCommandLineArgs.length;
          int newArgsLength = newArgs.length;
          System.arraycopy(newCommandLineArgs, 0, newCommandLineArgs = new String[newCommandLineArgsLength + newArgsLength - 1], 0, index);
          System.arraycopy(newArgs, 0, newCommandLineArgs, index, newArgsLength);
          index += newArgsLength;
        } else {
          newCommandLineArgs[(index++)] = arg;
        }
      }
      index = -1;
    } else {
      newCommandLineArgs = argv;
      for (int i = 0; i < argCount; i++) {
        newCommandLineArgs[i] = newCommandLineArgs[i].trim();
      }
    }
    argCount = newCommandLineArgs.length;
    this.expandedCommandLine = newCommandLineArgs;
    do
    {
      if (customEncoding != null) {
        throw new InvalidInputException(
          bind("configure.unexpectedCustomEncoding", currentArg, customEncoding));
      }

      currentArg = newCommandLineArgs[index];

      switch (mode) {
      case 0:
        if (currentArg.startsWith("[")) {
          throw new InvalidInputException(
            bind("configure.unexpectedBracket", 
            currentArg));
        }

        if (currentArg.endsWith("]"))
        {
          int encodingStart = currentArg.indexOf('[') + 1;
          if (encodingStart <= 1) {
            throw new InvalidInputException(
              bind("configure.unexpectedBracket", currentArg));
          }
          int encodingEnd = currentArg.length() - 1;
          if (encodingStart >= 1) {
            if (encodingStart < encodingEnd) {
              customEncoding = currentArg.substring(encodingStart, encodingEnd);
              try {
                new InputStreamReader(new ByteArrayInputStream(new byte[0]), customEncoding);
              } catch (UnsupportedEncodingException localUnsupportedEncodingException1) {
                throw new InvalidInputException(
                  bind("configure.unsupportedEncoding", customEncoding));
              }
            }
            currentArg = currentArg.substring(0, encodingStart - 1);
          }
        }

        if (currentArg.endsWith(".java")) {
          if (this.filenames == null) {
            this.filenames = new String[argCount - index];
            this.encodings = new String[argCount - index];
            this.destinationPaths = new String[argCount - index];
          } else if (filesCount == this.filenames.length) {
            int length = this.filenames.length;
            System.arraycopy(
              this.filenames, 
              0, 
              this.filenames = new String[length + argCount - index], 
              0, 
              length);
            System.arraycopy(
              this.encodings, 
              0, 
              this.encodings = new String[length + argCount - index], 
              0, 
              length);
            System.arraycopy(
              this.destinationPaths, 
              0, 
              this.destinationPaths = new String[length + argCount - index], 
              0, 
              length);
          }
          this.filenames[filesCount] = currentArg;
          this.encodings[(filesCount++)] = customEncoding;

          customEncoding = null;
          mode = 0;
        }
        else if (currentArg.equals("-log")) {
          if (this.log != null)
            throw new InvalidInputException(
              bind("configure.duplicateLog", currentArg));
          mode = 5;
        }
        else if (currentArg.equals("-repeat")) {
          if (this.repetitions > 0)
            throw new InvalidInputException(
              bind("configure.duplicateRepeat", currentArg));
          mode = 6;
        }
        else if (currentArg.equals("-maxProblems")) {
          if (this.maxProblems > 0)
            throw new InvalidInputException(
              bind("configure.duplicateMaxProblems", currentArg));
          mode = 11;
        }
        else if (currentArg.equals("-source")) {
          mode = 7;
        }
        else if (currentArg.equals("-encoding")) {
          mode = 8;
        }
        else if (currentArg.equals("-1.3")) {
          if (didSpecifyCompliance) {
            throw new InvalidInputException(
              bind("configure.duplicateCompliance", currentArg));
          }
          didSpecifyCompliance = true;
          this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.3");
          mode = 0;
        }
        else if (currentArg.equals("-1.4")) {
          if (didSpecifyCompliance) {
            throw new InvalidInputException(
              bind("configure.duplicateCompliance", currentArg));
          }
          didSpecifyCompliance = true;
          this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.4");
          mode = 0;
        }
        else if ((currentArg.equals("-1.5")) || (currentArg.equals("-5")) || (currentArg.equals("-5.0"))) {
          if (didSpecifyCompliance) {
            throw new InvalidInputException(
              bind("configure.duplicateCompliance", currentArg));
          }
          didSpecifyCompliance = true;
          this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.5");
          mode = 0;
        }
        else if ((currentArg.equals("-1.6")) || (currentArg.equals("-6")) || (currentArg.equals("-6.0"))) {
          if (didSpecifyCompliance) {
            throw new InvalidInputException(
              bind("configure.duplicateCompliance", currentArg));
          }
          didSpecifyCompliance = true;
          this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.6");
          mode = 0;
        }
        else if ((currentArg.equals("-1.7")) || (currentArg.equals("-7")) || (currentArg.equals("-7.0"))) {
          if (didSpecifyCompliance) {
            throw new InvalidInputException(
              bind("configure.duplicateCompliance", currentArg));
          }
          didSpecifyCompliance = true;
          this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.7");
          mode = 0;
        }
        else if (currentArg.equals("-d")) {
          if (this.destinationPath != null) {
            StringBuffer errorMessage = new StringBuffer();
            errorMessage.append(currentArg);
            if (index + 1 < argCount) {
              errorMessage.append(' ');
              errorMessage.append(newCommandLineArgs[(index + 1)]);
            }
            throw new InvalidInputException(
              bind("configure.duplicateOutputPath", errorMessage.toString()));
          }
          mode = 3;
        }
        else if ((currentArg.equals("-classpath")) || 
          (currentArg.equals("-cp"))) {
          mode = 1;
        }
        else if (currentArg.equals("-bootclasspath")) {
          if (bootclasspaths.size() > 0) {
            StringBuffer errorMessage = new StringBuffer();
            errorMessage.append(currentArg);
            if (index + 1 < argCount) {
              errorMessage.append(' ');
              errorMessage.append(newCommandLineArgs[(index + 1)]);
            }
            throw new InvalidInputException(
              bind("configure.duplicateBootClasspath", errorMessage.toString()));
          }
          mode = 9;
        }
        else if (currentArg.equals("-sourcepath")) {
          if (sourcepathClasspathArg != null) {
            StringBuffer errorMessage = new StringBuffer();
            errorMessage.append(currentArg);
            if (index + 1 < argCount) {
              errorMessage.append(' ');
              errorMessage.append(newCommandLineArgs[(index + 1)]);
            }
            throw new InvalidInputException(
              bind("configure.duplicateSourcepath", errorMessage.toString()));
          }
          mode = 13;
        }
        else if (currentArg.equals("-extdirs")) {
          if (extdirsClasspaths != null) {
            StringBuffer errorMessage = new StringBuffer();
            errorMessage.append(currentArg);
            if (index + 1 < argCount) {
              errorMessage.append(' ');
              errorMessage.append(newCommandLineArgs[(index + 1)]);
            }
            throw new InvalidInputException(
              bind("configure.duplicateExtDirs", errorMessage.toString()));
          }
          mode = 12;
        }
        else if (currentArg.equals("-endorseddirs")) {
          if (endorsedDirClasspaths != null) {
            StringBuffer errorMessage = new StringBuffer();
            errorMessage.append(currentArg);
            if (index + 1 < argCount) {
              errorMessage.append(' ');
              errorMessage.append(newCommandLineArgs[(index + 1)]);
            }
            throw new InvalidInputException(
              bind("configure.duplicateEndorsedDirs", errorMessage.toString()));
          }
          mode = 15;
        }
        else if (currentArg.equals("-progress")) {
          mode = 0;
          this.showProgress = true;
        }
        else if (currentArg.equals("-proceedOnError")) {
          mode = 0;
          this.proceedOnError = true;
        }
        else if (currentArg.equals("-time")) {
          mode = 0;
          this.timing = true;
        }
        else {
          if ((currentArg.equals("-version")) || 
            (currentArg.equals("-v"))) {
            this.logger.logVersion(true);
            this.proceed = false;
            return;
          }
          if (currentArg.equals("-showversion")) {
            printVersionRequired = true;
            mode = 0;
          }
          else if ("-deprecation".equals(currentArg)) {
            didSpecifyDeprecation = true;
            this.options.put("org.eclipse.jdt.core.compiler.problem.deprecation", "warning");
            mode = 0;
          }
          else if ((currentArg.equals("-help")) || (currentArg.equals("-?"))) {
            printUsageRequired = true;
            mode = 0;
          }
          else if ((currentArg.equals("-help:warn")) || 
            (currentArg.equals("-?:warn"))) {
            printUsageRequired = true;
            usageSection = "misc.usage.warn";
          }
          else if (currentArg.equals("-noExit")) {
            this.systemExitWhenFinished = false;
            mode = 0;
          }
          else if (currentArg.equals("-verbose")) {
            this.verbose = true;
            mode = 0;
          }
          else if (currentArg.equals("-referenceInfo")) {
            this.produceRefInfo = true;
            mode = 0;
          }
          else if (currentArg.equals("-inlineJSR")) {
            mode = 0;
            this.options.put(
              "org.eclipse.jdt.core.compiler.codegen.inlineJsrBytecode", 
              "enabled");
          }
          else if (currentArg.startsWith("-g")) {
            mode = 0;
            String debugOption = currentArg;
            int length = currentArg.length();
            if (length == 2) {
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.localVariable", 
                "generate");
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.lineNumber", 
                "generate");
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.sourceFile", 
                "generate");
            }
            else if (length > 3) {
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.localVariable", 
                "do not generate");
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.lineNumber", 
                "do not generate");
              this.options.put(
                "org.eclipse.jdt.core.compiler.debug.sourceFile", 
                "do not generate");
              if ((length != 7) || (!debugOption.equals("-g:none")))
              {
                StringTokenizer tokenizer = 
                  new StringTokenizer(debugOption.substring(3, debugOption.length()), ",");
                while (tokenizer.hasMoreTokens()) {
                  String token = tokenizer.nextToken();
                  if (token.equals("vars"))
                    this.options.put(
                      "org.eclipse.jdt.core.compiler.debug.localVariable", 
                      "generate");
                  else if (token.equals("lines"))
                    this.options.put(
                      "org.eclipse.jdt.core.compiler.debug.lineNumber", 
                      "generate");
                  else if (token.equals("source"))
                    this.options.put(
                      "org.eclipse.jdt.core.compiler.debug.sourceFile", 
                      "generate");
                  else
                    throw new InvalidInputException(
                      bind("configure.invalidDebugOption", debugOption));
                }
              }
            }
            else {
              throw new InvalidInputException(
                bind("configure.invalidDebugOption", debugOption));
            }
          } else if (currentArg.startsWith("-nowarn")) {
            disableWarnings();
            mode = 0;
          }
          else if (currentArg.startsWith("-warn")) {
            mode = 0;
            String warningOption = currentArg;
            int length = currentArg.length();
            if ((length == 10) && (warningOption.equals("-warn:none"))) {
              disableWarnings();
            }
            else {
              if (length <= 6)
                throw new InvalidInputException(
                  bind("configure.invalidWarningConfiguration", warningOption));
              boolean isEnabling;
              boolean isEnabling;
              int warnTokenStart;
              boolean isEnabling;
              switch (warningOption.charAt(6)) {
              case '+':
                int warnTokenStart = 7;
                isEnabling = true;
                break;
              case '-':
                int warnTokenStart = 7;
                isEnabling = false;
                break;
              case ',':
              default:
                warnTokenStart = 6;

                if (!didSpecifyWarnings) disableWarnings();
                isEnabling = true;
              }

              StringTokenizer tokenizer = 
                new StringTokenizer(warningOption.substring(warnTokenStart, warningOption.length()), ",");
              int tokenCounter = 0;

              if (didSpecifyDeprecation) {
                this.options.put("org.eclipse.jdt.core.compiler.problem.deprecation", "warning");
              }

              while (tokenizer.hasMoreTokens()) {
                String token = tokenizer.nextToken();
                tokenCounter++;
                handleWarningToken(token, isEnabling, useEnableJavadoc);
              }
              if (tokenCounter == 0)
                throw new InvalidInputException(
                  bind("configure.invalidWarningOption", currentArg));
              didSpecifyWarnings = true;
            }
          }
          else if (currentArg.equals("-target")) {
            mode = 4;
          }
          else if (currentArg.equals("-preserveAllLocals")) {
            this.options.put(
              "org.eclipse.jdt.core.compiler.codegen.unusedLocal", 
              "preserve");
            mode = 0;
          }
          else if (currentArg.equals("-enableJavadoc")) {
            mode = 0;
            this.options.put(
              "org.eclipse.jdt.core.compiler.doc.comment.support", 
              "enabled");
            useEnableJavadoc = true;
          }
          else if (currentArg.equals("-Xemacs")) {
            mode = 0;
            this.logger.setEmacs();
          }
          else if (currentArg.startsWith("-A")) {
            mode = 0;
          }
          else if (currentArg.equals("-processorpath")) {
            mode = 17;
          }
          else if (currentArg.equals("-processor")) {
            mode = 18;
          }
          else if (currentArg.equals("-proc:only")) {
            this.options.put(
              "org.eclipse.jdt.core.compiler.generateClassFiles", 
              "disabled");
            mode = 0;
          }
          else if (currentArg.equals("-proc:none")) {
            didSpecifyDisabledAnnotationProcessing = true;
            this.options.put(
              "org.eclipse.jdt.core.compiler.processAnnotations", 
              "disabled");
            mode = 0;
          }
          else if (currentArg.equals("-s")) {
            mode = 19;
          }
          else if ((currentArg.equals("-XprintProcessorInfo")) || 
            (currentArg.equals("-XprintRounds"))) {
            mode = 0;
          }
          else if (currentArg.startsWith("-X")) {
            mode = 0;
          }
          else if (currentArg.startsWith("-J")) {
            mode = 0;
          }
          else if (currentArg.equals("-O")) {
            mode = 0;
          }
          else if (currentArg.equals("-classNames")) {
            mode = 20; } 
        }break;
      case 4:
        if (this.didSpecifyTarget) {
          throw new InvalidInputException(
            bind("configure.duplicateTarget", currentArg));
        }
        this.didSpecifyTarget = true;
        if (currentArg.equals("1.1"))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.1");
        else if (currentArg.equals("1.2"))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.2");
        else if (currentArg.equals("1.3"))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.3");
        else if (currentArg.equals("1.4"))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4");
        else if ((currentArg.equals("1.5")) || (currentArg.equals("5")) || (currentArg.equals("5.0")))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.5");
        else if ((currentArg.equals("1.6")) || (currentArg.equals("6")) || (currentArg.equals("6.0")))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.6");
        else if ((currentArg.equals("1.7")) || (currentArg.equals("7")) || (currentArg.equals("7.0")))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.7");
        else if (currentArg.equals("jsr14"))
          this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "jsr14");
        else {
          throw new InvalidInputException(bind("configure.targetJDK", currentArg));
        }
        mode = 0;
        break;
      case 5:
        this.log = currentArg;
        mode = 0;
        break;
      case 6:
        try {
          this.repetitions = Integer.parseInt(currentArg);
          if (this.repetitions <= 0)
            throw new InvalidInputException(bind("configure.repetition", currentArg));
        }
        catch (NumberFormatException localNumberFormatException1) {
          throw new InvalidInputException(bind("configure.repetition", currentArg));
        }
        mode = 0;
        break;
      case 11:
        try {
          this.maxProblems = Integer.parseInt(currentArg);
          if (this.maxProblems <= 0) {
            throw new InvalidInputException(bind("configure.maxProblems", currentArg));
          }
          this.options.put("org.eclipse.jdt.core.compiler.maxProblemPerUnit", currentArg);
        } catch (NumberFormatException localNumberFormatException2) {
          throw new InvalidInputException(bind("configure.maxProblems", currentArg));
        }
        mode = 0;
        break;
      case 7:
        if (this.didSpecifySource) {
          throw new InvalidInputException(
            bind("configure.duplicateSource", currentArg));
        }
        this.didSpecifySource = true;
        if (currentArg.equals("1.3"))
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.3");
        else if (currentArg.equals("1.4"))
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.4");
        else if ((currentArg.equals("1.5")) || (currentArg.equals("5")) || (currentArg.equals("5.0")))
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.5");
        else if ((currentArg.equals("1.6")) || (currentArg.equals("6")) || (currentArg.equals("6.0")))
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.6");
        else if ((currentArg.equals("1.7")) || (currentArg.equals("7")) || (currentArg.equals("7.0")))
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.7");
        else {
          throw new InvalidInputException(bind("configure.source", currentArg));
        }
        mode = 0;
        break;
      case 8:
        if (didSpecifyDefaultEncoding)
          throw new InvalidInputException(
            bind("configure.duplicateDefaultEncoding", currentArg));
        try
        {
          new InputStreamReader(new ByteArrayInputStream(new byte[0]), currentArg);
        } catch (UnsupportedEncodingException localUnsupportedEncodingException2) {
          throw new InvalidInputException(
            bind("configure.unsupportedEncoding", currentArg));
        }
        this.options.put("org.eclipse.jdt.core.encoding", currentArg);
        didSpecifyDefaultEncoding = true;
        mode = 0;
        break;
      case 3:
        setDestinationPath(currentArg.equals("none") ? "none" : currentArg);
        mode = 0;
        break;
      case 1:
        mode = 0;
        index += processPaths(newCommandLineArgs, index, currentArg, classpaths);
        break;
      case 9:
        mode = 0;
        index += processPaths(newCommandLineArgs, index, currentArg, bootclasspaths);
        break;
      case 13:
        mode = 0;
        String[] sourcePaths = new String[1];
        index += processPaths(newCommandLineArgs, index, currentArg, sourcePaths);
        sourcepathClasspathArg = sourcePaths[0];
        break;
      case 12:
        if (currentArg.indexOf("[-d") != -1) {
          throw new InvalidInputException(
            bind("configure.unexpectedDestinationPathEntry", 
            "-extdir"));
        }
        StringTokenizer tokenizer = new StringTokenizer(currentArg, File.pathSeparator, false);
        extdirsClasspaths = new ArrayList(4);
        while (tokenizer.hasMoreTokens())
          extdirsClasspaths.add(tokenizer.nextToken());
        mode = 0;
        break;
      case 15:
        if (currentArg.indexOf("[-d") != -1)
          throw new InvalidInputException(
            bind("configure.unexpectedDestinationPathEntry", 
            "-endorseddirs"));
        StringTokenizer tokenizer = new StringTokenizer(currentArg, File.pathSeparator, false);
        endorsedDirClasspaths = new ArrayList(4);
        while (tokenizer.hasMoreTokens())
          endorsedDirClasspaths.add(tokenizer.nextToken());
        mode = 0;
        break;
      case 16:
        if (currentArg.endsWith("]"))
          customDestinationPath = currentArg.substring(0, 
            currentArg.length() - 1);
        else {
          throw new InvalidInputException(
            bind("configure.incorrectDestinationPathEntry", 
            "[-d " + currentArg));
        }

        break;
      case 17:
        mode = 0;
        break;
      case 18:
        mode = 0;
        break;
      case 19:
        mode = 0;
        break;
      case 20:
        StringTokenizer tokenizer = new StringTokenizer(currentArg, ",");
        if (this.classNames == null) {
          this.classNames = new String[4];
        }
        while (tokenizer.hasMoreTokens()) {
          if (this.classNames.length == classCount)
          {
            System.arraycopy(
              this.classNames, 
              0, 
              this.classNames = new String[classCount * 2], 
              0, 
              classCount);
          }
          this.classNames[(classCount++)] = tokenizer.nextToken();
        }
        mode = 0;
        break;
      case 2:
      case 10:
      case 14:
      }if (customDestinationPath == null) {
        if (File.separatorChar != '/') {
          currentArg = currentArg.replace('/', File.separatorChar);
        }
        if (currentArg.endsWith("[-d")) {
          currentSourceDirectory = currentArg.substring(0, 
            currentArg.length() - 3);
          mode = 16;
        }
        else {
          currentSourceDirectory = currentArg;
        }
      } else {
        File dir = new File(currentSourceDirectory);
        if (!dir.isDirectory()) {
          throw new InvalidInputException(
            bind("configure.unrecognizedOption", currentSourceDirectory));
        }
        String[] result = FileFinder.find(dir, ".JAVA");
        if ("none".equals(customDestinationPath)) {
          customDestinationPath = "none";
        }
        if (this.filenames != null)
        {
          int length = result.length;
          System.arraycopy(
            this.filenames, 
            0, 
            this.filenames = new String[length + filesCount], 
            0, 
            filesCount);
          System.arraycopy(
            this.encodings, 
            0, 
            this.encodings = new String[length + filesCount], 
            0, 
            filesCount);
          System.arraycopy(
            this.destinationPaths, 
            0, 
            this.destinationPaths = new String[length + filesCount], 
            0, 
            filesCount);
          System.arraycopy(result, 0, this.filenames, filesCount, length);
          for (int i = 0; i < length; i++) {
            this.encodings[(filesCount + i)] = customEncoding;
            this.destinationPaths[(filesCount + i)] = customDestinationPath;
          }
          filesCount += length;
          customEncoding = null;
          customDestinationPath = null;
          currentSourceDirectory = null;
        } else {
          this.filenames = result;
          filesCount = this.filenames.length;
          this.encodings = new String[filesCount];
          this.destinationPaths = new String[filesCount];
          for (int i = 0; i < filesCount; i++) {
            this.encodings[i] = customEncoding;
            this.destinationPaths[i] = customDestinationPath;
          }
          customEncoding = null;
          customDestinationPath = null;
          currentSourceDirectory = null;
        }
        mode = 0;
      }
      index++; } while (index < argCount);

    if ((printUsageRequired) || ((filesCount == 0) && (classCount == 0))) {
      if (usageSection == null)
        printUsage();
      else {
        printUsage(usageSection);
      }
      this.proceed = false;
      return;
    }

    if (this.log != null)
      this.logger.setLog(this.log);
    else {
      this.showProgress = false;
    }
    this.logger.logVersion(printVersionRequired);

    validateOptions(didSpecifyCompliance);

    if ((!didSpecifyDisabledAnnotationProcessing) && 
      (CompilerOptions.versionToJdkLevel(this.options.get("org.eclipse.jdt.core.compiler.compliance")) >= 3276800L)) {
      this.options.put("org.eclipse.jdt.core.compiler.processAnnotations", "enabled");
    }

    this.logger.logCommandLineArguments(newCommandLineArgs);
    this.logger.logOptions(this.options);
    if (this.repetitions == 0) {
      this.repetitions = 1;
    }
    if ((this.repetitions >= 3) && (this.timing)) {
      this.times = new long[this.repetitions];
      this.timesCounter = 0;
    }

    if (filesCount != 0) {
      System.arraycopy(
        this.filenames, 
        0, 
        this.filenames = new String[filesCount], 
        0, 
        filesCount);
    }

    if (classCount != 0) {
      System.arraycopy(
        this.classNames, 
        0, 
        this.classNames = new String[classCount], 
        0, 
        classCount);
    }

    setPaths(bootclasspaths, 
      sourcepathClasspathArg, 
      sourcepathClasspaths, 
      classpaths, 
      extdirsClasspaths, 
      endorsedDirClasspaths, 
      customEncoding);
  }

  protected void disableWarnings() {
    Object[] entries = this.options.entrySet().toArray();
    int i = 0; for (int max = entries.length; i < max; i++) {
      Map.Entry entry = (Map.Entry)entries[i];
      if ((entry.getKey() instanceof String))
      {
        if ((entry.getValue() instanceof String))
        {
          if (((String)entry.getValue()).equals("warning"))
            this.options.put(entry.getKey(), "ignore"); 
        }
      }
    }
    this.options.put("org.eclipse.jdt.core.compiler.taskTags", Util.EMPTY_STRING);
  }

  public String extractDestinationPathFromSourceFile(CompilationResult result) {
    ICompilationUnit compilationUnit = result.compilationUnit;
    if (compilationUnit != null) {
      char[] fileName = compilationUnit.getFileName();
      int lastIndex = CharOperation.lastIndexOf(File.separatorChar, fileName);
      if (lastIndex != -1) {
        String outputPathName = new String(fileName, 0, lastIndex);
        File output = new File(outputPathName);
        if ((output.exists()) && (output.isDirectory())) {
          return outputPathName;
        }
      }
    }
    return System.getProperty("user.dir");
  }

  public ICompilerRequestor getBatchRequestor()
  {
    return new ICompilerRequestor() {
      int lineDelta = 0;

      public void acceptResult(CompilationResult compilationResult) { if (compilationResult.lineSeparatorPositions != null) {
          int unitLineCount = compilationResult.lineSeparatorPositions.length;
          Main.this.lineCount += unitLineCount;
          this.lineDelta += unitLineCount;
          if ((Main.this.showProgress) && (this.lineDelta > 2000))
          {
            Main.this.logger.logProgress();
            this.lineDelta = 0;
          }
        }
        Main.this.logger.startLoggingSource(compilationResult);
        if ((compilationResult.hasProblems()) || (compilationResult.hasTasks())) {
          int localErrorCount = Main.this.logger.logProblems(compilationResult.getAllProblems(), compilationResult.compilationUnit.getContents(), Main.this);

          if ((Main.this.systemExitWhenFinished) && (!Main.this.proceedOnError) && (localErrorCount > 0))
          {
            CompilationUnitDeclaration[] queuedUnits = Main.this.batchCompiler.unitsToProcess;
            int i = 0; for (int length = Main.this.batchCompiler.totalUnits; i < length; i++) {
              CompilationUnitDeclaration queuedUnit = queuedUnits[i];
              if (queuedUnit != null) {
                CompilationResult result = queuedUnit.compilationResult;
                if ((result.hasProblems()) && (!result.hasBeenAccepted))
                  Main.this.logger.logProblems(result.getAllProblems(), result.compilationUnit.getContents(), Main.this);
              }
            }
            Main.this.logger.endLoggingSource();
            Main.this.logger.endLoggingSources();
            Main.this.logger.printStats();
            Main.this.logger.flush();
            Main.this.logger.close();
            System.exit(-1);
          }
        }
        Main.this.outputClassFiles(compilationResult);
        Main.this.logger.endLoggingSource();
      }
    };
  }

  public CompilationUnit[] getCompilationUnits()
    throws InvalidInputException
  {
    int fileCount = this.filenames.length;
    CompilationUnit[] units = new CompilationUnit[fileCount];
    HashtableOfObject knownFileNames = new HashtableOfObject(fileCount);

    String defaultEncoding = (String)this.options.get("org.eclipse.jdt.core.encoding");
    if (Util.EMPTY_STRING.equals(defaultEncoding)) {
      defaultEncoding = null;
    }
    for (int i = 0; i < fileCount; i++) {
      char[] charName = this.filenames[i].toCharArray();
      if (knownFileNames.get(charName) != null)
        throw new InvalidInputException(bind("unit.more", this.filenames[i]));
      knownFileNames.put(charName, charName);
      File file = new File(this.filenames[i]);
      if (!file.exists())
        throw new InvalidInputException(bind("unit.missing", this.filenames[i]));
      String encoding = this.encodings[i];
      if (encoding == null)
        encoding = defaultEncoding;
      units[i] = new CompilationUnit(null, this.filenames[i], encoding, 
        this.destinationPaths[i]);
    }
    return units;
  }

  public IErrorHandlingPolicy getHandlingPolicy()
  {
    return new IErrorHandlingPolicy() {
      public boolean proceedOnErrors() {
        return Main.this.proceedOnError;
      }
      public boolean stopOnFirstError() {
        return false;
      }
    };
  }

  public File getJavaHome()
  {
    if (!this.javaHomeChecked) {
      this.javaHomeChecked = true;
      String javaHome = System.getProperty("java.home");
      if (javaHome != null) {
        this.javaHomeCache = new File(javaHome);
        if (!this.javaHomeCache.exists())
          this.javaHomeCache = null;
      }
    }
    return this.javaHomeCache;
  }

  public FileSystem getLibraryAccess() {
    return new FileSystem(this.checkedClasspaths, this.filenames);
  }

  public IProblemFactory getProblemFactory()
  {
    return new DefaultProblemFactory(this.compilerLocale);
  }

  protected void initialize(PrintWriter outWriter, PrintWriter errWriter, boolean systemExit)
  {
    initialize(outWriter, errWriter, systemExit, null);
  }

  protected void initialize(PrintWriter outWriter, PrintWriter errWriter, boolean systemExit, Map customDefaultOptions)
  {
    this.logger = new Logger(this, outWriter, errWriter);
    this.proceed = true;
    this.out = outWriter;
    this.err = errWriter;
    this.systemExitWhenFinished = systemExit;
    this.options = new CompilerOptions().getMap();
    if (customDefaultOptions != null) {
      this.didSpecifySource = (customDefaultOptions.get("org.eclipse.jdt.core.compiler.source") != null);
      this.didSpecifyTarget = (customDefaultOptions.get("org.eclipse.jdt.core.compiler.codegen.targetPlatform") != null);
      for (Iterator iter = customDefaultOptions.entrySet().iterator(); iter.hasNext(); ) {
        Map.Entry entry = (Map.Entry)iter.next();
        this.options.put(entry.getKey(), entry.getValue());
      }
    } else {
      this.didSpecifySource = false;
      this.didSpecifyTarget = false;
    }
    this.classNames = null;
  }

  public void outputClassFiles(CompilationResult unitResult)
  {
    if ((unitResult != null) && ((!unitResult.hasErrors()) || (this.proceedOnError))) {
      ClassFile[] classFiles = unitResult.getClassFiles();
      String currentDestinationPath = null;
      boolean generateClasspathStructure = false;
      CompilationUnit compilationUnit = 
        (CompilationUnit)unitResult.compilationUnit;
      if (compilationUnit.destinationPath == null) {
        if (this.destinationPath == null) {
          currentDestinationPath = 
            extractDestinationPathFromSourceFile(unitResult);
        } else if (this.destinationPath != "none") {
          currentDestinationPath = this.destinationPath;
          generateClasspathStructure = true;
        }
      } else if (compilationUnit.destinationPath != "none") {
        currentDestinationPath = compilationUnit.destinationPath;
        generateClasspathStructure = true;
      }
      if (currentDestinationPath != null) {
        int i = 0; for (int fileCount = classFiles.length; i < fileCount; i++)
        {
          ClassFile classFile = classFiles[i];
          char[] filename = classFile.fileName();
          int length = filename.length;
          char[] relativeName = new char[length + 6];
          System.arraycopy(filename, 0, relativeName, 0, length);
          System.arraycopy(SuffixConstants.SUFFIX_class, 0, relativeName, length, 6);
          CharOperation.replace(relativeName, '/', File.separatorChar);
          String relativeStringName = new String(relativeName);
          try {
            if (this.compilerOptions.verbose) {
              this.out.println(
                Messages.bind(
                Messages.compilation_write, 
                new String[] { 
                String.valueOf(this.exportedClassFilesCounter + 1), 
                relativeStringName }));
            }
            ClassFile.writeToDisk(
              generateClasspathStructure, 
              currentDestinationPath, 
              relativeStringName, 
              classFile);
            LookupEnvironment env = this.batchCompiler.lookupEnvironment;
            if (classFile.isShared) env.classFilePool.release(classFile);
            this.logger.logClassFile(
              generateClasspathStructure, 
              currentDestinationPath, 
              relativeStringName);
            this.exportedClassFilesCounter += 1;
          } catch (IOException e) {
            this.logger.logNoClassFileCreated(currentDestinationPath, relativeStringName, e);
          }
        }
      }
    }
  }

  public void performCompilation()
    throws InvalidInputException
  {
    this.startTime = System.currentTimeMillis();

    FileSystem environment = getLibraryAccess();
    this.compilerOptions = new CompilerOptions(this.options);
    this.compilerOptions.performMethodsFullRecovery = false;
    this.compilerOptions.performStatementsRecovery = false;
    this.batchCompiler = 
      new Compiler(
      environment, 
      getHandlingPolicy(), 
      this.compilerOptions, 
      getBatchRequestor(), 
      getProblemFactory(), 
      this.out);

    if ((this.compilerOptions.complianceLevel >= 3276800L) && 
      (this.compilerOptions.processAnnotations)) {
      if (checkVMVersion(3276800L)) {
        initializeAnnotationProcessorManager();
        if (this.classNames != null)
          this.batchCompiler.setBinaryTypes(processClassNames(this.batchCompiler.lookupEnvironment));
      }
      else
      {
        this.logger.logIncorrectVMVersionForAnnotationProcessing();
      }

    }

    this.compilerOptions.verbose = this.verbose;
    this.compilerOptions.produceReferenceInfo = this.produceRefInfo;
    try {
      this.logger.startLoggingSources();
      this.batchCompiler.compile(getCompilationUnits());
    } finally {
      this.logger.endLoggingSources();
    }

    if (this.extraProblems != null) {
      this.logger.loggingExtraProblems(this);
      this.extraProblems = null;
    }
    this.logger.printStats();

    environment.cleanup();
  }

  private ReferenceBinding[] processClassNames(LookupEnvironment environment) throws InvalidInputException {
    int length = this.classNames.length;
    ReferenceBinding[] referenceBindings = new ReferenceBinding[length];
    for (int i = 0; i < length; i++) {
      String currentName = this.classNames[i];
      char[][] compoundName = (char[][])null;
      if (currentName.indexOf('.') != -1)
      {
        char[] typeName = currentName.toCharArray();
        compoundName = CharOperation.splitOn('.', typeName);
      } else {
        compoundName = new char[][] { currentName.toCharArray() };
      }
      ReferenceBinding type = environment.getType(compoundName);
      if ((type != null) && (type.isValidBinding())) {
        if (type.isBinaryBinding())
          referenceBindings[i] = type;
      }
      else {
        throw new InvalidInputException(
          bind("configure.invalidClassName", currentName));
      }
    }
    return referenceBindings;
  }
  protected void initializeAnnotationProcessorManager() {
    try {
      Class c = Class.forName("org.eclipse.jdt.internal.compiler.apt.dispatch.BatchAnnotationProcessorManager");
      AbstractAnnotationProcessorManager annotationManager = (AbstractAnnotationProcessorManager)c.newInstance();
      annotationManager.configure(this, this.expandedCommandLine);
      annotationManager.setErr(this.err);
      annotationManager.setOut(this.out);
      this.batchCompiler.annotationProcessorManager = annotationManager;
    }
    catch (ClassNotFoundException localClassNotFoundException) {
    }
    catch (InstantiationException localInstantiationException) {
      throw new AbortCompilation();
    }
    catch (IllegalAccessException localIllegalAccessException) {
      throw new AbortCompilation();
    }
    catch (UnsupportedClassVersionError localUnsupportedClassVersionError) {
      this.logger.logIncorrectVMVersionForAnnotationProcessing();
    }
  }

  public void printUsage() { printUsage("misc.usage"); }

  private void printUsage(String sectionID) {
    this.logger.logUsage(
      bind(
      sectionID, 
      new String[] { 
      System.getProperty("path.separator"), 
      bind("compiler.name"), 
      bind("compiler.version"), 
      bind("compiler.copyright") }));

    this.logger.flush();
  }

  public void processPathEntries(int defaultSize, ArrayList paths, String currentPath, String customEncoding, boolean isSourceOnly, boolean rejectDestinationPathOnJars)
    throws InvalidInputException
  {
    String currentClasspathName = null;
    String currentDestinationPath = null;
    ArrayList currentRuleSpecs = new ArrayList(defaultSize);
    StringTokenizer tokenizer = new StringTokenizer(currentPath, 
      File.pathSeparator + "[]", true);
    ArrayList tokens = new ArrayList();
    while (tokenizer.hasMoreTokens()) {
      tokens.add(tokenizer.nextToken());
    }

    int state = 0;
    String token = null;
    int cursor = 0; int tokensNb = tokens.size(); int bracket = -1;
    while ((cursor < tokensNb) && (state != 99)) {
      token = (String)tokens.get(cursor++);
      if (token.equals(File.pathSeparator)) {
        switch (state) {
        case 0:
        case 3:
        case 10:
          break;
        case 1:
        case 2:
        case 8:
          state = 3;
          addNewEntry(paths, currentClasspathName, currentRuleSpecs, 
            customEncoding, currentDestinationPath, isSourceOnly, 
            rejectDestinationPathOnJars);
          currentRuleSpecs.clear();
          break;
        case 6:
          state = 4;
          break;
        case 7:
          throw new InvalidInputException(
            bind("configure.incorrectDestinationPathEntry", 
            currentPath));
        case 11:
          cursor = bracket + 1;
          state = 5;
          break;
        case 4:
        case 5:
        case 9:
        default:
          state = 99; break;
        }
      } else if (token.equals("[")) {
        switch (state) {
        case 0:
          currentClasspathName = "";
        case 1:
          bracket = cursor - 1;
        case 11:
          state = 10;
          break;
        case 2:
          state = 9;
          break;
        case 8:
          state = 5;
          break;
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:
        case 9:
        case 10:
        default:
          state = 99; break;
        }
      } else if (token.equals("]")) {
        switch (state) {
        case 6:
          state = 2;
          break;
        case 7:
          state = 8;
          break;
        case 10:
          state = 11;
          break;
        case 8:
        case 9:
        case 11:
        default:
          state = 99; break;
        }
      }
      else {
        switch (state) {
        case 0:
        case 3:
          state = 1;
          currentClasspathName = token;
          break;
        case 5:
          if (token.startsWith("-d ")) {
            if (currentDestinationPath != null) {
              throw new InvalidInputException(
                bind("configure.duplicateDestinationPathEntry", 
                currentPath));
            }
            currentDestinationPath = token.substring(3).trim();
            state = 7;
          }break;
        case 4:
          if (currentDestinationPath != null) {
            throw new InvalidInputException(
              bind("configure.accessRuleAfterDestinationPath", 
              currentPath));
          }
          state = 6;
          currentRuleSpecs.add(token);
          break;
        case 9:
          if (!token.startsWith("-d ")) {
            state = 99;
          } else {
            currentDestinationPath = token.substring(3).trim();
            state = 7;
          }
          break;
        case 11:
          for (int i = bracket; i < cursor; i++) {
            currentClasspathName = currentClasspathName + (String)tokens.get(i);
          }
          state = 1;
          break;
        case 10:
          break;
        case 1:
        case 2:
        case 6:
        case 7:
        case 8: } state = 99;
      }

      if ((state == 11) && (cursor == tokensNb)) {
        cursor = bracket + 1;
        state = 5;
      }
    }
    switch (state) {
    case 3:
      break;
    case 1:
    case 2:
    case 8:
      addNewEntry(paths, currentClasspathName, currentRuleSpecs, 
        customEncoding, currentDestinationPath, isSourceOnly, 
        rejectDestinationPathOnJars);
      break;
    case 4:
    case 5:
    case 6:
    case 7:
    case 9:
    case 10:
    case 11:
    default:
      if (currentPath.length() != 0)
        this.logger.logIncorrectClasspath(currentPath);
      break;
    }
  }

  private int processPaths(String[] args, int index, String currentArg, ArrayList paths) throws InvalidInputException {
    int localIndex = index;
    int count = 0;
    int i = 0; for (int max = currentArg.length(); i < max; i++) {
      switch (currentArg.charAt(i)) {
      case '[':
        count++;
        break;
      case ']':
        count--;
      case '\\':
      }
    }
    if (count == 0) {
      paths.add(currentArg); } else {
      if (count > 1) {
        throw new InvalidInputException(
          bind("configure.unexpectedBracket", 
          currentArg));
      }
      StringBuffer currentPath = new StringBuffer(currentArg);
      while (true) {
        if (localIndex >= args.length) {
          throw new InvalidInputException(
            bind("configure.unexpectedBracket", 
            currentArg));
        }
        localIndex++;
        String nextArg = args[localIndex];
        int i = 0; for (int max = nextArg.length(); i < max; i++) {
          switch (nextArg.charAt(i)) {
          case '[':
            if (count > 1) {
              throw new InvalidInputException(
                bind("configure.unexpectedBracket", 
                nextArg));
            }
            count++;
            break;
          case ']':
            count--;
          case '\\':
          }
        }
        if (count == 0) {
          currentPath.append(' ');
          currentPath.append(nextArg);
          paths.add(currentPath.toString());
          return localIndex - index;
        }if (count < 0) {
          throw new InvalidInputException(
            bind("configure.unexpectedBracket", 
            nextArg));
        }
        currentPath.append(' ');
        currentPath.append(nextArg);
      }

    }

    return localIndex - index;
  }
  private int processPaths(String[] args, int index, String currentArg, String[] paths) throws InvalidInputException {
    int localIndex = index;
    int count = 0;
    int i = 0; for (int max = currentArg.length(); i < max; i++) {
      switch (currentArg.charAt(i)) {
      case '[':
        count++;
        break;
      case ']':
        count--;
      case '\\':
      }
    }
    if (count == 0) {
      paths[0] = currentArg;
    } else {
      StringBuffer currentPath = new StringBuffer(currentArg);
      while (true) {
        localIndex++;
        if (localIndex >= args.length) {
          throw new InvalidInputException(
            bind("configure.unexpectedBracket", 
            currentArg));
        }
        String nextArg = args[localIndex];
        int i = 0; for (int max = nextArg.length(); i < max; i++) {
          switch (nextArg.charAt(i)) {
          case '[':
            if (count > 1) {
              throw new InvalidInputException(
                bind("configure.unexpectedBracket", 
                currentArg));
            }
            count++;
            break;
          case ']':
            count--;
          case '\\':
          }
        }
        if (count == 0) {
          currentPath.append(' ');
          currentPath.append(nextArg);
          paths[0] = currentPath.toString();
          return localIndex - index;
        }if (count < 0) {
          throw new InvalidInputException(
            bind("configure.unexpectedBracket", 
            currentArg));
        }
        currentPath.append(' ');
        currentPath.append(nextArg);
      }

    }

    return localIndex - index;
  }

  public void relocalize()
  {
    relocalize(Locale.getDefault());
  }

  private void relocalize(Locale locale) {
    this.compilerLocale = locale;
    try {
      this.bundle = ResourceBundleFactory.getBundle(locale);
    } catch (MissingResourceException e) {
      System.out.println("Missing resource : " + "org.eclipse.jdt.internal.compiler.batch.messages".replace('.', '/') + ".properties for locale " + locale);
      throw e;
    }
  }

  public void setLocale(Locale locale)
  {
    relocalize(locale);
  }

  public void setDestinationPath(String dest)
  {
    this.destinationPath = dest;
  }

  protected void setPaths(ArrayList bootclasspaths, String sourcepathClasspathArg, ArrayList sourcepathClasspaths, ArrayList classpaths, ArrayList extdirsClasspaths, ArrayList endorsedDirClasspaths, String customEncoding)
    throws InvalidInputException
  {
    bootclasspaths = handleBootclasspath(bootclasspaths, customEncoding);

    classpaths = handleClasspath(classpaths, customEncoding);

    if (sourcepathClasspathArg != null) {
      processPathEntries(4, sourcepathClasspaths, 
        sourcepathClasspathArg, customEncoding, true, false);
    }

    extdirsClasspaths = handleExtdirs(extdirsClasspaths);

    endorsedDirClasspaths = handleEndorseddirs(endorsedDirClasspaths);

    bootclasspaths.addAll(endorsedDirClasspaths);
    bootclasspaths.addAll(extdirsClasspaths);
    bootclasspaths.addAll(sourcepathClasspaths);
    bootclasspaths.addAll(classpaths);
    classpaths = bootclasspaths;
    classpaths = FileSystem.ClasspathNormalizer.normalize(classpaths);
    this.checkedClasspaths = new FileSystem.Classpath[classpaths.size()];
    classpaths.toArray(this.checkedClasspaths);
    this.logger.logClasspath(this.checkedClasspaths);
  }
  protected void validateOptions(boolean didSpecifyCompliance) throws InvalidInputException {
    if (didSpecifyCompliance) {
      Object version = this.options.get("org.eclipse.jdt.core.compiler.compliance");
      if ("1.3".equals(version)) {
        if (!this.didSpecifySource) this.options.put("org.eclipse.jdt.core.compiler.source", "1.3");
        if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.1"); 
      }
      else if ("1.4".equals(version)) {
        if (this.didSpecifySource) {
          Object source = this.options.get("org.eclipse.jdt.core.compiler.source");
          if ("1.3".equals(source)) {
            if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.2"); 
          }
          else if (("1.4".equals(source)) && 
            (!this.didSpecifyTarget)) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4"); 
        }
        else
        {
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.3");
          if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.2"); 
        }
      }
      else if ("1.5".equals(version)) {
        if (this.didSpecifySource) {
          Object source = this.options.get("org.eclipse.jdt.core.compiler.source");
          if (("1.3".equals(source)) || 
            ("1.4".equals(source))) {
            if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4"); 
          }
          else if (("1.5".equals(source)) && 
            (!this.didSpecifyTarget)) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.5"); 
        }
        else
        {
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.5");
          if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.5"); 
        }
      }
      else if ("1.6".equals(version)) {
        if (this.didSpecifySource) {
          Object source = this.options.get("org.eclipse.jdt.core.compiler.source");
          if (("1.3".equals(source)) || 
            ("1.4".equals(source))) {
            if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4"); 
          }
          else if ((("1.5".equals(source)) || 
            ("1.6".equals(source))) && 
            (!this.didSpecifyTarget)) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.6"); 
        }
        else
        {
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.6");
          if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.6"); 
        }
      }
      else if ("1.7".equals(version)) {
        if (this.didSpecifySource) {
          Object source = this.options.get("org.eclipse.jdt.core.compiler.source");
          if (("1.3".equals(source)) || 
            ("1.4".equals(source))) {
            if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4"); 
          }
          else if (("1.5".equals(source)) || 
            ("1.6".equals(source))) {
            if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.6"); 
          }
          else if (("1.7".equals(source)) && 
            (!this.didSpecifyTarget)) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.7"); 
        }
        else
        {
          this.options.put("org.eclipse.jdt.core.compiler.source", "1.7");
          if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.7"); 
        }
      }
    }
    else if (this.didSpecifySource) {
      Object version = this.options.get("org.eclipse.jdt.core.compiler.source");

      if ("1.4".equals(version)) {
        if (!didSpecifyCompliance) this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.4");
        if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.4"); 
      }
      else if ("1.5".equals(version)) {
        if (!didSpecifyCompliance) this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.5");
        if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.5"); 
      }
      else if ("1.6".equals(version)) {
        if (!didSpecifyCompliance) this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.6");
        if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.6"); 
      }
      else if ("1.7".equals(version)) {
        if (!didSpecifyCompliance) this.options.put("org.eclipse.jdt.core.compiler.compliance", "1.7");
        if (!this.didSpecifyTarget) this.options.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", "1.7");
      }
    }

    Object sourceVersion = this.options.get("org.eclipse.jdt.core.compiler.source");
    Object compliance = this.options.get("org.eclipse.jdt.core.compiler.compliance");
    if ((sourceVersion.equals("1.7")) && 
      (CompilerOptions.versionToJdkLevel(compliance) < 3342336L))
    {
      throw new InvalidInputException(bind("configure.incompatibleComplianceForSource", (String)this.options.get("org.eclipse.jdt.core.compiler.compliance"), "1.7"));
    }if ((sourceVersion.equals("1.6")) && 
      (CompilerOptions.versionToJdkLevel(compliance) < 3276800L))
    {
      throw new InvalidInputException(bind("configure.incompatibleComplianceForSource", (String)this.options.get("org.eclipse.jdt.core.compiler.compliance"), "1.6"));
    }if ((sourceVersion.equals("1.5")) && 
      (CompilerOptions.versionToJdkLevel(compliance) < 3211264L))
    {
      throw new InvalidInputException(bind("configure.incompatibleComplianceForSource", (String)this.options.get("org.eclipse.jdt.core.compiler.compliance"), "1.5"));
    }if ((sourceVersion.equals("1.4")) && 
      (CompilerOptions.versionToJdkLevel(compliance) < 3145728L))
    {
      throw new InvalidInputException(bind("configure.incompatibleComplianceForSource", (String)this.options.get("org.eclipse.jdt.core.compiler.compliance"), "1.4"));
    }

    if (this.didSpecifyTarget) {
      Object targetVersion = this.options.get("org.eclipse.jdt.core.compiler.codegen.targetPlatform");

      if ("jsr14".equals(targetVersion))
      {
        if (CompilerOptions.versionToJdkLevel(sourceVersion) < 3211264L)
          throw new InvalidInputException(bind("configure.incompatibleTargetForGenericSource", (String)targetVersion, (String)sourceVersion));
      }
      else
      {
        if ((CompilerOptions.versionToJdkLevel(sourceVersion) >= 3342336L) && 
          (CompilerOptions.versionToJdkLevel(targetVersion) < 3342336L)) {
          throw new InvalidInputException(bind("configure.incompatibleTargetForSource", (String)targetVersion, "1.7"));
        }

        if ((CompilerOptions.versionToJdkLevel(sourceVersion) >= 3276800L) && 
          (CompilerOptions.versionToJdkLevel(targetVersion) < 3276800L)) {
          throw new InvalidInputException(bind("configure.incompatibleTargetForSource", (String)targetVersion, "1.6"));
        }

        if ((CompilerOptions.versionToJdkLevel(sourceVersion) >= 3211264L) && 
          (CompilerOptions.versionToJdkLevel(targetVersion) < 3211264L)) {
          throw new InvalidInputException(bind("configure.incompatibleTargetForSource", (String)targetVersion, "1.5"));
        }

        if ((CompilerOptions.versionToJdkLevel(sourceVersion) >= 3145728L) && 
          (CompilerOptions.versionToJdkLevel(targetVersion) < 3145728L)) {
          throw new InvalidInputException(bind("configure.incompatibleTargetForSource", (String)targetVersion, "1.4"));
        }

        if (CompilerOptions.versionToJdkLevel(compliance) < CompilerOptions.versionToJdkLevel(targetVersion))
          throw new InvalidInputException(bind("configure.incompatibleComplianceForTarget", (String)this.options.get("org.eclipse.jdt.core.compiler.compliance"), (String)targetVersion));
      }
    }
  }

  public static class Logger
  {
    private static final String CLASS = "class";
    private static final String CLASS_FILE = "classfile";
    private static final String CLASSPATH = "classpath";
    private static final String CLASSPATH_FILE = "FILE";
    private static final String CLASSPATH_FOLDER = "FOLDER";
    private static final String CLASSPATH_ID = "id";
    private static final String CLASSPATH_JAR = "JAR";
    private static final String CLASSPATHS = "classpaths";
    private static final String COMMAND_LINE_ARGUMENT = "argument";
    private static final String COMMAND_LINE_ARGUMENTS = "command_line";
    private static final String COMPILER = "compiler";
    private static final String COMPILER_COPYRIGHT = "copyright";
    private static final String COMPILER_NAME = "name";
    private static final String COMPILER_VERSION = "version";
    public static final int EMACS = 2;
    private static final String ERROR = "ERROR";
    private static final String ERROR_TAG = "error";
    private static final String EXCEPTION = "exception";
    private static final String EXTRA_PROBLEM_TAG = "extra_problem";
    private static final String EXTRA_PROBLEMS = "extra_problems";
    private static final HashtableOfInt FIELD_TABLE = new HashtableOfInt();
    private static final String KEY = "key";
    private static final String MESSAGE = "message";
    private static final String NUMBER_OF_CLASSFILES = "number_of_classfiles";
    private static final String NUMBER_OF_ERRORS = "errors";
    private static final String NUMBER_OF_LINES = "number_of_lines";
    private static final String NUMBER_OF_PROBLEMS = "problems";
    private static final String NUMBER_OF_TASKS = "tasks";
    private static final String NUMBER_OF_WARNINGS = "warnings";
    private static final String OPTION = "option";
    private static final String OPTIONS = "options";
    private static final String OUTPUT = "output";
    private static final String PACKAGE = "package";
    private static final String PATH = "path";
    private static final String PROBLEM_ARGUMENT = "argument";
    private static final String PROBLEM_ARGUMENT_VALUE = "value";
    private static final String PROBLEM_ARGUMENTS = "arguments";
    private static final String PROBLEM_CATEGORY_ID = "categoryID";
    private static final String ID = "id";
    private static final String PROBLEM_ID = "problemID";
    private static final String PROBLEM_LINE = "line";
    private static final String PROBLEM_OPTION_KEY = "optionKey";
    private static final String PROBLEM_MESSAGE = "message";
    private static final String PROBLEM_SEVERITY = "severity";
    private static final String PROBLEM_SOURCE_END = "charEnd";
    private static final String PROBLEM_SOURCE_START = "charStart";
    private static final String PROBLEM_SUMMARY = "problem_summary";
    private static final String PROBLEM_TAG = "problem";
    private static final String PROBLEMS = "problems";
    private static final String SOURCE = "source";
    private static final String SOURCE_CONTEXT = "source_context";
    private static final String SOURCE_END = "sourceEnd";
    private static final String SOURCE_START = "sourceStart";
    private static final String SOURCES = "sources";
    private static final String STATS = "stats";
    private static final String TASK = "task";
    private static final String TASKS = "tasks";
    private static final String TIME = "time";
    private static final String VALUE = "value";
    private static final String WARNING = "WARNING";
    public static final int XML = 1;
    private static final String XML_DTD_DECLARATION = "<!DOCTYPE compiler PUBLIC \"-//Eclipse.org//DTD Eclipse JDT 3.2.003 Compiler//EN\" \"http://www.eclipse.org/jdt/core/compiler_32_003.dtd\">";
    private PrintWriter err;
    private PrintWriter log;
    private Main main;
    private PrintWriter out;
    private HashMap parameters;
    int tagBits;

    static
    {
      try
      {
        Class tmp13_10 = Main.class$0; if (tmp13_10 == null) { tmp13_10;
          try { tmpTernaryOp = (Main.class$0 = Class.forName("org.eclipse.jdt.core.compiler.IProblem")); } catch (ClassNotFoundException localClassNotFoundException) { throw new NoClassDefFoundError(localClassNotFoundException.getMessage()); }  } Class c = tmp13_10;
        Field[] fields = c.getFields();
        int i = 0; for (int max = fields.length; i < max; i++) {
          Field field = fields[i];
          if (field.getType().equals(Integer.TYPE)) {
            Integer value = (Integer)field.get(null);
            FIELD_TABLE.put(value.intValue() & 0xFFFFFF, field.getName());
          }
        }
      } catch (SecurityException e) {
        e.printStackTrace();
      } catch (IllegalArgumentException e) {
        e.printStackTrace();
      } catch (IllegalAccessException e) {
        e.printStackTrace();
      }
    }

    public Logger(Main main, PrintWriter out, PrintWriter err)
    {
      this.out = out;
      this.err = err;
      this.parameters = new HashMap();
      this.main = main;
    }

    public String buildFileName(String outputPath, String relativeFileName)
    {
      char fileSeparatorChar = File.separatorChar;
      String fileSeparator = File.separator;

      outputPath = outputPath.replace('/', fileSeparatorChar);

      StringBuffer outDir = new StringBuffer(outputPath);
      if (!outputPath.endsWith(fileSeparator)) {
        outDir.append(fileSeparator);
      }
      StringTokenizer tokenizer = 
        new StringTokenizer(relativeFileName, fileSeparator);
      String token = tokenizer.nextToken();
      while (tokenizer.hasMoreTokens()) {
        outDir.append(token).append(fileSeparator);
        token = tokenizer.nextToken();
      }

      return token;
    }

    public void close() {
      if (this.log != null) {
        if ((this.tagBits & 0x1) != 0) {
          endTag("compiler");
          flush();
        }
        this.log.close();
      }
    }

    public void compiling()
    {
      printlnOut(this.main.bind("progress.compiling"));
    }

    private void endLoggingProblems()
    {
      endTag("problems");
    }
    private void endLoggingExtraProblems() {
      endTag("extra_problems");
    }
    public void endLoggingSource() {
      if ((this.tagBits & 0x1) != 0)
        endTag("source");
    }

    public void endLoggingSources()
    {
      if ((this.tagBits & 0x1) != 0)
        endTag("sources");
    }

    public void endLoggingTasks()
    {
      if ((this.tagBits & 0x1) != 0)
        endTag("tasks");
    }

    private void endTag(String name) {
      ((GenericXMLWriter)this.log).endTag(name, true, true);
    }

    private void extractContext(CategorizedProblem problem, char[] unitSource) {
      int startPosition = problem.getSourceStart();
      int endPosition = problem.getSourceEnd();
      if ((unitSource == null) && 
        (problem.getOriginatingFileName() != null)) {
        try {
          unitSource = Util.getFileCharContent(new File(new String(problem.getOriginatingFileName())), null);
        }
        catch (IOException localIOException)
        {
        }
      }
      int length = unitSource == null ? 0 : unitSource.length;
      if ((startPosition > endPosition) || 
        ((startPosition < 0) && (endPosition < 0)) || 
        (length <= 0) || 
        (endPosition > length)) {
        this.parameters.put("value", Messages.problem_noSourceInformation);
        this.parameters.put("sourceStart", "-1");
        this.parameters.put("sourceEnd", "-1");
        printTag("source_context", this.parameters, true, true);
        return;
      }

      for (int begin = startPosition >= length ? length - 1 : startPosition; begin > 0; begin--)
      {
        char c;
        if (((c = unitSource[(begin - 1)]) == '\n') || (c == '\r')) break;
      }
      for (int end = endPosition >= length ? length - 1 : endPosition; end + 1 < length; end++)
      {
        char c;
        if (((c = unitSource[(end + 1)]) == '\r') || (c == '\n'))
          break;
      }
      char c;
      while (((c = unitSource[begin]) == ' ') || (c == '\t'))
      {
        char c;
        begin++;
      }while (((c = unitSource[end]) == ' ') || (c == '\t')) end--;

      StringBuffer buffer = new StringBuffer();
      buffer.append(unitSource, begin, end - begin + 1);

      this.parameters.put("value", String.valueOf(buffer));
      this.parameters.put("sourceStart", Integer.toString(startPosition - begin));
      this.parameters.put("sourceEnd", Integer.toString(endPosition - begin));
      printTag("source_context", this.parameters, true, true);
    }

    public void flush() {
      this.out.flush();
      this.err.flush();
      if (this.log != null)
        this.log.flush();
    }

    private String getFieldName(int id) {
      return (String)FIELD_TABLE.get(id & 0xFFFFFF);
    }

    private String getProblemOptionKey(int problemID)
    {
      long irritant = ProblemReporter.getIrritant(problemID);
      return CompilerOptions.optionKeyFromIrritant(irritant);
    }

    public void logAverage(long[] times, long lineCount) {
      Arrays.sort(times);
      int length = times.length;
      long sum = 0L;
      int i = 1; for (int max = length - 1; i < max; i++) {
        sum += times[i];
      }
      long time = sum / (length - 2);
      printlnOut(this.main
        .bind("compile.averageTime", 
        new String[] { 
        String.valueOf(lineCount), 
        String.valueOf(time), 
        String.valueOf((int)(lineCount * 10000.0D / time) / 10.0D) }));
    }

    public void logClassFile(boolean generatePackagesStructure, String outputPath, String relativeFileName) {
      if ((this.tagBits & 0x1) != 0) {
        String fileName = null;
        if (generatePackagesStructure) {
          fileName = buildFileName(outputPath, relativeFileName);
        } else {
          char fileSeparatorChar = File.separatorChar;
          String fileSeparator = File.separator;

          outputPath = outputPath.replace('/', fileSeparatorChar);

          int indexOfPackageSeparator = relativeFileName.lastIndexOf(fileSeparatorChar);
          if (indexOfPackageSeparator == -1) {
            if (outputPath.endsWith(fileSeparator))
              fileName = outputPath + relativeFileName;
            else
              fileName = outputPath + fileSeparator + relativeFileName;
          }
          else {
            int length = relativeFileName.length();
            if (outputPath.endsWith(fileSeparator))
              fileName = outputPath + relativeFileName.substring(indexOfPackageSeparator + 1, length);
            else {
              fileName = outputPath + fileSeparator + relativeFileName.substring(indexOfPackageSeparator + 1, length);
            }
          }
        }
        File f = new File(fileName);
        try {
          this.parameters.put("path", f.getCanonicalPath());
          printTag("classfile", this.parameters, true, true);
        } catch (IOException e) {
          logNoClassFileCreated(outputPath, relativeFileName, e);
        }
      }
    }

    public void logClasspath(FileSystem.Classpath[] classpaths) { if (classpaths == null) return;
      if ((this.tagBits & 0x1) != 0) {
        int length = classpaths.length;
        if (length != 0)
        {
          printTag("classpaths", null, true, false);
          for (int i = 0; i < length; i++) {
            String classpath = classpaths[i].getPath();
            this.parameters.put("path", classpath);
            File f = new File(classpath);
            String id = null;
            if (f.isFile()) {
              if (Util.isArchiveFileName(classpath))
                id = "JAR";
              else
                id = "FILE";
            }
            else if (f.isDirectory()) {
              id = "FOLDER";
            }
            if (id != null) {
              this.parameters.put("id", id);
              printTag("classpath", this.parameters, true, true);
            }
          }
          endTag("classpaths");
        }
      } }

    public void logCommandLineArguments(String[] commandLineArguments)
    {
      if (commandLineArguments == null) return;
      if ((this.tagBits & 0x1) != 0) {
        int length = commandLineArguments.length;
        if (length != 0)
        {
          printTag("command_line", null, true, false);
          for (int i = 0; i < length; i++) {
            this.parameters.put("value", commandLineArguments[i]);
            printTag("argument", this.parameters, true, true);
          }
          endTag("command_line");
        }
      }
    }

    public void logException(Exception e)
    {
      StringWriter writer = new StringWriter();
      PrintWriter printWriter = new PrintWriter(writer);
      e.printStackTrace(printWriter);
      printWriter.flush();
      printWriter.close();
      String stackTrace = writer.getBuffer().toString();
      if ((this.tagBits & 0x1) != 0) {
        LineNumberReader reader = new LineNumberReader(new StringReader(stackTrace));

        int i = 0;
        StringBuffer buffer = new StringBuffer();
        String message = e.getMessage();
        if (message != null)
          buffer.append(message).append(Util.LINE_SEPARATOR);
        try
        {
          String line;
          while (((line = reader.readLine()) != null) && (i < 4))
          {
            String line;
            buffer.append(line).append(Util.LINE_SEPARATOR);
            i++;
          }
          reader.close();
        }
        catch (IOException localIOException) {
        }
        message = buffer.toString();
        this.parameters.put("message", message);
        this.parameters.put("class", e.getClass());
        printTag("exception", this.parameters, true, true);
      }
      String message = e.getMessage();
      if (message == null)
        printlnErr(stackTrace);
      else
        printlnErr(message);
    }

    public void logIncorrectClasspath(String wrongClasspath)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.incorrectClasspath", wrongClasspath));
        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main
        .bind("configure.incorrectClasspath", wrongClasspath));
    }

    public void logIncorrectEndorsedDirsEntry(String wrongPath)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.incorrectEndorsedDirsEntry", wrongPath));
        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main
        .bind("configure.incorrectEndorsedDirsEntry", wrongPath));
    }

    public void logIncorrectExtDirsEntry(String wrongPath)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.incorrectExtDirsEntry", wrongPath));
        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main
        .bind("configure.incorrectExtDirsEntry", wrongPath));
    }

    public void logIncorrectVMVersionForAnnotationProcessing() {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.incorrectVMVersionforAPT"));
        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main.bind("configure.incorrectVMVersionforAPT"));
    }

    public void logNoClassFileCreated(String outputDir, String relativeFileName, IOException e)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main
          .bind("output.noClassFileCreated", 
          new String[] { 
          outputDir, 
          relativeFileName, 
          e.getMessage() }));

        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main
        .bind("output.noClassFileCreated", 
        new String[] { 
        outputDir, 
        relativeFileName, 
        e.getMessage() }));
    }

    public void logNoClasspath()
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.noClasspath"));
        printTag("error", this.parameters, true, true);
      }
      printlnErr(this.main.bind("configure.noClasspath"));
    }

    public void logNumberOfClassFilesGenerated(int exportedClassFilesCounter)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("value", new Integer(exportedClassFilesCounter));
        printTag("number_of_classfiles", this.parameters, true, true);
      }
      if (exportedClassFilesCounter == 1)
        printlnOut(this.main.bind("compile.oneClassFileGenerated"));
      else
        printlnOut(this.main
          .bind("compile.severalClassFilesGenerated", 
          String.valueOf(exportedClassFilesCounter)));
    }

    public void logOptions(Map options)
    {
      if ((this.tagBits & 0x1) != 0) {
        printTag("options", null, true, false);
        Set entriesSet = options.entrySet();
        Object[] entries = entriesSet.toArray();
        Arrays.sort(entries, new Main.1(this));

        int i = 0; for (int max = entries.length; i < max; i++) {
          Map.Entry entry = (Map.Entry)entries[i];
          String key = (String)entry.getKey();
          this.parameters.put("key", key);
          this.parameters.put("value", entry.getValue());
          printTag("option", this.parameters, true, true);
        }
        endTag("options");
      }
    }

    private void logProblem(CategorizedProblem problem, int localErrorCount, int globalErrorCount, char[] unitSource)
    {
      if ((this.tagBits & 0x2) != 0) {
        String result = new String(problem.getOriginatingFileName()) + 
          ":" + 
          problem.getSourceLineNumber() + 
          ": " + (
          problem.isError() ? this.main.bind("output.emacs.error") : this.main.bind("output.emacs.warning")) + 
          ": " + 
          problem.getMessage();
        printlnErr(result);
        String errorReportSource = errorReportSource(problem, unitSource, this.tagBits);
        if (errorReportSource.length() != 0) printlnErr(errorReportSource); 
      }
      else { if (localErrorCount == 0) {
          printlnErr("----------");
        }
        printErr(problem.isError() ? 
          this.main.bind(
          "requestor.error", 
          Integer.toString(globalErrorCount), 
          new String(problem.getOriginatingFileName())) : 
          this.main.bind(
          "requestor.warning", 
          Integer.toString(globalErrorCount), 
          new String(problem.getOriginatingFileName())));
        try {
          String errorReportSource = errorReportSource(problem, unitSource, 0);
          printlnErr(errorReportSource);
          printlnErr(problem.getMessage());
        } catch (Exception localException) {
          printlnErr(this.main
            .bind("requestor.notRetrieveErrorMessage", problem.toString()));
        }
        printlnErr("----------"); }
    }

    public int logProblems(CategorizedProblem[] problems, char[] unitSource, Main currentMain)
    {
      int count = problems.length;
      int localErrorCount = 0;
      int localProblemCount = 0;
      if (count != 0) {
        int errors = 0;
        int warnings = 0;
        int tasks = 0;
        for (int i = 0; i < count; i++) {
          CategorizedProblem problem = problems[i];
          if (problem != null) {
            currentMain.globalProblemsCount += 1;
            logProblem(problem, localProblemCount, currentMain.globalProblemsCount, unitSource);
            localProblemCount++;
            if (problem.isError()) {
              localErrorCount++;
              errors++;
              currentMain.globalErrorsCount += 1;
            } else if (problem.getID() == 536871362) {
              currentMain.globalTasksCount += 1;
              tasks++;
            } else {
              currentMain.globalWarningsCount += 1;
              warnings++;
            }
          }
        }
        if ((this.tagBits & 0x1) != 0) {
          if (errors + warnings != 0) {
            startLoggingProblems(errors, warnings);
            for (int i = 0; i < count; i++) {
              CategorizedProblem problem = problems[i];
              if ((problem != null) && 
                (problem.getID() != 536871362)) {
                logXmlProblem(problem, unitSource);
              }
            }

            endLoggingProblems();
          }
          if (tasks != 0) {
            startLoggingTasks(tasks);
            for (int i = 0; i < count; i++) {
              CategorizedProblem problem = problems[i];
              if ((problem != null) && 
                (problem.getID() == 536871362)) {
                logXmlTask(problem, unitSource);
              }
            }

            endLoggingTasks();
          }
        }
      }
      return localErrorCount;
    }

    public void logProblemsSummary(int globalProblemsCount, int globalErrorsCount, int globalWarningsCount, int globalTasksCount)
    {
      if ((this.tagBits & 0x1) != 0)
      {
        this.parameters.put("problems", new Integer(globalProblemsCount));
        this.parameters.put("errors", new Integer(globalErrorsCount));
        this.parameters.put("warnings", new Integer(globalWarningsCount));
        this.parameters.put("tasks", new Integer(globalTasksCount));
        printTag("problem_summary", this.parameters, true, true);
      }
      if (globalProblemsCount == 1) {
        String message = null;
        if (globalErrorsCount == 1)
          message = this.main.bind("compile.oneError");
        else {
          message = this.main.bind("compile.oneWarning");
        }
        printErr(this.main.bind("compile.oneProblem", message));
      } else {
        String errorMessage = null;
        String warningMessage = null;
        if (globalErrorsCount > 0) {
          if (globalErrorsCount == 1)
            errorMessage = this.main.bind("compile.oneError");
          else {
            errorMessage = this.main.bind("compile.severalErrors", String.valueOf(globalErrorsCount));
          }
        }
        int warningsNumber = globalWarningsCount + globalTasksCount;
        if (warningsNumber > 0) {
          if (warningsNumber == 1)
            warningMessage = this.main.bind("compile.oneWarning");
          else {
            warningMessage = this.main.bind("compile.severalWarnings", String.valueOf(warningsNumber));
          }
        }
        if ((errorMessage == null) || (warningMessage == null)) {
          if (errorMessage == null)
            printErr(this.main
              .bind("compile.severalProblemsErrorsOrWarnings", 
              String.valueOf(globalProblemsCount), 
              warningMessage));
          else
            printErr(this.main
              .bind("compile.severalProblemsErrorsOrWarnings", 
              String.valueOf(globalProblemsCount), 
              errorMessage));
        }
        else {
          printErr(this.main
            .bind("compile.severalProblemsErrorsAndWarnings", 
            new String[] { 
            String.valueOf(globalProblemsCount), 
            errorMessage, 
            warningMessage }));
        }
      }

      if ((this.tagBits & 0x2) != 0)
        printlnErr();
    }

    public void logProgress()
    {
      printOut('.');
    }

    public void logRepetition(int i, int repetitions)
    {
      printlnOut(this.main
        .bind("compile.repetition", 
        String.valueOf(i + 1), String.valueOf(repetitions)));
    }

    public void logTiming(long time, long lineCount)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("value", new Long(time));
        printTag("time", this.parameters, true, true);
        this.parameters.put("value", new Long(lineCount));
        printTag("number_of_lines", this.parameters, true, true);
      }
      if (lineCount != 0L)
        printlnOut(this.main
          .bind("compile.instantTime", 
          new String[] { 
          String.valueOf(lineCount), 
          String.valueOf(time), 
          String.valueOf((int)(lineCount * 10000.0D / time) / 10.0D) }));
      else
        printlnOut(this.main.bind("compile.totalTime", String.valueOf(time)));
    }

    public void logUsage(String usage)
    {
      printlnOut(usage);
    }

    public void logVersion(boolean printToOut)
    {
      if ((this.log != null) && ((this.tagBits & 0x1) == 0)) {
        String version = this.main.bind("misc.version", 
          new String[] { 
          this.main.bind("compiler.name"), 
          this.main.bind("compiler.version"), 
          this.main.bind("compiler.copyright") });

        this.log.println("# " + version);
        if (printToOut) {
          this.out.println(version);
          this.out.flush();
        }
      } else if (printToOut) {
        String version = this.main.bind("misc.version", 
          new String[] { 
          this.main.bind("compiler.name"), 
          this.main.bind("compiler.version"), 
          this.main.bind("compiler.copyright") });

        this.out.println(version);
        this.out.flush();
      }
    }

    public void logWrongJDK()
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("message", this.main.bind("configure.requiresJDK1.2orAbove"));
        printTag("ERROR", this.parameters, true, true);
      }
      printlnErr(this.main.bind("configure.requiresJDK1.2orAbove"));
    }

    private void logXmlProblem(CategorizedProblem problem, char[] unitSource)
    {
      int sourceStart = problem.getSourceStart();
      int sourceEnd = problem.getSourceEnd();
      int id = problem.getID();
      this.parameters.put("id", getFieldName(id));
      this.parameters.put("problemID", new Integer(id));
      boolean isError = problem.isError();
      int severity = isError ? 1 : 0;
      this.parameters.put("severity", isError ? "ERROR" : "WARNING");
      this.parameters.put("line", new Integer(problem.getSourceLineNumber()));
      this.parameters.put("charStart", new Integer(sourceStart));
      this.parameters.put("charEnd", new Integer(sourceEnd));
      String problemOptionKey = getProblemOptionKey(id);
      if (problemOptionKey != null) {
        this.parameters.put("optionKey", problemOptionKey);
      }
      int categoryID = ProblemReporter.getProblemCategory(severity, id);
      this.parameters.put("categoryID", new Integer(categoryID));
      printTag("problem", this.parameters, true, false);
      this.parameters.put("value", problem.getMessage());
      printTag("message", this.parameters, true, true);
      extractContext(problem, unitSource);
      String[] arguments = problem.getArguments();
      int length = arguments.length;
      if (length != 0) {
        printTag("arguments", null, true, false);
        for (int i = 0; i < length; i++) {
          this.parameters.put("value", arguments[i]);
          printTag("argument", this.parameters, true, true);
        }
        endTag("arguments");
      }
      endTag("problem");
    }

    private void logXmlTask(CategorizedProblem problem, char[] unitSource)
    {
      this.parameters.put("line", new Integer(problem.getSourceLineNumber()));
      this.parameters.put("charStart", new Integer(problem.getSourceStart()));
      this.parameters.put("charEnd", new Integer(problem.getSourceEnd()));
      String problemOptionKey = getProblemOptionKey(problem.getID());
      if (problemOptionKey != null) {
        this.parameters.put("optionKey", problemOptionKey);
      }
      printTag("task", this.parameters, true, false);
      this.parameters.put("value", problem.getMessage());
      printTag("message", this.parameters, true, true);
      extractContext(problem, unitSource);
      endTag("task");
    }

    private void printErr(String s) {
      this.err.print(s);
      if (((this.tagBits & 0x1) == 0) && (this.log != null))
        this.log.print(s);
    }

    private void printlnErr()
    {
      this.err.println();
      if (((this.tagBits & 0x1) == 0) && (this.log != null))
        this.log.println();
    }

    private void printlnErr(String s) {
      this.err.println(s);
      if (((this.tagBits & 0x1) == 0) && (this.log != null))
        this.log.println(s);
    }

    private void printlnOut(String s) {
      this.out.println(s);
      if (((this.tagBits & 0x1) == 0) && (this.log != null))
        this.log.println(s);
    }

    public void printNewLine()
    {
      this.out.println();
    }

    private void printOut(char c) {
      this.out.print(c);
    }

    public void printStats() {
      boolean isTimed = this.main.timing;
      if ((this.tagBits & 0x1) != 0) {
        printTag("stats", null, true, false);
      }
      if (isTimed) {
        long time = System.currentTimeMillis() - this.main.startTime;
        logTiming(time, this.main.lineCount);
        if (this.main.times != null) {
          this.main.times[(this.main.timesCounter++)] = time;
        }
      }
      if (this.main.globalProblemsCount > 0) {
        logProblemsSummary(this.main.globalProblemsCount, this.main.globalErrorsCount, this.main.globalWarningsCount, this.main.globalTasksCount);
      }
      if ((this.main.exportedClassFilesCounter != 0) && (
        (this.main.showProgress) || (isTimed) || (this.main.verbose))) {
        logNumberOfClassFilesGenerated(this.main.exportedClassFilesCounter);
      }
      if ((this.tagBits & 0x1) != 0)
        endTag("stats");
    }

    private void printTag(String name, HashMap params, boolean insertNewLine, boolean closeTag)
    {
      ((GenericXMLWriter)this.log).printTag(name, this.parameters, true, insertNewLine, closeTag);
      this.parameters.clear();
    }

    public void setEmacs() {
      this.tagBits |= 2;
    }

    public void setLog(String logFileName) throws InvalidInputException {
      Date date = new Date();
      DateFormat dateFormat = DateFormat.getDateTimeInstance(3, 1, Locale.getDefault());
      try {
        int index = logFileName.lastIndexOf('.');
        if (index != -1) {
          if (logFileName.substring(index).toLowerCase().equals(".xml")) {
            this.log = new GenericXMLWriter(new FileOutputStream(logFileName, false), Util.LINE_SEPARATOR, true);
            this.tagBits |= 1;
            try
            {
              this.log.println("<!-- " + new String(dateFormat.format(date).getBytes(), "UTF-8") + " -->");
            } catch (UnsupportedEncodingException e) {
              e.printStackTrace();
            }
            this.log.println("<!DOCTYPE compiler PUBLIC \"-//Eclipse.org//DTD Eclipse JDT 3.2.003 Compiler//EN\" \"http://www.eclipse.org/jdt/core/compiler_32_003.dtd\">");
            this.parameters.put("name", this.main.bind("compiler.name"));
            this.parameters.put("version", this.main.bind("compiler.version"));
            this.parameters.put("copyright", this.main.bind("compiler.copyright"));
            printTag("compiler", this.parameters, true, false);
          } else {
            this.log = new PrintWriter(new FileOutputStream(logFileName, false));
            this.log.println("# " + dateFormat.format(date));
          }
        } else {
          this.log = new PrintWriter(new FileOutputStream(logFileName, false));
          this.log.println("# " + dateFormat.format(date));
        }
      } catch (FileNotFoundException localFileNotFoundException) {
        throw new InvalidInputException(this.main.bind("configure.cannotOpenLog", logFileName));
      }
    }

    private void startLoggingProblems(int errors, int warnings)
    {
      this.parameters.put("problems", new Integer(errors + warnings));
      this.parameters.put("errors", new Integer(errors));
      this.parameters.put("warnings", new Integer(warnings));
      printTag("problems", this.parameters, true, false);
    }

    private void startLoggingExtraProblems(int count) {
      this.parameters.put("problems", new Integer(count));
      printTag("extra_problems", this.parameters, true, false);
    }

    public void startLoggingSource(CompilationResult compilationResult) {
      if ((this.tagBits & 0x1) != 0) {
        ICompilationUnit compilationUnit = compilationResult.compilationUnit;
        if (compilationUnit != null) {
          char[] fileName = compilationUnit.getFileName();
          File f = new File(new String(fileName));
          if (fileName != null) {
            this.parameters.put("path", f.getAbsolutePath());
          }
          char[][] packageName = compilationResult.packageName;
          if (packageName != null) {
            this.parameters.put(
              "package", 
              new String(CharOperation.concatWith(packageName, File.separatorChar)));
          }
          CompilationUnit unit = (CompilationUnit)compilationUnit;
          String destinationPath = unit.destinationPath;
          if (destinationPath == null) {
            destinationPath = this.main.destinationPath;
          }
          if ((destinationPath != null) && (destinationPath != "none")) {
            if (File.separatorChar == '/')
              this.parameters.put("output", destinationPath);
            else {
              this.parameters.put("output", destinationPath.replace('/', File.separatorChar));
            }
          }
        }
        printTag("source", this.parameters, true, false);
      }
    }

    public void startLoggingSources() { if ((this.tagBits & 0x1) != 0)
        printTag("sources", null, true, false); }

    public void startLoggingTasks(int tasks)
    {
      if ((this.tagBits & 0x1) != 0) {
        this.parameters.put("tasks", new Integer(tasks));
        printTag("tasks", this.parameters, true, false);
      }
    }

    public void loggingExtraProblems(Main currentMain) {
      ArrayList problems = currentMain.extraProblems;
      int count = problems.size();
      int localErrorCount = 0;
      int localProblemCount = 0;
      if (count != 0) {
        int errors = 0;
        int warnings = 0;
        for (int i = 0; i < count; i++) {
          CategorizedProblem problem = (CategorizedProblem)problems.get(i);
          if (problem != null) {
            currentMain.globalProblemsCount += 1;
            logExtraProblem(problem, localProblemCount, currentMain.globalProblemsCount);
            localProblemCount++;
            if (problem.isError()) {
              localErrorCount++;
              errors++;
              currentMain.globalErrorsCount += 1;
            } else if (problem.isWarning()) {
              currentMain.globalWarningsCount += 1;
              warnings++;
            }
          }
        }
        if (((this.tagBits & 0x1) != 0) && 
          (errors + warnings != 0)) {
          startLoggingExtraProblems(count);
          for (int i = 0; i < count; i++) {
            CategorizedProblem problem = (CategorizedProblem)problems.get(i);
            if ((problem != null) && 
              (problem.getID() != 536871362)) {
              logXmlExtraProblem(problem, localProblemCount, currentMain.globalProblemsCount);
            }
          }

          endLoggingExtraProblems();
        }
      }
    }

    private void logXmlExtraProblem(CategorizedProblem problem, int globalErrorCount, int localErrorCount)
    {
      int sourceStart = problem.getSourceStart();
      int sourceEnd = problem.getSourceEnd();
      boolean isError = problem.isError();
      this.parameters.put("severity", isError ? "ERROR" : "WARNING");
      this.parameters.put("line", new Integer(problem.getSourceLineNumber()));
      this.parameters.put("charStart", new Integer(sourceStart));
      this.parameters.put("charEnd", new Integer(sourceEnd));
      printTag("extra_problem", this.parameters, true, false);
      this.parameters.put("value", problem.getMessage());
      printTag("message", this.parameters, true, true);
      extractContext(problem, null);
      endTag("extra_problem");
    }

    private void logExtraProblem(CategorizedProblem problem, int localErrorCount, int globalErrorCount) {
      char[] originatingFileName = problem.getOriginatingFileName();
      String fileName = 
        originatingFileName == null ? 
        this.main.bind("requestor.noFileNameSpecified") : 
        new String(originatingFileName);
      if ((this.tagBits & 0x2) != 0) {
        String result = fileName + 
          ":" + 
          problem.getSourceLineNumber() + 
          ": " + (
          problem.isError() ? this.main.bind("output.emacs.error") : this.main.bind("output.emacs.warning")) + 
          ": " + 
          problem.getMessage();
        printlnErr(result);
        String errorReportSource = errorReportSource(problem, null, this.tagBits);
        printlnErr(errorReportSource);
      } else {
        if (localErrorCount == 0) {
          printlnErr("----------");
        }
        printErr(problem.isError() ? 
          this.main.bind(
          "requestor.error", 
          Integer.toString(globalErrorCount), 
          new String(fileName)) : 
          this.main.bind(
          "requestor.warning", 
          Integer.toString(globalErrorCount), 
          new String(fileName)));
        String errorReportSource = errorReportSource(problem, null, 0);
        printlnErr(errorReportSource);
        printlnErr(problem.getMessage());
        printlnErr("----------");
      }
    }

    private String errorReportSource(CategorizedProblem problem, char[] unitSource, int bits)
    {
      int startPosition = problem.getSourceStart();
      int endPosition = problem.getSourceEnd();
      if ((unitSource == null) && 
        (problem.getOriginatingFileName() != null)) {
        try {
          unitSource = Util.getFileCharContent(new File(new String(problem.getOriginatingFileName())), null);
        }
        catch (IOException localIOException)
        {
        }
      }
      int length = unitSource == null ? 0 : unitSource.length;
      if ((startPosition > endPosition) || 
        ((startPosition < 0) && (endPosition < 0)) || 
        (length == 0)) {
        return Messages.problem_noSourceInformation;
      }
      StringBuffer errorBuffer = new StringBuffer();
      if ((bits & 0x2) == 0) {
        errorBuffer.append(' ').append(Messages.bind(Messages.problem_atLine, String.valueOf(problem.getSourceLineNumber())));
        errorBuffer.append(Util.LINE_SEPARATOR);
      }
      errorBuffer.append('\t');

      for (int begin = startPosition >= length ? length - 1 : startPosition; begin > 0; begin--)
      {
        char c;
        if (((c = unitSource[(begin - 1)]) == '\n') || (c == '\r')) break;
      }
      for (int end = endPosition >= length ? length - 1 : endPosition; end + 1 < length; end++)
      {
        char c;
        if (((c = unitSource[(end + 1)]) == '\r') || (c == '\n'))
          break;
      }
      char c;
      while (((c = unitSource[begin]) == ' ') || (c == '\t'))
      {
        char c;
        begin++;
      }

      errorBuffer.append(unitSource, begin, end - begin + 1);
      errorBuffer.append(Util.LINE_SEPARATOR).append("\t");

      for (int i = begin; i < startPosition; i++) {
        errorBuffer.append(unitSource[i] == '\t' ? '\t' : ' ');
      }
      for (int i = startPosition; i <= (endPosition >= length ? length - 1 : endPosition); i++) {
        errorBuffer.append('^');
      }
      return errorBuffer.toString();
    }
  }

  public static class ResourceBundleFactory
  {
    private static HashMap Cache = new HashMap();

    public static synchronized ResourceBundle getBundle(Locale locale) { ResourceBundle bundle = (ResourceBundle)Cache.get(locale);
      if (bundle == null) {
        bundle = ResourceBundle.getBundle("org.eclipse.jdt.internal.compiler.batch.messages", locale);
        Cache.put(locale, bundle);
      }
      return bundle;
    }
  }
}