package org.eclipse.jdt.internal.compiler.flow;

import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;

public abstract class FlowInfo
{
  public int tagBits;
  public static final int REACHABLE = 0;
  public static final int UNREACHABLE = 1;
  public static final int NULL_FLAG_MASK = 2;
  public static final int UNKNOWN = 0;
  public static final int NULL = 1;
  public static final int NON_NULL = -1;
  public static final UnconditionalFlowInfo DEAD_END = new UnconditionalFlowInfo();

  static { DEAD_END.tagBits = 1; }


  public abstract FlowInfo addInitializationsFrom(FlowInfo paramFlowInfo);

  public abstract FlowInfo addPotentialInitializationsFrom(FlowInfo paramFlowInfo);

  public FlowInfo asNegatedCondition()
  {
    return this;
  }

  public static FlowInfo conditional(FlowInfo initsWhenTrue, FlowInfo initsWhenFalse)
  {
    return new ConditionalFlowInfo(initsWhenTrue, initsWhenFalse);
  }

  public boolean cannotBeDefinitelyNullOrNonNull(LocalVariableBinding local)
  {
    return (isPotentiallyUnknown(local)) || (
      (isPotentiallyNonNull(local)) && (isPotentiallyNull(local)));
  }

  public boolean cannotBeNull(LocalVariableBinding local)
  {
    return (isDefinitelyNonNull(local)) || (isProtectedNonNull(local));
  }

  public boolean canOnlyBeNull(LocalVariableBinding local)
  {
    return (isDefinitelyNull(local)) || (isProtectedNull(local));
  }

  public abstract FlowInfo copy();

  public static UnconditionalFlowInfo initial(int maxFieldCount)
  {
    UnconditionalFlowInfo info = new UnconditionalFlowInfo();
    info.maxFieldCount = maxFieldCount;
    return info;
  }

  public abstract FlowInfo initsWhenFalse();

  public abstract FlowInfo initsWhenTrue();

  public abstract boolean isDefinitelyAssigned(FieldBinding paramFieldBinding);

  public abstract boolean isDefinitelyAssigned(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isDefinitelyNonNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isDefinitelyNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isDefinitelyUnknown(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isPotentiallyAssigned(FieldBinding paramFieldBinding);

  public abstract boolean isPotentiallyAssigned(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isPotentiallyNonNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isPotentiallyNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isPotentiallyUnknown(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isProtectedNonNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract boolean isProtectedNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsComparedEqualToNonNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsComparedEqualToNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsDefinitelyAssigned(FieldBinding paramFieldBinding);

  public abstract void markAsDefinitelyNonNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsDefinitelyNull(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsDefinitelyAssigned(LocalVariableBinding paramLocalVariableBinding);

  public abstract void markAsDefinitelyUnknown(LocalVariableBinding paramLocalVariableBinding);

  public static UnconditionalFlowInfo mergedOptimizedBranches(FlowInfo initsWhenTrue, boolean isOptimizedTrue, FlowInfo initsWhenFalse, boolean isOptimizedFalse, boolean allowFakeDeadBranch)
  {
    UnconditionalFlowInfo mergedInfo;
    UnconditionalFlowInfo mergedInfo;
    if (isOptimizedTrue)
    {
      UnconditionalFlowInfo mergedInfo;
      if ((initsWhenTrue == DEAD_END) && (allowFakeDeadBranch)) {
        mergedInfo = initsWhenFalse.setReachMode(1)
          .unconditionalInits();
      }
      else
        mergedInfo = 
          initsWhenTrue.addPotentialInitializationsFrom(
          initsWhenFalse.nullInfoLessUnconditionalCopy())
          .unconditionalInits();
    }
    else
    {
      UnconditionalFlowInfo mergedInfo;
      if (isOptimizedFalse)
      {
        UnconditionalFlowInfo mergedInfo;
        if ((initsWhenFalse == DEAD_END) && (allowFakeDeadBranch)) {
          mergedInfo = initsWhenTrue.setReachMode(1)
            .unconditionalInits();
        }
        else
          mergedInfo = 
            initsWhenFalse.addPotentialInitializationsFrom(
            initsWhenTrue.nullInfoLessUnconditionalCopy())
            .unconditionalInits();
      }
      else
      {
        mergedInfo = initsWhenTrue
          .mergedWith(initsWhenFalse.unconditionalInits());
      }
    }
    return mergedInfo;
  }

  public int reachMode()
  {
    return this.tagBits & 0x1;
  }

  public abstract FlowInfo safeInitsWhenTrue();

  public abstract FlowInfo setReachMode(int paramInt);

  public abstract UnconditionalFlowInfo mergedWith(UnconditionalFlowInfo paramUnconditionalFlowInfo);

  public abstract UnconditionalFlowInfo nullInfoLessUnconditionalCopy();

  public String toString()
  {
    if (this == DEAD_END) {
      return "FlowInfo.DEAD_END";
    }
    return super.toString();
  }

  public abstract UnconditionalFlowInfo unconditionalCopy();

  public abstract UnconditionalFlowInfo unconditionalFieldLessCopy();

  public abstract UnconditionalFlowInfo unconditionalInits();

  public abstract UnconditionalFlowInfo unconditionalInitsWithoutSideEffect();
}