package org.eclipse.jdt.internal.compiler.flow;

import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;

public class NullInfoRegistry extends UnconditionalFlowInfo
{
  public NullInfoRegistry(UnconditionalFlowInfo upstream)
  {
    this.maxFieldCount = upstream.maxFieldCount;
    if ((upstream.tagBits & 0x2) != 0)
    {
      long u1;
      long u2;
      long u3;
      long nu3;
      long u4;
      long nu4;
      this.nullBit2 = 
        ((u1 = upstream.nullBit1) & (
        u2 = upstream.nullBit2) & (
        nu3 = (u3 = upstream.nullBit3) ^ 0xFFFFFFFF) & (
        nu4 = (u4 = upstream.nullBit4) ^ 0xFFFFFFFF));
      long nu2;
      this.nullBit3 = (u1 & (nu2 = u2 ^ 0xFFFFFFFF) & u3 & nu4);
      this.nullBit4 = (u1 & nu2 & nu3 & u4);
      if ((this.nullBit2 | this.nullBit3 | this.nullBit4) != 0L) {
        this.tagBits |= 2;
      }
      if (upstream.extra != null) {
        this.extra = new long[6][];
        int length = upstream.extra[2].length;
        for (int i = 2; i < 6; i++) {
          this.extra[i] = new long[length];
        }
        for (int i = 0; i < tmp279_278; i++)
        {
          long tmp208_207 = upstream.extra[2][i]; u1 = tmp208_207;
          long tmp219_218 = upstream.extra[3][i]; u2 = tmp219_218;
          long tmp239_238 = ((u3 = upstream.extra[4][i]) ^ 0xFFFFFFFF); tmp208_207 = tmp239_238;
          long tmp259_258 = ((u4 = upstream.extra[5][i]) ^ 0xFFFFFFFF); tmp239_238 = tmp259_258;

          this.extra[3][i] = 
            (tmp208_207 & 
            tmp219_218 & 
            tmp239_238 & 
            tmp259_258);
          long tmp279_278 = (u2 ^ 0xFFFFFFFF); nu2 = tmp279_278; this.extra[4][i] = (u1 & tmp279_278 & u3 & tmp239_238);
          this.extra[5][i] = (u1 & nu2 & tmp208_207 & u4);
          if ((this.extra[3][i] | this.extra[4][i] | this.extra[5][i]) != 0L)
            this.tagBits |= 2;
        }
      }
    }
  }

  public NullInfoRegistry add(NullInfoRegistry other)
  {
    if ((other.tagBits & 0x2) == 0) {
      return this;
    }
    this.tagBits |= 2;
    this.nullBit1 |= other.nullBit1;
    this.nullBit2 |= other.nullBit2;
    this.nullBit3 |= other.nullBit3;
    this.nullBit4 |= other.nullBit4;
    if (other.extra != null) {
      if (this.extra == null) {
        this.extra = new long[6][];
        int i = 2; for (int length = other.extra[2].length; i < 6; i++)
          System.arraycopy(other.extra[i], 0, 
            this.extra[i] =  = new long[length], 0, length);
      }
      else {
        int length = this.extra[2].length; int otherLength = other.extra[2].length;
        if (otherLength > length)
          for (int i = 2; i < 6; i++) {
            System.arraycopy(this.extra[i], 0, 
              this.extra[i] =  = new long[otherLength], 0, length);
            System.arraycopy(other.extra[i], length, 
              this.extra[i], length, otherLength - length);
          }
        else if (otherLength < length) {
          length = otherLength;
        }
        for (int i = 2; i < 6; i++) {
          for (int j = 0; j < length; j++) {
            this.extra[i][j] |= other.extra[i][j];
          }
        }
      }
    }
    return this;
  }

  public void markAsComparedEqualToNonNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        this.nullBit1 |= 1L << position;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 2; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[2].length)) {
            for (int j = 2; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[vectorIndex + 1], 0, 
                oldLength);
            }
          }
        }
        this.extra[2][vectorIndex] |= 1L << position % 64;
      }
    }
  }

  public void markAsDefinitelyNonNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        this.nullBit3 |= 1L << position;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 2; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[2].length)) {
            for (int j = 2; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[vectorIndex + 1], 0, 
                oldLength);
            }
          }
        }
        this.extra[4][vectorIndex] |= 1L << position % 64;
      }
    }
  }

  public void markAsDefinitelyNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        this.nullBit2 |= 1L << position;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 2; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[2].length)) {
            for (int j = 2; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[vectorIndex + 1], 0, 
                oldLength);
            }
          }
        }
        this.extra[3][vectorIndex] |= 1L << position % 64;
      }
    }
  }

  public void markAsDefinitelyUnknown(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        this.nullBit4 |= 1L << position;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 2; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[2].length)) {
            for (int j = 2; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[vectorIndex + 1], 0, 
                oldLength);
            }
          }
        }
        this.extra[5][vectorIndex] |= 1L << position % 64;
      }
    }
  }

  public UnconditionalFlowInfo mitigateNullInfoOf(FlowInfo flowInfo)
  {
    if ((this.tagBits & 0x2) == 0) {
      return flowInfo.unconditionalInits();
    }

    boolean newCopy = false;
    UnconditionalFlowInfo source = flowInfo.unconditionalInits();
    long s1;
    long s3;
    long s4;
    long a2;
    long a4;
    long m1 = (s1 = source.nullBit1) & (s3 = source.nullBit3) & (
      s4 = source.nullBit4) & (
      (a2 = this.nullBit2) | (a4 = this.nullBit4));
    long s2;
    long a3;
    long m2 = s1 & (s2 = this.nullBit2) & (s3 ^ s4) & (
      (a3 = this.nullBit3) | a4);
    long ns3;
    long ns4;
    long ns2;
    long m3 = s1 & (s2 & (ns3 = s3 ^ 0xFFFFFFFF) & (ns4 = s4 ^ 0xFFFFFFFF) & (a3 | a4) | 
      (ns2 = s2 ^ 0xFFFFFFFF) & s3 & ns4 & (a2 | a4) | 
      ns2 & ns3 & s4 & (a2 | a3));
    long m;
    if ((m = m1 | m2 | m3) != 0L) {
      newCopy = true;
      source = source.unconditionalCopy();
      source.nullBit1 &= (m ^ 0xFFFFFFFF);
      long nm1;
      long nm2;
      source.nullBit2 &= (nm1 = m1 ^ 0xFFFFFFFF) & ((nm2 = m2 ^ 0xFFFFFFFF) | a4);
      source.nullBit3 &= (nm1 | a2) & nm2;
      source.nullBit4 &= nm1 & nm2;
    }
    if ((this.extra != null) && (source.extra != null)) {
      int length = this.extra[2].length; int sourceLength = source.extra[0].length;
      if (sourceLength < length) {
        length = sourceLength;
      }
      for (int i = 0; i < length; i++) {
        m1 = (s1 = source.extra[2][i]) & (s3 = source.extra[4][i]) & (
          s4 = source.extra[5][i]) & (
          (a2 = this.extra[3][i]) | (a4 = this.extra[5][i]));
        m2 = s1 & (s2 = this.extra[3][i]) & (s3 ^ s4) & (
          (a3 = this.extra[4][i]) | a4);
        m3 = s1 & (s2 & (ns3 = s3 ^ 0xFFFFFFFF) & (ns4 = s4 ^ 0xFFFFFFFF) & (a3 | a4) | 
          (ns2 = s2 ^ 0xFFFFFFFF) & s3 & ns4 & (a2 | a4) | 
          ns2 & ns3 & s4 & (a2 | a3));
        if ((m = m1 | m2 | m3) != 0L) {
          if (!newCopy) {
            newCopy = true;
            source = source.unconditionalCopy();
          }
          source.extra[2][i] &= (m ^ 0xFFFFFFFF);
          long tmp565_564 = (m1 ^ 0xFFFFFFFF); long nm1 = tmp565_564;
          long tmp574_573 = (m2 ^ 0xFFFFFFFF); long nm2 = tmp574_573; source.extra[3][i] &= tmp565_564 & (tmp574_573 | a4);
          source.extra[4][i] &= (nm1 | a2) & nm2;
          source.extra[5][i] &= nm1 & nm2;
        }
      }
    }
    return source;
  }

  public String toString() {
    if (this.extra == null) {
      return "NullInfoRegistry<" + this.nullBit1 + 
        this.nullBit2 + this.nullBit3 + this.nullBit4 + 
        ">";
    }

    String nullS = "NullInfoRegistry<[" + this.nullBit1 + 
      this.nullBit2 + this.nullBit3 + this.nullBit4;

    int i = 0;

    int ceil = this.extra[0].length > 3 ? 
      3 : 
      this.extra[0].length;
    for (; i < ceil; i++) {
      nullS = nullS + "," + this.extra[2][i] + 
        this.extra[3][i] + this.extra[4][i] + this.extra[5][i];
    }
    if (ceil < this.extra[0].length) {
      nullS = nullS + ",...";
    }
    return nullS + "]>";
  }
}