package org.eclipse.jdt.internal.compiler.flow;

import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class UnconditionalFlowInfo extends FlowInfo
{
  public static final boolean coverageTestFlag = false;
  public static int coverageTestId;
  public long definiteInits;
  public long potentialInits;
  public long nullBit1;
  public long nullBit2;
  public long nullBit3;
  public long nullBit4;
  public static final int extraLength = 6;
  public long[][] extra;
  public int maxFieldCount;
  public static final int BitCacheSize = 64;

  public FlowInfo addInitializationsFrom(FlowInfo inits)
  {
    if (this == DEAD_END)
      return this;
    if (inits == DEAD_END)
      return this;
    UnconditionalFlowInfo otherInits = inits.unconditionalInits();

    this.definiteInits |= otherInits.definiteInits;

    this.potentialInits |= otherInits.potentialInits;

    boolean thisHadNulls = (this.tagBits & 0x2) != 0;
    boolean otherHasNulls = (otherInits.tagBits & 0x2) != 0;

    if (otherHasNulls) {
      if (!thisHadNulls) {
        this.nullBit1 = otherInits.nullBit1;
        this.nullBit2 = otherInits.nullBit2;
        this.nullBit3 = otherInits.nullBit3;
        this.nullBit4 = otherInits.nullBit4;
      }
      else
      {
        long b1;
        long a1;
        long a3;
        long a4;
        long b2;
        long nb2;
        long b4;
        long nb4;
        long na4;
        long na3;
        long a2;
        long na2;
        long b3;
        long nb3;
        this.nullBit1 = 
          ((b1 = otherInits.nullBit1) | 
          (a1 = this.nullBit1) & ((a3 = this.nullBit3) & (
          a4 = this.nullBit4) & (nb2 = (b2 = otherInits.nullBit2) ^ 0xFFFFFFFF) & (
          nb4 = (b4 = otherInits.nullBit4) ^ 0xFFFFFFFF) | 
          ((na4 = a4 ^ 0xFFFFFFFF) | (na3 = a3 ^ 0xFFFFFFFF)) & (
          (na2 = (a2 = this.nullBit2) ^ 0xFFFFFFFF) & nb2 | 
          a2 & (nb3 = (b3 = otherInits.nullBit3) ^ 0xFFFFFFFF) & nb4)));
        long nb1;
        long na1;
        this.nullBit2 = 
          (b2 & (nb4 | nb3) | 
          na3 & na4 & b2 | 
          a2 & (nb3 & nb4 | 
          (nb1 = b1 ^ 0xFFFFFFFF) & (na3 | (na1 = a1 ^ 0xFFFFFFFF)) | 
          a1 & b2));
        this.nullBit3 = 
          (b3 & (nb1 & (b2 | a2 | na1) | 
          b1 & (b4 | nb2 | a1 & a3) | 
          na1 & na2 & na4) | 
          a3 & nb2 & nb4 | 
          nb1 & ((na2 & a4 | na1) & a3 | 
          a1 & na2 & na4 & b2));
        this.nullBit4 = 
          (nb1 & (a4 & (na3 & nb3 | (a3 | na2) & nb2) | 
          a1 & (a3 & nb2 & b4 | 
          a2 & b2 & (b4 | a3 & na4 & nb3))) | 
          b1 & (a3 & a4 & b4 | 
          na2 & na4 & nb3 & b4 | 
          a2 & ((b3 | a4) & b4 | 
          na3 & a4 & b2 & b3) | 
          na1 & (b4 | (a4 | a2) & b2 & b3)) | 
          (na1 & (na3 & nb3 | na2 & nb2) | 
          a1 & (nb2 & nb3 | a2 & a3)) & b4);
      }

      this.tagBits |= 2;
    }

    if ((this.extra != null) || (otherInits.extra != null)) {
      int mergeLimit = 0; int copyLimit = 0;
      if (this.extra != null) {
        if (otherInits.extra != null)
        {
          int length;
          int otherLength;
          if ((length = this.extra[0].length) < 
            (otherLength = otherInits.extra[0].length))
          {
            for (int j = 0; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[otherLength], 0, length);
            }
            mergeLimit = length;
            copyLimit = otherLength;
          }
          else
          {
            mergeLimit = otherLength;
          }

        }

      }
      else if (otherInits.extra != null)
      {
        this.extra = new long[6][];
        int otherLength;
        System.arraycopy(otherInits.extra[0], 0, 
          this.extra[0] =  = new long[otherLength = 
          otherInits.extra[0].length], 0, otherLength);
        System.arraycopy(otherInits.extra[1], 0, 
          this.extra[1] =  = new long[otherLength], 0, otherLength);
        if (otherHasNulls) {
          for (int j = 2; j < 6; j++) {
            System.arraycopy(otherInits.extra[j], 0, 
              this.extra[j] =  = new long[otherLength], 0, otherLength);
          }

        }
        else
        {
          for (int j = 2; j < 6; j++) {
            this.extra[j] = new long[otherLength];
          }

        }

      }

      for (int i = 0; i < mergeLimit; i++) {
        this.extra[0][i] |= otherInits.extra[0][i];
        this.extra[1][i] |= otherInits.extra[1][i];
      }
      for (; i < copyLimit; i++) {
        this.extra[0][i] = otherInits.extra[0][i];
        this.extra[1][i] = otherInits.extra[1][i];
      }

      if (!thisHadNulls) {
        if (copyLimit < mergeLimit) {
          copyLimit = mergeLimit;
        }
        mergeLimit = 0;
      }
      if (!otherHasNulls) {
        copyLimit = 0;
        mergeLimit = 0;
      }
      for (i = 0; i < mergeLimit; i++)
      {
        long tmp973_972 = otherInits.extra[2][i]; long b1 = tmp973_972;
        long tmp985_984 = this.extra[2][i]; long a1 = tmp985_984;
        long tmp997_996 = this.extra[4][i]; long a3 = tmp997_996;
        long tmp1009_1008 = this.extra[5][i]; long a4 = tmp1009_1008;
        long b2;
        long tmp1029_1028 = ((b2 = otherInits.extra[3][i]) ^ 0xFFFFFFFF); long nb2 = tmp1029_1028;
        long b4;
        long tmp1049_1048 = ((b4 = otherInits.extra[5][i]) ^ 0xFFFFFFFF); long nb4 = tmp1049_1048;
        long tmp1059_1058 = (a4 ^ 0xFFFFFFFF); long na4 = tmp1059_1058;
        long tmp1068_1067 = (a3 ^ 0xFFFFFFFF); long na3 = tmp1068_1067;
        long a2;
        long tmp1088_1087 = ((a2 = this.extra[3][i]) ^ 0xFFFFFFFF); long na2 = tmp1088_1087;
        long b3;
        long tmp1112_1111 = ((b3 = otherInits.extra[4][i]) ^ 0xFFFFFFFF); long nb3 = tmp1112_1111;

        this.extra[2][i] = 
          (tmp973_972 | 
          tmp985_984 & (tmp997_996 & 
          tmp1009_1008 & tmp1029_1028 & 
          tmp1049_1048 | 
          (tmp1059_1058 | tmp1068_1067) & (
          tmp1088_1087 & nb2 | 
          a2 & tmp1112_1111 & nb4)));
        long tmp1163_1162 = (b1 ^ 0xFFFFFFFF); long nb1 = tmp1163_1162;
        long tmp1174_1173 = (a1 ^ 0xFFFFFFFF); long na1 = tmp1174_1173;

        this.extra[3][i] = 
          (b2 & (nb4 | nb3) | 
          na3 & na4 & b2 | 
          a2 & (nb3 & nb4 | 
          tmp1163_1162 & (na3 | tmp1174_1173) | 
          a1 & b2));
        this.extra[4][i] = 
          (b3 & (nb1 & (b2 | a2 | na1) | 
          b1 & (b4 | nb2 | a1 & a3) | 
          na1 & na2 & na4) | 
          a3 & nb2 & nb4 | 
          nb1 & ((na2 & a4 | na1) & a3 | 
          a1 & na2 & na4 & b2));
        this.extra[5][i] = 
          (nb1 & (a4 & (na3 & nb3 | (a3 | na2) & nb2) | 
          a1 & (a3 & nb2 & b4 | 
          a2 & b2 & (b4 | a3 & na4 & nb3))) | 
          b1 & (a3 & a4 & b4 | 
          na2 & na4 & nb3 & b4 | 
          a2 & ((b3 | a4) & b4 | 
          na3 & a4 & b2 & b3) | 
          na1 & (b4 | (a4 | a2) & b2 & b3)) | 
          (na1 & (na3 & nb3 | na2 & nb2) | 
          a1 & (nb2 & nb3 | a2 & a3)) & b4);
      }

      for (; i < copyLimit; i++) {
        for (int j = 2; j < 6; j++) {
          this.extra[j][i] = otherInits.extra[j][i];
        }

      }

    }

    return this;
  }

  public FlowInfo addPotentialInitializationsFrom(FlowInfo inits) {
    if (this == DEAD_END) {
      return this;
    }
    if (inits == DEAD_END) {
      return this;
    }
    UnconditionalFlowInfo otherInits = inits.unconditionalInits();

    this.potentialInits |= otherInits.potentialInits;

    if (this.extra != null) {
      if (otherInits.extra != null)
      {
        int i = 0;
        int length;
        int otherLength;
        if ((length = this.extra[0].length) < (otherLength = otherInits.extra[0].length))
        {
          for (j = 0; j < 6; j++) {
            System.arraycopy(this.extra[j], 0, 
              this.extra[j] =  = new long[otherLength], 0, length);
          }
          for (; i < length; i++) {
            this.extra[1][i] |= otherInits.extra[1][i];
          }
          for (; i < otherLength; i++) {
            this.extra[1][i] = otherInits.extra[1][i];
          }
        }
        else
        {
          for (; i < otherLength; i++)
          {
            int j;
            this.extra[1][i] |= otherInits.extra[1][i];
          }
        }
      }
    }
    else if (otherInits.extra != null)
    {
      int otherLength = otherInits.extra[0].length;
      this.extra = new long[6][];
      for (int j = 0; j < 6; j++) {
        this.extra[j] = new long[otherLength];
      }
      System.arraycopy(otherInits.extra[1], 0, this.extra[1], 0, 
        otherLength);
    }
    addPotentialNullInfoFrom(otherInits);
    return this;
  }

  public UnconditionalFlowInfo addPotentialNullInfoFrom(UnconditionalFlowInfo otherInits)
  {
    if (((this.tagBits & 0x1) != 0) || 
      ((otherInits.tagBits & 0x1) != 0) || 
      ((otherInits.tagBits & 0x2) == 0)) {
      return this;
    }

    boolean thisHadNulls = (this.tagBits & 0x2) != 0;
    boolean thisHasNulls = false;
    long b2;
    long b3;
    long nb3;
    long b1;
    long nb1;
    long nb2;
    long b4;
    if (thisHadNulls)
    {
      long a1;
      long a3;
      long a4;
      long b2;
      long nb2;
      long b4;
      long nb4;
      long b1;
      long b3;
      long a2;
      long na2;
      long na4;
      long na3;
      long nb3;
      this.nullBit1 = 
        ((a1 = this.nullBit1) & (
        (a3 = this.nullBit3) & (a4 = this.nullBit4) & (
        (nb2 = (b2 = otherInits.nullBit2) ^ 0xFFFFFFFF) & (
        nb4 = (b4 = otherInits.nullBit4) ^ 0xFFFFFFFF) | 
        (b1 = otherInits.nullBit1) & (b3 = otherInits.nullBit3)) | 
        (na2 = (a2 = this.nullBit2) ^ 0xFFFFFFFF) & (
        b1 & b3 | ((na4 = a4 ^ 0xFFFFFFFF) | (na3 = a3 ^ 0xFFFFFFFF)) & nb2) | 
        a2 & ((na4 | na3) & ((nb3 = b3 ^ 0xFFFFFFFF) & nb4 | b1 & b2))));
      long nb1;
      long na1;
      this.nullBit2 = 
        (b2 & (nb3 | (nb1 = b1 ^ 0xFFFFFFFF)) | 
        a2 & (nb3 & nb4 | b2 | na3 | (na1 = a1 ^ 0xFFFFFFFF)));
      this.nullBit3 = 
        (b3 & (nb1 & b2 | 
        a2 & (nb2 | a3) | 
        na1 & nb2 | 
        a1 & na2 & na4 & b1) | 
        a3 & (nb2 & nb4 | na2 & a4 | na1) | 
        a1 & na2 & na4 & b2);
      this.nullBit4 = 
        (na3 & (nb1 & nb3 & b4 | 
        a4 & (nb3 | b1 & b2)) | 
        nb2 & (na3 & b1 & nb3 | na2 & (nb1 & b4 | b1 & nb3 | a4)) | 
        a3 & (a4 & (nb2 | b1 & b3) | 
        a1 & a2 & (nb1 & b4 | na4 & (b2 | b1) & nb3)));

      if ((this.nullBit2 | this.nullBit3 | this.nullBit4) != 0L)
        thisHasNulls = true;
    }
    else {
      this.nullBit1 = 0L;
      this.nullBit2 = 
        ((b2 = otherInits.nullBit2) & (
        (nb3 = (b3 = otherInits.nullBit3) ^ 0xFFFFFFFF) | (
        nb1 = (b1 = otherInits.nullBit1) ^ 0xFFFFFFFF)));
      this.nullBit3 = (b3 & (nb1 | (nb2 = b2 ^ 0xFFFFFFFF)));
      this.nullBit4 = ((b1 ^ 0xFFFFFFFF) & (b3 ^ 0xFFFFFFFF) & (b4 = otherInits.nullBit4) | (b2 ^ 0xFFFFFFFF) & (b1 & (b3 ^ 0xFFFFFFFF) | (b1 ^ 0xFFFFFFFF) & b4));

      if ((this.nullBit2 | this.nullBit3 | this.nullBit4) != 0L) {
        thisHasNulls = true;
      }
    }

    if (otherInits.extra != null) {
      int mergeLimit = 0; int copyLimit = otherInits.extra[0].length;
      if (this.extra == null) {
        this.extra = new long[6][];
        for (int j = 0; j < 6; j++) {
          this.extra[j] = new long[copyLimit];
        }

      }
      else
      {
        mergeLimit = copyLimit;
        if (mergeLimit > this.extra[0].length) {
          mergeLimit = this.extra[0].length;
          for (int j = 0; j < 6; j++) {
            System.arraycopy(this.extra[j], 0, 
              this.extra[j] =  = new long[copyLimit], 0, 
              mergeLimit);
          }
          if (!thisHadNulls) {
            mergeLimit = 0;
          }

        }

      }

      for (int i = 0; i < mergeLimit; i++)
      {
        long tmp728_727 = this.extra[2][i]; long a1 = tmp728_727;
        long tmp740_739 = this.extra[4][i]; long a3 = tmp740_739;
        long tmp752_751 = this.extra[5][i]; long a4 = tmp752_751;
        long tmp772_771 = ((b2 = otherInits.extra[3][i]) ^ 0xFFFFFFFF); nb2 = tmp772_771;
        long tmp791_790 = ((b4 = otherInits.extra[5][i]) ^ 0xFFFFFFFF); long nb4 = tmp791_790;
        long tmp804_803 = otherInits.extra[2][i]; b1 = tmp804_803;
        long tmp816_815 = otherInits.extra[4][i]; b3 = tmp816_815;
        long a2;
        long tmp838_837 = ((a2 = this.extra[3][i]) ^ 0xFFFFFFFF); long na2 = tmp838_837;
        long tmp852_851 = (a4 ^ 0xFFFFFFFF); long na4 = tmp852_851;
        long tmp861_860 = (a3 ^ 0xFFFFFFFF); long na3 = tmp861_860;
        long tmp884_883 = (b3 ^ 0xFFFFFFFF); nb3 = tmp884_883;

        this.extra[2][i] = 
          (tmp728_727 & (
          tmp740_739 & tmp752_751 & (
          tmp772_771 & 
          tmp791_790 | 
          tmp804_803 & tmp816_815) | 
          tmp838_837 & (
          b1 & b3 | (tmp852_851 | tmp861_860) & nb2) | 
          a2 & ((na4 | na3) & (tmp884_883 & nb4 | b1 & b2))));
        long tmp919_918 = (b1 ^ 0xFFFFFFFF); nb1 = tmp919_918;
        long tmp943_942 = (a1 ^ 0xFFFFFFFF); long na1 = tmp943_942;

        this.extra[3][i] = 
          (b2 & (nb3 | tmp919_918) | 
          a2 & (nb3 & nb4 | b2 | na3 | tmp943_942));
        this.extra[4][i] = 
          (b3 & (nb1 & b2 | 
          a2 & (nb2 | a3) | 
          na1 & nb2 | 
          a1 & na2 & na4 & b1) | 
          a3 & (nb2 & nb4 | na2 & a4 | na1) | 
          a1 & na2 & na4 & b2);
        this.extra[5][i] = 
          (na3 & (nb1 & nb3 & b4 | 
          a4 & (nb3 | b1 & b2)) | 
          nb2 & (na3 & b1 & nb3 | na2 & (nb1 & b4 | b1 & nb3 | a4)) | 
          a3 & (a4 & (nb2 | b1 & b3) | 
          a1 & a2 & (nb1 & b4 | na4 & (b2 | b1) & nb3)));
        if ((this.extra[3][i] | this.extra[4][i] | this.extra[5][i]) != 0L) {
          thisHasNulls = true;
        }

      }

      for (; i < copyLimit; i++) {
        this.extra[2][i] = 0L;
        long tmp1201_1200 = otherInits.extra[3][i]; b2 = tmp1201_1200;
        long tmp1220_1219 = ((b3 = otherInits.extra[4][i]) ^ 0xFFFFFFFF); nb3 = tmp1220_1219;
        long tmp1239_1238 = ((b1 = otherInits.extra[2][i]) ^ 0xFFFFFFFF); nb1 = tmp1239_1238;

        this.extra[3][i] = 
          (tmp1201_1200 & (
          tmp1220_1219 | 
          tmp1239_1238));
        long tmp1263_1262 = (b2 ^ 0xFFFFFFFF); nb2 = tmp1263_1262; this.extra[4][i] = (b3 & (nb1 | tmp1263_1262));
        long tmp1299_1298 = otherInits.extra[5][i]; b4 = tmp1299_1298; this.extra[5][i] = ((b1 ^ 0xFFFFFFFF) & (b3 ^ 0xFFFFFFFF) & tmp1299_1298 | (b2 ^ 0xFFFFFFFF) & (b1 & (b3 ^ 0xFFFFFFFF) | (b1 ^ 0xFFFFFFFF) & b4));
        if ((this.extra[3][i] | this.extra[4][i] | this.extra[5][i]) != 0L) {
          thisHasNulls = true;
        }

      }

    }

    if (thisHasNulls) {
      this.tagBits |= 2;
    }
    else {
      this.tagBits &= 2;
    }
    return this;
  }

  public final boolean cannotBeDefinitelyNullOrNonNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (
        ((this.nullBit1 ^ 0xFFFFFFFF) & (
        this.nullBit2 & this.nullBit3 | this.nullBit4) | 
        (this.nullBit2 ^ 0xFFFFFFFF) & (this.nullBit3 ^ 0xFFFFFFFF) & this.nullBit4) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length)
      return false;
    long a2;
    long a3;
    long a4;
    return (
      ((this.extra[2][vectorIndex] ^ 0xFFFFFFFF) & (
      (a2 = this.extra[3][vectorIndex]) & (a3 = this.extra[4][vectorIndex]) | (a4 = this.extra[5][vectorIndex])) | 
      (a2 ^ 0xFFFFFFFF) & (a3 ^ 0xFFFFFFFF) & a4) & 
      1L << position % 64) != 0L;
  }

  public final boolean cannotBeNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit1 & this.nullBit3 & (
        this.nullBit2 & this.nullBit4 | this.nullBit2 ^ 0xFFFFFFFF) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[4][vectorIndex] & (
      this.extra[3][vectorIndex] & this.extra[5][vectorIndex] | 
      this.extra[3][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  public final boolean canOnlyBeNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit1 & this.nullBit2 & (
        this.nullBit3 ^ 0xFFFFFFFF | this.nullBit4 ^ 0xFFFFFFFF) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (
      this.extra[4][vectorIndex] ^ 0xFFFFFFFF | this.extra[5][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  public FlowInfo copy()
  {
    if (this == DEAD_END) {
      return this;
    }
    UnconditionalFlowInfo copy = new UnconditionalFlowInfo();

    copy.definiteInits = this.definiteInits;
    copy.potentialInits = this.potentialInits;
    boolean hasNullInfo = (this.tagBits & 0x2) != 0;
    if (hasNullInfo) {
      copy.nullBit1 = this.nullBit1;
      copy.nullBit2 = this.nullBit2;
      copy.nullBit3 = this.nullBit3;
      copy.nullBit4 = this.nullBit4;
    }
    copy.tagBits = this.tagBits;
    copy.maxFieldCount = this.maxFieldCount;
    if (this.extra != null)
    {
      copy.extra = new long[6][];
      int length;
      System.arraycopy(this.extra[0], 0, 
        copy.extra[0] =  = new long[length = this.extra[0].length], 0, 
        length);
      System.arraycopy(this.extra[1], 0, 
        copy.extra[1] =  = new long[length], 0, length);
      if (hasNullInfo) {
        for (int j = 2; j < 6; j++) {
          System.arraycopy(this.extra[j], 0, 
            copy.extra[j] =  = new long[length], 0, length);
        }
      }
      else {
        for (int j = 2; j < 6; j++) {
          copy.extra[j] = new long[length];
        }
      }
    }
    return copy;
  }

  public UnconditionalFlowInfo discardInitializationInfo()
  {
    if (this == DEAD_END) {
      return this;
    }
    this.definiteInits = 
      (this.potentialInits = 0L);
    if (this.extra != null) {
      int i = 0; for (int length = this.extra[0].length; i < length; i++)
      {
        long tmp54_53 = 0L; this.extra[1][i] = tmp54_53; this.extra[0][i] = tmp54_53;
      }
    }
    return this;
  }

  public UnconditionalFlowInfo discardNonFieldInitializations()
  {
    int limit = this.maxFieldCount;
    if (limit < 64) {
      long mask = (1L << limit) - 1L;
      this.definiteInits &= mask;
      this.potentialInits &= mask;
      this.nullBit1 &= mask;
      this.nullBit2 &= mask;
      this.nullBit3 &= mask;
      this.nullBit4 &= mask;
    }

    if (this.extra == null) {
      return this;
    }
    int length = this.extra[0].length;
    int vectorIndex;
    if ((vectorIndex = limit / 64 - 1) >= length) {
      return this;
    }
    if (vectorIndex >= 0)
    {
      long mask = (1L << limit % 64) - 1L;
      for (int j = 0; j < 6; j++) {
        this.extra[j][vectorIndex] &= mask;
      }
    }
    for (int i = vectorIndex + 1; i < length; i++) {
      for (int j = 0; j < 6; j++) {
        this.extra[j][i] = 0L;
      }
    }
    return this;
  }

  public FlowInfo initsWhenFalse() {
    return this;
  }

  public FlowInfo initsWhenTrue() {
    return this;
  }

  private final boolean isDefinitelyAssigned(int position)
  {
    if (position < 64)
    {
      return (this.definiteInits & 1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[0][vectorIndex] & 
      1L << position % 64) != 0L;
  }

  public final boolean isDefinitelyAssigned(FieldBinding field)
  {
    if ((this.tagBits & 0x1) != 0) {
      return true;
    }
    return isDefinitelyAssigned(field.id);
  }

  public final boolean isDefinitelyAssigned(LocalVariableBinding local)
  {
    if (((this.tagBits & 0x1) != 0) && ((local.declaration.bits & 0x40000000) != 0)) {
      return true;
    }
    return isDefinitelyAssigned(local.id + this.maxFieldCount);
  }

  public final boolean isDefinitelyNonNull(LocalVariableBinding local)
  {
    if (((this.tagBits & 0x1) != 0) || 
      ((this.tagBits & 0x2) == 0)) {
      return false;
    }
    if (((local.type.tagBits & 0x2) != 0L) || 
      (local.constant() != Constant.NotAConstant)) {
      return true;
    }
    int position = local.id + this.maxFieldCount;
    if (position < 64) {
      return (this.nullBit1 & this.nullBit3 & (this.nullBit2 ^ 0xFFFFFFFF | this.nullBit4) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[4][vectorIndex] & (
      this.extra[3][vectorIndex] ^ 0xFFFFFFFF | this.extra[5][vectorIndex]) & 
      1L << position % 64) != 0L;
  }

  public final boolean isDefinitelyNull(LocalVariableBinding local)
  {
    if (((this.tagBits & 0x1) != 0) || 
      ((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L)) {
      return false;
    }
    int position = local.id + this.maxFieldCount;
    if (position < 64) {
      return (this.nullBit1 & this.nullBit2 & (
        this.nullBit3 ^ 0xFFFFFFFF | this.nullBit4 ^ 0xFFFFFFFF) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (
      this.extra[4][vectorIndex] ^ 0xFFFFFFFF | this.extra[5][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  public final boolean isDefinitelyUnknown(LocalVariableBinding local)
  {
    if (((this.tagBits & 0x1) != 0) || 
      ((this.tagBits & 0x2) == 0)) {
      return false;
    }
    int position = local.id + this.maxFieldCount;
    if (position < 64) {
      return (this.nullBit1 & this.nullBit4 & (
        this.nullBit2 ^ 0xFFFFFFFF) & (this.nullBit3 ^ 0xFFFFFFFF) & 1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[5][vectorIndex] & (
      this.extra[3][vectorIndex] ^ 0xFFFFFFFF) & (this.extra[4][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  private final boolean isPotentiallyAssigned(int position)
  {
    if (position < 64)
    {
      return (this.potentialInits & 1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[1][vectorIndex] & 
      1L << position % 64) != 0L;
  }

  public final boolean isPotentiallyAssigned(FieldBinding field) {
    return isPotentiallyAssigned(field.id);
  }

  public final boolean isPotentiallyAssigned(LocalVariableBinding local)
  {
    if (local.constant() != Constant.NotAConstant) {
      return true;
    }
    return isPotentiallyAssigned(local.id + this.maxFieldCount);
  }

  public final boolean isPotentiallyNonNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit3 & (this.nullBit1 ^ 0xFFFFFFFF | this.nullBit2 ^ 0xFFFFFFFF) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[4][vectorIndex] & (
      this.extra[2][vectorIndex] ^ 0xFFFFFFFF | this.extra[3][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  public final boolean isPotentiallyNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit2 & (this.nullBit1 ^ 0xFFFFFFFF | this.nullBit3 ^ 0xFFFFFFFF) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[3][vectorIndex] & (
      this.extra[2][vectorIndex] ^ 0xFFFFFFFF | this.extra[4][vectorIndex] ^ 0xFFFFFFFF) & 
      1L << position % 64) != 0L;
  }

  public final boolean isPotentiallyUnknown(LocalVariableBinding local)
  {
    if (((this.tagBits & 0x1) != 0) || 
      ((this.tagBits & 0x2) == 0)) {
      return false;
    }
    int position = local.id + this.maxFieldCount;
    if (position < 64) {
      return (this.nullBit4 & (
        this.nullBit1 ^ 0xFFFFFFFF | (this.nullBit2 ^ 0xFFFFFFFF) & (this.nullBit3 ^ 0xFFFFFFFF)) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[5][vectorIndex] & (
      this.extra[2][vectorIndex] ^ 0xFFFFFFFF | 
      (this.extra[3][vectorIndex] ^ 0xFFFFFFFF) & (this.extra[4][vectorIndex] ^ 0xFFFFFFFF)) & 
      1L << position % 64) != 0L;
  }

  public final boolean isProtectedNonNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit1 & this.nullBit3 & this.nullBit4 & 1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & 
      this.extra[4][vectorIndex] & 
      this.extra[5][vectorIndex] & 
      1L << position % 64) != 0L;
  }

  public final boolean isProtectedNull(LocalVariableBinding local) {
    if (((this.tagBits & 0x2) == 0) || 
      ((local.type.tagBits & 0x2) != 0L))
      return false;
    int position;
    if ((position = local.id + this.maxFieldCount) < 64)
    {
      return (this.nullBit1 & this.nullBit2 & (
        this.nullBit3 ^ this.nullBit4) & 
        1L << position) != 0L;
    }

    if (this.extra == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = position / 64 - 1) >= 
      this.extra[0].length) {
      return false;
    }
    return (this.extra[2][vectorIndex] & this.extra[3][vectorIndex] & (
      this.extra[4][vectorIndex] ^ this.extra[5][vectorIndex]) & 
      1L << position % 64) != 0L;
  }

  public void markAsComparedEqualToNonNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        long mask;
        long a1;
        long a2;
        long na2;
        long a3;
        long a4;
        if (((mask = 1L << position) & (
          a1 = this.nullBit1) & (
          na2 = (a2 = this.nullBit2) ^ 0xFFFFFFFF) & (
          (a3 = this.nullBit3) ^ 0xFFFFFFFF) & (
          a4 = this.nullBit4)) != 
          0L) {
          this.nullBit4 &= (mask ^ 0xFFFFFFFF);
        } else if ((mask & a1 & na2 & a3) == 0L) {
          this.nullBit4 |= mask;
          if ((mask & a1) == 0L) {
            if ((mask & a2 & (a3 ^ a4)) != 0L) {
              this.nullBit2 &= (mask ^ 0xFFFFFFFF);
            }
            else if ((mask & (a2 | a3 | a4)) == 0L) {
              this.nullBit2 |= mask;
            }
          }
        }
        this.nullBit1 |= mask;
        this.nullBit3 |= mask;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 0; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[0].length)) {
            int newLength = vectorIndex + 1;
            for (int j = 0; j < 6; j++)
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[newLength], 0, 
                oldLength);
          }
        }
        long mask;
        long a1;
        long a2;
        long na2;
        long a3;
        long a4;
        if (((mask = 1L << position % 64) & (
          a1 = this.extra[2][vectorIndex]) & (
          na2 = (a2 = this.extra[3][vectorIndex]) ^ 0xFFFFFFFF) & (
          (a3 = this.extra[4][vectorIndex]) ^ 0xFFFFFFFF) & (
          a4 = this.extra[5][vectorIndex])) != 
          0L) {
          this.extra[5][vectorIndex] &= (mask ^ 0xFFFFFFFF);
        } else if ((mask & a1 & na2 & a3) == 0L) {
          this.extra[5][vectorIndex] |= mask;
          if ((mask & a1) == 0L) {
            if ((mask & a2 & (a3 ^ a4)) != 0L) {
              this.extra[3][vectorIndex] &= (mask ^ 0xFFFFFFFF);
            }
            else if ((mask & (a2 | a3 | a4)) == 0L) {
              this.extra[3][vectorIndex] |= mask;
            }
          }
        }
        this.extra[2][vectorIndex] |= mask;
        this.extra[4][vectorIndex] |= mask;
      }
    }
  }

  public void markAsComparedEqualToNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        long mask;
        if (((mask = 1L << position) & this.nullBit1) != 0L) {
          if ((mask & (
            this.nullBit2 ^ 0xFFFFFFFF | this.nullBit3 | 
            this.nullBit4 ^ 0xFFFFFFFF)) != 0L)
            this.nullBit4 &= (mask ^ 0xFFFFFFFF);
        }
        else if ((mask & this.nullBit4) != 0L) {
          this.nullBit3 &= (mask ^ 0xFFFFFFFF);
        }
        else if ((mask & this.nullBit2) != 0L) {
          this.nullBit3 &= (mask ^ 0xFFFFFFFF);
          this.nullBit4 |= mask;
        } else {
          this.nullBit3 |= mask;
        }

        this.nullBit1 |= mask;
        this.nullBit2 |= mask;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        long mask = 1L << position % 64;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 0; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[0].length)) {
            int newLength = vectorIndex + 1;
            for (int j = 0; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[newLength], 0, 
                oldLength);
            }

          }

        }

        if ((mask & this.extra[2][vectorIndex]) != 0L) {
          if ((mask & (
            this.extra[3][vectorIndex] ^ 0xFFFFFFFF | this.extra[4][vectorIndex] | 
            this.extra[5][vectorIndex] ^ 0xFFFFFFFF)) != 0L)
            this.extra[5][vectorIndex] &= (mask ^ 0xFFFFFFFF);
        }
        else if ((mask & this.extra[5][vectorIndex]) != 0L) {
          this.extra[4][vectorIndex] &= (mask ^ 0xFFFFFFFF);
        }
        else if ((mask & this.extra[3][vectorIndex]) != 0L) {
          this.extra[4][vectorIndex] &= (mask ^ 0xFFFFFFFF);
          this.extra[5][vectorIndex] |= mask;
        } else {
          this.extra[4][vectorIndex] |= mask;
        }

        this.extra[2][vectorIndex] |= mask;
        this.extra[3][vectorIndex] |= mask;
      }
    }
  }

  private final void markAsDefinitelyAssigned(int position)
  {
    if (this != DEAD_END)
    {
      if (position < 64)
      {
        long mask;
        this.definiteInits |= (mask = 1L << position);
        this.potentialInits |= mask;
      }
      else
      {
        int vectorIndex = position / 64 - 1;
        if (this.extra == null) {
          int length = vectorIndex + 1;
          this.extra = new long[6][];
          for (int j = 0; j < 6; j++)
            this.extra[j] = new long[length];
        }
        else
        {
          int oldLength;
          if (vectorIndex >= (oldLength = this.extra[0].length))
            for (int j = 0; j < 6; j++)
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[vectorIndex + 1], 0, 
                oldLength);
        }
        long tmp166_165 = (1L << position % 64); long mask = tmp166_165;

        this.extra[0][vectorIndex] |= 
          tmp166_165;
        this.extra[1][vectorIndex] |= mask;
      }
    }
  }

  public void markAsDefinitelyAssigned(FieldBinding field) {
    if (this != DEAD_END)
      markAsDefinitelyAssigned(field.id);
  }

  public void markAsDefinitelyAssigned(LocalVariableBinding local) {
    if (this != DEAD_END)
      markAsDefinitelyAssigned(local.id + this.maxFieldCount);
  }

  public void markAsDefinitelyNonNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        long mask;
        this.nullBit1 |= (mask = 1L << position);
        this.nullBit3 |= mask;

        this.nullBit2 &= (mask ^= -1L);
        this.nullBit4 &= mask;
      }
      else
      {
        int vectorIndex;
        long tmp113_112 = (1L << position % 64); long mask = tmp113_112;

        this.extra[2][(vectorIndex = position / 64 - 1)] |= 
          tmp113_112;
        this.extra[4][vectorIndex] |= mask;
        long tmp145_144 = (mask ^ 0xFFFFFFFF); mask = tmp145_144; this.extra[3][vectorIndex] &= tmp145_144;
        this.extra[5][vectorIndex] &= mask;
      }
    }
  }

  public void markAsDefinitelyNull(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        long mask;
        this.nullBit1 |= (mask = 1L << position);
        this.nullBit2 |= mask;

        this.nullBit3 &= (mask ^= -1L);
        this.nullBit4 &= mask;
      }
      else
      {
        int vectorIndex;
        long tmp113_112 = (1L << position % 64); long mask = tmp113_112;

        this.extra[2][(vectorIndex = position / 64 - 1)] |= 
          tmp113_112;
        this.extra[3][vectorIndex] |= mask;
        long tmp145_144 = (mask ^ 0xFFFFFFFF); mask = tmp145_144; this.extra[4][vectorIndex] &= tmp145_144;
        this.extra[5][vectorIndex] &= mask;
      }
    }
  }

  public void markAsDefinitelyUnknown(LocalVariableBinding local)
  {
    if (this != DEAD_END) {
      this.tagBits |= 2;
      int position;
      if ((position = local.id + this.maxFieldCount) < 64)
      {
        long mask;
        this.nullBit1 |= (mask = 1L << position);
        this.nullBit4 |= mask;

        this.nullBit2 &= (mask ^= -1L);
        this.nullBit3 &= mask;
      }
      else
      {
        int vectorIndex;
        long tmp113_112 = (1L << position % 64); long mask = tmp113_112;

        this.extra[2][(vectorIndex = position / 64 - 1)] |= 
          tmp113_112;
        this.extra[5][vectorIndex] |= mask;
        long tmp145_144 = (mask ^ 0xFFFFFFFF); mask = tmp145_144; this.extra[3][vectorIndex] &= tmp145_144;
        this.extra[4][vectorIndex] &= mask;
      }
    }
  }

  public UnconditionalFlowInfo mergedWith(UnconditionalFlowInfo otherInits)
  {
    if (((otherInits.tagBits & 0x1) != 0) && (this != DEAD_END))
    {
      return this;
    }
    if ((this.tagBits & 0x1) != 0)
    {
      return (UnconditionalFlowInfo)otherInits.copy();
    }

    this.definiteInits &= otherInits.definiteInits;

    this.potentialInits |= otherInits.potentialInits;

    boolean thisHasNulls = (this.tagBits & 0x2) != 0;
    boolean otherHasNulls = (otherInits.tagBits & 0x2) != 0;
    boolean thisHadNulls = thisHasNulls;

    if (thisHadNulls) {
      if (otherHasNulls)
      {
        long a2;
        long a3;
        long a4;
        long b1;
        long b2;
        long nb2;
        long a1;
        long b3;
        long b4;
        long na2;
        long nb4;
        long na4;
        long na3;
        long nb3;
        this.nullBit1 = 
          ((a2 = this.nullBit2) & (a3 = this.nullBit3) & (
          a4 = this.nullBit4) & (b1 = otherInits.nullBit1) & (
          nb2 = (b2 = otherInits.nullBit2) ^ 0xFFFFFFFF) | 
          (a1 = this.nullBit1) & (b1 & (a3 & a4 & (b3 = otherInits.nullBit3) & (
          b4 = otherInits.nullBit4) | 
          (na2 = a2 ^ 0xFFFFFFFF) & nb2 & (
          (nb4 = b4 ^ 0xFFFFFFFF) | (na4 = a4 ^ 0xFFFFFFFF) | 
          (na3 = a3 ^ 0xFFFFFFFF) & (nb3 = b3 ^ 0xFFFFFFFF)) | 
          a2 & b2 & ((na4 | na3) & (nb4 | nb3))) | 
          na2 & b2 & b3 & b4));
        long nb1;
        long na1;
        this.nullBit2 = 
          (b2 & (nb3 | (nb1 = b1 ^ 0xFFFFFFFF) | a3 & (a4 | (na1 = a1 ^ 0xFFFFFFFF)) & nb4) | 
          a2 & (b2 | na4 & b3 & (b4 | nb1) | na3 | na1));
        this.nullBit3 = 
          (b3 & (nb2 & b4 | nb1 | a3 & (na4 & nb4 | a4 & b4)) | 
          a3 & (na2 & a4 | na1) | 
          (a2 | na1) & b1 & nb2 & nb4 | 
          a1 & na2 & na4 & (b2 | nb1));
        this.nullBit4 = 
          (na3 & (nb1 & nb3 & b4 | 
          b1 & (nb2 & nb3 | a4 & b2 & nb4) | 
          na1 & a4 & (nb3 | b1 & b2)) | 
          a3 & a4 & (b3 & b4 | b1 & nb2) | 
          na2 & (nb1 & b4 | b1 & nb3 | na1 & a4) & nb2 | 
          a1 & (na3 & (nb3 & b4 | 
          b1 & b2 & b3 & nb4 | 
          na2 & (nb3 | nb2)) | 
          na2 & b3 & b4 | 
          a2 & (nb1 & b4 | a3 & na4 & b1) & nb3));
      }
      else
      {
        long a1 = this.nullBit1;
        this.nullBit1 = 0L;
        long a2;
        long a3;
        long na1;
        long na3;
        this.nullBit2 = ((a2 = this.nullBit2) & (na3 = (a3 = this.nullBit3) ^ 0xFFFFFFFF | (na1 = a1 ^ 0xFFFFFFFF)));
        long na2;
        long a4;
        this.nullBit3 = (a3 & ((na2 = a2 ^ 0xFFFFFFFF) & (a4 = this.nullBit4) | na1) | a1 & na2 & (a4 ^ 0xFFFFFFFF));
        this.nullBit4 = ((na3 | na2) & na1 & a4 | a1 & na3 & na2);
      }

    }
    else if (otherHasNulls) {
      this.nullBit1 = 0L;
      long b2;
      long b3;
      long b1;
      long nb1;
      long nb3;
      this.nullBit2 = ((b2 = otherInits.nullBit2) & (nb3 = (b3 = otherInits.nullBit3) ^ 0xFFFFFFFF | (nb1 = (b1 = otherInits.nullBit1) ^ 0xFFFFFFFF)));
      long nb2;
      long b4;
      this.nullBit3 = (b3 & ((nb2 = b2 ^ 0xFFFFFFFF) & (b4 = otherInits.nullBit4) | nb1) | b1 & nb2 & (b4 ^ 0xFFFFFFFF));
      this.nullBit4 = ((nb3 | nb2) & nb1 & b4 | b1 & nb3 & nb2);

      thisHasNulls = 
        (this.nullBit2 != 0L) || 
        (this.nullBit3 != 0L) || 
        (this.nullBit4 != 0L);
    }

    if ((this.extra != null) || (otherInits.extra != null)) {
      int mergeLimit = 0; int copyLimit = 0; int resetLimit = 0;

      if (this.extra != null) {
        if (otherInits.extra != null)
        {
          int length;
          int otherLength;
          if ((length = this.extra[0].length) < 
            (otherLength = otherInits.extra[0].length))
          {
            for (int j = 0; j < 6; j++) {
              System.arraycopy(this.extra[j], 0, 
                this.extra[j] =  = new long[otherLength], 0, length);
            }
            mergeLimit = length;
            copyLimit = otherLength;
          }
          else
          {
            mergeLimit = otherLength;
            resetLimit = length;
          }

        }
        else
        {
          resetLimit = this.extra[0].length;
        }

      }
      else if (otherInits.extra != null)
      {
        int otherLength = otherInits.extra[0].length;
        this.extra = new long[6][];
        for (int j = 0; j < 6; j++) {
          this.extra[j] = new long[otherLength];
        }
        System.arraycopy(otherInits.extra[1], 0, 
          this.extra[1], 0, otherLength);
        copyLimit = otherLength;
      }

      for (int i = 0; i < mergeLimit; i++) {
        this.extra[0][i] &= otherInits.extra[0][i];
        this.extra[1][i] |= otherInits.extra[1][i];
      }
      for (; i < copyLimit; i++) {
        this.extra[1][i] = otherInits.extra[1][i];
      }
      for (; i < resetLimit; i++) {
        this.extra[0][i] = 0L;
      }

      if (!otherHasNulls) {
        if (resetLimit < mergeLimit) {
          resetLimit = mergeLimit;
        }
        copyLimit = 0;
        mergeLimit = 0;
      }
      if (!thisHadNulls) {
        resetLimit = 0;
      }

      for (i = 0; i < mergeLimit; i++)
      {
        long tmp1195_1194 = this.extra[3][i]; long a2 = tmp1195_1194;
        long tmp1207_1206 = this.extra[4][i]; long a3 = tmp1207_1206;
        long tmp1220_1219 = this.extra[5][i]; long a4 = tmp1220_1219;
        long tmp1233_1232 = otherInits.extra[2][i]; long b1 = tmp1233_1232;
        long b2;
        long tmp1253_1252 = ((b2 = otherInits.extra[3][i]) ^ 0xFFFFFFFF); long nb2 = tmp1253_1252;
        long tmp1266_1265 = this.extra[2][i]; long a1 = tmp1266_1265;
        long tmp1285_1284 = otherInits.extra[4][i]; long b3 = tmp1285_1284;
        long tmp1298_1297 = otherInits.extra[5][i]; long b4 = tmp1298_1297;
        long tmp1308_1307 = (a2 ^ 0xFFFFFFFF); long na2 = tmp1308_1307;
        long tmp1320_1319 = (b4 ^ 0xFFFFFFFF); long nb4 = tmp1320_1319;
        long tmp1329_1328 = (a4 ^ 0xFFFFFFFF); long na4 = tmp1329_1328;
        long tmp1339_1338 = (a3 ^ 0xFFFFFFFF); long na3 = tmp1339_1338;
        long tmp1348_1347 = (b3 ^ 0xFFFFFFFF); long nb3 = tmp1348_1347;

        this.extra[2][i] = 
          (tmp1195_1194 & tmp1207_1206 & 
          tmp1220_1219 & tmp1233_1232 & 
          tmp1253_1252 | 
          tmp1266_1265 & (b1 & (a3 & a4 & tmp1285_1284 & 
          tmp1298_1297 | 
          tmp1308_1307 & nb2 & (
          tmp1320_1319 | tmp1329_1328 | 
          tmp1339_1338 & tmp1348_1347) | 
          a2 & b2 & ((na4 | na3) & (nb4 | nb3))) | 
          na2 & b2 & b3 & b4));
        long tmp1407_1406 = (b1 ^ 0xFFFFFFFF); long nb1 = tmp1407_1406;
        long tmp1421_1420 = (a1 ^ 0xFFFFFFFF); long na1 = tmp1421_1420; this.extra[3][i] = 
          (b2 & (nb3 | tmp1407_1406 | a3 & (a4 | tmp1421_1420) & nb4) | 
          a2 & (b2 | na4 & b3 & (b4 | nb1) | na3 | na1));
        this.extra[4][i] = 
          (b3 & (nb2 & b4 | nb1 | a3 & (na4 & nb4 | a4 & b4)) | 
          a3 & (na2 & a4 | na1) | 
          (a2 | na1) & b1 & nb2 & nb4 | 
          a1 & na2 & na4 & (b2 | nb1));
        this.extra[5][i] = 
          (na3 & (nb1 & nb3 & b4 | 
          b1 & (nb2 & nb3 | a4 & b2 & nb4) | 
          na1 & a4 & (nb3 | b1 & b2)) | 
          a3 & a4 & (b3 & b4 | b1 & nb2) | 
          na2 & (nb1 & b4 | b1 & nb3 | na1 & a4) & nb2 | 
          a1 & (na3 & (nb3 & b4 | 
          b1 & b2 & b3 & nb4 | 
          na2 & (nb3 | nb2)) | 
          na2 & b3 & b4 | 
          a2 & (nb1 & b4 | a3 & na4 & b1) & nb3));
        thisHasNulls = (thisHasNulls) || 
          (this.extra[3][i] != 0L) || 
          (this.extra[4][i] != 0L) || 
          (this.extra[5][i] != 0L);
      }

      for (; i < copyLimit; i++) {
        this.extra[2][i] = 0L;
        long tmp1783_1782 = otherInits.extra[3][i]; long b2 = tmp1783_1782;
        long b3;
        long b1;
        long nb1;
        long tmp1822_1821 = ((b3 = otherInits.extra[4][i]) ^ 0xFFFFFFFF | (nb1 = (b1 = otherInits.extra[2][i]) ^ 0xFFFFFFFF)); long nb3 = tmp1822_1821; this.extra[3][i] = (tmp1783_1782 & tmp1822_1821);
        long tmp1843_1842 = (b2 ^ 0xFFFFFFFF); long nb2 = tmp1843_1842;
        long tmp1855_1854 = otherInits.extra[5][i]; long b4 = tmp1855_1854; this.extra[4][i] = (b3 & (tmp1843_1842 & tmp1855_1854 | nb1) | b1 & nb2 & (b4 ^ 0xFFFFFFFF));
        this.extra[5][i] = ((nb3 | nb2) & nb1 & b4 | b1 & nb3 & nb2);
        thisHasNulls = (thisHasNulls) || 
          (this.extra[3][i] != 0L) || 
          (this.extra[4][i] != 0L) || 
          (this.extra[5][i] != 0L);
      }

      for (; i < resetLimit; i++) {
        long a1 = this.extra[2][i];
        this.extra[2][i] = 0L;
        long tmp2009_2008 = this.extra[3][i]; long a2 = tmp2009_2008;
        long a3;
        long na1;
        long tmp2038_2037 = ((a3 = this.extra[4][i]) ^ 0xFFFFFFFF | (na1 = a1 ^ 0xFFFFFFFF)); long na3 = tmp2038_2037; this.extra[3][i] = (tmp2009_2008 & tmp2038_2037);
        long tmp2059_2058 = (a2 ^ 0xFFFFFFFF); long na2 = tmp2059_2058;
        long tmp2071_2070 = this.extra[5][i]; long a4 = tmp2071_2070; this.extra[4][i] = (a3 & (tmp2059_2058 & tmp2071_2070 | na1) | a1 & na2 & (a4 ^ 0xFFFFFFFF));
        this.extra[5][i] = ((na3 | na2) & na1 & a4 | a1 & na3 & na2);
        thisHasNulls = (thisHasNulls) || 
          (this.extra[3][i] != 0L) || 
          (this.extra[4][i] != 0L) || 
          (this.extra[5][i] != 0L);
      }

    }

    if (thisHasNulls) {
      this.tagBits |= 2;
    }
    else {
      this.tagBits &= -3;
    }
    return this;
  }

  static int numberOfEnclosingFields(ReferenceBinding type)
  {
    int count = 0;
    type = type.enclosingType();
    while (type != null) {
      count += type.fieldCount();
      type = type.enclosingType();
    }
    return count;
  }

  public UnconditionalFlowInfo nullInfoLessUnconditionalCopy() {
    if (this == DEAD_END) {
      return this;
    }
    UnconditionalFlowInfo copy = new UnconditionalFlowInfo();
    copy.definiteInits = this.definiteInits;
    copy.potentialInits = this.potentialInits;
    this.tagBits &= -3;
    copy.maxFieldCount = this.maxFieldCount;
    if (this.extra != null)
    {
      copy.extra = new long[6][];
      int length;
      System.arraycopy(this.extra[0], 0, 
        copy.extra[0] =  = 
        new long[length = this.extra[0].length], 0, length);
      System.arraycopy(this.extra[1], 0, 
        copy.extra[1] =  = new long[length], 0, length);
      for (int j = 2; j < 6; j++) {
        copy.extra[j] = new long[length];
      }
    }
    return copy;
  }

  public FlowInfo safeInitsWhenTrue() {
    return copy();
  }

  public FlowInfo setReachMode(int reachMode) {
    if ((reachMode == 0) && (this != DEAD_END)) {
      this.tagBits &= -2;
    }
    else {
      if ((this.tagBits & 0x1) == 0)
      {
        this.potentialInits = 0L;
        if (this.extra != null) {
          int i = 0; int length = this.extra[0].length;
          for (; i < length; i++) {
            this.extra[1][i] = 0L;
          }
        }
      }
      this.tagBits |= 1;
    }
    return this;
  }

  public String toString()
  {
    if (this == DEAD_END) {
      return "FlowInfo.DEAD_END";
    }
    if ((this.tagBits & 0x2) != 0) {
      if (this.extra == null) {
        return "FlowInfo<def: " + this.definiteInits + 
          ", pot: " + this.potentialInits + 
          ", reachable:" + ((this.tagBits & 0x1) == 0) + 
          ", null: " + this.nullBit1 + 
          this.nullBit2 + this.nullBit3 + this.nullBit4 + 
          ">";
      }

      String def = "FlowInfo<def:[" + this.definiteInits;
      String pot = "], pot:[" + this.potentialInits;
      String nullS = ", null:[" + this.nullBit1 + 
        this.nullBit2 + this.nullBit3 + this.nullBit4;

      int i = 0;

      int ceil = this.extra[0].length > 3 ? 
        3 : 
        this.extra[0].length;
      for (; i < ceil; i++) {
        def = def + "," + this.extra[0][i];
        pot = pot + "," + this.extra[1][i];
        nullS = nullS + "," + this.extra[2][i] + 
          this.extra[3][i] + this.extra[4][i] + this.extra[5][i];
      }
      if (ceil < this.extra[0].length) {
        def = def + ",...";
        pot = pot + ",...";
        nullS = nullS + ",...";
      }
      return def + pot + 
        "], reachable:" + ((this.tagBits & 0x1) == 0) + 
        nullS + 
        "]>";
    }

    if (this.extra == null) {
      return "FlowInfo<def: " + this.definiteInits + 
        ", pot: " + this.potentialInits + 
        ", reachable:" + ((this.tagBits & 0x1) == 0) + 
        ", no null info>";
    }

    String def = "FlowInfo<def:[" + this.definiteInits;
    String pot = "], pot:[" + this.potentialInits;

    int i = 0;

    int ceil = this.extra[0].length > 3 ? 
      3 : 
      this.extra[0].length;
    for (; i < ceil; i++) {
      def = def + "," + this.extra[0][i];
      pot = pot + "," + this.extra[1][i];
    }
    if (ceil < this.extra[0].length) {
      def = def + ",...";
      pot = pot + ",...";
    }
    return def + pot + 
      "], reachable:" + ((this.tagBits & 0x1) == 0) + 
      ", no null info>";
  }

  public UnconditionalFlowInfo unconditionalCopy()
  {
    return (UnconditionalFlowInfo)copy();
  }

  public UnconditionalFlowInfo unconditionalFieldLessCopy()
  {
    UnconditionalFlowInfo copy = new UnconditionalFlowInfo();
    copy.tagBits = this.tagBits;
    copy.maxFieldCount = this.maxFieldCount;
    int limit = this.maxFieldCount;
    if (limit < 64)
    {
      long mask;
      this.definiteInits &= (mask = (1L << limit) - 1L ^ 0xFFFFFFFF);
      this.potentialInits &= mask;
      this.nullBit1 &= mask;
      this.nullBit2 &= mask;
      this.nullBit3 &= mask;
      this.nullBit4 &= mask;
    }

    if (this.extra == null)
      return copy;
    int vectorIndex;
    int length;
    if ((vectorIndex = limit / 64 - 1) >= 
      (length = this.extra[0].length)) {
      return copy;
    }

    copy.extra = new long[6][];
    int copyStart;
    if ((copyStart = vectorIndex + 1) < length) {
      int copyLength = length - copyStart;
      for (int j = 0; j < 6; j++) {
        System.arraycopy(this.extra[j], copyStart, 
          copy.extra[j] =  = new long[length], copyStart, 
          copyLength);
      }
    }
    else if (vectorIndex >= 0) {
      for (int j = 0; j < 6; j++) {
        copy.extra[j] = new long[length];
      }
    }
    if (vectorIndex >= 0) {
      long mask = (1L << limit % 64) - 1L ^ 0xFFFFFFFF;
      for (int j = 0; j < 6; j++) {
        copy.extra[j][vectorIndex] = 
          (this.extra[j][vectorIndex] & mask);
      }
    }
    return copy;
  }

  public UnconditionalFlowInfo unconditionalInits()
  {
    return this;
  }

  public UnconditionalFlowInfo unconditionalInitsWithoutSideEffect() {
    return this;
  }

  public static class AssertionFailedException extends RuntimeException
  {
    private static final long serialVersionUID = 1827352841030089703L;

    public AssertionFailedException(String message)
    {
      super();
    }
  }
}