package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
import org.eclipse.jdt.internal.compiler.util.SimpleSet;

class MethodVerifier15 extends MethodVerifier
{
  MethodVerifier15(LookupEnvironment environment)
  {
    super(environment);
  }
  boolean areMethodsCompatible(MethodBinding one, MethodBinding two) {
    MethodBinding sub = computeSubstituteMethod(two, one);
    return (sub != null) && (doesSubstituteMethodOverride(one, sub)) && (areReturnTypesCompatible(one, sub));
  }
  boolean areParametersEqual(MethodBinding one, MethodBinding two) {
    TypeBinding[] oneArgs = one.parameters;
    TypeBinding[] twoArgs = two.parameters;
    if (oneArgs == twoArgs) return true;

    int length = oneArgs.length;
    if (length != twoArgs.length) return false;

    if (one.declaringClass.isInterface()) {
      for (int i = 0; i < length; i++) {
        if (!areTypesEqual(oneArgs[i], twoArgs[i]))
          return false;
      }
    }
    else
    {
      for (int i = 0; i < length; i++) {
        if (!areTypesEqual(oneArgs[i], twoArgs[i])) {
          if ((oneArgs[i].leafComponentType().isRawType()) && 
            (oneArgs[i].dimensions() == twoArgs[i].dimensions()) && (oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType())))
          {
            for (int j = 0; j < i; j++) {
              if (oneArgs[j].leafComponentType().isParameterizedType())
                return false;
            }
            break;
          }

          return false;
        }
      }

      for (i++; i < length; i++) {
        if (!areTypesEqual(oneArgs[i], twoArgs[i])) {
          if ((!oneArgs[i].leafComponentType().isRawType()) || 
            (oneArgs[i].dimensions() != twoArgs[i].dimensions()) || (!oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType())))
          {
            return false;
          } } else if (oneArgs[i].leafComponentType().isParameterizedType()) {
          return false;
        }
      }
    }
    return true;
  }
  boolean areReturnTypesCompatible(MethodBinding one, MethodBinding two) {
    if (one.returnType == two.returnType) return true;
    return areReturnTypesCompatible0(one, two);
  }
  boolean areTypesEqual(TypeBinding one, TypeBinding two) {
    if (one == two) return true;

    if ((one.isParameterizedType()) && (two.isParameterizedType())) {
      return (one.isEquivalentTo(two)) && (two.isEquivalentTo(one));
    }

    return false;
  }
  boolean canSkipInheritedMethods() {
    if ((this.type.superclass() != null) && (
      (this.type.superclass().isAbstract()) || (this.type.superclass().isParameterizedType())))
      return false;
    return this.type.superInterfaces() == Binding.NO_SUPERINTERFACES;
  }

  boolean canSkipInheritedMethods(MethodBinding one, MethodBinding two) {
    return (two == null) || (
      (one.declaringClass == two.declaringClass) && (!one.declaringClass.isParameterizedType()));
  }
  void checkConcreteInheritedMethod(MethodBinding concreteMethod, MethodBinding[] abstractMethods) {
    super.checkConcreteInheritedMethod(concreteMethod, abstractMethods);

    int i = 0; for (int l = abstractMethods.length; i < l; i++) {
      MethodBinding abstractMethod = abstractMethods[i];
      if (concreteMethod.isVarargs() != abstractMethod.isVarargs()) {
        problemReporter().varargsConflict(concreteMethod, abstractMethod, this.type);
      }

      MethodBinding originalInherited = abstractMethod.original();
      if (originalInherited.returnType != concreteMethod.returnType) {
        if (abstractMethod.returnType.leafComponentType().isParameterizedType()) {
          if (concreteMethod.returnType.leafComponentType().isRawType())
            problemReporter().unsafeReturnTypeOverride(concreteMethod, originalInherited, this.type);
        } else if ((abstractMethod.hasSubstitutedReturnType()) && (originalInherited.returnType.leafComponentType().isTypeVariable()) && 
          (((TypeVariableBinding)originalInherited.returnType.leafComponentType()).declaringElement == originalInherited)) {
          TypeBinding currentReturnType = concreteMethod.returnType.leafComponentType();
          if ((!currentReturnType.isTypeVariable()) || (((TypeVariableBinding)currentReturnType).declaringElement != concreteMethod)) {
            problemReporter().unsafeReturnTypeOverride(concreteMethod, originalInherited, this.type);
          }
        }
      }

      if ((originalInherited.declaringClass.isInterface()) && 
        (this.type.superclass.erasure().findSuperTypeWithSameErasure(originalInherited.declaringClass) == null))
        this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
    }
  }

  void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMethod, MethodBinding[] allInheritedMethods) {
    if (currentMethod.isVarargs() != inheritedMethod.isVarargs()) {
      problemReporter(currentMethod).varargsConflict(currentMethod, inheritedMethod, this.type);
    }

    MethodBinding originalInherited = inheritedMethod.original();
    if (originalInherited.returnType != currentMethod.returnType)
    {
      if ((inheritedMethod.returnType.leafComponentType().isParameterizedType()) && (currentMethod.returnType.leafComponentType().isRawType())) {
        problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, originalInherited, this.type);
      } else if ((inheritedMethod.hasSubstitutedReturnType()) && (originalInherited.returnType.leafComponentType().isTypeVariable()) && 
        (((TypeVariableBinding)originalInherited.returnType.leafComponentType()).declaringElement == originalInherited)) {
        TypeBinding currentReturnType = currentMethod.returnType.leafComponentType();
        if ((!currentReturnType.isTypeVariable()) || (((TypeVariableBinding)currentReturnType).declaringElement != currentMethod)) {
          problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, originalInherited, this.type);
        }
      }
    }

    if (this.type.addSyntheticBridgeMethod(originalInherited, currentMethod.original()) != null) {
      int i = 0; for (int l = allInheritedMethods.length; i < l; i++) {
        MethodBinding otherInheritedMethod = allInheritedMethods[i];
        MethodBinding otherOriginal = otherInheritedMethod.original();

        if ((otherOriginal != originalInherited) && (otherOriginal != otherInheritedMethod) && 
          (!inheritedMethod.areParametersEqual(otherInheritedMethod)))
        {
          if ((otherInheritedMethod.declaringClass.erasure() == inheritedMethod.declaringClass.erasure()) || 
            (otherInheritedMethod.declaringClass.findSuperTypeWithSameErasure(inheritedMethod.declaringClass) == null))
          {
            if (detectInheritedNameClash(originalInherited, otherOriginal))
              return;
          }
        }
      }
    }
  }

  void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMethod)
  {
    if ((currentMethod.declaringClass.isInterface()) || (currentMethod.isStatic())) return;

    if (!detectNameClash(currentMethod, inheritedMethod)) {
      TypeBinding[] currentParams = currentMethod.parameters;
      TypeBinding[] inheritedParams = inheritedMethod.parameters;
      int length = currentParams.length;
      if (length != inheritedParams.length) return;

      for (int i = 0; i < length; i++) {
        if ((currentParams[i] != inheritedParams[i]) && (
          (currentParams[i].isBaseType() != inheritedParams[i].isBaseType()) || (!inheritedParams[i].isCompatibleWith(currentParams[i]))))
          return;
      }
      ReferenceBinding[] interfacesToVisit = (ReferenceBinding[])null;
      int nextPosition = 0;
      ReferenceBinding superType = inheritedMethod.declaringClass;
      ReferenceBinding[] itsInterfaces = superType.superInterfaces();
      if (itsInterfaces != Binding.NO_SUPERINTERFACES) {
        nextPosition = itsInterfaces.length;
        interfacesToVisit = itsInterfaces;
      }
      superType = superType.superclass();
      while ((superType != null) && (superType.isValidBinding())) {
        MethodBinding[] methods = superType.getMethods(currentMethod.selector);
        int m = 0; for (int n = methods.length; m < n; m++) {
          MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
          if ((substitute != null) && (!doesSubstituteMethodOverride(currentMethod, substitute)) && (detectNameClash(currentMethod, substitute)))
            return;
        }
        if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES)
          if (interfacesToVisit == null) {
            interfacesToVisit = itsInterfaces;
            nextPosition = interfacesToVisit.length;
          } else {
            int itsLength = itsInterfaces.length;
            if (nextPosition + itsLength >= interfacesToVisit.length)
              System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
            for (int a = 0; a < itsLength; a++) {
              ReferenceBinding next = itsInterfaces[a];
              int b = 0;
              while (next != interfacesToVisit[b])
              {
                b++; if (b >= nextPosition)
                {
                  interfacesToVisit[(nextPosition++)] = next;
                }
              }
            }
          }
        superType = superType.superclass();
      }

      for (int i = 0; i < nextPosition; i++) {
        superType = interfacesToVisit[i];
        if (superType.isValidBinding()) {
          MethodBinding[] methods = superType.getMethods(currentMethod.selector);
          int m = 0; for (int n = methods.length; m < n; m++) {
            MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
            if ((substitute != null) && (!doesSubstituteMethodOverride(currentMethod, substitute)) && (detectNameClash(currentMethod, substitute)))
              return;
          }
          if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
            int itsLength = itsInterfaces.length;
            if (nextPosition + itsLength >= interfacesToVisit.length)
              System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
            for (int a = 0; a < itsLength; a++) {
              ReferenceBinding next = itsInterfaces[a];
              int b = 0;
              while (next != interfacesToVisit[b])
              {
                b++; if (b >= nextPosition)
                {
                  interfacesToVisit[(nextPosition++)] = next; } 
              }
            }
          }
        }
      }
    }
  }

  void checkInheritedMethods(MethodBinding inheritedMethod, MethodBinding otherInheritedMethod) { if (inheritedMethod.declaringClass.erasure() == otherInheritedMethod.declaringClass.erasure()) {
      if (inheritedMethod.areParameterErasuresEqual(otherInheritedMethod)) {
        problemReporter().duplicateInheritedMethods(this.type, inheritedMethod, otherInheritedMethod);
      }
    }
    else if (inheritedMethod.declaringClass.findSuperTypeWithSameErasure(otherInheritedMethod.declaringClass) != null)
    {
      return;
    }

    if ((inheritedMethod.declaringClass.isInterface()) || (inheritedMethod.isStatic())) return;

    detectInheritedNameClash(inheritedMethod.original(), otherInheritedMethod.original()); }

  void checkInheritedMethods(MethodBinding[] methods, int length) {
    int count = length;
    int[] skip = new int[count];
    int i = 0; for (int l = length - 1; i < l; i++) {
      if (skip[i] != -1) {
        MethodBinding method = methods[i];
        MethodBinding[] duplicates = (MethodBinding[])null;
        for (int j = i + 1; j <= l; j++) {
          MethodBinding method2 = methods[j];
          if ((method.declaringClass == method2.declaringClass) && (areMethodsCompatible(method, method2))) {
            skip[j] = -1;
            if (duplicates == null)
              duplicates = new MethodBinding[length];
            duplicates[j] = method2;
          }
        }
        if (duplicates != null)
        {
          int concreteCount = method.isAbstract() ? 0 : 1;
          MethodBinding methodToKeep = method;
          int m = 0; for (int s = duplicates.length; m < s; m++) {
            if ((duplicates[m] != null) && 
              (!duplicates[m].isAbstract())) {
              methodToKeep = duplicates[m];
              concreteCount++;
            }
          }

          if (concreteCount != 1) {
            int m = 0; for (int s = duplicates.length; m < s; m++)
              if (duplicates[m] != null) {
                problemReporter().duplicateInheritedMethods(this.type, method, duplicates[m]);
                count--;
                if (methodToKeep == duplicates[m])
                  methods[i] = null;
                else
                  methods[m] = null;
              }
          }
        }
      }
    }
    if (count < length) {
      if (count == 1) return;
      MethodBinding[] newMethods = new MethodBinding[count];
      int i = length;
      do { if (methods[i] != null)
          newMethods[(--count)] = methods[i];
        i--; } while (i >= 0);

      methods = newMethods;
      length = newMethods.length;
    }

    super.checkInheritedMethods(methods, length);
  }

  boolean checkInheritedReturnTypes(MethodBinding[] methods, int length)
  {
    int[] areIncompatible = (int[])null;

    int i = 0;
    int j;
    for (int l = this.type.isAbstract() ? length - 2 : 0; i <= l; 
      j < length)
    {
      MethodBinding method = methods[(i++)];
      j = i; continue;
      if (!areReturnTypesCompatible(method, methods[j])) {
        if (this.type.isInterface()) {
          int m = length;
          do { if (methods[m].declaringClass.id == 1)
              break;
            m--; } while (m >= 0);
        }

        if (isUnsafeReturnTypeOverride(method, methods[j])) {
          problemReporter(method).unsafeReturnTypeOverride(method, methods[j], this.type);
        }
        else {
          if (areIncompatible == null)
            areIncompatible = new int[length];
          areIncompatible[(i - 1)] = -1;
          areIncompatible[j] = -1;
        }
      }
      j++;
    }

    if (areIncompatible == null) {
      return true;
    }
    int count = 0;
    for (int i = 0; i < length; i++)
      if (areIncompatible[i] == -1) count++;
    if (count == length) {
      problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, methods, length);
      return false;
    }
    MethodBinding[] methodsToReport = new MethodBinding[count];
    int i = 0; for (int index = 0; i < length; i++)
      if (areIncompatible[i] == -1)
        methodsToReport[(index++)] = methods[i];
    problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, methodsToReport, count);
    return false;
  }
  void checkMethods() {
    boolean mustImplementAbstractMethods = mustImplementAbstractMethods();
    boolean skipInheritedMethods = (mustImplementAbstractMethods) && (canSkipInheritedMethods());
    char[][] methodSelectors = this.inheritedMethods.keyTable;
    int s = methodSelectors.length;
    do { if (methodSelectors[s] != null)
      {
        MethodBinding[] current = (MethodBinding[])this.currentMethods.get(methodSelectors[s]);
        if ((current != null) || (!skipInheritedMethods))
        {
          MethodBinding[] inherited = (MethodBinding[])this.inheritedMethods.valueTable[s];
          if ((inherited.length == 1) && (current == null)) {
            if ((mustImplementAbstractMethods) && (inherited[0].isAbstract()))
              checkAbstractMethod(inherited[0]);
          }
          else
          {
            int index = -1;
            MethodBinding[] matchingInherited = new MethodBinding[inherited.length];
            byte[] foundMatch = new byte[inherited.length];
            if (current != null) {
              int i = 0; for (int length1 = current.length; i < length1; i++) {
                MethodBinding currentMethod = current[i];
                int j = 0; for (int length2 = inherited.length; j < length2; j++) {
                  MethodBinding inheritedMethod = computeSubstituteMethod(inherited[j], currentMethod);
                  if (inheritedMethod != null) {
                    if ((foundMatch[j] == 0) && (doesSubstituteMethodOverride(currentMethod, inheritedMethod))) {
                      matchingInherited[(++index)] = inheritedMethod;
                      foundMatch[j] = 1;
                    } else {
                      checkForNameClash(currentMethod, inheritedMethod);
                    }
                  }
                }
                if (index >= 0)
                {
                  checkAgainstInheritedMethods(currentMethod, matchingInherited, index + 1, inherited);
                  while (index >= 0) matchingInherited[(index--)] = null;
                }
              }
            }

            int i = 0; for (int length = inherited.length; i < length; i++)
              if (foundMatch[i] != 1)
              {
                MethodBinding inheritedMethod = inherited[i];
                matchingInherited[(++index)] = inheritedMethod;
                for (int j = i + 1; j < length; j++) {
                  MethodBinding otherInheritedMethod = inherited[j];
                  if ((foundMatch[j] != 1) && (!canSkipInheritedMethods(inheritedMethod, otherInheritedMethod)))
                  {
                    otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
                    if (otherInheritedMethod != null)
                      if ((inheritedMethod.declaringClass != otherInheritedMethod.declaringClass) && 
                        (doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod))) {
                        matchingInherited[(++index)] = otherInheritedMethod;
                        foundMatch[j] = 1;
                      } else {
                        checkInheritedMethods(inheritedMethod, otherInheritedMethod);
                      }
                  }
                }
                if (index != -1)
                {
                  if (index > 0)
                    checkInheritedMethods(matchingInherited, index + 1);
                  else if ((mustImplementAbstractMethods) && (index == 0) && (matchingInherited[0].isAbstract()))
                    checkAbstractMethod(matchingInherited[0]);
                  while (index >= 0) matchingInherited[(index--)] = null;
                }
              }
          }
        }
      }
      s--; } while (s >= 0);
  }

  void checkTypeVariableMethods(TypeParameter typeParameter)
  {
    char[][] methodSelectors = this.inheritedMethods.keyTable;
    int s = methodSelectors.length;
    do { if (methodSelectors[s] != null) {
        MethodBinding[] inherited = (MethodBinding[])this.inheritedMethods.valueTable[s];
        if (inherited.length != 1)
        {
          int index = -1;
          MethodBinding[] matchingInherited = new MethodBinding[inherited.length];
          int i = 0; for (int length = inherited.length; i < length; i++) {
            while (index >= 0) matchingInherited[(index--)] = null;
            MethodBinding inheritedMethod = inherited[i];
            if (inheritedMethod != null) {
              matchingInherited[(++index)] = inheritedMethod;
              for (int j = i + 1; j < length; j++) {
                MethodBinding otherInheritedMethod = inherited[j];
                if (!canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
                {
                  otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
                  if ((otherInheritedMethod != null) && (doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod))) {
                    matchingInherited[(++index)] = otherInheritedMethod;
                    inherited[j] = null;
                  }
                }
              }
            }
            if (index > 0) {
              MethodBinding first = matchingInherited[0];
              int count = index + 1;
              do count--; while ((count > 0) && (areReturnTypesCompatible(first, matchingInherited[count])));
              if (count > 0) {
                problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(typeParameter, matchingInherited, index + 1);
                break;
              }
            }
          }
        }
      }
      s--; } while (s >= 0);
  }

  MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBinding currentMethod)
  {
    if (inheritedMethod == null) return null;
    if (currentMethod.parameters.length != inheritedMethod.parameters.length) return null;

    if ((currentMethod.declaringClass instanceof BinaryTypeBinding))
      ((BinaryTypeBinding)currentMethod.declaringClass).resolveTypesFor(currentMethod);
    if ((inheritedMethod.declaringClass instanceof BinaryTypeBinding)) {
      ((BinaryTypeBinding)inheritedMethod.declaringClass).resolveTypesFor(inheritedMethod);
    }
    TypeVariableBinding[] inheritedTypeVariables = inheritedMethod.typeVariables;
    if (inheritedTypeVariables == Binding.NO_TYPE_VARIABLES) return inheritedMethod;
    int inheritedLength = inheritedTypeVariables.length;
    TypeVariableBinding[] typeVariables = currentMethod.typeVariables;
    int length = typeVariables.length;
    if ((length > 0) && (inheritedLength != length)) return inheritedMethod;
    TypeBinding[] arguments = new TypeBinding[inheritedLength];
    if (inheritedLength <= length) {
      System.arraycopy(typeVariables, 0, arguments, 0, inheritedLength);
    } else {
      System.arraycopy(typeVariables, 0, arguments, 0, length);
      for (int i = length; i < inheritedLength; i++)
        arguments[i] = inheritedTypeVariables[i].upperBound();
    }
    ParameterizedGenericMethodBinding substitute = 
      this.environment.createParameterizedGenericMethod(inheritedMethod, arguments);

    for (int i = 0; i < inheritedLength; i++) {
      TypeVariableBinding inheritedTypeVariable = inheritedTypeVariables[i];
      TypeBinding argument = arguments[i];
      if ((argument instanceof TypeVariableBinding)) {
        TypeVariableBinding typeVariable = (TypeVariableBinding)argument;
        if (typeVariable.firstBound == inheritedTypeVariable.firstBound) {
          if (typeVariable.firstBound == null)
            continue;
        } else if ((typeVariable.firstBound != null) && (inheritedTypeVariable.firstBound != null) && 
          (typeVariable.firstBound.isClass() != inheritedTypeVariable.firstBound.isClass())) {
          return inheritedMethod;
        }
        if (Scope.substitute(substitute, inheritedTypeVariable.superclass) != typeVariable.superclass)
          return inheritedMethod;
        int interfaceLength = inheritedTypeVariable.superInterfaces.length;
        ReferenceBinding[] interfaces = typeVariable.superInterfaces;
        if (interfaceLength != interfaces.length) {
          return inheritedMethod;
        }
        for (int j = 0; j < interfaceLength; j++) {
          TypeBinding superType = Scope.substitute(substitute, inheritedTypeVariable.superInterfaces[j]);
          int k = 0;
          while (superType != interfaces[k])
          {
            k++; if (k >= interfaceLength)
            {
              return inheritedMethod; } 
          }
        } } else if (inheritedTypeVariable.boundCheck(substitute, argument) != 0) {
        return inheritedMethod;
      }
    }
    return substitute;
  }
  boolean detectInheritedNameClash(MethodBinding inherited, MethodBinding otherInherited) {
    if ((!inherited.areParameterErasuresEqual(otherInherited)) || (inherited.returnType.erasure() != otherInherited.returnType.erasure())) return false;

    problemReporter().inheritedMethodsHaveNameClash(this.type, inherited, otherInherited);
    return true;
  }
  boolean detectNameClash(MethodBinding current, MethodBinding inherited) {
    MethodBinding original = inherited.original();
    if ((!current.areParameterErasuresEqual(original)) || (current.returnType.erasure() != original.returnType.erasure())) return false;

    problemReporter(current).methodNameClash(current, 
      inherited.declaringClass.isRawType() ? inherited : original);
    return true;
  }
  public boolean doesMethodOverride(MethodBinding method, MethodBinding inheritedMethod) {
    MethodBinding substitute = computeSubstituteMethod(inheritedMethod, method);
    return (substitute != null) && (doesSubstituteMethodOverride(method, substitute));
  }

  boolean doesSubstituteMethodOverride(MethodBinding method, MethodBinding substituteMethod)
  {
    if (!areParametersEqual(method, substituteMethod))
    {
      if ((substituteMethod.hasSubstitutedParameters()) && (method.areParameterErasuresEqual(substituteMethod)))
        return (method.typeVariables == Binding.NO_TYPE_VARIABLES) && (!hasGenericParameter(method));
      return false;
    }

    if ((substituteMethod instanceof ParameterizedGenericMethodBinding))
    {
      return (!hasGenericParameter(method)) || (method.typeVariables != Binding.NO_TYPE_VARIABLES);
    }

    return method.typeVariables == Binding.NO_TYPE_VARIABLES;
  }
  boolean hasGenericParameter(MethodBinding method) {
    if (method.genericSignature() == null) return false;

    TypeBinding[] params = method.parameters;
    int i = 0; for (int l = params.length; i < l; i++) {
      TypeBinding param = params[i].leafComponentType();
      if ((param instanceof ReferenceBinding)) {
        int modifiers = ((ReferenceBinding)param).modifiers;
        if ((modifiers & 0x40000000) != 0)
          return true;
      }
    }
    return false;
  }

  boolean doTypeVariablesClash(MethodBinding one, MethodBinding substituteTwo) {
    return (one.typeVariables != Binding.NO_TYPE_VARIABLES) && (!(substituteTwo instanceof ParameterizedGenericMethodBinding));
  }
  SimpleSet findSuperinterfaceCollisions(ReferenceBinding superclass, ReferenceBinding[] superInterfaces) {
    ReferenceBinding[] interfacesToVisit = (ReferenceBinding[])null;
    int nextPosition = 0;
    ReferenceBinding[] itsInterfaces = superInterfaces;
    if (itsInterfaces != Binding.NO_SUPERINTERFACES) {
      nextPosition = itsInterfaces.length;
      interfacesToVisit = itsInterfaces;
    }

    boolean isInconsistent = this.type.isHierarchyInconsistent();
    ReferenceBinding superType = superclass;
    while ((superType != null) && (superType.isValidBinding())) {
      isInconsistent |= superType.isHierarchyInconsistent();
      if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES)
        if (interfacesToVisit == null) {
          interfacesToVisit = itsInterfaces;
          nextPosition = interfacesToVisit.length;
        } else {
          int itsLength = itsInterfaces.length;
          if (nextPosition + itsLength >= interfacesToVisit.length)
            System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
          for (int a = 0; a < itsLength; a++) {
            ReferenceBinding next = itsInterfaces[a];
            int b = 0;
            while (next != interfacesToVisit[b])
            {
              b++; if (b >= nextPosition)
              {
                interfacesToVisit[(nextPosition++)] = next;
              }
            }
          }
        }
      superType = superType.superclass();
    }

    for (int i = 0; i < nextPosition; i++) {
      superType = interfacesToVisit[i];
      if (superType.isValidBinding()) {
        isInconsistent |= superType.isHierarchyInconsistent();
        if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
          int itsLength = itsInterfaces.length;
          if (nextPosition + itsLength >= interfacesToVisit.length)
            System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
          for (int a = 0; a < itsLength; a++) {
            ReferenceBinding next = itsInterfaces[a];
            int b = 0;
            while (next != interfacesToVisit[b])
            {
              b++; if (b >= nextPosition)
              {
                interfacesToVisit[(nextPosition++)] = next;
              }
            }
          }
        }
      }
    }
    if (!isInconsistent) return null;
    SimpleSet copy = null;
    for (int i = 0; i < nextPosition; i++) {
      ReferenceBinding current = interfacesToVisit[i];
      if (current.isValidBinding()) {
        TypeBinding erasure = current.erasure();
        for (int j = i + 1; j < nextPosition; j++) {
          ReferenceBinding next = interfacesToVisit[j];
          if ((next.isValidBinding()) && (next.erasure() == erasure)) {
            if (copy == null)
              copy = new SimpleSet(nextPosition);
            copy.add(interfacesToVisit[i]);
            copy.add(interfacesToVisit[j]);
          }
        }
      }
    }
    return copy;
  }

  boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBinding existingMethod, ReferenceBinding superType) {
    if ((inheritedMethod.original() != inheritedMethod) && (existingMethod.declaringClass.isInterface())) {
      return false;
    }
    inheritedMethod = computeSubstituteMethod(inheritedMethod, existingMethod);

    return (inheritedMethod != null) && 
      (inheritedMethod.returnType == existingMethod.returnType) && 
      (super.isInterfaceMethodImplemented(inheritedMethod, existingMethod, superType));
  }

  boolean isUnsafeReturnTypeOverride(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    if (currentMethod.returnType == inheritedMethod.returnType.erasure()) {
      TypeBinding[] currentParams = currentMethod.parameters;
      TypeBinding[] inheritedParams = inheritedMethod.parameters;
      int i = 0; for (int l = currentParams.length; i < l; i++)
        if (!areTypesEqual(currentParams[i], inheritedParams[i]))
          return true;
    }
    if ((currentMethod.typeVariables == Binding.NO_TYPE_VARIABLES) && 
      (inheritedMethod.original().typeVariables != Binding.NO_TYPE_VARIABLES) && 
      (currentMethod.returnType.erasure().findSuperTypeWithSameErasure(inheritedMethod.returnType.erasure()) != null)) {
      return true;
    }
    return false;
  }
  boolean reportIncompatibleReturnTypeError(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    if (isUnsafeReturnTypeOverride(currentMethod, inheritedMethod)) {
      problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, inheritedMethod, this.type);
      return false;
    }
    return super.reportIncompatibleReturnTypeError(currentMethod, inheritedMethod);
  }
  void verify(SourceTypeBinding someType) {
    if (someType.isAnnotationType()) {
      someType.detectAnnotationCycle();
    }
    super.verify(someType);

    int i = someType.typeVariables.length;
    do { TypeVariableBinding var = someType.typeVariables[i];

      if ((var.superInterfaces != Binding.NO_SUPERINTERFACES) && (
        (var.superInterfaces.length != 1) || (var.superclass.id != 1)))
      {
        this.currentMethods = new HashtableOfObject(0);
        ReferenceBinding superclass = var.superclass();
        if (superclass.kind() == 4100)
          superclass = (ReferenceBinding)superclass.erasure();
        ReferenceBinding[] itsInterfaces = var.superInterfaces();
        ReferenceBinding[] superInterfaces = new ReferenceBinding[itsInterfaces.length];
        int j = itsInterfaces.length;
        do { superInterfaces[j] = (itsInterfaces[j].kind() == 4100 ? 
            (ReferenceBinding)itsInterfaces[j].erasure() : 
            itsInterfaces[j]);

          j--; } while (j >= 0);

        computeInheritedMethods(superclass, superInterfaces);
        checkTypeVariableMethods(someType.scope.referenceContext.typeParameters[i]);
      }
      i--; } while (i >= 0);
  }
}