package org.jvnet.jaxbw.intellij;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public final class FileInfo
{
  private final String fileName;
  private final Map<Integer, LineInfo> messages;

  public FileInfo(String fileName)
  {
    this.fileName = fileName;
    this.messages = new HashMap();
  }

  public final String getFileName()
  {
    return this.fileName;
  }

  public final synchronized void addError(int line, String message)
  {
    if (!this.messages.containsKey(Integer.valueOf(line))) {
      LineInfo newInfo = new LineInfo(line);
      this.messages.put(Integer.valueOf(line), newInfo);
    }
    LineInfo ei = (LineInfo)this.messages.get(Integer.valueOf(line));
    ei.addMessage(MessageKind.ERROR, message);
  }

  public final synchronized void addWarning(int line, String message)
  {
    if (!this.messages.containsKey(Integer.valueOf(line))) {
      LineInfo newInfo = new LineInfo(line);
      this.messages.put(Integer.valueOf(line), newInfo);
    }
    LineInfo ei = (LineInfo)this.messages.get(Integer.valueOf(line));
    ei.addMessage(MessageKind.WARNING, message);
  }

  public final synchronized MessageInfo addMessage(int line, MessageKind kind, String message)
  {
    if (!this.messages.containsKey(Integer.valueOf(line))) {
      LineInfo newInfo = new LineInfo(line);
      this.messages.put(Integer.valueOf(line), newInfo);
    }
    LineInfo ei = (LineInfo)this.messages.get(Integer.valueOf(line));
    return ei.addMessage(kind, message);
  }

  public final synchronized Collection<LineInfo> getMessages()
  {
    return this.messages.values();
  }

  public final synchronized LineInfo getMessages(int lineNumber) {
    return (LineInfo)this.messages.get(Integer.valueOf(lineNumber));
  }

  public final void remove(MessageInfo mInfo)
  {
    int lineNumber = mInfo.getLineNumber();
    LineInfo lInfo = (LineInfo)this.messages.get(Integer.valueOf(lineNumber));
    if (lInfo != null) {
      lInfo.messages.remove(mInfo);
      if (lInfo.messages.size() == 0)
        this.messages.remove(Integer.valueOf(lineNumber));
    }
  }

  public void clearMessages(MessageKind kind) {
    if (kind == null) {
      this.messages.clear();
    } else {
      for (LineInfo lInfo : this.messages.values()) {
        Iterator it = lInfo.messages.iterator();
        while (it.hasNext()) {
          MessageInfo mInfo = (MessageInfo)it.next();
          if (mInfo.kind == kind)
            it.remove();
        }
      }
      Iterator it = this.messages.entrySet().iterator();
      while (it.hasNext()) {
        Map.Entry eInfo = (Map.Entry)it.next();
        if (((LineInfo)eInfo.getValue()).messages.size() == 0)
          it.remove();
      }
    }
  }

  public static final class LineInfo
  {
    private final int lineNumber;
    private final List<FileInfo.MessageInfo> messages;

    public LineInfo(int lineNumber)
    {
      this.lineNumber = lineNumber;
      this.messages = new LinkedList();
    }

    public final List<FileInfo.MessageInfo> getMessages()
    {
      return this.messages;
    }

    public final int getLineNumber()
    {
      return this.lineNumber;
    }

    public final FileInfo.MessageInfo addMessage(FileInfo.MessageKind kind, String text)
    {
      FileInfo.MessageInfo newMessage = new FileInfo.MessageInfo(this.lineNumber, kind, text);

      this.messages.add(newMessage);
      return newMessage;
    }
  }

  public static final class MessageInfo
  {
    private int lineNumber;
    private FileInfo.MessageKind kind;
    private String text;

    public MessageInfo(int lineNumber, FileInfo.MessageKind kind, String text)
    {
      this.lineNumber = lineNumber;
      this.kind = kind;
      this.text = text;
    }

    public int getLineNumber()
    {
      return this.lineNumber;
    }

    public FileInfo.MessageKind getKind()
    {
      return this.kind;
    }

    public String getText()
    {
      return this.text;
    }
  }

  public static enum MessageKind
  {
    ERROR(0), 

    WARNING(1), 

    INFO(2), 

    SUCCESS(3);

    private int severity;

    private MessageKind(int severity)
    {
      this.severity = severity;
    }
  }
}