package org.jvnet.jaxbw.intellij;

import com.intellij.openapi.progress.ProgressIndicator;
import com.sun.tools.xjc.XJCListener;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import org.xml.sax.SAXParseException;

public class XjcHarvesterListener extends XJCListener
{
  private ProgressIndicator monitor;
  private Map<String, FileInfo> messages;
  private List<String> generatedFileNames;
  private final Logger _logger = Logger.getLogger(XjcHarvesterListener.class.getName());

  public XjcHarvesterListener(ProgressIndicator monitor)
  {
    this.monitor = monitor;
    this.messages = new HashMap();
    this.generatedFileNames = new LinkedList();
  }

  private void addMessage(String filename, int lineNumber, FileInfo.MessageKind kind, String text)
  {
    if (filename == null)
      filename = "null";
    filename = FilePathInfo.normalize(filename);
    if (!this.messages.containsKey(filename)) {
      FileInfo fInfo = new FileInfo(filename);
      this.messages.put(filename, fInfo);
    }
    FileInfo fInfo = (FileInfo)this.messages.get(filename);
    fInfo.addMessage(lineNumber - 1, kind, text);
  }

  public void message(String message)
  {
    this.monitor.setText2(message);
  }

  public void generatedFile(String s)
  {
    this._logger.info("/xjc/file/ " + s);
    if (s.endsWith("package-info.java"))
      return;
    this.generatedFileNames.add(s);
    this.monitor.setText2("Generated file '" + s + "'");
  }

  public void error(SAXParseException spe)
  {
    this._logger.info("/xjc/ Error [" + spe.getSystemId() + ":" + spe.getLineNumber() + "] " + spe.getMessage());

    addMessage(spe.getSystemId(), spe.getLineNumber(), FileInfo.MessageKind.ERROR, spe.getMessage());
  }

  public void fatalError(SAXParseException spe)
  {
    this._logger.info("/xjc/ Fatal error [" + spe.getSystemId() + ":" + spe.getLineNumber() + "] " + spe.getMessage());

    addMessage(spe.getSystemId(), spe.getLineNumber(), FileInfo.MessageKind.ERROR, spe.getMessage());
  }

  public void info(SAXParseException spe)
  {
    this._logger.info("/xjc/ Info [" + spe.getSystemId() + ":" + spe.getLineNumber() + "] " + spe.getMessage());
  }

  public void warning(SAXParseException spe)
  {
    this._logger.info("/xjc/ Warning [" + spe.getSystemId() + ":" + spe.getLineNumber() + "] " + spe.getMessage());

    addMessage(spe.getSystemId(), spe.getLineNumber(), FileInfo.MessageKind.WARNING, spe.getMessage());
  }

  public Map<String, FileInfo> getMessages()
  {
    return this.messages;
  }

  public List<String> getGeneratedFileNames()
  {
    return this.generatedFileNames;
  }
}