package org.jvnet.jaxbw.intellij.wizard;

import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import org.jvnet.flamingo.wizard.WizardSettings;
import org.jvnet.flamingo.wizard.WizardStep;

public final class OutputDirectoryStep
  implements WizardStep
{
  private final XjcWizard owner;
  private String directoryName;
  private File directoryFile;
  private final JComponent component;
  private final JTextField directoryTf;
  private final JButton selectButton;
  private final JButton removeButton;

  public OutputDirectoryStep(XjcWizard owner)
  {
    this.owner = owner;

    JPanel panel = new JPanel();
    panel.setLayout(new FlowLayout(1));

    this.directoryTf = new JTextField();
    this.directoryTf.setColumns(30);
    this.directoryTf.setEditable(false);

    this.selectButton = new JButton("Select");
    this.selectButton.setMargin(new Insets(0, 0, 0, 0));
    this.selectButton.addActionListener(new SelectActionListener(this));

    this.removeButton = new JButton("Remove");
    this.removeButton.setMargin(new Insets(0, 0, 0, 0));
    this.removeButton.addActionListener(new RemoveActionListener(this));
    this.removeButton.setEnabled(false);
    panel.add(this.directoryTf);
    panel.add(this.selectButton);
    panel.add(this.removeButton);
    panel.setOpaque(false);

    this.component = panel;
  }

  private void removeDirectoryName()
  {
    this.directoryFile = null;
    this.directoryName = null;
    this.directoryTf.setText("");
    this.removeButton.setEnabled(false);

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        OutputDirectoryStep.this.owner.update();
      }
    });
  }

  private void setDirectoryName(File file)
  {
    this.directoryFile = file;
    this.directoryName = this.directoryFile.getAbsolutePath();
    this.directoryTf.setText(this.directoryFile.getAbsolutePath());
    this.removeButton.setEnabled(true);

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        OutputDirectoryStep.this.owner.update();
      }
    });
  }

  public final boolean canProceed()
  {
    return (this.directoryName != null) && (this.directoryName.length() > 0);
  }

  public void exportSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    settings.setRootOutputDirectory(this.directoryName);
  }

  public void importSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    this.directoryName = settings.getRootOutputDirectory();
    this.directoryTf.setText(this.directoryName);
  }

  public final JComponent getComponent()
  {
    return this.component;
  }

  public final String getTitle()
  {
    return "Choose root output directory";
  }

  public final boolean isSupportsDirectory()
  {
    return true;
  }

  public final String getCurrentDirectory()
    throws UnsupportedOperationException
  {
    return this.directoryName;
  }

  public final void setCurrentDirectory(String str)
    throws UnsupportedOperationException
  {
    if (str == null) {
      return;
    }

    if (this.directoryName == null)
      setDirectoryName(new File(str));
  }

  public final String getDescription()
  {
    return "Specify the root directory for the generated classes. The generated classes will be put in this directory under the package that you have chosen";
  }

  public Icon getStepIcon(int width, int height)
  {
    return ImageCreator.getWizardImage(width, height);
  }

  private static final class RemoveActionListener
    implements ActionListener
  {
    private final OutputDirectoryStep step;

    public RemoveActionListener(OutputDirectoryStep step)
    {
      this.step = step;
    }

    public final void actionPerformed(ActionEvent e) {
      this.step.removeDirectoryName();
    }
  }

  private static final class SelectActionListener
    implements ActionListener
  {
    private final OutputDirectoryStep step;

    public SelectActionListener(OutputDirectoryStep step)
    {
      this.step = step;
    }

    public final void actionPerformed(ActionEvent e)
    {
      JFileChooser chooser = new JFileChooser(this.step.directoryFile);
      chooser.setFileSelectionMode(1);

      int returnVal = chooser.showOpenDialog(this.step.owner);
      if (returnVal == 0)
        this.step.setDirectoryName(chooser.getSelectedFile());
    }
  }
}