package org.quartz.ee.jta;

import javax.transaction.SystemException;
import javax.transaction.UserTransaction;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.core.JobRunShell;
import org.quartz.spi.TriggerFiredBundle;
import org.slf4j.Logger;

public class JTAJobRunShell extends JobRunShell
{
  private UserTransaction ut;

  public JTAJobRunShell(Scheduler scheduler, TriggerFiredBundle bndle)
  {
    super(scheduler, bndle);
  }

  protected void begin()
    throws SchedulerException
  {
    cleanupUserTransaction();

    boolean beganSuccessfully = false;
    try {
      getLog().debug("Looking up UserTransaction.");
      this.ut = UserTransactionHelper.lookupUserTransaction();

      getLog().debug("Beginning UserTransaction.");
      this.ut.begin();

      beganSuccessfully = true;
    } catch (SchedulerException se) {
      throw se;
    }
    catch (Exception nse) {
      throw new SchedulerException("JTAJobRunShell could not start UserTransaction.", nse);
    }
    finally {
      if (!beganSuccessfully)
        cleanupUserTransaction();
    }
  }

  protected void complete(boolean successfulExecution)
    throws SchedulerException
  {
    if (this.ut == null)
      return;
    try
    {
      try
      {
        if (this.ut.getStatus() == 1) {
          getLog().debug("UserTransaction marked for rollback only.");
          successfulExecution = false;
        }
      } catch (SystemException e) {
        throw new SchedulerException("JTAJobRunShell could not read UserTransaction status.", e);
      }

      if (successfulExecution)
        try {
          getLog().debug("Committing UserTransaction.");
          this.ut.commit();
        } catch (Exception nse) {
          throw new SchedulerException("JTAJobRunShell could not commit UserTransaction.", nse);
        }
      else {
        try
        {
          getLog().debug("Rolling-back UserTransaction.");
          this.ut.rollback();
        } catch (Exception nse) {
          throw new SchedulerException("JTAJobRunShell could not rollback UserTransaction.", nse);
        }
      }
    }
    finally
    {
      cleanupUserTransaction();
    }
  }

  public void passivate()
  {
    cleanupUserTransaction();
    super.passivate();
  }

  private void cleanupUserTransaction() {
    if (this.ut != null) {
      UserTransactionHelper.returnUserTransaction(this.ut);
      this.ut = null;
    }
  }
}