package org.quartz.impl.calendar;

import java.io.Serializable;
import java.util.Date;
import java.util.TimeZone;

public class BaseCalendar
  implements org.quartz.Calendar, Serializable, Cloneable
{
  static final long serialVersionUID = 3106623404629760239L;
  private org.quartz.Calendar baseCalendar;
  private String description;
  private TimeZone timeZone;

  public BaseCalendar()
  {
  }

  public BaseCalendar(org.quartz.Calendar baseCalendar)
  {
    setBaseCalendar(baseCalendar);
  }

  public BaseCalendar(TimeZone timeZone)
  {
    setTimeZone(timeZone);
  }

  public BaseCalendar(org.quartz.Calendar baseCalendar, TimeZone timeZone)
  {
    setBaseCalendar(baseCalendar);
    setTimeZone(timeZone);
  }

  public Object clone()
  {
    try {
      BaseCalendar clone = (BaseCalendar)super.clone();
      if (getBaseCalendar() != null) {
        clone.baseCalendar = ((Cloneable)getBaseCalendar().clone());
      }
      if (getTimeZone() != null)
        clone.timeZone = ((TimeZone)getTimeZone().clone());
      return clone; } catch (CloneNotSupportedException ex) {
    }
    throw new IncompatibleClassChangeError("Not Cloneable.");
  }

  public void setBaseCalendar(org.quartz.Calendar baseCalendar)
  {
    this.baseCalendar = baseCalendar;
  }

  public org.quartz.Calendar getBaseCalendar()
  {
    return this.baseCalendar;
  }

  public String getDescription()
  {
    return this.description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public TimeZone getTimeZone()
  {
    return this.timeZone;
  }

  public void setTimeZone(TimeZone timeZone)
  {
    this.timeZone = timeZone;
  }

  public boolean isTimeIncluded(long timeStamp)
  {
    if (timeStamp <= 0L) {
      throw new IllegalArgumentException("timeStamp must be greater 0");
    }

    if ((this.baseCalendar != null) && 
      (!this.baseCalendar.isTimeIncluded(timeStamp))) return false;

    return true;
  }

  public long getNextIncludedTime(long timeStamp)
  {
    if (timeStamp <= 0L) {
      throw new IllegalArgumentException("timeStamp must be greater 0");
    }

    if (this.baseCalendar != null) {
      return this.baseCalendar.getNextIncludedTime(timeStamp);
    }

    return timeStamp;
  }

  protected java.util.Calendar createJavaCalendar(long timeStamp)
  {
    java.util.Calendar calendar = createJavaCalendar();
    calendar.setTime(new Date(timeStamp));
    return calendar;
  }

  protected java.util.Calendar createJavaCalendar()
  {
    return getTimeZone() == null ? java.util.Calendar.getInstance() : java.util.Calendar.getInstance(getTimeZone());
  }

  protected java.util.Calendar getStartOfDayJavaCalendar(long timeInMillis)
  {
    java.util.Calendar startOfDay = createJavaCalendar(timeInMillis);
    startOfDay.set(11, 0);
    startOfDay.set(12, 0);
    startOfDay.set(13, 0);
    startOfDay.set(14, 0);
    return startOfDay;
  }

  protected java.util.Calendar getEndOfDayJavaCalendar(long timeInMillis)
  {
    java.util.Calendar endOfDay = createJavaCalendar(timeInMillis);
    endOfDay.set(11, 23);
    endOfDay.set(12, 59);
    endOfDay.set(13, 59);
    endOfDay.set(14, 999);
    return endOfDay;
  }
}