package org.quartz.impl.calendar;

import java.io.Serializable;
import java.util.Date;
import java.util.TimeZone;

public class MonthlyCalendar extends BaseCalendar
  implements org.quartz.Calendar, Serializable
{
  static final long serialVersionUID = 419164961091807944L;
  private static final int MAX_DAYS_IN_MONTH = 31;
  private boolean[] excludeDays = new boolean[31];

  private boolean excludeAll = false;

  public MonthlyCalendar() {
    this(null, null);
  }

  public MonthlyCalendar(org.quartz.Calendar baseCalendar) {
    this(baseCalendar, null);
  }

  public MonthlyCalendar(TimeZone timeZone) {
    this(null, timeZone);
  }

  public MonthlyCalendar(org.quartz.Calendar baseCalendar, TimeZone timeZone) {
    super(baseCalendar, timeZone);

    this.excludeAll = areAllDaysExcluded();
  }

  public Object clone()
  {
    MonthlyCalendar clone = (MonthlyCalendar)super.clone();
    clone.excludeDays = ((boolean[])this.excludeDays.clone());
    return clone;
  }

  public boolean[] getDaysExcluded()
  {
    return this.excludeDays;
  }

  public boolean isDayExcluded(int day)
  {
    if ((day < 1) || (day > 31)) {
      throw new IllegalArgumentException("The day parameter must be in the range of 1 to 31");
    }

    return this.excludeDays[(day - 1)];
  }

  public void setDaysExcluded(boolean[] days)
  {
    if (days == null) {
      throw new IllegalArgumentException("The days parameter cannot be null.");
    }

    if (days.length < 31) {
      throw new IllegalArgumentException("The days parameter must have a length of at least 31 elements.");
    }

    this.excludeDays = days;
    this.excludeAll = areAllDaysExcluded();
  }

  public void setDayExcluded(int day, boolean exclude)
  {
    if ((day < 1) || (day > 31)) {
      throw new IllegalArgumentException("The day parameter must be in the range of 1 to 31");
    }

    this.excludeDays[(day - 1)] = exclude;
    this.excludeAll = areAllDaysExcluded();
  }

  public boolean areAllDaysExcluded()
  {
    for (int i = 1; i <= 31; i++) {
      if (!isDayExcluded(i)) {
        return false;
      }
    }

    return true;
  }

  public boolean isTimeIncluded(long timeStamp)
  {
    if (this.excludeAll == true) {
      return false;
    }

    if (!super.isTimeIncluded(timeStamp)) return false;

    java.util.Calendar cl = createJavaCalendar(timeStamp);
    int day = cl.get(5);

    return !isDayExcluded(day);
  }

  public long getNextIncludedTime(long timeStamp)
  {
    if (this.excludeAll == true) {
      return 0L;
    }

    long baseTime = super.getNextIncludedTime(timeStamp);
    if ((baseTime > 0L) && (baseTime > timeStamp)) {
      timeStamp = baseTime;
    }

    java.util.Calendar cl = getStartOfDayJavaCalendar(timeStamp);
    int day = cl.get(5);

    if (!isDayExcluded(day)) {
      return timeStamp;
    }

    while (isDayExcluded(day) == true) {
      cl.add(5, 1);
      day = cl.get(5);
    }

    return cl.getTime().getTime();
  }
}