package org.quartz.impl.jdbcjobstore;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.quartz.spi.ClassLoadHelper;
import org.slf4j.Logger;

public class SybaseDelegate extends StdJDBCDelegate
{
  public SybaseDelegate(Logger log, String tablePrefix, String schedName, String instanceId, ClassLoadHelper classLoadHelper)
  {
    super(log, tablePrefix, schedName, instanceId, classLoadHelper);
  }

  public SybaseDelegate(Logger log, String tablePrefix, String schedName, String instanceId, ClassLoadHelper classLoadHelper, Boolean useProperties) {
    super(log, tablePrefix, schedName, instanceId, classLoadHelper, useProperties);
  }

  protected Object getObjectFromBlob(ResultSet rs, String colName)
    throws ClassNotFoundException, IOException, SQLException
  {
    InputStream binaryInput = rs.getBinaryStream(colName);

    if ((binaryInput == null) || (binaryInput.available() == 0)) {
      return null;
    }

    Object obj = null;

    ObjectInputStream in = new ObjectInputStream(binaryInput);
    try {
      obj = in.readObject();
    } finally {
      in.close();
    }

    return obj;
  }

  protected Object getJobDataFromBlob(ResultSet rs, String colName)
    throws ClassNotFoundException, IOException, SQLException
  {
    if (canUseProperties()) {
      InputStream binaryInput = rs.getBinaryStream(colName);
      return binaryInput;
    }
    return getObjectFromBlob(rs, colName);
  }

  protected void setBytes(PreparedStatement ps, int index, ByteArrayOutputStream baos)
    throws SQLException
  {
    ps.setBytes(index, baos == null ? null : baos.toByteArray());
  }
}