package org.quartz.impl.jdbcjobstore;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import org.slf4j.Logger;

public class UpdateLockRowSemaphore extends DBSemaphore
{
  public static final String UPDATE_FOR_LOCK = "UPDATE {0}LOCKS SET LOCK_NAME = LOCK_NAME WHERE SCHED_NAME = {1} AND LOCK_NAME = ? ";
  public static final String INSERT_LOCK = "INSERT INTO {0}LOCKS(SCHED_NAME, LOCK_NAME) VALUES ({1}, ?)";

  public UpdateLockRowSemaphore()
  {
    super("QRTZ_", null, "UPDATE {0}LOCKS SET LOCK_NAME = LOCK_NAME WHERE SCHED_NAME = {1} AND LOCK_NAME = ? ", "INSERT INTO {0}LOCKS(SCHED_NAME, LOCK_NAME) VALUES ({1}, ?)");
  }

  protected void executeSQL(Connection conn, String lockName, String expandedSQL, String expandedInsertSQL)
    throws LockException
  {
    PreparedStatement ps = null;

    int count = 0;
    do {
      count++;
      try {
        ps = conn.prepareStatement(expandedSQL);
        ps.setString(1, lockName);

        if (getLog().isDebugEnabled()) {
          getLog().debug("Lock '" + lockName + "' is being obtained: " + Thread.currentThread().getName());
        }

        int numUpdate = ps.executeUpdate();

        if (numUpdate < 1) {
          getLog().debug("Inserting new lock row for lock: '" + lockName + "' being obtained by thread: " + Thread.currentThread().getName());

          ps.close();
          ps = null;
          ps = conn.prepareStatement(expandedInsertSQL);
          ps.setString(1, lockName);

          int res = ps.executeUpdate();

          if (res != 1) {
            if (count < 3) {
              try
              {
                Thread.sleep(1000L);
              } catch (InterruptedException ignore) {
                Thread.currentThread().interrupt();
              }

            }

            throw new SQLException(Util.rtp("No row exists, and one could not be inserted in table {0}LOCKS for lock named: " + lockName, getTablePrefix(), getSchedulerNameLiteral()));
          }

          if (ps == null) break;
          try {
            ps.close();
          }
          catch (Exception ignore)
          {
          }
        }
      }
      catch (SQLException sqle)
      {
        if (getLog().isDebugEnabled()) {
          getLog().debug("Lock '" + lockName + "' was not obtained by: " + Thread.currentThread().getName() + (count < 3 ? " - will try again." : ""));
        }

        if (count < 3) {
          try
          {
            Thread.sleep(1000L);
          } catch (InterruptedException ignore) {
            Thread.currentThread().interrupt();
          }

        }

        throw new LockException("Failure obtaining db row lock: " + sqle.getMessage(), sqle);
      }
      finally {
        if (ps != null)
          try {
            ps.close();
          } catch (Exception ignore) {
          }
      }
    }
    while (count < 2);
  }

  protected String getUpdateLockRowSQL() {
    return getSQL();
  }

  public void setUpdateLockRowSQL(String updateLockRowSQL) {
    setSQL(updateLockRowSQL);
  }
}