package org.quartz.jobs.ee.jms;

import javax.jms.Message;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

public final class SendTopicMessageJob
  implements Job
{
  public void execute(JobExecutionContext jobCtx)
    throws JobExecutionException
  {
    TopicConnection conn = null;

    TopicSession sess = null;

    TopicPublisher publisher = null;
    try
    {
      JobDataMap dataMap = jobCtx.getMergedJobDataMap();

      Context namingCtx = JmsHelper.getInitialContext(dataMap);

      TopicConnectionFactory connFactory = (TopicConnectionFactory)namingCtx.lookup(dataMap.getString("jms.connection.factory"));

      if (!JmsHelper.isDestinationSecure(dataMap)) {
        conn = connFactory.createTopicConnection();
      } else {
        String user = dataMap.getString("jms.user");

        String password = dataMap.getString("jms.password");

        conn = connFactory.createTopicConnection(user, password);
      }

      boolean useTransaction = JmsHelper.useTransaction(dataMap);

      int ackMode = dataMap.getInt("jms.acknowledge");

      sess = conn.createTopicSession(useTransaction, ackMode);

      Topic topic = (Topic)namingCtx.lookup(dataMap.getString("jms.destination"));

      publisher = sess.createPublisher(topic);

      JmsMessageFactory messageFactory = JmsHelper.getMessageFactory(dataMap.getString("jms.message.factory.class.name"));

      Message msg = messageFactory.createMessage(dataMap, sess);

      publisher.publish(msg);
    } catch (Exception e) {
      throw new JobExecutionException(e);
    } finally {
      JmsHelper.closeResource(publisher);

      JmsHelper.closeResource(sess);

      JmsHelper.closeResource(conn);
    }
  }
}