package devmgr.versioned.jrpc;

import devmgr.trace.Trace;
import java.io.IOException;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Hashtable;

public abstract class RPCServer
{
  private int m_Program = 0;
  private int m_Version = 0;
  private int m_TcpTimeout = 0;
  private boolean m_UdpStarted = false;
  private boolean m_TcpStarted = false;
  private Thread m_UdpThread = null;
  private Thread m_TcpListenerThread = null;
  private ThreadGroup m_ThreadGroup = null;
  private boolean m_ThreadGroupStopped = false;
  private boolean m_RunAsDaemon = false;
  private Hashtable m_ReqDetailsTable = new Hashtable();

  public RPCServer(int paramInt1, int paramInt2)
  {
    this.m_Program = paramInt1;
    this.m_Version = paramInt2;
    this.m_ThreadGroup = new ThreadGroup(toString());
  }

  public void setDaemon(boolean paramBoolean)
  {
    this.m_RunAsDaemon = paramBoolean;
  }

  public synchronized int startUDP(int paramInt)
    throws RPCError, IOException
  {
    int i = paramInt;
    if (this.m_UdpStarted)
      throw new RPCError("UDP_SERVER_ACTIVE");
    DatagramSocket localDatagramSocket;
    if (paramInt != 0)
    {
      localDatagramSocket = new DatagramSocket(i);
    }
    else
    {
      localDatagramSocket = new DatagramSocket();
      i = localDatagramSocket.getLocalPort();
    }
    this.m_UdpThread = new Thread(this.m_ThreadGroup, new UDPThread(localDatagramSocket));
    this.m_UdpThread.setDaemon(this.m_RunAsDaemon);
    this.m_UdpThread.start();
    this.m_UdpStarted = true;
    return i;
  }

  public synchronized int startTCP(int paramInt)
    throws RPCError, IOException
  {
    int i = paramInt;
    if (this.m_TcpStarted)
      throw new RPCError("TCP_SERVER_ACTIVE");
    ServerSocket localServerSocket = new ServerSocket(i);
    i = localServerSocket.getLocalPort();
    this.m_TcpListenerThread = new Thread(this.m_ThreadGroup, new TCPListenerThread(localServerSocket));
    this.m_TcpListenerThread.setDaemon(this.m_RunAsDaemon);
    this.m_TcpListenerThread.start();
    this.m_TcpStarted = true;
    return i;
  }

  public synchronized void halt()
  {
    try
    {
      this.m_ThreadGroup = null;
      this.m_ThreadGroupStopped = true;
    }
    catch (Exception localException)
    {
      Trace.printStackTrace(79, localException);
    }
  }

  public synchronized void setTcpTimeout(int paramInt)
  {
    this.m_TcpTimeout = paramInt;
  }

  public synchronized int getTcpTimeout()
  {
    return this.m_TcpTimeout;
  }

  public RPCRequestDetails getRequestDetails()
  {
    return (RPCRequestDetails)this.m_ReqDetailsTable.get(Thread.currentThread());
  }

  public void nullHandler()
    throws RPCError
  {
  }

  public abstract XDRType processCall(int paramInt, XDRInputStream paramXDRInputStream)
    throws RPCError;

  protected void exceptionHandler(Exception paramException)
  {
    Trace.println(79, "Exception in RPC server: " + paramException.toString());
    Trace.printStackTrace(79, paramException);
  }

  private XDRMessageIOV transact(XDRMessageIOV paramXDRMessageIOV, boolean paramBoolean, InetAddress paramInetAddress, int paramInt)
    throws RPCError
  {
    XDRMessageIOV localXDRMessageIOV = null;
    XDRInputStream localXDRInputStream = new XDRInputStream(paramXDRMessageIOV);
    XDROutputStream localXDROutputStream = null;
    RPCMsg localRPCMsg1 = new RPCMsg();
    RPCMsg localRPCMsg2 = new RPCMsg();
    XDRType localXDRType = null;
    int i = 0;
    int j = 0;
    Thread localThread = Thread.currentThread();
    RPCAcceptedReply localRPCAcceptedReply = localRPCMsg1.getBody().getReplyMsg().getAcceptedReply();
    RPCAcceptedStatus localRPCAcceptedStatus = localRPCAcceptedReply.getBody().getStatus();
    Trace.println(4, "Received call from " + paramInetAddress);
    if (Trace.isTracing(5))
      RPCDebug.dumpIOV(5, paramXDRMessageIOV);
    localRPCMsg2.xdrDecode(localXDRInputStream);
    if ((localRPCMsg2.getBody().getMsgType().getValue() == 0) && (localRPCMsg2.getBody().getCallMsg().getRpcVersion() == 2))
    {
      localRPCMsg1.setXid(localRPCMsg2.getXid());
      localRPCMsg1.getBody().getMsgType().setValue(1);
      localRPCAcceptedStatus.setValue(0);
      localRPCAcceptedReply.setVerifier(localRPCMsg2.getBody().getCallMsg().getVerifier());
      if (localRPCMsg2.getBody().getCallMsg().getProgramNo() != this.m_Program)
      {
        localRPCAcceptedStatus.setValue(1);
      }
      else if (localRPCMsg2.getBody().getCallMsg().getVersionNo() != this.m_Version)
      {
        localRPCAcceptedStatus.setValue(2);
        localRPCAcceptedReply.getBody().getVersions().setLow(this.m_Version);
        localRPCAcceptedReply.getBody().getVersions().setHigh(this.m_Version);
      }
      else
      {
        RPCRequestDetails localRPCRequestDetails = new RPCRequestDetails(localRPCMsg2, paramInetAddress, paramInt, paramBoolean);
        this.m_ReqDetailsTable.put(localThread, localRPCRequestDetails);
        try
        {
          i = 0;
          j = 0;
          localXDRType = processCall(localRPCMsg2.getBody().getCallMsg().getProcedure(), localXDRInputStream);
        }
        catch (RPCError localRPCError)
        {
          Trace.printStackTrace(79, localRPCError);
          if (localRPCError.getMessage().equals("SUPPRESS_REPLY"))
            j = 1;
          else
            i = 1;
        }
        finally
        {
          this.m_ReqDetailsTable.remove(localThread);
        }
        if (j == 0)
          if (i != 0)
            localRPCAcceptedStatus.setValue(4);
          else if (localXDRType == null)
            localRPCAcceptedStatus.setValue(3);
          else
            localRPCAcceptedStatus.setValue(0);
      }
      if (j == 0)
      {
        localXDROutputStream = new XDROutputStream();
        localRPCMsg1.xdrEncode(localXDROutputStream);
        if (localXDRType != null)
          localXDRType.xdrEncode(localXDROutputStream);
        localXDRMessageIOV = localXDROutputStream.getMessage();
        Trace.println(4, "Sending reply to " + paramInetAddress);
        if (Trace.isTracing(6))
          RPCDebug.dumpIOV(6, localXDRMessageIOV);
      }
    }
    return localXDRMessageIOV;
  }

  public boolean allowConnection(InetAddress paramInetAddress)
  {
    return true;
  }

  private synchronized void startTcpHandler(Socket paramSocket)
    throws IOException
  {
    if (!allowConnection(paramSocket.getInetAddress()))
    {
      paramSocket.close();
    }
    else
    {
      Thread localThread = new Thread(this.m_ThreadGroup, new TCPConnectionThread(paramSocket));
      localThread.setDaemon(this.m_RunAsDaemon);
      localThread.start();
    }
  }

  class UDPThread
    implements Runnable
  {
    private UDPChannel m_Channel = null;
    private int m_Port;

    UDPThread(DatagramSocket arg2)
    {
      DatagramSocket localDatagramSocket;
      this.m_Channel = new UDPChannel(localDatagramSocket);
      this.m_Port = localDatagramSocket.getLocalPort();
    }

    protected void finalize()
      throws Throwable
    {
      if (this.m_Channel != null)
        this.m_Channel.close();
    }

    public void run()
    {
      XDRMessageIOV localXDRMessageIOV1 = null;
      XDRMessageIOV localXDRMessageIOV2 = null;
      UDPMessageBundle localUDPMessageBundle1 = null;
      UDPMessageBundle localUDPMessageBundle2 = null;
      StringBuffer localStringBuffer = new StringBuffer();
      localStringBuffer.append(getClass().getName());
      localStringBuffer.append("[" + this.m_Port + "]");
      Thread.currentThread().setName(localStringBuffer.toString());
      Trace.printf(4, "Server UDP thread running on port %d", new Integer(this.m_Port));
      while (!RPCServer.this.m_ThreadGroupStopped)
        try
        {
          localUDPMessageBundle2 = this.m_Channel.receiveMsg();
          localXDRMessageIOV1 = localUDPMessageBundle2.getMsg();
          localXDRMessageIOV2 = RPCServer.this.transact(localXDRMessageIOV1, false, localUDPMessageBundle2.getAddress(), localUDPMessageBundle2.getPort());
          if (localXDRMessageIOV2 != null)
          {
            localUDPMessageBundle1 = new UDPMessageBundle(localXDRMessageIOV2, localUDPMessageBundle2.getAddress(), localUDPMessageBundle2.getPort());
            this.m_Channel.sendMsg(localUDPMessageBundle1);
            localUDPMessageBundle1 = null;
          }
        }
        catch (Exception localException)
        {
          RPCServer.this.exceptionHandler(localException);
        }
    }
  }

  class TCPListenerThread
    implements Runnable
  {
    private ServerSocket m_Sock;

    TCPListenerThread(ServerSocket arg2)
    {
      Object localObject;
      this.m_Sock = localObject;
    }

    protected void finalize()
      throws Throwable
    {
    }

    public void run()
    {
      int i = 0;
      StringBuffer localStringBuffer = new StringBuffer();
      int j = this.m_Sock.getLocalPort();
      localStringBuffer.append(getClass().getName());
      localStringBuffer.append("[" + j + "]");
      Thread.currentThread().setName(localStringBuffer.toString());
      Trace.printf(4, "Server TCP listener running on port %d", new Integer(j));
      while ((i == 0) && (!RPCServer.this.m_ThreadGroupStopped))
        try
        {
          Socket localSocket = this.m_Sock.accept();
          RPCServer.this.startTcpHandler(localSocket);
        }
        catch (IOException localIOException1)
        {
          RPCServer.this.exceptionHandler(localIOException1);
          i = 1;
        }
      try
      {
        this.m_Sock.close();
      }
      catch (IOException localIOException2)
      {
        Trace.printStackTrace(79, localIOException2);
      }
    }
  }

  class TCPConnectionThread
    implements Runnable
  {
    private TCPChannel m_Channel = null;
    private InetAddress m_RemoteAddress = null;
    private int m_RemotePort;

    TCPConnectionThread(Socket arg2)
      throws IOException
    {
      Socket localSocket;
      this.m_RemoteAddress = localSocket.getInetAddress();
      this.m_RemotePort = localSocket.getPort();
      this.m_Channel = new TCPChannel(localSocket);
      this.m_Channel.setTimeout(RPCServer.this.getTcpTimeout());
    }

    protected void finalize()
      throws Throwable
    {
      if (this.m_Channel != null)
        this.m_Channel.close();
    }

    public void run()
    {
      int i = 0;
      StringBuffer localStringBuffer = new StringBuffer();
      localStringBuffer.append(getClass().getName());
      localStringBuffer.append("[" + this.m_RemoteAddress.toString() + ":" + this.m_RemotePort + "]");
      Thread.currentThread().setName(localStringBuffer.toString());
      Trace.println(4, "Server TCP thread started");
      while ((i == 0) && (!RPCServer.this.m_ThreadGroupStopped))
        try
        {
          XDRMessageIOV localXDRMessageIOV1 = this.m_Channel.receiveMsg();
          XDRMessageIOV localXDRMessageIOV2 = RPCServer.this.transact(localXDRMessageIOV1, true, this.m_RemoteAddress, this.m_RemotePort);
          if (localXDRMessageIOV2 != null)
          {
            this.m_Channel.sendMsg(localXDRMessageIOV2);
            localXDRMessageIOV2 = null;
          }
        }
        catch (Exception localException1)
        {
          RPCServer.this.exceptionHandler(localException1);
          i = 1;
        }
      Trace.println(4, "Server TCP thread exiting");
      try
      {
        this.m_Channel.close();
        this.m_Channel = null;
      }
      catch (Exception localException2)
      {
        Trace.printStackTrace(79, localException2);
      }
    }
  }
}