package devmgr.versioned.jrpc;

import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

public class TCPChannel
{
  private Socket m_Sock = null;
  private OutputStream m_Out = null;
  private InputStream m_In = null;

  public TCPChannel(Socket paramSocket)
    throws IOException
  {
    this.m_Sock = paramSocket;
    this.m_Out = this.m_Sock.getOutputStream();
    this.m_In = this.m_Sock.getInputStream();
  }

  protected void finalize()
    throws Throwable
  {
    if (this.m_In != null)
      this.m_In.close();
    if (this.m_Out != null)
      this.m_Out.close();
    if (this.m_Sock != null)
      this.m_Sock.close();
  }

  public void setTimeout(int paramInt)
    throws IOException
  {
    this.m_Sock.setSoTimeout(paramInt * 1000);
  }

  public void close()
    throws IOException
  {
    this.m_In.close();
    this.m_In = null;
    this.m_Out.close();
    this.m_Out = null;
    this.m_Sock.close();
    this.m_Sock = null;
  }

  public InetAddress getLocalAddress()
  {
    return this.m_Sock.getLocalAddress();
  }

  public void sendMsg(XDRMessageIOV paramXDRMessageIOV)
    throws RPCError, IOException
  {
    int i = paramXDRMessageIOV.getMessageSize();
    int j = 0;
    int m = 0;
    byte[] arrayOfByte2 = new byte[4];
    while (j < i)
    {
      byte[] arrayOfByte1 = paramXDRMessageIOV.getBuffer(m++);
      int n = Math.min(arrayOfByte1.length, i - j);
      int k = n;
      if (j + n >= i)
        k |= -2147483648;
      arrayOfByte2[0] = ((byte)(k >>> 24));
      arrayOfByte2[1] = ((byte)(k >>> 16));
      arrayOfByte2[2] = ((byte)(k >>> 8));
      arrayOfByte2[3] = ((byte)(k >>> 0));
      this.m_Out.write(arrayOfByte2);
      this.m_Out.write(arrayOfByte1, 0, n);
      j += n;
    }
    this.m_Out.flush();
  }

  public XDRMessageIOV receiveMsg()
    throws RPCError, IOException
  {
    XDRMessageIOV localXDRMessageIOV = new XDRMessageIOV();
    byte[] arrayOfByte1 = new byte[4];
    int i;
    for (int j = 0; j == 0; j = (i & 0x80000000) != 0 ? 1 : 0)
    {
      readFully(arrayOfByte1, 0, arrayOfByte1.length);
      i = (arrayOfByte1[0] & 0xFF) << 24 | (arrayOfByte1[1] & 0xFF) << 16 | (arrayOfByte1[2] & 0xFF) << 8 | (arrayOfByte1[3] & 0xFF) << 0;
      int k;
      if ((k = i & 0x7FFFFFFF) != 0)
      {
        byte[] arrayOfByte2 = new byte[k];
        readFully(arrayOfByte2, 0, k);
        localXDRMessageIOV.addFullBuffer(arrayOfByte2);
      }
    }
    return localXDRMessageIOV;
  }

  private void readFully(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
    throws RPCError, IOException
  {
    int i = 0;
    while (i < paramInt2)
    {
      int j;
      try
      {
        j = this.m_In.read(paramArrayOfByte, paramInt1 + i, paramInt2 - i);
      }
      catch (InterruptedIOException localInterruptedIOException)
      {
        throw new RPCError("TIMEOUT");
      }
      if (j <= 0)
        throw new RPCError("CONNECTION_LOST");
      i += j;
    }
  }

  public InetAddress getServerAddress()
  {
    return this.m_Sock.getInetAddress();
  }
}