package devmgr.versioned.jrpc;

public class XDRInputStream
{
  private static final int XDR_UNIT_SIZE = 4;
  private XDRMessageIOV m_Msg;
  private int m_TotalBytes;
  private int m_TotalConsumed;
  private int m_NextBuf;
  private byte[] m_Buf;
  private int m_BufSize;
  private int m_BufPos;

  public XDRInputStream(XDRMessageIOV paramXDRMessageIOV)
  {
    this.m_Msg = paramXDRMessageIOV;
    this.m_TotalBytes = this.m_Msg.getMessageSize();
    this.m_TotalConsumed = 0;
    this.m_NextBuf = 0;
    this.m_Buf = null;
    this.m_BufSize = 0;
    this.m_BufPos = 0;
  }

  public int getPosition()
  {
    return this.m_TotalConsumed;
  }

  public void setPosition(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= this.m_TotalBytes))
    {
      this.m_TotalConsumed = this.m_TotalBytes;
      this.m_Buf = null;
      this.m_BufSize = 0;
      this.m_BufPos = 0;
      this.m_NextBuf = 0;
    }
    else
    {
      this.m_NextBuf = 0;
      this.m_Buf = this.m_Msg.getBuffer(this.m_NextBuf++);
      this.m_BufSize = Math.min(this.m_Buf.length, this.m_TotalBytes);
      this.m_TotalConsumed = 0;
      while (this.m_TotalConsumed + this.m_BufSize < paramInt)
      {
        this.m_TotalConsumed += this.m_BufSize;
        this.m_Buf = this.m_Msg.getBuffer(this.m_NextBuf++);
        this.m_BufSize = Math.min(this.m_Buf.length, this.m_TotalBytes - this.m_TotalConsumed);
      }
      this.m_BufPos = (paramInt - this.m_TotalConsumed);
      this.m_TotalConsumed = paramInt;
    }
  }

  public boolean getBoolean()
    throws RPCError
  {
    int i = getInt();
    boolean bool;
    if ((i & 0xFF) == 0)
      bool = false;
    else
      bool = true;
    return bool;
  }

  public char getChar()
    throws RPCError
  {
    int i = getInt();
    return (char)(i & 0xFF);
  }

  public short getShort()
    throws RPCError
  {
    return (short)(getInt() & 0xFFFF);
  }

  public int getInt()
    throws RPCError
  {
    int i;
    int j;
    int k;
    int m;
    if (this.m_BufPos + 3 >= this.m_BufSize)
    {
      i = getByte() & 0xFF;
      j = getByte() & 0xFF;
      k = getByte() & 0xFF;
      m = getByte() & 0xFF;
    }
    else
    {
      i = this.m_Buf[(this.m_BufPos++)] & 0xFF;
      j = this.m_Buf[(this.m_BufPos++)] & 0xFF;
      k = this.m_Buf[(this.m_BufPos++)] & 0xFF;
      m = this.m_Buf[(this.m_BufPos++)] & 0xFF;
      this.m_TotalConsumed += 4;
    }
    return i << 24 | j << 16 | k << 8 | m << 0;
  }

  public long getLong()
    throws RPCError
  {
    return getInt() << 32 | getInt() & 0xFFFFFFFF;
  }

  public float getFloat()
    throws RPCError
  {
    int i = getInt();
    return Float.intBitsToFloat(i);
  }

  public double getDouble()
    throws RPCError
  {
    long l = getLong();
    return Double.longBitsToDouble(l);
  }

  public String getString()
    throws RPCError
  {
    int i = getInt();
    return fetchString(i);
  }

  public String getString(int paramInt)
    throws RPCError
  {
    int i = getInt();
    if (i > paramInt)
      throw new RPCError("XDR_MAX_LENGTH");
    return fetchString(i);
  }

  public byte[] getVariableOpaque()
    throws RPCError
  {
    int i = getInt();
    return getFixedOpaque(i);
  }

  public byte[] getVariableOpaque(int paramInt)
    throws RPCError
  {
    int i = getInt();
    if (i > paramInt)
      throw new RPCError("XDR_MAX_LENGTH");
    return getFixedOpaque(i);
  }

  public byte[] getFixedOpaque(int paramInt)
    throws RPCError
  {
    int i = calculatePad(paramInt);
    byte[] arrayOfByte = new byte[paramInt];
    int j;
    if (this.m_BufPos + paramInt - 1 >= this.m_BufSize)
    {
      for (j = 0; j < paramInt; j++)
        arrayOfByte[j] = getByte();
    }
    else
    {
      System.arraycopy(this.m_Buf, this.m_BufPos, arrayOfByte, 0, paramInt);
      this.m_BufPos += paramInt;
      this.m_TotalConsumed += paramInt;
    }
    if (this.m_BufPos + i - 1 >= this.m_BufSize)
    {
      for (j = 0; j < i; j++)
        getByte();
    }
    else
    {
      this.m_BufPos += i;
      this.m_TotalConsumed += i;
    }
    return arrayOfByte;
  }

  public int getRemainingLength()
  {
    return this.m_TotalBytes - this.m_TotalConsumed;
  }

  public void skip(int paramInt)
    throws RPCError
  {
    for (int i = 0; i < paramInt; i++)
      getByte();
  }

  private byte getByte()
    throws RPCError
  {
    if (this.m_BufPos >= this.m_BufSize)
    {
      if (this.m_TotalConsumed >= this.m_TotalBytes)
        throw new RPCError("XDR_UNDERFLOW");
      this.m_Buf = this.m_Msg.getBuffer(this.m_NextBuf++);
      this.m_BufPos = 0;
      this.m_BufSize = Math.min(this.m_Buf.length, this.m_TotalBytes - this.m_TotalConsumed);
    }
    this.m_TotalConsumed += 1;
    return this.m_Buf[(this.m_BufPos++)];
  }

  private int calculatePad(int paramInt)
  {
    int i = 0;
    int j;
    if ((j = paramInt % 4) != 0)
      i = 4 - j;
    return i;
  }

  private String fetchString(int paramInt)
    throws RPCError
  {
    int i = calculatePad(paramInt);
    char[] arrayOfChar = new char[paramInt];
    int j;
    if (this.m_BufPos + paramInt - 1 >= this.m_BufSize)
    {
      for (j = 0; j < paramInt; j++)
        arrayOfChar[j] = ((char)getByte());
    }
    else
    {
      for (j = 0; j < paramInt; j++)
        arrayOfChar[j] = ((char)this.m_Buf[(this.m_BufPos++)]);
      this.m_TotalConsumed += paramInt;
    }
    if (this.m_BufPos + i - 1 >= this.m_BufSize)
    {
      for (j = 0; j < i; j++)
        getByte();
    }
    else
    {
      this.m_BufPos += i;
      this.m_TotalConsumed += i;
    }
    return String.copyValueOf(arrayOfChar);
  }
}