package devmgr.versioned.symbol;

import devmgr.versioned.jrpc.RPCError;
import devmgr.versioned.jrpc.XDRInputStream;
import devmgr.versioned.jrpc.XDROutputStream;
import devmgr.versioned.jrpc.XDRType;

public class FeatureParams
  implements XDRType, SYMbolAPIConstants
{
  private int maxHotSpares;
  private int[] cacheBlockSizes;
  private int maxDCEDrives;
  private int maxReadAheadMultiplier;
  private int minMediaScanPeriod;
  private int maxMediaScanPeriod;
  private int[] supportedSegSizes;
  private int numStandardSegSizes;
  private SegmentSizeTransition[] segTransitions;
  private int mappableLUNCount;
  private int maxPartitionCount;
  private int maxMappingsPerVolume;
  private int maxVolumes;
  private int maxSnapshots;
  private int maxSnapshotsPerBase;
  private int minRepositorySize;
  private int maxMirrors;
  private int maxVolCopys;
  private int maxConcurrentBackgroundCopys;
  private int maxUtmLunValue;
  private byte[] reserved1;
  private byte[] reserved2;
  private boolean battery_not_in_configuration;
  private PhysicalDriveType[] supportedDriveTypes;
  private int currentMaxMirrors;
  private int maxVolumesPerGroup;
  private int maxDriveDownloadFiles;
  private int maxMgmtClientRecords;
  private int maxMgmtClientRecordSize;
  private ControllerRedundancyMode redundancyMode = new ControllerRedundancyMode();

  public FeatureParams()
  {
  }

  public FeatureParams(FeatureParams paramFeatureParams)
  {
    this.maxHotSpares = paramFeatureParams.maxHotSpares;
    this.cacheBlockSizes = paramFeatureParams.cacheBlockSizes;
    this.maxDCEDrives = paramFeatureParams.maxDCEDrives;
    this.maxReadAheadMultiplier = paramFeatureParams.maxReadAheadMultiplier;
    this.minMediaScanPeriod = paramFeatureParams.minMediaScanPeriod;
    this.maxMediaScanPeriod = paramFeatureParams.maxMediaScanPeriod;
    this.supportedSegSizes = paramFeatureParams.supportedSegSizes;
    this.numStandardSegSizes = paramFeatureParams.numStandardSegSizes;
    this.segTransitions = paramFeatureParams.segTransitions;
    this.mappableLUNCount = paramFeatureParams.mappableLUNCount;
    this.maxPartitionCount = paramFeatureParams.maxPartitionCount;
    this.maxMappingsPerVolume = paramFeatureParams.maxMappingsPerVolume;
    this.maxVolumes = paramFeatureParams.maxVolumes;
    this.maxSnapshots = paramFeatureParams.maxSnapshots;
    this.maxSnapshotsPerBase = paramFeatureParams.maxSnapshotsPerBase;
    this.minRepositorySize = paramFeatureParams.minRepositorySize;
    this.maxMirrors = paramFeatureParams.maxMirrors;
    this.maxVolCopys = paramFeatureParams.maxVolCopys;
    this.maxConcurrentBackgroundCopys = paramFeatureParams.maxConcurrentBackgroundCopys;
    this.maxUtmLunValue = paramFeatureParams.maxUtmLunValue;
    this.reserved1 = paramFeatureParams.reserved1;
    this.reserved2 = paramFeatureParams.reserved2;
    this.battery_not_in_configuration = paramFeatureParams.battery_not_in_configuration;
    this.supportedDriveTypes = paramFeatureParams.supportedDriveTypes;
    this.currentMaxMirrors = paramFeatureParams.currentMaxMirrors;
    this.maxVolumesPerGroup = paramFeatureParams.maxVolumesPerGroup;
    this.maxDriveDownloadFiles = paramFeatureParams.maxDriveDownloadFiles;
    this.maxMgmtClientRecords = paramFeatureParams.maxMgmtClientRecords;
    this.maxMgmtClientRecordSize = paramFeatureParams.maxMgmtClientRecordSize;
    this.redundancyMode = paramFeatureParams.redundancyMode;
  }

  public int getMaxHotSpares()
  {
    return this.maxHotSpares;
  }

  public void setMaxHotSpares(int paramInt)
  {
    this.maxHotSpares = paramInt;
  }

  public int[] getCacheBlockSizes()
  {
    return this.cacheBlockSizes;
  }

  public void setCacheBlockSizes(int[] paramArrayOfInt)
  {
    this.cacheBlockSizes = paramArrayOfInt;
  }

  public int getMaxDCEDrives()
  {
    return this.maxDCEDrives;
  }

  public void setMaxDCEDrives(int paramInt)
  {
    this.maxDCEDrives = paramInt;
  }

  public int getMaxReadAheadMultiplier()
  {
    return this.maxReadAheadMultiplier;
  }

  public void setMaxReadAheadMultiplier(int paramInt)
  {
    this.maxReadAheadMultiplier = paramInt;
  }

  public int getMinMediaScanPeriod()
  {
    return this.minMediaScanPeriod;
  }

  public void setMinMediaScanPeriod(int paramInt)
  {
    this.minMediaScanPeriod = paramInt;
  }

  public int getMaxMediaScanPeriod()
  {
    return this.maxMediaScanPeriod;
  }

  public void setMaxMediaScanPeriod(int paramInt)
  {
    this.maxMediaScanPeriod = paramInt;
  }

  public int[] getSupportedSegSizes()
  {
    return this.supportedSegSizes;
  }

  public void setSupportedSegSizes(int[] paramArrayOfInt)
  {
    this.supportedSegSizes = paramArrayOfInt;
  }

  public int getNumStandardSegSizes()
  {
    return this.numStandardSegSizes;
  }

  public void setNumStandardSegSizes(int paramInt)
  {
    this.numStandardSegSizes = paramInt;
  }

  public SegmentSizeTransition[] getSegTransitions()
  {
    return this.segTransitions;
  }

  public void setSegTransitions(SegmentSizeTransition[] paramArrayOfSegmentSizeTransition)
  {
    this.segTransitions = paramArrayOfSegmentSizeTransition;
  }

  public int getMappableLUNCount()
  {
    return this.mappableLUNCount;
  }

  public void setMappableLUNCount(int paramInt)
  {
    this.mappableLUNCount = paramInt;
  }

  public int getMaxPartitionCount()
  {
    return this.maxPartitionCount;
  }

  public void setMaxPartitionCount(int paramInt)
  {
    this.maxPartitionCount = paramInt;
  }

  public int getMaxMappingsPerVolume()
  {
    return this.maxMappingsPerVolume;
  }

  public void setMaxMappingsPerVolume(int paramInt)
  {
    this.maxMappingsPerVolume = paramInt;
  }

  public int getMaxVolumes()
  {
    return this.maxVolumes;
  }

  public void setMaxVolumes(int paramInt)
  {
    this.maxVolumes = paramInt;
  }

  public int getMaxSnapshots()
  {
    return this.maxSnapshots;
  }

  public void setMaxSnapshots(int paramInt)
  {
    this.maxSnapshots = paramInt;
  }

  public int getMaxSnapshotsPerBase()
  {
    return this.maxSnapshotsPerBase;
  }

  public void setMaxSnapshotsPerBase(int paramInt)
  {
    this.maxSnapshotsPerBase = paramInt;
  }

  public int getMinRepositorySize()
  {
    return this.minRepositorySize;
  }

  public void setMinRepositorySize(int paramInt)
  {
    this.minRepositorySize = paramInt;
  }

  public int getMaxMirrors()
  {
    return this.maxMirrors;
  }

  public void setMaxMirrors(int paramInt)
  {
    this.maxMirrors = paramInt;
  }

  public int getMaxVolCopys()
  {
    return this.maxVolCopys;
  }

  public void setMaxVolCopys(int paramInt)
  {
    this.maxVolCopys = paramInt;
  }

  public int getMaxConcurrentBackgroundCopys()
  {
    return this.maxConcurrentBackgroundCopys;
  }

  public void setMaxConcurrentBackgroundCopys(int paramInt)
  {
    this.maxConcurrentBackgroundCopys = paramInt;
  }

  public int getMaxUtmLunValue()
  {
    return this.maxUtmLunValue;
  }

  public void setMaxUtmLunValue(int paramInt)
  {
    this.maxUtmLunValue = paramInt;
  }

  public byte[] getReserved1()
  {
    return this.reserved1;
  }

  public void setReserved1(byte[] paramArrayOfByte)
  {
    this.reserved1 = paramArrayOfByte;
  }

  public byte[] getReserved2()
  {
    return this.reserved2;
  }

  public void setReserved2(byte[] paramArrayOfByte)
  {
    this.reserved2 = paramArrayOfByte;
  }

  public boolean getBattery_not_in_configuration()
  {
    return this.battery_not_in_configuration;
  }

  public void setBattery_not_in_configuration(boolean paramBoolean)
  {
    this.battery_not_in_configuration = paramBoolean;
  }

  public PhysicalDriveType[] getSupportedDriveTypes()
  {
    return this.supportedDriveTypes;
  }

  public void setSupportedDriveTypes(PhysicalDriveType[] paramArrayOfPhysicalDriveType)
  {
    this.supportedDriveTypes = paramArrayOfPhysicalDriveType;
  }

  public int getCurrentMaxMirrors()
  {
    return this.currentMaxMirrors;
  }

  public void setCurrentMaxMirrors(int paramInt)
  {
    this.currentMaxMirrors = paramInt;
  }

  public int getMaxVolumesPerGroup()
  {
    return this.maxVolumesPerGroup;
  }

  public void setMaxVolumesPerGroup(int paramInt)
  {
    this.maxVolumesPerGroup = paramInt;
  }

  public int getMaxDriveDownloadFiles()
  {
    return this.maxDriveDownloadFiles;
  }

  public void setMaxDriveDownloadFiles(int paramInt)
  {
    this.maxDriveDownloadFiles = paramInt;
  }

  public int getMaxMgmtClientRecords()
  {
    return this.maxMgmtClientRecords;
  }

  public void setMaxMgmtClientRecords(int paramInt)
  {
    this.maxMgmtClientRecords = paramInt;
  }

  public int getMaxMgmtClientRecordSize()
  {
    return this.maxMgmtClientRecordSize;
  }

  public void setMaxMgmtClientRecordSize(int paramInt)
  {
    this.maxMgmtClientRecordSize = paramInt;
  }

  public ControllerRedundancyMode getRedundancyMode()
  {
    return this.redundancyMode;
  }

  public void setRedundancyMode(ControllerRedundancyMode paramControllerRedundancyMode)
  {
    this.redundancyMode = paramControllerRedundancyMode;
  }

  public void xdrEncode(XDROutputStream paramXDROutputStream)
    throws RPCError
  {
    int i = paramXDROutputStream.prepareLength();
    paramXDROutputStream.putInt(this.maxHotSpares);
    int j = this.cacheBlockSizes == null ? 0 : this.cacheBlockSizes.length;
    paramXDROutputStream.putInt(j);
    for (int k = 0; k < j; k++)
      paramXDROutputStream.putInt(this.cacheBlockSizes[k]);
    paramXDROutputStream.putInt(this.maxDCEDrives);
    paramXDROutputStream.putInt(this.maxReadAheadMultiplier);
    paramXDROutputStream.putInt(this.minMediaScanPeriod);
    paramXDROutputStream.putInt(this.maxMediaScanPeriod);
    j = this.supportedSegSizes == null ? 0 : this.supportedSegSizes.length;
    paramXDROutputStream.putInt(j);
    for (k = 0; k < j; k++)
      paramXDROutputStream.putInt(this.supportedSegSizes[k]);
    paramXDROutputStream.putInt(this.numStandardSegSizes);
    j = this.segTransitions == null ? 0 : this.segTransitions.length;
    paramXDROutputStream.putInt(j);
    for (k = 0; k < j; k++)
      this.segTransitions[k].xdrEncode(paramXDROutputStream);
    paramXDROutputStream.putInt(this.mappableLUNCount);
    paramXDROutputStream.putInt(this.maxPartitionCount);
    paramXDROutputStream.putInt(this.maxMappingsPerVolume);
    paramXDROutputStream.putInt(this.maxVolumes);
    paramXDROutputStream.putInt(this.maxSnapshots);
    paramXDROutputStream.putInt(this.maxSnapshotsPerBase);
    paramXDROutputStream.putInt(this.minRepositorySize);
    paramXDROutputStream.putInt(this.maxMirrors);
    paramXDROutputStream.putInt(this.maxVolCopys);
    paramXDROutputStream.putInt(this.maxConcurrentBackgroundCopys);
    paramXDROutputStream.putInt(this.maxUtmLunValue);
    paramXDROutputStream.putFixedOpaque(this.reserved1, 8);
    paramXDROutputStream.putVariableOpaque(this.reserved2);
    paramXDROutputStream.putBoolean(this.battery_not_in_configuration);
    j = this.supportedDriveTypes == null ? 0 : this.supportedDriveTypes.length;
    paramXDROutputStream.putInt(j);
    for (k = 0; k < j; k++)
      this.supportedDriveTypes[k].xdrEncode(paramXDROutputStream);
    paramXDROutputStream.putInt(this.currentMaxMirrors);
    paramXDROutputStream.putInt(this.maxVolumesPerGroup);
    paramXDROutputStream.putInt(this.maxDriveDownloadFiles);
    paramXDROutputStream.putInt(this.maxMgmtClientRecords);
    paramXDROutputStream.putInt(this.maxMgmtClientRecordSize);
    this.redundancyMode.xdrEncode(paramXDROutputStream);
    paramXDROutputStream.setLength(i);
  }

  public void xdrDecode(XDRInputStream paramXDRInputStream)
    throws RPCError
  {
    int i = paramXDRInputStream.getInt() + paramXDRInputStream.getPosition();
    if (paramXDRInputStream.getPosition() < i)
      this.maxHotSpares = paramXDRInputStream.getInt();
    int j;
    int k;
    if (paramXDRInputStream.getPosition() < i)
    {
      j = paramXDRInputStream.getInt();
      this.cacheBlockSizes = new int[j];
      for (k = 0; k < j; k++)
        this.cacheBlockSizes[k] = paramXDRInputStream.getInt();
    }
    if (paramXDRInputStream.getPosition() < i)
      this.maxDCEDrives = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxReadAheadMultiplier = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.minMediaScanPeriod = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxMediaScanPeriod = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
    {
      j = paramXDRInputStream.getInt();
      this.supportedSegSizes = new int[j];
      for (k = 0; k < j; k++)
        this.supportedSegSizes[k] = paramXDRInputStream.getInt();
    }
    if (paramXDRInputStream.getPosition() < i)
      this.numStandardSegSizes = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
    {
      j = paramXDRInputStream.getInt();
      this.segTransitions = new SegmentSizeTransition[j];
      for (k = 0; k < j; k++)
      {
        this.segTransitions[k] = new SegmentSizeTransition();
        this.segTransitions[k].xdrDecode(paramXDRInputStream);
      }
    }
    if (paramXDRInputStream.getPosition() < i)
      this.mappableLUNCount = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxPartitionCount = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxMappingsPerVolume = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxVolumes = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxSnapshots = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxSnapshotsPerBase = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.minRepositorySize = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxMirrors = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxVolCopys = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxConcurrentBackgroundCopys = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxUtmLunValue = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.reserved1 = paramXDRInputStream.getFixedOpaque(8);
    if (paramXDRInputStream.getPosition() < i)
      this.reserved2 = paramXDRInputStream.getVariableOpaque();
    if (paramXDRInputStream.getPosition() < i)
      this.battery_not_in_configuration = paramXDRInputStream.getBoolean();
    if (paramXDRInputStream.getPosition() < i)
    {
      j = paramXDRInputStream.getInt();
      this.supportedDriveTypes = new PhysicalDriveType[j];
      for (k = 0; k < j; k++)
      {
        this.supportedDriveTypes[k] = new PhysicalDriveType();
        this.supportedDriveTypes[k].xdrDecode(paramXDRInputStream);
      }
    }
    if (paramXDRInputStream.getPosition() < i)
      this.currentMaxMirrors = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxVolumesPerGroup = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxDriveDownloadFiles = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxMgmtClientRecords = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.maxMgmtClientRecordSize = paramXDRInputStream.getInt();
    if (paramXDRInputStream.getPosition() < i)
      this.redundancyMode.xdrDecode(paramXDRInputStream);
    paramXDRInputStream.skip(i - paramXDRInputStream.getPosition());
  }
}