package devmgr.versioned.symbol;

import devmgr.versioned.jrpc.RPCAuthGenerator;
import devmgr.versioned.jrpc.RPCAuthType;
import devmgr.versioned.jrpc.RPCError;
import devmgr.versioned.jrpc.RPCOpaqueAuth;
import devmgr.versioned.jrpc.XDRMessageIOV;
import devmgr.versioned.jrpc.XDROutputStream;
import java.net.InetAddress;
import java.security.MessageDigest;

public class SYMbolAuthGenerator
  implements RPCAuthGenerator
{
  private static final int DIGEST_LENGTH = 20;
  private int m_ClientIp;
  private SAIdentifier m_ArrayId;
  private long m_ConfigGeneration;
  private MessageDigest m_Sha;
  private byte[] m_Password;

  public SYMbolAuthGenerator(InetAddress paramInetAddress, SAIdentifier paramSAIdentifier)
    throws RPCError
  {
    byte[] arrayOfByte = paramInetAddress.getAddress();
    for (int i = 0; i < 4; i++)
      this.m_ClientIp = (this.m_ClientIp << 8 | arrayOfByte[i] & 0xFF);
    this.m_ArrayId = paramSAIdentifier;
    this.m_Password = new byte[0];
    this.m_ConfigGeneration = 0L;
    try
    {
      this.m_Sha = MessageDigest.getInstance("SHA");
    }
    catch (Exception localException)
    {
      throw new RPCError("AUTH_ALGORITHM");
    }
  }

  public RPCOpaqueAuth getInitialAuthData()
    throws RPCError
  {
    RPCAuthType localRPCAuthType = new RPCAuthType();
    RPCOpaqueAuth localRPCOpaqueAuth = new RPCOpaqueAuth();
    SYMbolAuthentication localSYMbolAuthentication = new SYMbolAuthentication();
    XDROutputStream localXDROutputStream = new XDROutputStream();
    localSYMbolAuthentication.setClientIp(this.m_ClientIp);
    localSYMbolAuthentication.setArrayId(this.m_ArrayId);
    localSYMbolAuthentication.setConfigGeneration(this.m_ConfigGeneration);
    localSYMbolAuthentication.setDigest(new byte[20]);
    localSYMbolAuthentication.xdrEncode(localXDROutputStream);
    XDRMessageIOV localXDRMessageIOV = localXDROutputStream.getMessage();
    int i = localXDRMessageIOV.getMessageSize();
    localRPCOpaqueAuth.setInfo(new byte[i]);
    localXDRMessageIOV.copyData(localRPCOpaqueAuth.getInfo());
    localRPCAuthType.setValue(-1430533119);
    localRPCOpaqueAuth.setType(localRPCAuthType);
    return localRPCOpaqueAuth;
  }

  public synchronized RPCOpaqueAuth getFinalAuthData(RPCOpaqueAuth paramRPCOpaqueAuth, XDRMessageIOV paramXDRMessageIOV)
    throws RPCError
  {
    int i = paramXDRMessageIOV.getMessageSize();
    int j = 0;
    int k = 0;
    this.m_Sha.reset();
    this.m_Sha.update(this.m_Password);
    while (j < i)
    {
      byte[] arrayOfByte2 = paramXDRMessageIOV.getBuffer(k);
      int m = Math.min(arrayOfByte2.length, i - j);
      this.m_Sha.update(arrayOfByte2, 0, m);
      k++;
      j += m;
    }
    byte[] arrayOfByte1 = this.m_Sha.digest();
    this.m_Sha.reset();
    this.m_Sha.update(this.m_Password);
    this.m_Sha.update(arrayOfByte1);
    arrayOfByte1 = this.m_Sha.digest();
    int n = paramRPCOpaqueAuth.getInfo().length - 20;
    System.arraycopy(arrayOfByte1, 0, paramRPCOpaqueAuth.getInfo(), n, 20);
    return paramRPCOpaqueAuth;
  }

  public void setConfigGeneration(long paramLong)
  {
    this.m_ConfigGeneration = paramLong;
  }

  public void setPassword(String paramString)
  {
    this.m_Password = UnicodeTranslator.getBytes(paramString);
  }
}