package org.eclipse.jdt.internal.compiler.problem;

import java.io.CharConversionException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.MessageFormat;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.IErrorHandlingPolicy;
import org.eclipse.jdt.internal.compiler.IProblemFactory;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AbstractVariableDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.AnnotationMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Argument;
import org.eclipse.jdt.internal.compiler.ast.ArrayAllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.ArrayQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.ArrayReference;
import org.eclipse.jdt.internal.compiler.ast.ArrayTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Assignment;
import org.eclipse.jdt.internal.compiler.ast.BinaryExpression;
import org.eclipse.jdt.internal.compiler.ast.Block;
import org.eclipse.jdt.internal.compiler.ast.BranchStatement;
import org.eclipse.jdt.internal.compiler.ast.CaseStatement;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.CompoundAssignment;
import org.eclipse.jdt.internal.compiler.ast.ConditionalExpression;
import org.eclipse.jdt.internal.compiler.ast.ConstructorDeclaration;
import org.eclipse.jdt.internal.compiler.ast.EqualExpression;
import org.eclipse.jdt.internal.compiler.ast.ExplicitConstructorCall;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldReference;
import org.eclipse.jdt.internal.compiler.ast.ImportReference;
import org.eclipse.jdt.internal.compiler.ast.Initializer;
import org.eclipse.jdt.internal.compiler.ast.InstanceOfExpression;
import org.eclipse.jdt.internal.compiler.ast.IntLiteral;
import org.eclipse.jdt.internal.compiler.ast.LabeledStatement;
import org.eclipse.jdt.internal.compiler.ast.Literal;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.LongLiteral;
import org.eclipse.jdt.internal.compiler.ast.MemberValuePair;
import org.eclipse.jdt.internal.compiler.ast.MessageSend;
import org.eclipse.jdt.internal.compiler.ast.MethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.NameReference;
import org.eclipse.jdt.internal.compiler.ast.NumberLiteral;
import org.eclipse.jdt.internal.compiler.ast.ParameterizedQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.ParameterizedSingleTypeReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedAllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Reference;
import org.eclipse.jdt.internal.compiler.ast.ReturnStatement;
import org.eclipse.jdt.internal.compiler.ast.SingleNameReference;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.ast.SwitchStatement;
import org.eclipse.jdt.internal.compiler.ast.ThisReference;
import org.eclipse.jdt.internal.compiler.ast.TryStatement;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.ast.UnaryExpression;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticArgumentBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
import org.eclipse.jdt.internal.compiler.parser.Parser;
import org.eclipse.jdt.internal.compiler.parser.RecoveryScanner;
import org.eclipse.jdt.internal.compiler.parser.Scanner;
import org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
import org.eclipse.jdt.internal.compiler.util.Messages;

public class ProblemReporter extends ProblemHandler
{
  public ReferenceContext referenceContext;
  private Scanner positionScanner;

  public static long getIrritant(int problemID)
  {
    switch (problemID)
    {
    case 16777381:
      return 8L;
    case 268435844:
      return 1024L;
    case 67108974:
      return 1L;
    case 67109274:
      return 2L;
    case 67109277:
    case 67109278:
      return 16384L;
    case 16777221:
    case 33554505:
    case 67108967:
    case 67109276:
    case 134217861:
      return 4L;
    case 536870973:
      return 16L;
    case 536870974:
      return 32L;
    case 536871063:
      return 64L;
    case 33554622:
    case 33554623:
    case 67109056:
    case 67109057:
      return 128L;
    case 536871173:
    case 536871177:
      return 256L;
    case 536871352:
      return 512L;
    case 536871353:
      return 8589934592L;
    case 570425420:
    case 603979893:
      return 2048L;
    case 553648146:
    case 570425422:
    case 603979895:
      return 268435456L;
    case 536871090:
      return 8192L;
    case 553648135:
    case 570425421:
    case 603979894:
    case 603979910:
      return 32768L;
    case 536871002:
    case 536871006:
    case 536871007:
    case 570425435:
      return 65536L;
    case 570425436:
    case 570425437:
      return 131072L;
    case 16777249:
    case 16777787:
    case 16777792:
    case 16777793:
      return 549755813888L;
    case 536871091:
      return 262144L;
    case 536871092:
    case 553648316:
      return 524288L;
    case 536871372:
      return 134217728L;
    case 553648309:
    case 553648311:
      return 67108864L;
    case 536871096:
      return 16777216L;
    case 536871097:
    case 536871098:
      return 8388608L;
    case 570425423:
      return 4194304L;
    case 536871101:
      return 536870912L;
    case 16777746:
    case 16777747:
    case 16777748:
    case 16777752:
    case 16777761:
    case 16777785:
    case 16777786:
    case 67109423:
    case 67109438:
      return 1073741824L;
    case 16777788:
      return 35184372088832L;
    case 67109491:
      return 1099511627776L;
    case 536871540:
    case 536871541:
    case 536871542:
      return 4398046511104L;
    case 16777753:
      return 2147483648L;
    case 536871008:
      return 4294967296L;
    case 16777523:
      return 17179869184L;
    case 16777496:
      return 8796093022208L;
    case 67109665:
    case 134218530:
      return 34359738368L;
    case 536871363:
      return 68719476736L;
    case 536871364:
      return 1125899906842624L;
    case 536871365:
    case 536871366:
    case 536871367:
    case 536871368:
    case 536871369:
    case 536871370:
      return 2251799813685248L;
    case 536871632:
    case 536871633:
      return 137438953472L;
    case 33555193:
      return 2199023255552L;
    case 16777842:
      return 274877906944L;
    case 536871543:
      return 17592186044416L;
    case 536871111:
      return 70368744177664L;
    case -1610612272:
    case -1610612271:
    case -1610612270:
    case -1610612269:
    case -1610612268:
    case -1610612267:
    case -1610612266:
    case -1610612264:
    case -1610612263:
    case -1610612262:
    case -1610612260:
    case -1610612258:
    case -1610612257:
    case -1610612256:
    case -1610612255:
    case -1610612254:
    case -1610612253:
    case -1610612252:
    case -1610612251:
    case -1610612249:
    case -1610612248:
    case -1610612247:
    case -1610612246:
    case -1610612245:
    case -1610612244:
    case -1610612243:
    case -1610612242:
    case -1610612241:
    case -1610612240:
    case -1610612239:
    case -1610612238:
    case -1610612237:
    case -1610612236:
    case -1610612235:
    case -1610612234:
    case -1610612233:
    case -1610612232:
    case -1610612231:
    case -1610612230:
    case -1610612229:
    case -1610612228:
    case -1610612227:
    case -1610612226:
    case -1610612225:
    case -1610612224:
    case -1610612223:
    case -1610612221:
    case -1610612220:
    case -1610612219:
    case -1610612218:
    case -1610612217:
    case -1610611886:
    case -1610611885:
    case -1610611884:
    case -1610611883:
    case -1610611882:
    case -1610611881:
    case -1610611880:
    case -1610611879:
    case -1610611878:
    case -1610611877:
      return 33554432L;
    case -1610612265:
    case -1610612261:
    case -1610612259:
      return 2097152L;
    case -1610612250:
      return 1048576L;
    case 536870971:
      return 140737488355328L;
    case 536871106:
      return 281474976710656L;
    case 67109280:
      return 562949953421312L;
    }
    return 0L;
  }

  public static int getProblemCategory(int severity, int problemID)
  {
    if ((severity & 0x80) == 0)
    {
      long irritant = getIrritant(problemID);
      int irritantInt = (int)irritant;
      if (irritantInt == irritant) {
        switch (irritantInt) {
        case 1:
        case 128:
        case 512:
        case 2048:
        case 4194304:
        case 134217728:
        case 268435456:
          return 80;
        case 8:
        case 64:
        case 8192:
        case 262144:
        case 524288:
        case 16777216:
          return 90;
        case 2:
        case 16384:
        case 65536:
        case 131072:
          return 100;
        case 16:
        case 32:
        case 1024:
        case 32768:
        case 8388608:
        case 67108864:
        case 536870912:
          return 120;
        case 4:
          return 110;
        case 256:
          return 140;
        case 4096:
          return 0;
        case 1048576:
        case 2097152:
        case 33554432:
        case 33554436:
          return 70;
        case 1073741824:
          return 130;
        }

      }
      else
      {
        irritantInt = (int)(irritant >>> 32);
        switch (irritantInt) {
        case 0:
        case 2:
        case 32:
        case 64:
        case 256:
        case 1024:
        case 32768:
          return 80;
        case 1:
        case 8:
        case 16:
        case 512:
        case 65536:
        case 131072:
        case 262144:
        case 524288:
          return 90;
        case 128:
          return 100;
        case 4096:
        case 16384:
          return 120;
        case 4:
        case 2048:
          return 150;
        case 8192:
          return 130;
        }

      }

    }

    switch (problemID) {
    case 16777540:
    case 536871612:
      return 10;
    }

    if ((problemID & 0x40000000) != 0)
      return 20;
    if ((problemID & 0x10000000) != 0)
      return 30;
    if ((problemID & 0x1000000) != 0)
      return 40;
    if ((problemID & 0xE000000) != 0) {
      return 50;
    }
    return 60;
  }
  public ProblemReporter(IErrorHandlingPolicy policy, CompilerOptions options, IProblemFactory problemFactory) {
    super(policy, options, problemFactory);
  }
  public void abortDueToInternalError(String errorMessage) {
    abortDueToInternalError(errorMessage, null);
  }
  public void abortDueToInternalError(String errorMessage, ASTNode location) {
    String[] arguments = { errorMessage };
    handle(
      0, 
      arguments, 
      arguments, 
      159, 
      location == null ? 0 : location.sourceStart, 
      location == null ? 0 : location.sourceEnd);
  }

  public void abstractMethodCannotBeOverridden(SourceTypeBinding type, MethodBinding concreteMethod) {
    handle(
      67109275, 
      new String[] { 
      new String(type.sourceName()), 
      new String(
      CharOperation.concat(
      concreteMethod.declaringClass.readableName(), 
      concreteMethod.readableName(), 
      '.')) }, 
      new String[] { 
      new String(type.sourceName()), 
      new String(
      CharOperation.concat(
      concreteMethod.declaringClass.shortReadableName(), 
      concreteMethod.shortReadableName(), 
      '.')) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }

  public void abstractMethodInAbstractClass(SourceTypeBinding type, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { new String(type.sourceName()), new String(methodDecl.selector) };
    handle(
      67109227, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void abstractMethodMustBeImplemented(SourceTypeBinding type, MethodBinding abstractMethod) {
    handle(
      67109264, 
      new String[] { 
      new String(abstractMethod.selector), 
      typesAsString(abstractMethod.isVarargs(), abstractMethod.parameters, false), 
      new String(abstractMethod.declaringClass.readableName()), 
      new String(type.readableName()) }, 
      new String[] { 
      new String(abstractMethod.selector), 
      typesAsString(abstractMethod.isVarargs(), abstractMethod.parameters, true), 
      new String(abstractMethod.declaringClass.shortReadableName()), 
      new String(type.shortReadableName()) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void abstractMethodNeedingNoBody(AbstractMethodDeclaration method) {
    handle(
      603979889, 
      NoArgument, 
      NoArgument, 
      method.sourceStart, 
      method.sourceEnd, 
      method, 
      method.compilationResult());
  }
  public void alreadyDefinedLabel(char[] labelName, ASTNode location) {
    String[] arguments = { new String(labelName) };
    handle(
      536871083, 
      arguments, 
      arguments, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void annotationCannotOverrideMethod(MethodBinding overrideMethod, MethodBinding inheritedMethod) {
    ASTNode location = overrideMethod.sourceMethod();
    handle(
      67109480, 
      new String[] { 
      new String(overrideMethod.declaringClass.readableName()), 
      new String(inheritedMethod.declaringClass.readableName()), 
      new String(inheritedMethod.selector), 
      typesAsString(inheritedMethod.isVarargs(), inheritedMethod.parameters, false) }, 
      new String[] { 
      new String(overrideMethod.declaringClass.shortReadableName()), 
      new String(inheritedMethod.declaringClass.shortReadableName()), 
      new String(inheritedMethod.selector), 
      typesAsString(inheritedMethod.isVarargs(), inheritedMethod.parameters, true) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void annotationCircularity(TypeBinding sourceType, TypeBinding otherType, TypeReference reference) {
    if (sourceType == otherType)
      handle(
        16777822, 
        new String[] { new String(sourceType.readableName()) }, 
        new String[] { new String(sourceType.shortReadableName()) }, 
        reference.sourceStart, 
        reference.sourceEnd);
    else
      handle(
        16777823, 
        new String[] { new String(sourceType.readableName()), new String(otherType.readableName()) }, 
        new String[] { new String(sourceType.shortReadableName()), new String(otherType.shortReadableName()) }, 
        reference.sourceStart, 
        reference.sourceEnd); 
  }

  public void annotationMembersCannotHaveParameters(AnnotationMethodDeclaration annotationMethodDeclaration) { handle(
      1610613353, 
      NoArgument, 
      NoArgument, 
      annotationMethodDeclaration.sourceStart, 
      annotationMethodDeclaration.sourceEnd); }

  public void annotationMembersCannotHaveTypeParameters(AnnotationMethodDeclaration annotationMethodDeclaration) {
    handle(
      1610613354, 
      NoArgument, 
      NoArgument, 
      annotationMethodDeclaration.sourceStart, 
      annotationMethodDeclaration.sourceEnd);
  }
  public void annotationTypeDeclarationCannotHaveConstructor(ConstructorDeclaration constructorDeclaration) {
    handle(
      1610613360, 
      NoArgument, 
      NoArgument, 
      constructorDeclaration.sourceStart, 
      constructorDeclaration.sourceEnd);
  }
  public void annotationTypeDeclarationCannotHaveSuperclass(TypeDeclaration typeDeclaration) {
    handle(
      1610613355, 
      NoArgument, 
      NoArgument, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }
  public void annotationTypeDeclarationCannotHaveSuperinterfaces(TypeDeclaration typeDeclaration) {
    handle(
      1610613356, 
      NoArgument, 
      NoArgument, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }
  public void annotationTypeUsedAsSuperinterface(SourceTypeBinding type, TypeReference superInterfaceRef, ReferenceBinding superType) {
    handle(
      16777842, 
      new String[] { new String(superType.readableName()), new String(type.sourceName()) }, 
      new String[] { new String(superType.shortReadableName()), new String(type.sourceName()) }, 
      superInterfaceRef.sourceStart, 
      superInterfaceRef.sourceEnd);
  }
  public void annotationValueMustBeAnnotation(TypeBinding annotationType, char[] name, Expression value, TypeBinding expectedType) {
    String str = new String(name);
    handle(
      536871537, 
      new String[] { new String(annotationType.readableName()), str, new String(expectedType.readableName()) }, 
      new String[] { new String(annotationType.shortReadableName()), str, new String(expectedType.readableName()) }, 
      value.sourceStart, 
      value.sourceEnd);
  }
  public void annotationValueMustBeArrayInitializer(TypeBinding annotationType, char[] name, Expression value) {
    String str = new String(name);
    handle(
      536871544, 
      new String[] { new String(annotationType.readableName()), str }, 
      new String[] { new String(annotationType.shortReadableName()), str }, 
      value.sourceStart, 
      value.sourceEnd);
  }
  public void annotationValueMustBeClassLiteral(TypeBinding annotationType, char[] name, Expression value) {
    String str = new String(name);
    handle(
      536871524, 
      new String[] { new String(annotationType.readableName()), str }, 
      new String[] { new String(annotationType.shortReadableName()), str }, 
      value.sourceStart, 
      value.sourceEnd);
  }
  public void annotationValueMustBeConstant(TypeBinding annotationType, char[] name, Expression value, boolean isEnum) {
    String str = new String(name);
    if (isEnum)
      handle(
        536871545, 
        new String[] { new String(annotationType.readableName()), str }, 
        new String[] { new String(annotationType.shortReadableName()), str }, 
        value.sourceStart, 
        value.sourceEnd);
    else
      handle(
        536871525, 
        new String[] { new String(annotationType.readableName()), str }, 
        new String[] { new String(annotationType.shortReadableName()), str }, 
        value.sourceStart, 
        value.sourceEnd);
  }

  public void anonymousClassCannotExtendFinalClass(Expression expression, TypeBinding type) {
    handle(
      16777245, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void argumentTypeCannotBeVoid(SourceTypeBinding type, AbstractMethodDeclaration methodDecl, Argument arg) {
    String[] arguments = { new String(methodDecl.selector), new String(arg.name) };
    handle(
      67109228, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void argumentTypeCannotBeVoidArray(Argument arg) {
    handle(
      536870966, 
      NoArgument, 
      NoArgument, 
      arg.type.sourceStart, 
      arg.type.sourceEnd);
  }
  public void arrayConstantsOnlyInArrayInitializers(int sourceStart, int sourceEnd) {
    handle(
      1610612944, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void assignmentHasNoEffect(AbstractVariableDeclaration location, char[] name) {
    int severity = computeSeverity(536871090);
    if (severity == -1) return;
    String[] arguments = { new String(name) };
    int start = location.sourceStart;
    int end = location.sourceEnd;
    if (location.initialization != null) {
      end = location.initialization.sourceEnd;
    }
    handle(
      536871090, 
      arguments, 
      arguments, 
      severity, 
      start, 
      end);
  }
  public void assignmentHasNoEffect(Assignment location, char[] name) {
    int severity = computeSeverity(536871090);
    if (severity == -1) return;
    String[] arguments = { new String(name) };
    handle(
      536871090, 
      arguments, 
      arguments, 
      severity, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void attemptToReturnNonVoidExpression(ReturnStatement returnStatement, TypeBinding expectedType) {
    handle(
      67108969, 
      new String[] { new String(expectedType.readableName()) }, 
      new String[] { new String(expectedType.shortReadableName()) }, 
      returnStatement.sourceStart, 
      returnStatement.sourceEnd);
  }
  public void attemptToReturnVoidValue(ReturnStatement returnStatement) {
    handle(
      67108970, 
      NoArgument, 
      NoArgument, 
      returnStatement.sourceStart, 
      returnStatement.sourceEnd);
  }
  public void autoboxing(Expression expression, TypeBinding originalType, TypeBinding convertedType) {
    if (this.options.getSeverity(137438953472L) == -1) return;
    handle(
      originalType.isBaseType() ? 536871632 : 536871633, 
      new String[] { new String(originalType.readableName()), new String(convertedType.readableName()) }, 
      new String[] { new String(originalType.shortReadableName()), new String(convertedType.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void boundCannotBeArray(ASTNode location, TypeBinding type) {
    handle(
      16777784, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void boundMustBeAnInterface(ASTNode location, TypeBinding type) {
    handle(
      16777745, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void bytecodeExceeds64KLimit(AbstractMethodDeclaration location) {
    MethodBinding method = location.binding;
    if (location.isConstructor())
      handle(
        536870981, 
        new String[] { new String(location.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { new String(location.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
        159, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        536870975, 
        new String[] { new String(location.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { new String(location.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
        159, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void bytecodeExceeds64KLimit(TypeDeclaration location) {
    handle(
      536870976, 
      NoArgument, 
      NoArgument, 
      159, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void cannotAllocateVoidArray(Expression expression) {
    handle(
      536870966, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void cannotAssignToFinalField(FieldBinding field, ASTNode location) {
    handle(
      33554512, 
      new String[] { 
      field.declaringClass == null ? "array" : new String(field.declaringClass.readableName()), 
      new String(field.readableName()) }, 
      new String[] { 
      field.declaringClass == null ? "array" : new String(field.declaringClass.shortReadableName()), 
      new String(field.shortReadableName()) }, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void cannotAssignToFinalLocal(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.readableName()) };
    handle(
      536870970, 
      arguments, 
      arguments, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void cannotAssignToFinalOuterLocal(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.readableName()) };
    handle(
      536870972, 
      arguments, 
      arguments, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void cannotDefineDimensionsAndInitializer(ArrayAllocationExpression expresssion) {
    handle(
      536871070, 
      NoArgument, 
      NoArgument, 
      expresssion.sourceStart, 
      expresssion.sourceEnd);
  }
  public void cannotDireclyInvokeAbstractMethod(MessageSend messageSend, MethodBinding method) {
    handle(
      67108968, 
      new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
      messageSend.sourceStart, 
      messageSend.sourceEnd);
  }
  public void cannotExtendEnum(SourceTypeBinding type, TypeReference superclass, TypeBinding superTypeBinding) {
    String name = new String(type.sourceName());
    String superTypeFullName = new String(superTypeBinding.readableName());
    String superTypeShortName = new String(superTypeBinding.shortReadableName());
    if (superTypeShortName.equals(name)) superTypeShortName = superTypeFullName;
    handle(
      16777972, 
      new String[] { superTypeFullName, name }, 
      new String[] { superTypeShortName, name }, 
      superclass.sourceStart, 
      superclass.sourceEnd);
  }
  public void cannotImportPackage(ImportReference importRef) {
    String[] arguments = { CharOperation.toString(importRef.tokens) };
    handle(
      268435843, 
      arguments, 
      arguments, 
      importRef.sourceStart, 
      importRef.sourceEnd);
  }
  public void cannotInstantiate(TypeReference typeRef, TypeBinding type) {
    handle(
      16777373, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      typeRef.sourceStart, 
      typeRef.sourceEnd);
  }
  public void cannotInvokeSuperConstructorInEnum(ExplicitConstructorCall constructorCall, MethodBinding enumConstructor) {
    handle(
      67109621, 
      new String[] { 
      new String(enumConstructor.declaringClass.sourceName()), 
      typesAsString(enumConstructor.isVarargs(), enumConstructor.parameters, false) }, 
      new String[] { 
      new String(enumConstructor.declaringClass.sourceName()), 
      typesAsString(enumConstructor.isVarargs(), enumConstructor.parameters, true) }, 
      constructorCall.sourceStart, 
      constructorCall.sourceEnd);
  }
  public void cannotReadSource(CompilationUnitDeclaration unit, AbortCompilationUnit abortException, boolean verbose) {
    String fileName = new String(unit.compilationResult.fileName);
    if ((abortException.exception instanceof CharConversionException))
    {
      String encoding = abortException.encoding;
      if (encoding == null) {
        encoding = System.getProperty("file.encoding");
      }
      String[] arguments = { fileName, encoding };
      handle(
        536871613, 
        arguments, 
        arguments, 
        0, 
        0);
      return;
    }
    StringWriter stringWriter = new StringWriter();
    PrintWriter writer = new PrintWriter(stringWriter);
    if (verbose) {
      abortException.exception.printStackTrace(writer);
    } else {
      writer.print(abortException.exception.getClass().getName());
      writer.print(':');
      writer.print(abortException.exception.getMessage());
    }
    String exceptionTrace = stringWriter.toString();
    String[] arguments = { fileName, exceptionTrace };
    handle(
      536871614, 
      arguments, 
      arguments, 
      0, 
      0);
  }
  public void cannotReferToNonFinalOuterLocal(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.readableName()) };
    handle(
      536870937, 
      arguments, 
      arguments, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void cannotReturnInInitializer(ASTNode location) {
    handle(
      536871074, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void cannotThrowNull(ASTNode expression) {
    handle(
      536871089, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void cannotThrowType(ASTNode exception, TypeBinding expectedType) {
    handle(
      16777536, 
      new String[] { new String(expectedType.readableName()) }, 
      new String[] { new String(expectedType.shortReadableName()) }, 
      exception.sourceStart, 
      exception.sourceEnd);
  }
  public void cannotUseQualifiedEnumConstantInCaseLabel(Reference location, FieldBinding field) {
    handle(
      33555187, 
      new String[] { String.valueOf(field.declaringClass.readableName()), String.valueOf(field.name) }, 
      new String[] { String.valueOf(field.declaringClass.shortReadableName()), String.valueOf(field.name) }, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void cannotUseSuperInCodeSnippet(int start, int end) {
    handle(
      536871334, 
      NoArgument, 
      NoArgument, 
      159, 
      start, 
      end);
  }
  public void cannotUseSuperInJavaLangObject(ASTNode reference) {
    handle(
      16777217, 
      NoArgument, 
      NoArgument, 
      reference.sourceStart, 
      reference.sourceEnd);
  }
  public void caseExpressionMustBeConstant(Expression expression) {
    handle(
      536871065, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void classExtendFinalClass(SourceTypeBinding type, TypeReference superclass, TypeBinding superTypeBinding) {
    String name = new String(type.sourceName());
    String superTypeFullName = new String(superTypeBinding.readableName());
    String superTypeShortName = new String(superTypeBinding.shortReadableName());
    if (superTypeShortName.equals(name)) superTypeShortName = superTypeFullName;
    handle(
      16777529, 
      new String[] { superTypeFullName, name }, 
      new String[] { superTypeShortName, name }, 
      superclass.sourceStart, 
      superclass.sourceEnd);
  }
  public void codeSnippetMissingClass(String missing, int start, int end) {
    String[] arguments = { missing };
    handle(
      536871332, 
      arguments, 
      arguments, 
      159, 
      start, 
      end);
  }
  public void codeSnippetMissingMethod(String className, String missingMethod, String argumentTypes, int start, int end) {
    String[] arguments = { className, missingMethod, argumentTypes };
    handle(
      536871333, 
      arguments, 
      arguments, 
      159, 
      start, 
      end);
  }

  public int computeSeverity(int problemID)
  {
    switch (problemID) {
    case 536871362:
      return 0;
    case 67109667:
      return 0;
    case 16777538:
      return 0;
    case -1610612270:
    case -1610612269:
    case -1610612268:
    case -1610612264:
    case -1610612263:
    case -1610612262:
    case -1610612258:
    case -1610612256:
    case -1610612255:
    case -1610612254:
    case -1610612248:
    case -1610612246:
    case -1610612244:
    case -1610612242:
    case -1610612240:
    case -1610612238:
    case -1610612236:
    case -1610612235:
    case -1610612234:
    case -1610612233:
    case -1610612231:
    case -1610612229:
    case -1610612228:
    case -1610612227:
    case -1610612226:
    case -1610612225:
    case -1610612220:
    case -1610612219:
    case -1610611886:
    case -1610611885:
    case -1610611884:
    case -1610611883:
    case -1610611882:
    case -1610611881:
    case -1610611880:
    case -1610611879:
    case -1610611878:
    case -1610611877:
      if (!this.options.reportInvalidJavadocTags) {
        return -1;
      }

      break;
    case -1610612245:
    case -1610612241:
    case -1610612237:
    case -1610612230:
      if ((!this.options.reportInvalidJavadocTags) || (!this.options.reportInvalidJavadocTagsDeprecatedRef)) {
        return -1;
      }

      break;
    case -1610612271:
    case -1610612247:
    case -1610612243:
    case -1610612239:
    case -1610612232:
      if ((!this.options.reportInvalidJavadocTags) || (!this.options.reportInvalidJavadocTagsNotVisibleRef)) {
        return -1;
      }
      break;
    }
    long irritant = getIrritant(problemID);
    if (irritant != 0L) {
      if (((problemID & 0x80000000) != 0) && (!this.options.docCommentSupport))
        return -1;
      return this.options.getSeverity(irritant);
    }
    return 129;
  }
  public void conditionalArgumentsIncompatibleTypes(ConditionalExpression expression, TypeBinding trueType, TypeBinding falseType) {
    handle(
      16777232, 
      new String[] { new String(trueType.readableName()), new String(falseType.readableName()) }, 
      new String[] { new String(trueType.sourceName()), new String(falseType.sourceName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void conflictingImport(ImportReference importRef) {
    String[] arguments = { CharOperation.toString(importRef.tokens) };
    handle(
      268435841, 
      arguments, 
      arguments, 
      importRef.sourceStart, 
      importRef.sourceEnd);
  }

  public void constantOutOfFormat(NumberLiteral literal)
  {
    if (((literal instanceof LongLiteral)) || ((literal instanceof IntLiteral))) {
      char[] source = literal.source();
      try
      {
        String Radix;
        int radix;
        String Radix;
        if ((source[1] == 'x') || (source[1] == 'X')) {
          int radix = 16;
          Radix = "Hex";
        } else {
          radix = 8;
          Radix = "Octal";
        }

        int place = -1;
        for (int i = radix == 8 ? 1 : 2; i < source.length; i++) {
          if (ScannerHelper.digit(source[i], radix) == -1) {
            place = i;
            break;
          }
        }
        String[] arguments = { 
          new String(literal.literalType(null).readableName()), 
          Radix + " " + new String(source) + " (digit " + new String(new char[] { source[place] }) + ")" };

        handle(
          536871066, 
          arguments, 
          arguments, 
          literal.sourceStart, 
          literal.sourceEnd);
        return;
      }
      catch (IndexOutOfBoundsException localIndexOutOfBoundsException)
      {
        constantOutOfRange(literal, literal.literalType(null)); } 
    }
  }

  public void constantOutOfRange(Literal literal, TypeBinding literalType) { String[] arguments = { new String(literalType.readableName()), new String(literal.source()) };
    handle(
      536871066, 
      arguments, 
      arguments, 
      literal.sourceStart, 
      literal.sourceEnd); }

  public void corruptedSignature(TypeBinding enclosingType, char[] signature, int position) {
    handle(
      536871612, 
      new String[] { new String(enclosingType.readableName()), new String(signature), String.valueOf(position) }, 
      new String[] { new String(enclosingType.shortReadableName()), new String(signature), String.valueOf(position) }, 
      159, 
      0, 
      0);
  }
  public void deprecatedField(FieldBinding field, ASTNode location) {
    int severity = computeSeverity(33554505);
    if (severity == -1) return;
    handle(
      33554505, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      severity, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void deprecatedMethod(MethodBinding method, ASTNode location) {
    boolean isConstructor = method.isConstructor();
    int severity = computeSeverity(isConstructor ? 134217861 : 67108967);
    if (severity == -1) return;
    if (isConstructor)
      handle(
        134217861, 
        new String[] { new String(method.declaringClass.readableName()), typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { new String(method.declaringClass.shortReadableName()), typesAsString(method.isVarargs(), method.parameters, true) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        67108967, 
        new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void deprecatedType(TypeBinding type, ASTNode location) {
    if (location == null) return;
    int severity = computeSeverity(16777221);
    if (severity == -1) return;
    type = type.leafComponentType();
    handle(
      16777221, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      severity, 
      location.sourceStart, 
      nodeSourceEnd(null, location));
  }
  public void disallowedTargetForAnnotation(Annotation annotation) {
    handle(
      16777838, 
      new String[] { new String(annotation.resolvedType.readableName()) }, 
      new String[] { new String(annotation.resolvedType.shortReadableName()) }, 
      annotation.sourceStart, 
      annotation.sourceEnd);
  }
  public void duplicateAnnotation(Annotation annotation) {
    handle(
      16777824, 
      new String[] { new String(annotation.resolvedType.readableName()) }, 
      new String[] { new String(annotation.resolvedType.shortReadableName()) }, 
      annotation.sourceStart, 
      annotation.sourceEnd);
  }
  public void duplicateAnnotationValue(TypeBinding annotationType, MemberValuePair memberValuePair) {
    String name = new String(memberValuePair.name);
    handle(
      536871522, 
      new String[] { name, new String(annotationType.readableName()) }, 
      new String[] { name, new String(annotationType.shortReadableName()) }, 
      memberValuePair.sourceStart, 
      memberValuePair.sourceEnd);
  }
  public void duplicateBounds(ASTNode location, TypeBinding type) {
    handle(
      16777783, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void duplicateCase(CaseStatement caseStatement) {
    handle(
      33554602, 
      NoArgument, 
      NoArgument, 
      caseStatement.sourceStart, 
      caseStatement.sourceEnd);
  }
  public void duplicateDefaultCase(ASTNode statement) {
    handle(
      536871078, 
      NoArgument, 
      NoArgument, 
      statement.sourceStart, 
      statement.sourceEnd);
  }

  public void duplicateEnumSpecialMethod(SourceTypeBinding type, AbstractMethodDeclaration methodDecl) {
    MethodBinding method = methodDecl.binding;
    handle(
      67109618, 
      new String[] { 
      new String(methodDecl.selector), 
      new String(method.declaringClass.readableName()), 
      typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { 
      new String(methodDecl.selector), 
      new String(method.declaringClass.shortReadableName()), 
      typesAsString(method.isVarargs(), method.parameters, true) }, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void duplicateFieldInType(SourceTypeBinding type, FieldDeclaration fieldDecl) {
    handle(
      33554772, 
      new String[] { new String(type.sourceName()), new String(fieldDecl.name) }, 
      new String[] { new String(type.shortReadableName()), new String(fieldDecl.name) }, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }

  public void duplicateImport(ImportReference importRef) {
    String[] arguments = { CharOperation.toString(importRef.tokens) };
    handle(
      268435842, 
      arguments, 
      arguments, 
      importRef.sourceStart, 
      importRef.sourceEnd);
  }
  public void duplicateInheritedMethods(SourceTypeBinding type, MethodBinding inheritedMethod1, MethodBinding inheritedMethod2) {
    handle(
      67109429, 
      new String[] { 
      new String(inheritedMethod1.selector), 
      new String(inheritedMethod1.declaringClass.readableName()), 
      typesAsString(inheritedMethod1.isVarargs(), inheritedMethod1.original().parameters, false), 
      typesAsString(inheritedMethod2.isVarargs(), inheritedMethod2.original().parameters, false) }, 
      new String[] { 
      new String(inheritedMethod1.selector), 
      new String(inheritedMethod1.declaringClass.shortReadableName()), 
      typesAsString(inheritedMethod1.isVarargs(), inheritedMethod1.original().parameters, true), 
      typesAsString(inheritedMethod2.isVarargs(), inheritedMethod2.original().parameters, true) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void duplicateInitializationOfBlankFinalField(FieldBinding field, Reference reference) {
    String[] arguments = { new String(field.readableName()) };
    handle(
      33554514, 
      arguments, 
      arguments, 
      nodeSourceStart(field, reference), 
      nodeSourceEnd(field, reference));
  }
  public void duplicateInitializationOfFinalLocal(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.readableName()) };
    handle(
      536870969, 
      arguments, 
      arguments, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }

  public void duplicateMethodInType(SourceTypeBinding type, AbstractMethodDeclaration methodDecl) {
    MethodBinding method = methodDecl.binding;
    boolean duplicateErasure = false;
    if ((method.modifiers & 0x40000000) != 0)
    {
      int i = 0; for (int length = method.parameters.length; i < length; i++) {
        if ((method.parameters[i].tagBits & 0x20000000) != 0L) {
          duplicateErasure = true;
          break;
        }
      }
    }
    if (duplicateErasure) {
      int length = method.parameters.length;
      TypeBinding[] erasures = new TypeBinding[length];
      for (int i = 0; i < length; i++) {
        erasures[i] = method.parameters[i].erasure();
      }
      handle(
        16777743, 
        new String[] { 
        new String(methodDecl.selector), 
        new String(method.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false), 
        typesAsString(method.isVarargs(), erasures, false) }, 
        new String[] { 
        new String(methodDecl.selector), 
        new String(method.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true), 
        typesAsString(method.isVarargs(), erasures, true) }, 
        methodDecl.sourceStart, 
        methodDecl.sourceEnd);
    } else {
      handle(
        67109219, 
        new String[] { 
        new String(methodDecl.selector), 
        new String(method.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(methodDecl.selector), 
        new String(method.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        methodDecl.sourceStart, 
        methodDecl.sourceEnd);
    }
  }

  public void duplicateModifierForField(ReferenceBinding type, FieldDeclaration fieldDecl)
  {
    String[] arguments = { new String(fieldDecl.name) };
    handle(
      33554773, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void duplicateModifierForMethod(ReferenceBinding type, AbstractMethodDeclaration methodDecl) {
    handle(
      67109221, 
      new String[] { new String(type.sourceName()), new String(methodDecl.selector) }, 
      new String[] { new String(type.shortReadableName()), new String(methodDecl.selector) }, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void duplicateModifierForType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777517, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void duplicateModifierForVariable(LocalDeclaration localDecl, boolean complainForArgument) {
    String[] arguments = { new String(localDecl.name) };
    handle(
      complainForArgument ? 
      67109232 : 
      67109259, 
      arguments, 
      arguments, 
      localDecl.sourceStart, 
      localDecl.sourceEnd);
  }
  public void duplicateNestedType(TypeDeclaration typeDecl) {
    String[] arguments = { new String(typeDecl.name) };
    handle(
      16777535, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd);
  }
  public void duplicateSuperinterface(SourceTypeBinding type, TypeReference reference, ReferenceBinding superType) {
    handle(
      16777530, 
      new String[] { 
      new String(superType.readableName()), 
      new String(type.sourceName()) }, 
      new String[] { 
      new String(superType.shortReadableName()), 
      new String(type.sourceName()) }, 
      reference.sourceStart, 
      reference.sourceEnd);
  }
  public void duplicateTargetInTargetAnnotation(TypeBinding annotationType, NameReference reference) {
    FieldBinding field = reference.fieldBinding();
    String name = new String(field.name);
    handle(
      536871533, 
      new String[] { name, new String(annotationType.readableName()) }, 
      new String[] { name, new String(annotationType.shortReadableName()) }, 
      nodeSourceStart(field, reference), 
      nodeSourceEnd(field, reference));
  }
  public void duplicateTypeParameterInType(TypeParameter typeParameter) {
    handle(
      536871432, 
      new String[] { new String(typeParameter.name) }, 
      new String[] { new String(typeParameter.name) }, 
      typeParameter.sourceStart, 
      typeParameter.sourceEnd);
  }
  public void duplicateTypes(CompilationUnitDeclaration compUnitDecl, TypeDeclaration typeDecl) {
    String[] arguments = { new String(compUnitDecl.getFileName()), new String(typeDecl.name) };
    this.referenceContext = typeDecl;
    handle(
      16777539, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd, 
      compUnitDecl.compilationResult);
  }
  public void emptyControlFlowStatement(int sourceStart, int sourceEnd) {
    handle(
      553648316, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void enumAbstractMethodMustBeImplemented(AbstractMethodDeclaration method) {
    MethodBinding abstractMethod = method.binding;
    handle(
      67109622, 
      new String[] { 
      new String(abstractMethod.selector), 
      typesAsString(abstractMethod.isVarargs(), abstractMethod.parameters, false), 
      new String(abstractMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(abstractMethod.selector), 
      typesAsString(abstractMethod.isVarargs(), abstractMethod.parameters, true), 
      new String(abstractMethod.declaringClass.shortReadableName()) }, 
      method.sourceStart(), 
      method.sourceEnd());
  }
  public void enumConstantsCannotBeSurroundedByParenthesis(Expression expression) {
    handle(
      1610613178, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void enumStaticFieldUsedDuringInitialization(FieldBinding field, ASTNode location) {
    handle(
      33555194, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void enumSwitchCannotTargetField(Reference reference, FieldBinding field) {
    handle(
      33555191, 
      new String[] { String.valueOf(field.declaringClass.readableName()), String.valueOf(field.name) }, 
      new String[] { String.valueOf(field.declaringClass.shortReadableName()), String.valueOf(field.name) }, 
      nodeSourceStart(field, reference), 
      nodeSourceEnd(field, reference));
  }
  public void errorNoMethodFor(MessageSend messageSend, TypeBinding recType, TypeBinding[] params) {
    StringBuffer buffer = new StringBuffer();
    StringBuffer shortBuffer = new StringBuffer();
    int i = 0; for (int length = params.length; i < length; i++) {
      if (i != 0) {
        buffer.append(", ");
        shortBuffer.append(", ");
      }
      buffer.append(new String(params[i].readableName()));
      shortBuffer.append(new String(params[i].shortReadableName()));
    }

    int id = recType.isArrayType() ? 67108980 : 67108978;
    handle(
      id, 
      new String[] { new String(recType.readableName()), new String(messageSend.selector), buffer.toString() }, 
      new String[] { new String(recType.shortReadableName()), new String(messageSend.selector), shortBuffer.toString() }, 
      messageSend.sourceStart, 
      messageSend.sourceEnd);
  }
  public void errorThisSuperInStatic(ASTNode reference) {
    String[] arguments = { reference.isSuper() ? "super" : "this" };
    handle(
      536871112, 
      arguments, 
      arguments, 
      reference.sourceStart, 
      reference.sourceEnd);
  }
  public void expressionShouldBeAVariable(Expression expression) {
    handle(
      1610612959, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void fieldHiding(FieldDeclaration fieldDecl, Binding hiddenVariable) {
    FieldBinding field = fieldDecl.binding;
    if ((CharOperation.equals(TypeConstants.SERIALVERSIONUID, field.name)) && 
      (field.isStatic()) && 
      (field.isFinal()) && 
      (TypeBinding.LONG == field.type)) {
      return;
    }
    if ((CharOperation.equals(TypeConstants.SERIALPERSISTENTFIELDS, field.name)) && 
      (field.isStatic()) && 
      (field.isFinal()) && 
      (field.type.dimensions() == 1) && 
      (CharOperation.equals(TypeConstants.CharArray_JAVA_IO_OBJECTSTREAMFIELD, field.type.leafComponentType().readableName()))) {
      return;
    }
    boolean isLocal = hiddenVariable instanceof LocalVariableBinding;
    int severity = computeSeverity(isLocal ? 570425436 : 570425437);
    if (severity == -1) return;
    if (isLocal) {
      handle(
        570425436, 
        new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
        new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
        severity, 
        nodeSourceStart(hiddenVariable, fieldDecl), 
        nodeSourceEnd(hiddenVariable, fieldDecl));
    } else if ((hiddenVariable instanceof FieldBinding)) {
      FieldBinding hiddenField = (FieldBinding)hiddenVariable;
      handle(
        570425437, 
        new String[] { new String(field.declaringClass.readableName()), new String(field.name), new String(hiddenField.declaringClass.readableName()) }, 
        new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name), new String(hiddenField.declaringClass.shortReadableName()) }, 
        severity, 
        nodeSourceStart(hiddenField, fieldDecl), 
        nodeSourceEnd(hiddenField, fieldDecl));
    }
  }

  public void fieldsOrThisBeforeConstructorInvocation(ThisReference reference) { handle(
      134217866, 
      NoArgument, 
      NoArgument, 
      reference.sourceStart, 
      reference.sourceEnd); }

  public void finallyMustCompleteNormally(Block finallyBlock) {
    handle(
      536871096, 
      NoArgument, 
      NoArgument, 
      finallyBlock.sourceStart, 
      finallyBlock.sourceEnd);
  }
  public void finalMethodCannotBeOverridden(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    handle(
      67109265, 
      new String[] { new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      currentMethod.sourceStart(), 
      currentMethod.sourceEnd());
  }
  public void finalVariableBound(TypeVariableBinding typeVariable, TypeReference typeRef) {
    int severity = computeSeverity(16777753);
    if (severity == -1) return;
    handle(
      16777753, 
      new String[] { new String(typeVariable.sourceName), new String(typeRef.resolvedType.readableName()) }, 
      new String[] { new String(typeVariable.sourceName), new String(typeRef.resolvedType.shortReadableName()) }, 
      severity, 
      typeRef.sourceStart, 
      typeRef.sourceEnd);
  }

  public void forbiddenReference(FieldBinding field, ASTNode location, String messageTemplate, int problemId) {
    handle(
      problemId, 
      new String[] { new String(field.readableName()) }, 
      new String[] { 
      MessageFormat.format(messageTemplate, 
      new String[] { 
      new String(field.shortReadableName()), 
      new String(field.declaringClass.shortReadableName()) }) }, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }

  public void forbiddenReference(MethodBinding method, ASTNode location, String messageTemplate, int problemId) {
    if (method.isConstructor())
      handle(
        problemId, 
        new String[] { new String(method.readableName()) }, 
        new String[] { 
        MessageFormat.format(messageTemplate, 
        new String[] { new String(method.shortReadableName()) }) }, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        problemId, 
        new String[] { new String(method.readableName()) }, 
        new String[] { 
        MessageFormat.format(messageTemplate, 
        new String[] { 
        new String(method.shortReadableName()), 
        new String(method.declaringClass.shortReadableName()) }) }, 
        location.sourceStart, 
        location.sourceEnd); 
  }

  public void forbiddenReference(TypeBinding type, ASTNode location, String messageTemplate, int problemId) { if (location == null) return;
    int severity = computeSeverity(problemId);
    if (severity == -1) return;

    handle(
      problemId, 
      new String[] { new String(type.readableName()) }, 
      new String[] { MessageFormat.format(messageTemplate, new String[] { new String(type.shortReadableName()) }) }, 
      severity, 
      location.sourceStart, 
      location.sourceEnd); }

  public void forwardReference(Reference reference, int indexInQualification, TypeBinding type) {
    handle(
      570425419, 
      NoArgument, 
      NoArgument, 
      reference.sourceStart, 
      reference.sourceEnd);
  }
  public void forwardTypeVariableReference(ASTNode location, TypeVariableBinding type) {
    handle(
      16777744, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void genericTypeCannotExtendThrowable(TypeDeclaration typeDecl) {
    ASTNode location = typeDecl.binding.isAnonymousType() ? typeDecl.allocation.type : typeDecl.superclass;
    handle(
      16777773, 
      new String[] { new String(typeDecl.binding.readableName()) }, 
      new String[] { new String(typeDecl.binding.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }

  private void handle(int problemId, String[] problemArguments, String[] messageArguments, int problemStartPosition, int problemEndPosition)
  {
    handle(
      problemId, 
      problemArguments, 
      messageArguments, 
      problemStartPosition, 
      problemEndPosition, 
      this.referenceContext, 
      this.referenceContext == null ? null : this.referenceContext.compilationResult());
    this.referenceContext = null;
  }

  private void handle(int problemId, String[] problemArguments, String[] messageArguments, int problemStartPosition, int problemEndPosition, CompilationResult unitResult)
  {
    handle(
      problemId, 
      problemArguments, 
      messageArguments, 
      problemStartPosition, 
      problemEndPosition, 
      this.referenceContext, 
      unitResult);
    this.referenceContext = null;
  }

  private void handle(int problemId, String[] problemArguments, String[] messageArguments, int severity, int problemStartPosition, int problemEndPosition)
  {
    handle(
      problemId, 
      problemArguments, 
      messageArguments, 
      severity, 
      problemStartPosition, 
      problemEndPosition, 
      this.referenceContext, 
      this.referenceContext == null ? null : this.referenceContext.compilationResult());
    this.referenceContext = null;
  }

  public void hiddenCatchBlock(ReferenceBinding exceptionType, ASTNode location) {
    handle(
      16777381, 
      new String[] { 
      new String(exceptionType.readableName()) }, 
      new String[] { 
      new String(exceptionType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }

  public void hierarchyCircularity(SourceTypeBinding sourceType, ReferenceBinding superType, TypeReference reference) {
    int start = 0;
    int end = 0;

    if (reference == null) {
      start = sourceType.sourceStart();
      end = sourceType.sourceEnd();
    } else {
      start = reference.sourceStart;
      end = reference.sourceEnd;
    }

    if (sourceType == superType)
      handle(
        16777532, 
        new String[] { new String(sourceType.readableName()) }, 
        new String[] { new String(sourceType.shortReadableName()) }, 
        start, 
        end);
    else
      handle(
        16777533, 
        new String[] { new String(sourceType.readableName()), new String(superType.readableName()) }, 
        new String[] { new String(sourceType.shortReadableName()), new String(superType.shortReadableName()) }, 
        start, 
        end);
  }

  public void hierarchyHasProblems(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777543, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalAbstractModifierCombinationForMethod(ReferenceBinding type, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { new String(type.sourceName()), new String(methodDecl.selector) };
    handle(
      67109226, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void illegalAccessFromTypeVariable(TypeVariableBinding variable, ASTNode location) {
    String[] arguments = { new String(variable.sourceName) };
    handle(
      16777791, 
      arguments, 
      arguments, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void illegalClassLiteralForTypeVariable(TypeVariableBinding variable, ASTNode location) {
    String[] arguments = { new String(variable.sourceName) };
    handle(
      16777774, 
      arguments, 
      arguments, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void illegalExtendedDimensions(AnnotationMethodDeclaration annotationTypeMemberDeclaration) {
    handle(
      67109465, 
      NoArgument, 
      NoArgument, 
      annotationTypeMemberDeclaration.sourceStart, 
      annotationTypeMemberDeclaration.sourceEnd);
  }
  public void illegalExtendedDimensions(Argument argument) {
    handle(
      1610613536, 
      NoArgument, 
      NoArgument, 
      argument.sourceStart, 
      argument.sourceEnd);
  }
  public void illegalGenericArray(TypeBinding leafComponentType, ASTNode location) {
    handle(
      16777751, 
      new String[] { new String(leafComponentType.readableName()) }, 
      new String[] { new String(leafComponentType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void illegalInstanceOfGenericType(TypeBinding checkedType, ASTNode location) {
    if (checkedType.isTypeVariable()) {
      handle(
        536871459, 
        new String[] { new String(checkedType.readableName()), new String(checkedType.erasure().readableName()) }, 
        new String[] { new String(checkedType.shortReadableName()), new String(checkedType.erasure().shortReadableName()) }, 
        location.sourceStart, 
        location.sourceEnd);
      return;
    }
    handle(
      536871458, 
      new String[] { new String(checkedType.readableName()), new String(checkedType.erasure().sourceName()) }, 
      new String[] { new String(checkedType.shortReadableName()), new String(checkedType.erasure().sourceName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void illegalLocalTypeDeclaration(TypeDeclaration typeDeclaration) {
    if (isRecoveredName(typeDeclaration.name)) return;

    int problemID = 0;
    if ((typeDeclaration.modifiers & 0x4000) != 0)
      problemID = 536870943;
    else if ((typeDeclaration.modifiers & 0x2000) != 0)
      problemID = 536870942;
    else if ((typeDeclaration.modifiers & 0x200) != 0) {
      problemID = 536870938;
    }
    if (problemID != 0) {
      String[] arguments = { new String(typeDeclaration.name) };
      handle(
        problemID, 
        arguments, 
        arguments, 
        typeDeclaration.sourceStart, 
        typeDeclaration.sourceEnd);
    }
  }

  public void illegalModifierCombinationFinalAbstractForClass(SourceTypeBinding type) { String[] arguments = { new String(type.sourceName()) };
    handle(
      16777524, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd()); }

  public void illegalModifierCombinationFinalVolatileForField(ReferenceBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(fieldDecl.name) };

    handle(
      33554777, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void illegalModifierForAnnotationField(FieldDeclaration fieldDecl) {
    String name = new String(fieldDecl.name);
    handle(
      536871527, 
      new String[] { 
      new String(fieldDecl.binding.declaringClass.readableName()), 
      name }, 
      new String[] { 
      new String(fieldDecl.binding.declaringClass.shortReadableName()), 
      name }, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void illegalModifierForAnnotationMember(AbstractMethodDeclaration methodDecl) {
    handle(
      67109464, 
      new String[] { 
      new String(methodDecl.binding.declaringClass.readableName()), 
      new String(methodDecl.selector) }, 
      new String[] { 
      new String(methodDecl.binding.declaringClass.shortReadableName()), 
      new String(methodDecl.selector) }, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void illegalModifierForAnnotationMemberType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777820, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForAnnotationType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777819, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForClass(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777518, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForEnum(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777966, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForEnumConstant(ReferenceBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(fieldDecl.name) };
    handle(
      33555183, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }

  public void illegalModifierForEnumConstructor(AbstractMethodDeclaration constructor) {
    handle(
      67109624, 
      NoArgument, 
      NoArgument, 
      constructor.sourceStart, 
      constructor.sourceEnd);
  }
  public void illegalModifierForField(ReferenceBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(fieldDecl.name) };
    handle(
      33554774, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void illegalModifierForInterface(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777519, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }

  public void illegalModifierForInterfaceField(FieldDeclaration fieldDecl) {
    String name = new String(fieldDecl.name);
    handle(
      33554775, 
      new String[] { 
      new String(fieldDecl.binding.declaringClass.readableName()), 
      name }, 
      new String[] { 
      new String(fieldDecl.binding.declaringClass.shortReadableName()), 
      name }, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void illegalModifierForInterfaceMethod(AbstractMethodDeclaration methodDecl) {
    handle(
      67109223, 
      new String[] { 
      new String(methodDecl.binding.declaringClass.readableName()), 
      new String(methodDecl.selector), 
      typesAsString(methodDecl.binding.isVarargs(), methodDecl.binding.parameters, false) }, 
      new String[] { 
      new String(methodDecl.binding.declaringClass.shortReadableName()), 
      new String(methodDecl.selector), 
      typesAsString(methodDecl.binding.isVarargs(), methodDecl.binding.parameters, true) }, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void illegalModifierForLocalClass(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777522, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForLocalEnum(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777968, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForMemberClass(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777520, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForMemberEnum(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777969, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForMemberInterface(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777521, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalModifierForMethod(AbstractMethodDeclaration methodDecl) {
    handle(
      67109222, 
      new String[] { 
      new String(methodDecl.selector), 
      typesAsString(methodDecl.binding.isVarargs(), methodDecl.binding.parameters, false), 
      new String(methodDecl.binding.declaringClass.readableName()) }, 
      new String[] { 
      new String(methodDecl.selector), 
      typesAsString(methodDecl.binding.isVarargs(), methodDecl.binding.parameters, true), 
      new String(methodDecl.binding.declaringClass.shortReadableName()) }, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void illegalModifierForVariable(LocalDeclaration localDecl, boolean complainAsArgument) {
    String[] arguments = { new String(localDecl.name) };
    handle(
      complainAsArgument ? 
      67109220 : 
      67109260, 
      arguments, 
      arguments, 
      localDecl.sourceStart, 
      localDecl.sourceEnd);
  }
  public void illegalPrimitiveOrArrayTypeForEnclosingInstance(TypeBinding enclosingType, ASTNode location) {
    handle(
      16777243, 
      new String[] { new String(enclosingType.readableName()) }, 
      new String[] { new String(enclosingType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void illegalQualifiedParameterizedTypeAllocation(TypeReference qualifiedTypeReference, TypeBinding allocatedType) {
    handle(
      16777782, 
      new String[] { new String(allocatedType.readableName()), new String(allocatedType.enclosingType().readableName()) }, 
      new String[] { new String(allocatedType.shortReadableName()), new String(allocatedType.enclosingType().shortReadableName()) }, 
      qualifiedTypeReference.sourceStart, 
      qualifiedTypeReference.sourceEnd);
  }
  public void illegalStaticModifierForMemberType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777527, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalUsageOfQualifiedTypeReference(QualifiedTypeReference qualifiedTypeReference) {
    StringBuffer buffer = new StringBuffer();
    char[][] tokens = qualifiedTypeReference.tokens;
    for (int i = 0; i < tokens.length; i++) {
      if (i > 0) buffer.append('.');
      buffer.append(tokens[i]);
    }
    String[] arguments = { String.valueOf(buffer) };
    handle(
      1610612934, 
      arguments, 
      arguments, 
      qualifiedTypeReference.sourceStart, 
      qualifiedTypeReference.sourceEnd);
  }
  public void illegalVararg(Argument argType, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { CharOperation.toString(argType.type.getTypeName()), new String(methodDecl.selector) };
    handle(
      67109279, 
      arguments, 
      arguments, 
      argType.sourceStart, 
      argType.sourceEnd);
  }
  public void illegalVisibilityModifierCombinationForField(ReferenceBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(fieldDecl.name) };
    handle(
      33554776, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void illegalVisibilityModifierCombinationForMemberType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777526, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalVisibilityModifierCombinationForMethod(ReferenceBinding type, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { new String(type.sourceName()), new String(methodDecl.selector) };
    handle(
      67109224, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void illegalVisibilityModifierForInterfaceMemberType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      16777525, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void illegalVoidExpression(ASTNode location) {
    handle(
      536871076, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void importProblem(ImportReference importRef, Binding expectedImport) {
    if ((expectedImport instanceof FieldBinding)) {
      int id = 33554502;
      FieldBinding field = (FieldBinding)expectedImport;
      String[] readableArguments = (String[])null;
      String[] shortArguments = (String[])null;
      switch (expectedImport.problemId()) {
      case 2:
        id = 33554503;
        readableArguments = new String[] { CharOperation.toString(importRef.tokens), new String(field.declaringClass.readableName()) };
        shortArguments = new String[] { CharOperation.toString(importRef.tokens), new String(field.declaringClass.shortReadableName()) };
        break;
      case 3:
        id = 33554504;
        readableArguments = new String[] { new String(field.readableName()) };
        shortArguments = new String[] { new String(field.readableName()) };
        break;
      case 8:
        id = 16777219;
        readableArguments = new String[] { new String(field.declaringClass.leafComponentType().readableName()) };
        shortArguments = new String[] { new String(field.declaringClass.leafComponentType().shortReadableName()) };
      case 4:
      case 5:
      case 6:
      case 7: } handle(
        id, 
        readableArguments, 
        shortArguments, 
        nodeSourceStart(field, importRef), 
        nodeSourceEnd(field, importRef));
      return;
    }

    if (expectedImport.problemId() == 1) {
      char[][] tokens = (expectedImport instanceof ProblemReferenceBinding) ? 
        ((ProblemReferenceBinding)expectedImport).compoundName : 
        importRef.tokens;
      String[] arguments = { CharOperation.toString(tokens) };
      handle(
        268435846, 
        arguments, 
        arguments, 
        importRef.sourceStart, 
        (int)importRef.sourcePositions[(tokens.length - 1)]);
      return;
    }
    if (expectedImport.problemId() == 14) {
      char[][] tokens = importRef.tokens;
      String[] arguments = { CharOperation.toString(tokens) };
      handle(
        268435847, 
        arguments, 
        arguments, 
        importRef.sourceStart, 
        (int)importRef.sourcePositions[(tokens.length - 1)]);
      return;
    }
    invalidType(importRef, (TypeBinding)expectedImport);
  }
  public void incompatibleExceptionInThrowsClause(SourceTypeBinding type, MethodBinding currentMethod, MethodBinding inheritedMethod, ReferenceBinding exceptionType) {
    if (type == currentMethod.declaringClass)
    {
      int id;
      int id;
      if ((currentMethod.declaringClass.isInterface()) && 
        (!inheritedMethod.isPublic()))
        id = 67109278;
      else {
        id = 67109266;
      }
      handle(
        id, 
        new String[] { 
        new String(exceptionType.sourceName()), 
        new String(
        CharOperation.concat(
        inheritedMethod.declaringClass.readableName(), 
        inheritedMethod.readableName(), 
        '.')) }, 
        new String[] { 
        new String(exceptionType.sourceName()), 
        new String(
        CharOperation.concat(
        inheritedMethod.declaringClass.shortReadableName(), 
        inheritedMethod.shortReadableName(), 
        '.')) }, 
        currentMethod.sourceStart(), 
        currentMethod.sourceEnd());
    } else {
      handle(
        67109267, 
        new String[] { 
        new String(exceptionType.sourceName()), 
        new String(
        CharOperation.concat(
        currentMethod.declaringClass.sourceName(), 
        currentMethod.readableName(), 
        '.')), 
        new String(
        CharOperation.concat(
        inheritedMethod.declaringClass.readableName(), 
        inheritedMethod.readableName(), 
        '.')) }, 
        new String[] { 
        new String(exceptionType.sourceName()), 
        new String(
        CharOperation.concat(
        currentMethod.declaringClass.sourceName(), 
        currentMethod.shortReadableName(), 
        '.')), 
        new String(
        CharOperation.concat(
        inheritedMethod.declaringClass.shortReadableName(), 
        inheritedMethod.shortReadableName(), 
        '.')) }, 
        type.sourceStart(), 
        type.sourceEnd());
    }
  }
  public void incompatibleReturnType(MethodBinding currentMethod, MethodBinding inheritedMethod) { StringBuffer methodSignature = new StringBuffer();
    methodSignature
      .append(inheritedMethod.declaringClass.readableName())
      .append('.')
      .append(inheritedMethod.readableName());

    StringBuffer shortSignature = new StringBuffer();
    shortSignature
      .append(inheritedMethod.declaringClass.shortReadableName())
      .append('.')
      .append(inheritedMethod.shortReadableName());

    ReferenceBinding declaringClass = currentMethod.declaringClass;
    int id;
    int id;
    if ((declaringClass.isInterface()) && 
      (!inheritedMethod.isPublic()))
      id = 67109277;
    else {
      id = 67109268;
    }
    AbstractMethodDeclaration method = currentMethod.sourceMethod();
    int sourceStart = 0;
    int sourceEnd = 0;
    if (method == null) {
      if ((declaringClass instanceof SourceTypeBinding)) {
        SourceTypeBinding sourceTypeBinding = (SourceTypeBinding)declaringClass;
        sourceStart = sourceTypeBinding.sourceStart();
        sourceEnd = sourceTypeBinding.sourceEnd();
      }
    } else if (method.isConstructor()) {
      sourceStart = method.sourceStart;
      sourceEnd = method.sourceEnd;
    } else {
      TypeReference returnType = ((MethodDeclaration)method).returnType;
      sourceStart = returnType.sourceStart;
      if ((returnType instanceof ParameterizedSingleTypeReference)) {
        ParameterizedSingleTypeReference typeReference = (ParameterizedSingleTypeReference)returnType;
        TypeReference[] typeArguments = typeReference.typeArguments;
        if (typeArguments[(typeArguments.length - 1)].sourceEnd > typeReference.sourceEnd)
          sourceEnd = retrieveClosingAngleBracketPosition(typeReference.sourceEnd);
        else
          sourceEnd = returnType.sourceEnd;
      }
      else if ((returnType instanceof ParameterizedQualifiedTypeReference)) {
        ParameterizedQualifiedTypeReference typeReference = (ParameterizedQualifiedTypeReference)returnType;
        sourceEnd = retrieveClosingAngleBracketPosition(typeReference.sourceEnd);
      } else {
        sourceEnd = returnType.sourceEnd;
      }
    }
    handle(
      id, 
      new String[] { methodSignature.toString() }, 
      new String[] { shortSignature.toString() }, 
      sourceStart, 
      sourceEnd); }

  public void incorrectArityForParameterizedType(ASTNode location, TypeBinding type, TypeBinding[] argumentTypes) {
    if (location == null) {
      handle(
        16777741, 
        new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false) }, 
        new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true) }, 
        131, 
        0, 
        0);
      return;
    }
    handle(
      16777741, 
      new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false) }, 
      new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void incorrectLocationForNonEmptyDimension(ArrayAllocationExpression expression, int index) {
    handle(
      536871114, 
      NoArgument, 
      NoArgument, 
      expression.dimensions[index].sourceStart, 
      expression.dimensions[index].sourceEnd);
  }
  public void incorrectSwitchType(Expression expression, TypeBinding testType) {
    handle(
      16777385, 
      new String[] { new String(testType.readableName()) }, 
      new String[] { new String(testType.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void indirectAccessToStaticField(ASTNode location, FieldBinding field) {
    int severity = computeSeverity(570425422);
    if (severity == -1) return;
    handle(
      570425422, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      severity, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void indirectAccessToStaticMethod(ASTNode location, MethodBinding method) {
    int severity = computeSeverity(603979895);
    if (severity == -1) return;
    handle(
      603979895, 
      new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
      severity, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void inheritedMethodReducesVisibility(SourceTypeBinding type, MethodBinding concreteMethod, MethodBinding[] abstractMethods) {
    StringBuffer concreteSignature = new StringBuffer();
    concreteSignature
      .append(concreteMethod.declaringClass.readableName())
      .append('.')
      .append(concreteMethod.readableName());
    StringBuffer shortSignature = new StringBuffer();
    shortSignature
      .append(concreteMethod.declaringClass.shortReadableName())
      .append('.')
      .append(concreteMethod.shortReadableName());
    handle(
      67109269, 
      new String[] { 
      concreteSignature.toString(), 
      new String(abstractMethods[0].declaringClass.readableName()) }, 
      new String[] { 
      new String(shortSignature.toString()), 
      new String(abstractMethods[0].declaringClass.shortReadableName()) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void inheritedMethodsHaveIncompatibleReturnTypes(ASTNode location, MethodBinding[] inheritedMethods, int length) {
    StringBuffer methodSignatures = new StringBuffer();
    StringBuffer shortSignatures = new StringBuffer();
    int i = length;
    do { methodSignatures
        .append(inheritedMethods[i].declaringClass.readableName())
        .append('.')
        .append(inheritedMethods[i].readableName());
      shortSignatures
        .append(inheritedMethods[i].declaringClass.shortReadableName())
        .append('.')
        .append(inheritedMethods[i].shortReadableName());
      if (i != 0) {
        methodSignatures.append(", ");
        shortSignatures.append(", ");
      }
      i--; } while (i >= 0);

    handle(
      67109268, 
      new String[] { methodSignatures.toString() }, 
      new String[] { shortSignatures.toString() }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void inheritedMethodsHaveIncompatibleReturnTypes(SourceTypeBinding type, MethodBinding[] inheritedMethods, int length) {
    StringBuffer methodSignatures = new StringBuffer();
    StringBuffer shortSignatures = new StringBuffer();
    int i = length;
    do { methodSignatures
        .append(inheritedMethods[i].declaringClass.readableName())
        .append('.')
        .append(inheritedMethods[i].readableName());
      shortSignatures
        .append(inheritedMethods[i].declaringClass.shortReadableName())
        .append('.')
        .append(inheritedMethods[i].shortReadableName());
      if (i != 0) {
        methodSignatures.append(", ");
        shortSignatures.append(", ");
      }
      i--; } while (i >= 0);

    handle(
      67109268, 
      new String[] { methodSignatures.toString() }, 
      new String[] { shortSignatures.toString() }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void inheritedMethodsHaveNameClash(SourceTypeBinding type, MethodBinding oneMethod, MethodBinding twoMethod) {
    handle(
      67109424, 
      new String[] { 
      new String(oneMethod.selector), 
      typesAsString(oneMethod.original().isVarargs(), oneMethod.original().parameters, false), 
      new String(oneMethod.declaringClass.readableName()), 
      typesAsString(twoMethod.original().isVarargs(), twoMethod.original().parameters, false), 
      new String(twoMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(oneMethod.selector), 
      typesAsString(oneMethod.original().isVarargs(), oneMethod.original().parameters, true), 
      new String(oneMethod.declaringClass.shortReadableName()), 
      typesAsString(twoMethod.original().isVarargs(), twoMethod.original().parameters, true), 
      new String(twoMethod.declaringClass.shortReadableName()) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void initializerMustCompleteNormally(FieldDeclaration fieldDecl) {
    handle(
      536871075, 
      NoArgument, 
      NoArgument, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void innerTypesCannotDeclareStaticInitializers(ReferenceBinding innerType, Initializer initializer) {
    handle(
      536870936, 
      new String[] { new String(innerType.readableName()) }, 
      new String[] { new String(innerType.shortReadableName()) }, 
      initializer.sourceStart, 
      initializer.sourceStart);
  }
  public void interfaceCannotHaveConstructors(ConstructorDeclaration constructor) {
    handle(
      1610612943, 
      NoArgument, 
      NoArgument, 
      constructor.sourceStart, 
      constructor.sourceEnd, 
      constructor, 
      constructor.compilationResult());
  }
  public void interfaceCannotHaveInitializers(SourceTypeBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(type.sourceName()) };

    handle(
      16777516, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void invalidAnnotationMemberType(MethodDeclaration methodDecl) {
    handle(
      16777821, 
      new String[] { 
      new String(methodDecl.binding.returnType.readableName()), 
      new String(methodDecl.selector), 
      new String(methodDecl.binding.declaringClass.readableName()) }, 
      new String[] { 
      new String(methodDecl.binding.returnType.shortReadableName()), 
      new String(methodDecl.selector), 
      new String(methodDecl.binding.declaringClass.shortReadableName()) }, 
      methodDecl.returnType.sourceStart, 
      methodDecl.returnType.sourceEnd);
  }

  public void invalidBreak(ASTNode location) {
    handle(
      536871084, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void invalidConstructor(Statement statement, MethodBinding targetConstructor) {
    boolean insideDefaultConstructor = 
      ((this.referenceContext instanceof ConstructorDeclaration)) && 
      (((ConstructorDeclaration)this.referenceContext).isDefaultConstructor());
    boolean insideImplicitConstructorCall = 
      ((statement instanceof ExplicitConstructorCall)) && 
      (((ExplicitConstructorCall)statement).accessMode == 1);

    int sourceStart = statement.sourceStart;
    int sourceEnd = statement.sourceEnd;
    if ((statement instanceof AllocationExpression)) {
      AllocationExpression allocation = (AllocationExpression)statement;
      if (allocation.enumConstant != null) {
        sourceStart = allocation.enumConstant.sourceStart;
        sourceEnd = allocation.enumConstant.sourceEnd;
      }
    }

    int id = 134217858;
    MethodBinding shownConstructor = targetConstructor;
    switch (targetConstructor.problemId()) {
    case 1:
      if (insideDefaultConstructor)
        id = 134217868;
      else if (insideImplicitConstructorCall)
        id = 134217871;
      else {
        id = 134217858;
      }
      break;
    case 2:
      if (insideDefaultConstructor)
        id = 134217869;
      else if (insideImplicitConstructorCall)
        id = 134217872;
      else {
        id = 134217859;
      }
      ProblemMethodBinding problemConstructor = (ProblemMethodBinding)targetConstructor;
      if (problemConstructor.closestMatch != null) {
        shownConstructor = problemConstructor.closestMatch.original();
      }
      break;
    case 3:
      if (insideDefaultConstructor)
        id = 134217870;
      else if (insideImplicitConstructorCall)
        id = 134217873;
      else {
        id = 134217860;
      }
      break;
    case 10:
      ProblemMethodBinding problemConstructor = (ProblemMethodBinding)targetConstructor;
      ParameterizedGenericMethodBinding substitutedConstructor = (ParameterizedGenericMethodBinding)problemConstructor.closestMatch;
      shownConstructor = substitutedConstructor.original();
      int augmentedLength = problemConstructor.parameters.length;
      TypeBinding inferredTypeArgument = problemConstructor.parameters[(augmentedLength - 2)];
      TypeVariableBinding typeParameter = (TypeVariableBinding)problemConstructor.parameters[(augmentedLength - 1)];
      TypeBinding[] invocationArguments = new TypeBinding[augmentedLength - 2];
      System.arraycopy(problemConstructor.parameters, 0, invocationArguments, 0, augmentedLength - 2);
      handle(
        16777760, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(false, invocationArguments, false), 
        new String(inferredTypeArgument.readableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(false, invocationArguments, true), 
        new String(inferredTypeArgument.shortReadableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, true) }, 
        sourceStart, 
        sourceEnd);
      return;
    case 11:
      ProblemMethodBinding problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      if (shownConstructor.typeVariables == Binding.NO_TYPE_VARIABLES)
        handle(
          16777767, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
          new String(shownConstructor.declaringClass.readableName()), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
          new String(shownConstructor.declaringClass.shortReadableName()), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
          sourceStart, 
          sourceEnd);
      else {
        handle(
          16777768, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
          new String(shownConstructor.declaringClass.readableName()), 
          typesAsString(false, shownConstructor.typeVariables, false), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
          new String(shownConstructor.declaringClass.shortReadableName()), 
          typesAsString(false, shownConstructor.typeVariables, true), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
          sourceStart, 
          sourceEnd);
      }
      return;
    case 12:
      ProblemMethodBinding problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      handle(
        16777769, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownConstructor).typeArguments, false), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownConstructor).typeArguments, true), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
        sourceStart, 
        sourceEnd);
      return;
    case 13:
      ProblemMethodBinding problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      handle(
        16777771, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
        sourceStart, 
        sourceEnd);
      return;
    case 0:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    default:
      needImplementation();
    }

    handle(
      id, 
      new String[] { new String(targetConstructor.declaringClass.readableName()), typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false) }, 
      new String[] { new String(targetConstructor.declaringClass.shortReadableName()), typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true) }, 
      sourceStart, 
      sourceEnd);
  }
  public void invalidContinue(ASTNode location) {
    handle(
      536871085, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }

  public void invalidEnclosingType(Expression expression, TypeBinding type, ReferenceBinding enclosingType) {
    if (enclosingType.isAnonymousType()) enclosingType = enclosingType.superclass();
    if ((enclosingType.sourceName != null) && (enclosingType.sourceName.length == 0)) return;

    int flag = 16777218;
    switch (type.problemId()) {
    case 1:
      flag = 16777218;
      break;
    case 2:
      flag = 16777219;
      break;
    case 3:
      flag = 16777220;
      break;
    case 4:
      flag = 16777222;
      break;
    case 0:
    default:
      needImplementation();
    }

    handle(
      flag, 
      new String[] { new String(enclosingType.readableName()) + "." + new String(type.readableName()) }, 
      new String[] { new String(enclosingType.shortReadableName()) + "." + new String(type.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }

  public void invalidExplicitConstructorCall(ASTNode location) {
    handle(
      1207959691, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void invalidExpressionAsStatement(Expression expression) {
    handle(
      1610612958, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidField(FieldReference fieldRef, TypeBinding searchedType) {
    if (isRecoveredName(fieldRef.token)) return;

    int id = 33554502;
    FieldBinding field = fieldRef.binding;
    switch (field.problemId()) {
    case 1:
      id = 33554502;

      break;
    case 2:
      handle(
        33554503, 
        new String[] { new String(fieldRef.token), new String(field.declaringClass.readableName()) }, 
        new String[] { new String(fieldRef.token), new String(field.declaringClass.shortReadableName()) }, 
        nodeSourceStart(field, fieldRef), 
        nodeSourceEnd(field, fieldRef));
      return;
    case 3:
      id = 33554504;
      break;
    case 7:
      id = 33554506;
      break;
    case 6:
      id = 134217863;
      break;
    case 5:
      id = 33554628;
      break;
    case 8:
      handle(
        16777219, 
        new String[] { new String(searchedType.leafComponentType().readableName()) }, 
        new String[] { new String(searchedType.leafComponentType().shortReadableName()) }, 
        fieldRef.receiver.sourceStart, 
        fieldRef.receiver.sourceEnd);
      return;
    case 0:
    case 4:
    default:
      needImplementation();
    }

    String[] arguments = { 
      new String(field.readableName()) };
    handle(
      id, 
      arguments, 
      arguments, 
      nodeSourceStart(field, fieldRef), 
      nodeSourceEnd(field, fieldRef));
  }
  public void invalidField(NameReference nameRef, FieldBinding field) {
    if ((nameRef instanceof QualifiedNameReference)) { QualifiedNameReference ref = (QualifiedNameReference)nameRef;
      if (!isRecoveredName(ref.tokens));
    } else {
      SingleNameReference ref = (SingleNameReference)nameRef;
      if (isRecoveredName(ref.token)) return;
    }
    int id = 33554502;
    switch (field.problemId()) {
    case 1:
      id = 33554502;
      break;
    case 2:
      char[] name = field.readableName();
      name = CharOperation.lastSegment(name, '.');
      handle(
        33554503, 
        new String[] { new String(name), new String(field.declaringClass.readableName()) }, 
        new String[] { new String(name), new String(field.declaringClass.shortReadableName()) }, 
        nodeSourceStart(field, nameRef), 
        nodeSourceEnd(field, nameRef));
      return;
    case 3:
      id = 33554504;
      break;
    case 7:
      id = 33554506;
      break;
    case 6:
      id = 134217863;
      break;
    case 5:
      id = 33554628;
      break;
    case 8:
      handle(
        16777219, 
        new String[] { new String(field.declaringClass.leafComponentType().readableName()) }, 
        new String[] { new String(field.declaringClass.leafComponentType().shortReadableName()) }, 
        nameRef.sourceStart, 
        nameRef.sourceEnd);
      return;
    case 0:
    case 4:
    default:
      needImplementation();
    }

    String[] arguments = { 
      new String(field.readableName()) };
    handle(
      id, 
      arguments, 
      arguments, 
      nameRef.sourceStart, 
      nameRef.sourceEnd);
  }

  public void invalidField(QualifiedNameReference nameRef, FieldBinding field, int index, TypeBinding searchedType)
  {
    if (isRecoveredName(nameRef.tokens)) return;

    if (searchedType.isBaseType()) {
      handle(
        33554653, 
        new String[] { 
        new String(searchedType.readableName()), 
        CharOperation.toString(CharOperation.subarray(nameRef.tokens, 0, index)), 
        new String(nameRef.tokens[index]) }, 
        new String[] { 
        new String(searchedType.sourceName()), 
        CharOperation.toString(CharOperation.subarray(nameRef.tokens, 0, index)), 
        new String(nameRef.tokens[index]) }, 
        nameRef.sourceStart, 
        (int)nameRef.sourcePositions[index]);
      return;
    }

    int id = 33554502;
    switch (field.problemId()) {
    case 1:
      id = 33554502;

      break;
    case 2:
      String fieldName = new String(nameRef.tokens[index]);
      handle(
        33554503, 
        new String[] { fieldName, new String(field.declaringClass.readableName()) }, 
        new String[] { fieldName, new String(field.declaringClass.shortReadableName()) }, 
        nodeSourceStart(field, nameRef), 
        nodeSourceEnd(field, nameRef));
      return;
    case 3:
      id = 33554504;
      break;
    case 7:
      id = 33554506;
      break;
    case 6:
      id = 134217863;
      break;
    case 5:
      id = 33554628;
      break;
    case 8:
      handle(
        16777219, 
        new String[] { new String(searchedType.leafComponentType().readableName()) }, 
        new String[] { new String(searchedType.leafComponentType().shortReadableName()) }, 
        nameRef.sourceStart, 
        nameRef.sourceEnd);
      return;
    case 0:
    case 4:
    default:
      needImplementation();
    }

    String[] arguments = { 
      CharOperation.toString(CharOperation.subarray(nameRef.tokens, 0, index + 1)) };
    handle(
      id, 
      arguments, 
      arguments, 
      nameRef.sourceStart, 
      (int)nameRef.sourcePositions[index]);
  }
  public void invalidFileNameForPackageAnnotations(Annotation annotation) {
    handle(
      1610613338, 
      NoArgument, 
      NoArgument, 
      annotation.sourceStart, 
      annotation.sourceEnd);
  }

  public void invalidMethod(MessageSend messageSend, MethodBinding method) {
    if (isRecoveredName(messageSend.selector)) return;

    int id = 67108964;
    MethodBinding shownMethod = method;
    switch (method.problemId()) {
    case 1:
      id = 67108964;
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      if (problemMethod.closestMatch != null) { shownMethod = problemMethod.closestMatch;
        String closestParameterTypeNames = typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false);
        String parameterTypeNames = typesAsString(false, problemMethod.parameters, false);
        String closestParameterTypeShortNames = typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true);
        String parameterTypeShortNames = typesAsString(false, problemMethod.parameters, true);
        handle(
          67108979, 
          new String[] { 
          new String(shownMethod.declaringClass.readableName()), 
          new String(shownMethod.selector), 
          closestParameterTypeNames, 
          parameterTypeNames }, 
          new String[] { 
          new String(shownMethod.declaringClass.shortReadableName()), 
          new String(shownMethod.selector), 
          closestParameterTypeShortNames, 
          parameterTypeShortNames }, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
        return; } break;
    case 2:
      id = 67108965;
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      if (problemMethod.closestMatch != null) {
        shownMethod = problemMethod.closestMatch.original();
      }
      break;
    case 3:
      id = 67108966;
      break;
    case 5:
      id = 67109059;
      break;
    case 6:
      id = 134217864;
      break;
    case 7:
      id = 603979977;
      break;
    case 8:
      handle(
        16777219, 
        new String[] { new String(method.declaringClass.leafComponentType().readableName()) }, 
        new String[] { new String(method.declaringClass.leafComponentType().shortReadableName()) }, 
        messageSend.receiver.sourceStart, 
        messageSend.receiver.sourceEnd);
      return;
    case 10:
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      ParameterizedGenericMethodBinding substitutedMethod = (ParameterizedGenericMethodBinding)problemMethod.closestMatch;
      shownMethod = substitutedMethod.original();
      int augmentedLength = problemMethod.parameters.length;
      TypeBinding inferredTypeArgument = problemMethod.parameters[(augmentedLength - 2)];
      TypeVariableBinding typeParameter = (TypeVariableBinding)problemMethod.parameters[(augmentedLength - 1)];
      TypeBinding[] invocationArguments = new TypeBinding[augmentedLength - 2];
      System.arraycopy(problemMethod.parameters, 0, invocationArguments, 0, augmentedLength - 2);
      handle(
        16777759, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(false, invocationArguments, false), 
        new String(inferredTypeArgument.readableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(false, invocationArguments, true), 
        new String(inferredTypeArgument.shortReadableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, true) }, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 11:
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      if (shownMethod.typeVariables == Binding.NO_TYPE_VARIABLES)
        handle(
          16777764, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
          new String(shownMethod.declaringClass.readableName()), 
          typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
          new String(shownMethod.declaringClass.shortReadableName()), 
          typesAsString(method.isVarargs(), method.parameters, true) }, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
      else {
        handle(
          16777765, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
          new String(shownMethod.declaringClass.readableName()), 
          typesAsString(false, shownMethod.typeVariables, false), 
          typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
          new String(shownMethod.declaringClass.shortReadableName()), 
          typesAsString(false, shownMethod.typeVariables, true), 
          typesAsString(method.isVarargs(), method.parameters, true) }, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
      }
      return;
    case 12:
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      handle(
        16777766, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownMethod).typeArguments, false), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownMethod).typeArguments, true), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 13:
      ProblemMethodBinding problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      handle(
        16777770, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 0:
    case 4:
    case 9:
    default:
      needImplementation();
    }

    handle(
      id, 
      new String[] { 
      new String(method.declaringClass.readableName()), 
      new String(shownMethod.selector), typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false) }, 
      new String[] { 
      new String(method.declaringClass.shortReadableName()), 
      new String(shownMethod.selector), typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true) }, 
      (int)(messageSend.nameSourcePosition >>> 32), 
      (int)messageSend.nameSourcePosition);
  }

  public void invalidNullToSynchronize(Expression expression) {
    handle(
      536871088, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidOperator(BinaryExpression expression, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      536871072, 
      new String[] { 
      expression.operatorToString(), 
      leftName + ", " + rightName }, 
      new String[] { 
      expression.operatorToString(), 
      leftShortName + ", " + rightShortName }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidOperator(CompoundAssignment assign, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      536871072, 
      new String[] { 
      assign.operatorToString(), 
      leftName + ", " + rightName }, 
      new String[] { 
      assign.operatorToString(), 
      leftShortName + ", " + rightShortName }, 
      assign.sourceStart, 
      assign.sourceEnd);
  }
  public void invalidOperator(UnaryExpression expression, TypeBinding type) {
    handle(
      536871072, 
      new String[] { expression.operatorToString(), new String(type.readableName()) }, 
      new String[] { expression.operatorToString(), new String(type.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidParameterizedExceptionType(TypeBinding exceptionType, ASTNode location) {
    handle(
      16777750, 
      new String[] { new String(exceptionType.readableName()) }, 
      new String[] { new String(exceptionType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void invalidParenthesizedExpression(ASTNode reference) {
    handle(
      1610612961, 
      NoArgument, 
      NoArgument, 
      reference.sourceStart, 
      reference.sourceEnd);
  }
  public void invalidType(ASTNode location, TypeBinding type) {
    if ((type instanceof ReferenceBinding)) {
      if (!isRecoveredName(((ReferenceBinding)type).compoundName));
    }
    else if ((type instanceof ArrayBinding)) {
      TypeBinding leafType = ((ArrayBinding)type).leafComponentType;
      if (((leafType instanceof ReferenceBinding)) && 
        (isRecoveredName(((ReferenceBinding)leafType).compoundName))) return;

    }

    int id = 16777218;
    switch (type.problemId()) {
    case 1:
      id = 16777218;
      break;
    case 2:
      id = 16777219;
      break;
    case 3:
      id = 16777220;
      break;
    case 4:
      id = 16777222;
      break;
    case 5:
      id = 16777413;
      break;
    case 7:
      id = 536871434;
      break;
    case 9:
      id = 536871433;
      break;
    case 0:
    case 6:
    case 8:
    default:
      needImplementation();
    }

    int end = location.sourceEnd;

    if ((location instanceof QualifiedNameReference)) {
      QualifiedNameReference ref = (QualifiedNameReference)location;
      if (isRecoveredName(ref.tokens)) return;
      if (ref.indexOfFirstFieldBinding >= 1)
        end = (int)ref.sourcePositions[(ref.indexOfFirstFieldBinding - 1)];
    } else if ((location instanceof ParameterizedQualifiedTypeReference))
    {
      ParameterizedQualifiedTypeReference ref = (ParameterizedQualifiedTypeReference)location;
      if (isRecoveredName(ref.tokens)) return;
      if ((type instanceof ReferenceBinding)) {
        char[][] name = ((ReferenceBinding)type).compoundName;
        end = (int)ref.sourcePositions[(name.length - 1)];
      }
    } else if ((location instanceof ArrayQualifiedTypeReference)) {
      ArrayQualifiedTypeReference arrayQualifiedTypeReference = (ArrayQualifiedTypeReference)location;
      if (isRecoveredName(arrayQualifiedTypeReference.tokens)) return;
      long[] positions = arrayQualifiedTypeReference.sourcePositions;
      end = (int)positions[(positions.length - 1)];
    } else if ((location instanceof QualifiedTypeReference)) {
      QualifiedTypeReference ref = (QualifiedTypeReference)location;
      if (isRecoveredName(ref.tokens)) return;
      if ((type instanceof ReferenceBinding)) {
        char[][] name = ((ReferenceBinding)type).compoundName;
        if (name.length <= ref.sourcePositions.length)
          end = (int)ref.sourcePositions[(name.length - 1)];
      }
    } else if ((location instanceof ImportReference)) {
      ImportReference ref = (ImportReference)location;
      if (isRecoveredName(ref.tokens)) return;
      if ((type instanceof ReferenceBinding)) {
        char[][] name = ((ReferenceBinding)type).compoundName;
        end = (int)ref.sourcePositions[(name.length - 1)];
      }
    } else if ((location instanceof ArrayTypeReference)) {
      ArrayTypeReference arrayTypeReference = (ArrayTypeReference)location;
      if (isRecoveredName(arrayTypeReference.token)) return;
      end = arrayTypeReference.originalSourceEnd;
    }
    handle(
      id, 
      new String[] { new String(type.leafComponentType().readableName()) }, 
      new String[] { new String(type.leafComponentType().shortReadableName()) }, 
      location.sourceStart, 
      end);
  }
  public void invalidTypeForCollection(Expression expression) {
    handle(
      536871493, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidTypeReference(Expression expression) {
    handle(
      536871115, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidTypeToSynchronize(Expression expression, TypeBinding type) {
    handle(
      536871087, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidTypeVariableAsException(TypeBinding exceptionType, ASTNode location) {
    handle(
      16777749, 
      new String[] { new String(exceptionType.readableName()) }, 
      new String[] { new String(exceptionType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void invalidUnaryExpression(Expression expression) {
    handle(
      1610612942, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void invalidUsageOfAnnotation(Annotation annotation) {
    handle(
      1610613332, 
      NoArgument, 
      NoArgument, 
      annotation.sourceStart, 
      annotation.sourceEnd);
  }
  public void invalidUsageOfAnnotationDeclarations(TypeDeclaration annotationTypeDeclaration) {
    handle(
      1610613333, 
      NoArgument, 
      NoArgument, 
      annotationTypeDeclaration.sourceStart, 
      annotationTypeDeclaration.sourceEnd);
  }
  public void invalidUsageOfEnumDeclarations(TypeDeclaration enumDeclaration) {
    handle(
      1610613330, 
      NoArgument, 
      NoArgument, 
      enumDeclaration.sourceStart, 
      enumDeclaration.sourceEnd);
  }
  public void invalidUsageOfForeachStatements(LocalDeclaration elementVariable, Expression collection) {
    handle(
      1610613328, 
      NoArgument, 
      NoArgument, 
      elementVariable.declarationSourceStart, 
      collection.sourceEnd);
  }
  public void invalidUsageOfStaticImports(ImportReference staticImport) {
    handle(
      1610613327, 
      NoArgument, 
      NoArgument, 
      staticImport.declarationSourceStart, 
      staticImport.declarationSourceEnd);
  }
  public void invalidUsageOfTypeArguments(TypeReference firstTypeReference, TypeReference lastTypeReference) {
    handle(
      1610613329, 
      NoArgument, 
      NoArgument, 
      firstTypeReference.sourceStart, 
      lastTypeReference.sourceEnd);
  }
  public void invalidUsageOfTypeParameters(TypeParameter firstTypeParameter, TypeParameter lastTypeParameter) {
    handle(
      1610613326, 
      NoArgument, 
      NoArgument, 
      firstTypeParameter.declarationSourceStart, 
      lastTypeParameter.declarationSourceEnd);
  }
  public void invalidUsageOfVarargs(Argument argument) {
    handle(
      1610613331, 
      NoArgument, 
      NoArgument, 
      argument.type.sourceStart, 
      argument.sourceEnd);
  }
  public void illegalUsageOfWildcard(TypeReference wildcard) {
    handle(
      1610613314, 
      NoArgument, 
      NoArgument, 
      wildcard.sourceStart, 
      wildcard.sourceEnd);
  }
  public void isClassPathCorrect(char[][] wellKnownTypeName, CompilationUnitDeclaration compUnitDecl, Object location) {
    this.referenceContext = compUnitDecl;
    String[] arguments = { CharOperation.toString(wellKnownTypeName) };
    int start = 0; int end = 0;
    if (location != null) {
      if ((location instanceof InvocationSite)) {
        InvocationSite site = (InvocationSite)location;
        start = site.sourceStart();
        end = site.sourceEnd();
      } else if ((location instanceof ASTNode)) {
        ASTNode node = (ASTNode)location;
        start = node.sourceStart();
        end = node.sourceEnd();
      }
    }
    handle(
      16777540, 
      arguments, 
      arguments, 
      start, 
      end);
  }
  private boolean isIdentifier(int token) {
    return token == 26;
  }
  private boolean isKeyword(int token) {
    switch (token) {
    case 15:
    case 32:
    case 33:
    case 34:
    case 35:
    case 36:
    case 37:
    case 38:
    case 39:
    case 40:
    case 41:
    case 42:
    case 43:
    case 44:
    case 45:
    case 46:
    case 54:
    case 55:
    case 56:
    case 57:
    case 58:
    case 59:
    case 60:
    case 61:
    case 62:
    case 63:
    case 64:
    case 72:
    case 73:
    case 74:
    case 75:
    case 76:
    case 77:
    case 78:
    case 79:
    case 80:
    case 81:
    case 82:
    case 83:
    case 95:
    case 96:
    case 97:
    case 99:
    case 100:
    case 101:
    case 102:
    case 103:
    case 104:
    case 105:
    case 106:
      return true;
    case 16:
    case 17:
    case 18:
    case 19:
    case 20:
    case 21:
    case 22:
    case 23:
    case 24:
    case 25:
    case 26:
    case 27:
    case 28:
    case 29:
    case 30:
    case 31:
    case 47:
    case 48:
    case 49:
    case 50:
    case 51:
    case 52:
    case 53:
    case 65:
    case 66:
    case 67:
    case 68:
    case 69:
    case 70:
    case 71:
    case 84:
    case 85:
    case 86:
    case 87:
    case 88:
    case 89:
    case 90:
    case 91:
    case 92:
    case 93:
    case 94:
    case 98: } return false;
  }

  private boolean isLiteral(int token) {
    switch (token) {
    case 47:
    case 48:
    case 49:
    case 50:
    case 51:
    case 52:
      return true;
    }
    return false;
  }

  private boolean isRecoveredName(char[] simpleName) {
    return simpleName == RecoveryScanner.FAKE_IDENTIFIER;
  }
  private boolean isRecoveredName(char[][] qualifiedName) {
    if (qualifiedName == null) return false;

    for (int i = 0; i < qualifiedName.length; i++) {
      if (qualifiedName[i] == RecoveryScanner.FAKE_IDENTIFIER) return true;
    }

    return false;
  }
  public void javadocAmbiguousMethodReference(int sourceStart, int sourceEnd, Binding fieldBinding, int modifiers) {
    int severity = computeSeverity(-1610612225);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { new String(fieldBinding.readableName()) };
      handle(
        -1610612225, 
        arguments, 
        arguments, 
        severity, 
        sourceStart, 
        sourceEnd);
    }
  }

  public void javadocDeprecatedField(FieldBinding field, ASTNode location, int modifiers) {
    int severity = computeSeverity(-1610612245);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(
        -1610612245, 
        new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
        new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
        severity, 
        nodeSourceStart(field, location), 
        nodeSourceEnd(field, location));
  }

  public void javadocDeprecatedMethod(MethodBinding method, ASTNode location, int modifiers)
  {
    boolean isConstructor = method.isConstructor();
    int severity = computeSeverity(isConstructor ? -1610612241 : -1610612237);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      if (isConstructor)
        handle(
          -1610612241, 
          new String[] { new String(method.declaringClass.readableName()), typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { new String(method.declaringClass.shortReadableName()), typesAsString(method.isVarargs(), method.parameters, true) }, 
          severity, 
          location.sourceStart, 
          location.sourceEnd);
      else
        handle(
          -1610612237, 
          new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
          severity, 
          location.sourceStart, 
          location.sourceEnd);
  }

  public void javadocDeprecatedType(TypeBinding type, ASTNode location, int modifiers)
  {
    if (location == null) return;
    int severity = computeSeverity(-1610612230);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      if ((type.isMemberType()) && ((type instanceof ReferenceBinding)) && (!javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, ((ReferenceBinding)type).modifiers)))
        handle(-1610612271, NoArgument, NoArgument, location.sourceStart, location.sourceEnd);
      else
        handle(
          -1610612230, 
          new String[] { new String(type.readableName()) }, 
          new String[] { new String(type.shortReadableName()) }, 
          severity, 
          location.sourceStart, 
          location.sourceEnd);
  }

  public void javadocDuplicatedParamTag(char[] token, int sourceStart, int sourceEnd, int modifiers)
  {
    int severity = computeSeverity(-1610612263);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { String.valueOf(token) };
      handle(
        -1610612263, 
        arguments, 
        arguments, 
        severity, 
        sourceStart, 
        sourceEnd);
    }
  }

  public void javadocDuplicatedReturnTag(int sourceStart, int sourceEnd) { handle(-1610612260, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocDuplicatedTag(char[] tagName, int sourceStart, int sourceEnd) {
    String[] arguments = { new String(tagName) };
    handle(
      -1610612272, 
      arguments, 
      arguments, 
      sourceStart, 
      sourceEnd);
  }
  public void javadocDuplicatedThrowsClassName(TypeReference typeReference, int modifiers) {
    int severity = computeSeverity(-1610612256);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { String.valueOf(typeReference.resolvedType.sourceName()) };
      handle(
        -1610612256, 
        arguments, 
        arguments, 
        severity, 
        typeReference.sourceStart, 
        typeReference.sourceEnd);
    }
  }

  public void javadocEmptyReturnTag(int sourceStart, int sourceEnd, int modifiers) { int severity = computeSeverity(-1610612220);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612220, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocErrorNoMethodFor(MessageSend messageSend, TypeBinding recType, TypeBinding[] params, int modifiers)
  {
    int id = recType.isArrayType() ? -1610612234 : -1610612236;
    int severity = computeSeverity(id);
    if (severity == -1) return;
    StringBuffer buffer = new StringBuffer();
    StringBuffer shortBuffer = new StringBuffer();
    int i = 0; for (int length = params.length; i < length; i++) {
      if (i != 0) {
        buffer.append(", ");
        shortBuffer.append(", ");
      }
      buffer.append(new String(params[i].readableName()));
      shortBuffer.append(new String(params[i].shortReadableName()));
    }
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(
        id, 
        new String[] { new String(recType.readableName()), new String(messageSend.selector), buffer.toString() }, 
        new String[] { new String(recType.shortReadableName()), new String(messageSend.selector), shortBuffer.toString() }, 
        severity, 
        messageSend.sourceStart, 
        messageSend.sourceEnd);
  }

  public void javadocHiddenReference(int sourceStart, int sourceEnd, Scope scope, int modifiers) {
    Scope currentScope = scope;
    while (currentScope.parent.kind != 4) {
      if (!javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, currentScope.getDeclarationModifiers())) {
        return;
      }
      currentScope = currentScope.parent;
    }
    String[] arguments = { this.options.getVisibilityString(this.options.reportInvalidJavadocTagsVisibility), this.options.getVisibilityString(modifiers) };
    handle(-1610612271, arguments, arguments, sourceStart, sourceEnd);
  }

  public void javadocInvalidConstructor(Statement statement, MethodBinding targetConstructor, int modifiers) {
    if (!javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) return;
    int sourceStart = statement.sourceStart;
    int sourceEnd = statement.sourceEnd;
    if ((statement instanceof AllocationExpression)) {
      AllocationExpression allocation = (AllocationExpression)statement;
      if (allocation.enumConstant != null) {
        sourceStart = allocation.enumConstant.sourceStart;
        sourceEnd = allocation.enumConstant.sourceEnd;
      }
    }
    int id = -1610612244;
    ProblemMethodBinding problemConstructor = null;
    MethodBinding shownConstructor = null;
    switch (targetConstructor.problemId()) {
    case 1:
      id = -1610612244;
      break;
    case 2:
      id = -1610612243;
      break;
    case 3:
      id = -1610612242;
      break;
    case 10:
      int severity = computeSeverity(-1610611881);
      if (severity == -1) return;
      problemConstructor = (ProblemMethodBinding)targetConstructor;
      ParameterizedGenericMethodBinding substitutedConstructor = (ParameterizedGenericMethodBinding)problemConstructor.closestMatch;
      shownConstructor = substitutedConstructor.original();

      int augmentedLength = problemConstructor.parameters.length;
      TypeBinding inferredTypeArgument = problemConstructor.parameters[(augmentedLength - 2)];
      TypeVariableBinding typeParameter = (TypeVariableBinding)problemConstructor.parameters[(augmentedLength - 1)];
      TypeBinding[] invocationArguments = new TypeBinding[augmentedLength - 2];
      System.arraycopy(problemConstructor.parameters, 0, invocationArguments, 0, augmentedLength - 2);

      handle(
        -1610611881, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(false, invocationArguments, false), 
        new String(inferredTypeArgument.readableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(false, invocationArguments, true), 
        new String(inferredTypeArgument.shortReadableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, true) }, 
        severity, 
        sourceStart, 
        sourceEnd);
      return;
    case 11:
      problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      boolean noTypeVariables = shownConstructor.typeVariables == Binding.NO_TYPE_VARIABLES;
      int severity = computeSeverity(noTypeVariables ? -1610611880 : -1610611879);
      if (severity == -1) return;
      if (noTypeVariables)
        handle(
          -1610611880, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
          new String(shownConstructor.declaringClass.readableName()), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
          new String(shownConstructor.declaringClass.shortReadableName()), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
          severity, 
          sourceStart, 
          sourceEnd);
      else {
        handle(
          -1610611879, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
          new String(shownConstructor.declaringClass.readableName()), 
          typesAsString(false, shownConstructor.typeVariables, false), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
          new String[] { 
          new String(shownConstructor.declaringClass.sourceName()), 
          typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
          new String(shownConstructor.declaringClass.shortReadableName()), 
          typesAsString(false, shownConstructor.typeVariables, true), 
          typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
          severity, 
          sourceStart, 
          sourceEnd);
      }
      return;
    case 12:
      int severity = computeSeverity(-1610611878);
      if (severity == -1) return;
      problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      handle(
        -1610611878, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownConstructor).typeArguments, false), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownConstructor).typeArguments, true), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
        severity, 
        sourceStart, 
        sourceEnd);
      return;
    case 13:
      int severity = computeSeverity(-1610611877);
      if (severity == -1) return;
      problemConstructor = (ProblemMethodBinding)targetConstructor;
      shownConstructor = problemConstructor.closestMatch;
      handle(
        -1610611877, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, false), 
        new String(shownConstructor.declaringClass.readableName()), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
        new String[] { 
        new String(shownConstructor.declaringClass.sourceName()), 
        typesAsString(shownConstructor.isVarargs(), shownConstructor.parameters, true), 
        new String(shownConstructor.declaringClass.shortReadableName()), 
        typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
        severity, 
        sourceStart, 
        sourceEnd);
      return;
    case 0:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    default:
      needImplementation();
    }

    int severity = computeSeverity(id);
    if (severity == -1) return;
    handle(
      id, 
      new String[] { new String(targetConstructor.declaringClass.readableName()), typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, false) }, 
      new String[] { new String(targetConstructor.declaringClass.shortReadableName()), typesAsString(targetConstructor.isVarargs(), targetConstructor.parameters, true) }, 
      severity, 
      statement.sourceStart, 
      statement.sourceEnd);
  }

  public void javadocInvalidField(int sourceStart, int sourceEnd, Binding fieldBinding, TypeBinding searchedType, int modifiers)
  {
    int id = -1610612248;
    switch (fieldBinding.problemId()) {
    case 1:
      id = -1610612248;
      break;
    case 2:
      id = -1610612247;
      break;
    case 3:
      id = -1610612246;
      break;
    case 0:
    default:
      needImplementation();
    }

    int severity = computeSeverity(id);
    if (severity == -1) return;

    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { new String(fieldBinding.readableName()) };
      handle(
        id, 
        arguments, 
        arguments, 
        severity, 
        sourceStart, 
        sourceEnd);
    }
  }

  public void javadocInvalidMemberTypeQualification(int sourceStart, int sourceEnd, int modifiers) { if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612270, NoArgument, NoArgument, sourceStart, sourceEnd);
  }

  public void javadocInvalidMethod(MessageSend messageSend, MethodBinding method, int modifiers)
  {
    if (!javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) return;

    ProblemMethodBinding problemMethod = null;
    MethodBinding shownMethod = null;
    int id = -1610612240;
    switch (method.problemId()) {
    case 1:
      id = -1610612240;
      problemMethod = (ProblemMethodBinding)method;
      if (problemMethod.closestMatch != null) {
        int severity = computeSeverity(-1610612235);
        if (severity == -1) return;
        String closestParameterTypeNames = typesAsString(problemMethod.closestMatch.isVarargs(), problemMethod.closestMatch.parameters, false);
        String parameterTypeNames = typesAsString(method.isVarargs(), method.parameters, false);
        String closestParameterTypeShortNames = typesAsString(problemMethod.closestMatch.isVarargs(), problemMethod.closestMatch.parameters, true);
        String parameterTypeShortNames = typesAsString(method.isVarargs(), method.parameters, true);
        if (closestParameterTypeShortNames.equals(parameterTypeShortNames)) {
          closestParameterTypeShortNames = closestParameterTypeNames;
          parameterTypeShortNames = parameterTypeNames;
        }
        handle(
          -1610612235, 
          new String[] { 
          new String(problemMethod.closestMatch.declaringClass.readableName()), 
          new String(problemMethod.closestMatch.selector), 
          closestParameterTypeNames, 
          parameterTypeNames }, 
          new String[] { 
          new String(problemMethod.closestMatch.declaringClass.shortReadableName()), 
          new String(problemMethod.closestMatch.selector), 
          closestParameterTypeShortNames, 
          parameterTypeShortNames }, 
          severity, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
        return;
      }

      break;
    case 2:
      id = -1610612239;
      break;
    case 3:
      id = -1610612238;
      break;
    case 10:
      int severity = computeSeverity(-1610611886);
      if (severity == -1) return;
      problemMethod = (ProblemMethodBinding)method;
      ParameterizedGenericMethodBinding substitutedMethod = (ParameterizedGenericMethodBinding)problemMethod.closestMatch;
      shownMethod = substitutedMethod.original();
      int augmentedLength = problemMethod.parameters.length;
      TypeBinding inferredTypeArgument = problemMethod.parameters[(augmentedLength - 2)];
      TypeVariableBinding typeParameter = (TypeVariableBinding)problemMethod.parameters[(augmentedLength - 1)];
      TypeBinding[] invocationArguments = new TypeBinding[augmentedLength - 2];
      System.arraycopy(problemMethod.parameters, 0, invocationArguments, 0, augmentedLength - 2);
      handle(
        -1610611886, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(false, invocationArguments, false), 
        new String(inferredTypeArgument.readableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(false, invocationArguments, true), 
        new String(inferredTypeArgument.shortReadableName()), 
        new String(typeParameter.sourceName), 
        parameterBoundAsString(typeParameter, true) }, 
        severity, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 11:
      problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      boolean noTypeVariables = shownMethod.typeVariables == Binding.NO_TYPE_VARIABLES;
      int severity = computeSeverity(noTypeVariables ? -1610611885 : -1610611884);
      if (severity == -1) return;
      if (noTypeVariables)
        handle(
          -1610611885, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
          new String(shownMethod.declaringClass.readableName()), 
          typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
          new String(shownMethod.declaringClass.shortReadableName()), 
          typesAsString(method.isVarargs(), method.parameters, true) }, 
          severity, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
      else {
        handle(
          -1610611884, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
          new String(shownMethod.declaringClass.readableName()), 
          typesAsString(false, shownMethod.typeVariables, false), 
          typesAsString(method.isVarargs(), method.parameters, false) }, 
          new String[] { 
          new String(shownMethod.selector), 
          typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
          new String(shownMethod.declaringClass.shortReadableName()), 
          typesAsString(false, shownMethod.typeVariables, true), 
          typesAsString(method.isVarargs(), method.parameters, true) }, 
          severity, 
          (int)(messageSend.nameSourcePosition >>> 32), 
          (int)messageSend.nameSourcePosition);
      }
      return;
    case 12:
      int severity = computeSeverity(-1610611883);
      if (severity == -1) return;
      problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      handle(
        -1610611883, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownMethod).typeArguments, false), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(false, ((ParameterizedGenericMethodBinding)shownMethod).typeArguments, true), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        severity, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 13:
      int severity = computeSeverity(-1610611882);
      if (severity == -1) return;
      problemMethod = (ProblemMethodBinding)method;
      shownMethod = problemMethod.closestMatch;
      handle(
        -1610611882, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, false), 
        new String(shownMethod.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(shownMethod.selector), 
        typesAsString(shownMethod.isVarargs(), shownMethod.parameters, true), 
        new String(shownMethod.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        severity, 
        (int)(messageSend.nameSourcePosition >>> 32), 
        (int)messageSend.nameSourcePosition);
      return;
    case 0:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    default:
      needImplementation();
    }

    int severity = computeSeverity(id);
    if (severity == -1) return;

    handle(
      id, 
      new String[] { 
      new String(method.declaringClass.readableName()), 
      new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { 
      new String(method.declaringClass.shortReadableName()), 
      new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
      severity, 
      (int)(messageSend.nameSourcePosition >>> 32), 
      (int)messageSend.nameSourcePosition);
  }
  public void javadocInvalidParamTagName(int sourceStart, int sourceEnd) {
    handle(-1610612217, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidParamTypeParameter(int sourceStart, int sourceEnd) {
    handle(-1610612267, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidReference(int sourceStart, int sourceEnd) {
    handle(-1610612253, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidSeeReferenceArgs(int sourceStart, int sourceEnd) {
    handle(-1610612251, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidSeeUrlReference(int sourceStart, int sourceEnd) {
    handle(-1610612252, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidTag(int sourceStart, int sourceEnd) {
    handle(-1610612249, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidThrowsClass(int sourceStart, int sourceEnd) {
    handle(-1610612257, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocInvalidThrowsClassName(TypeReference typeReference, int modifiers) {
    int severity = computeSeverity(-1610612255);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { String.valueOf(typeReference.resolvedType.sourceName()) };
      handle(
        -1610612255, 
        arguments, 
        arguments, 
        severity, 
        typeReference.sourceStart, 
        typeReference.sourceEnd);
    }
  }

  public void javadocInvalidType(ASTNode location, TypeBinding type, int modifiers) { if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      int id = -1610612233;
      switch (type.problemId()) {
      case 1:
        id = -1610612233;
        break;
      case 2:
        id = -1610612232;
        break;
      case 3:
        id = -1610612231;
        break;
      case 4:
        id = -1610612229;
        break;
      case 5:
        id = -1610612226;
        break;
      case 7:
        id = -1610612268;
        break;
      case 0:
      case 6:
      default:
        needImplementation();
      }

      int severity = computeSeverity(id);
      if (severity == -1) return;
      handle(
        id, 
        new String[] { new String(type.readableName()) }, 
        new String[] { new String(type.shortReadableName()) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
    } }

  public void javadocInvalidValueReference(int sourceStart, int sourceEnd, int modifiers) {
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612219, NoArgument, NoArgument, sourceStart, sourceEnd); 
  }

  public void javadocMalformedSeeReference(int sourceStart, int sourceEnd) { handle(-1610612223, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocMissing(int sourceStart, int sourceEnd, int modifiers) {
    int severity = computeSeverity(-1610612250);
    if (severity == -1) return;
    boolean overriding = (modifiers & 0x30000000) != 0;
    boolean report = (this.options.getSeverity(1048576L) != -1) && (
      (!overriding) || (this.options.reportMissingJavadocCommentsOverriding));
    if (report) {
      String arg = javadocVisibilityArgument(this.options.reportMissingJavadocCommentsVisibility, modifiers);
      if (arg != null) {
        String[] arguments = { arg };
        handle(
          -1610612250, 
          arguments, 
          arguments, 
          severity, 
          sourceStart, 
          sourceEnd);
      }
    }
  }

  public void javadocMissingHashCharacter(int sourceStart, int sourceEnd, String ref) { int severity = computeSeverity(-1610612221);
    if (severity == -1) return;
    String[] arguments = { ref };
    handle(
      -1610612221, 
      arguments, 
      arguments, 
      severity, 
      sourceStart, 
      sourceEnd); }

  public void javadocMissingIdentifier(int sourceStart, int sourceEnd, int modifiers) {
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612269, NoArgument, NoArgument, sourceStart, sourceEnd); 
  }

  public void javadocMissingParamName(int sourceStart, int sourceEnd, int modifiers) { if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612264, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocMissingParamTag(char[] name, int sourceStart, int sourceEnd, int modifiers) {
    int severity = computeSeverity(-1610612265);
    if (severity == -1) return;
    boolean overriding = (modifiers & 0x30000000) != 0;
    boolean report = (this.options.getSeverity(2097152L) != -1) && (
      (!overriding) || (this.options.reportMissingJavadocTagsOverriding));
    if ((report) && (javadocVisibility(this.options.reportMissingJavadocTagsVisibility, modifiers))) {
      String[] arguments = { String.valueOf(name) };
      handle(
        -1610612265, 
        arguments, 
        arguments, 
        severity, 
        sourceStart, 
        sourceEnd);
    }
  }

  public void javadocMissingReference(int sourceStart, int sourceEnd, int modifiers) { if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612254, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocMissingReturnTag(int sourceStart, int sourceEnd, int modifiers) {
    boolean overriding = (modifiers & 0x30000000) != 0;
    boolean report = (this.options.getSeverity(2097152L) != -1) && (
      (!overriding) || (this.options.reportMissingJavadocTagsOverriding));
    if ((report) && (javadocVisibility(this.options.reportMissingJavadocTagsVisibility, modifiers)))
      handle(-1610612261, NoArgument, NoArgument, sourceStart, sourceEnd);
  }

  public void javadocMissingThrowsClassName(int sourceStart, int sourceEnd, int modifiers) {
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers))
      handle(-1610612258, NoArgument, NoArgument, sourceStart, sourceEnd); 
  }

  public void javadocMissingThrowsTag(TypeReference typeRef, int modifiers) { int severity = computeSeverity(-1610612259);
    if (severity == -1) return;
    boolean overriding = (modifiers & 0x30000000) != 0;
    boolean report = (this.options.getSeverity(2097152L) != -1) && (
      (!overriding) || (this.options.reportMissingJavadocTagsOverriding));
    if ((report) && (javadocVisibility(this.options.reportMissingJavadocTagsVisibility, modifiers))) {
      String[] arguments = { String.valueOf(typeRef.resolvedType.sourceName()) };
      handle(
        -1610612259, 
        arguments, 
        arguments, 
        severity, 
        typeRef.sourceStart, 
        typeRef.sourceEnd);
    } }

  public void javadocUndeclaredParamTagName(char[] token, int sourceStart, int sourceEnd, int modifiers) {
    int severity = computeSeverity(-1610612262);
    if (severity == -1) return;
    if (javadocVisibility(this.options.reportInvalidJavadocTagsVisibility, modifiers)) {
      String[] arguments = { String.valueOf(token) };
      handle(
        -1610612262, 
        arguments, 
        arguments, 
        severity, 
        sourceStart, 
        sourceEnd);
    }
  }

  public void javadocUnexpectedTag(int sourceStart, int sourceEnd) { handle(-1610612266, NoArgument, NoArgument, sourceStart, sourceEnd); }

  public void javadocUnexpectedText(int sourceStart, int sourceEnd) {
    handle(-1610612218, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  public void javadocUnterminatedInlineTag(int sourceStart, int sourceEnd) {
    handle(-1610612224, NoArgument, NoArgument, sourceStart, sourceEnd);
  }
  private boolean javadocVisibility(int visibility, int modifiers) {
    if (modifiers < 0) return true;
    switch (modifiers & 0x7) {
    case 1:
      return true;
    case 4:
      return visibility != 1;
    case 0:
      return (visibility == 0) || (visibility == 2);
    case 2:
      return visibility == 2;
    case 3:
    }return true;
  }
  private String javadocVisibilityArgument(int visibility, int modifiers) {
    String argument = null;
    switch (modifiers & 0x7) {
    case 1:
      argument = "public";
      break;
    case 4:
      if (visibility != 1) {
        argument = "protected";
      }
      break;
    case 0:
      if ((visibility == 0) || (visibility == 2)) {
        argument = "default";
      }
      break;
    case 2:
      if (visibility == 2)
        argument = "private";
      break;
    case 3:
    }
    return argument;
  }
  public void localVariableHiding(LocalDeclaration local, Binding hiddenVariable, boolean isSpecialArgHidingField) {
    if ((hiddenVariable instanceof LocalVariableBinding)) {
      int id = (local instanceof Argument) ? 
        536871006 : 
        536871002;
      int severity = computeSeverity(id);
      if (severity == -1) return;
      String[] arguments = { new String(local.name) };
      handle(
        id, 
        arguments, 
        arguments, 
        severity, 
        nodeSourceStart(hiddenVariable, local), 
        nodeSourceEnd(hiddenVariable, local));
    } else if ((hiddenVariable instanceof FieldBinding)) {
      if ((isSpecialArgHidingField) && (!this.options.reportSpecialParameterHidingField)) {
        return;
      }
      int id = (local instanceof Argument) ? 
        536871007 : 
        570425435;
      int severity = computeSeverity(id);
      if (severity == -1) return;
      FieldBinding field = (FieldBinding)hiddenVariable;
      handle(
        id, 
        new String[] { new String(local.name), new String(field.declaringClass.readableName()) }, 
        new String[] { new String(local.name), new String(field.declaringClass.shortReadableName()) }, 
        severity, 
        local.sourceStart, 
        local.sourceEnd);
    }
  }

  public void localVariableNonNullComparedToNull(LocalVariableBinding local, ASTNode location) { int severity = computeSeverity(536871370);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871370, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location)); }

  public void localVariableNullComparedToNonNull(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871366);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871366, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariableNullInstanceof(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871368);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871368, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariableNullReference(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871363);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871363, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariablePotentialNullReference(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871364);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871364, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariableRedundantCheckOnNonNull(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871369);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871369, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariableRedundantCheckOnNull(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871365);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871365, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void localVariableRedundantNullAssignment(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536871367);
    if (severity == -1) return;
    String[] arguments = { new String(local.name) };
    handle(
      536871367, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void methodMustOverride(AbstractMethodDeclaration method) {
    MethodBinding binding = method.binding;
    handle(
      this.options.sourceLevel == 3211264L ? 67109487 : 67109498, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, false), new String(binding.declaringClass.readableName()) }, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, true), new String(binding.declaringClass.shortReadableName()) }, 
      method.sourceStart, 
      method.sourceEnd);
  }
  public void methodNameClash(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    handle(
      67109424, 
      new String[] { 
      new String(currentMethod.selector), 
      typesAsString(currentMethod.isVarargs(), currentMethod.parameters, false), 
      new String(currentMethod.declaringClass.readableName()), 
      typesAsString(inheritedMethod.isVarargs(), inheritedMethod.parameters, false), 
      new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(currentMethod.selector), 
      typesAsString(currentMethod.isVarargs(), currentMethod.parameters, true), 
      new String(currentMethod.declaringClass.shortReadableName()), 
      typesAsString(inheritedMethod.isVarargs(), inheritedMethod.parameters, true), 
      new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      currentMethod.sourceStart(), 
      currentMethod.sourceEnd());
  }
  public void methodNeedBody(AbstractMethodDeclaration methodDecl) {
    handle(
      603979883, 
      NoArgument, 
      NoArgument, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void methodNeedingNoBody(MethodDeclaration methodDecl) {
    handle(
      (methodDecl.modifiers & 0x100) != 0 ? 603979888 : 603979889, 
      NoArgument, 
      NoArgument, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }

  public void methodWithConstructorName(MethodDeclaration methodDecl) {
    handle(
      67108974, 
      NoArgument, 
      NoArgument, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void missingDeprecatedAnnotationForField(FieldDeclaration field) {
    int severity = computeSeverity(536871540);
    if (severity == -1) return;
    FieldBinding binding = field.binding;
    handle(
      536871540, 
      new String[] { new String(binding.declaringClass.readableName()), new String(binding.name) }, 
      new String[] { new String(binding.declaringClass.shortReadableName()), new String(binding.name) }, 
      severity, 
      nodeSourceStart(binding, field), 
      nodeSourceEnd(binding, field));
  }
  public void missingDeprecatedAnnotationForMethod(AbstractMethodDeclaration method) {
    int severity = computeSeverity(536871541);
    if (severity == -1) return;
    MethodBinding binding = method.binding;
    handle(
      536871541, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, false), new String(binding.declaringClass.readableName()) }, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, true), new String(binding.declaringClass.shortReadableName()) }, 
      severity, 
      method.sourceStart, 
      method.sourceEnd);
  }
  public void missingDeprecatedAnnotationForType(TypeDeclaration type) {
    int severity = computeSeverity(536871542);
    if (severity == -1) return;
    TypeBinding binding = type.binding;
    handle(
      536871542, 
      new String[] { new String(binding.readableName()) }, 
      new String[] { new String(binding.shortReadableName()) }, 
      severity, 
      type.sourceStart, 
      type.sourceEnd);
  }
  public void missingEnumConstantCase(SwitchStatement switchStatement, FieldBinding enumConstant) {
    handle(
      33555193, 
      new String[] { new String(enumConstant.declaringClass.readableName()), new String(enumConstant.name) }, 
      new String[] { new String(enumConstant.declaringClass.shortReadableName()), new String(enumConstant.name) }, 
      switchStatement.expression.sourceStart, 
      switchStatement.expression.sourceEnd);
  }
  public void missingOverrideAnnotation(AbstractMethodDeclaration method) {
    int severity = computeSeverity(67109491);
    if (severity == -1) return;
    MethodBinding binding = method.binding;
    handle(
      67109491, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, false), new String(binding.declaringClass.readableName()) }, 
      new String[] { new String(binding.selector), typesAsString(binding.isVarargs(), binding.parameters, true), new String(binding.declaringClass.shortReadableName()) }, 
      severity, 
      method.sourceStart, 
      method.sourceEnd);
  }
  public void missingReturnType(AbstractMethodDeclaration methodDecl) {
    handle(
      16777327, 
      NoArgument, 
      NoArgument, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void missingSemiColon(Expression expression) {
    handle(
      1610612960, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }

  public void missingSerialVersion(TypeDeclaration typeDecl) {
    String[] arguments = { new String(typeDecl.name) };
    handle(
      536871008, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd);
  }
  public void missingValueForAnnotationMember(Annotation annotation, char[] memberName) {
    String memberString = new String(memberName);
    handle(
      16777825, 
      new String[] { new String(annotation.resolvedType.readableName()), memberString }, 
      new String[] { new String(annotation.resolvedType.shortReadableName()), memberString }, 
      annotation.sourceStart, 
      annotation.sourceEnd);
  }
  public void mustDefineDimensionsOrInitializer(ArrayAllocationExpression expression) {
    handle(
      536871071, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void mustSpecifyPackage(CompilationUnitDeclaration compUnitDecl) {
    String[] arguments = { new String(compUnitDecl.getFileName()) };
    handle(
      536871238, 
      arguments, 
      arguments, 
      compUnitDecl.sourceStart, 
      compUnitDecl.sourceStart + 1);
  }
  public void mustUseAStaticMethod(MessageSend messageSend, MethodBinding method) {
    handle(
      603979977, 
      new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
      messageSend.sourceStart, 
      messageSend.sourceEnd);
  }
  public void nativeMethodsCannotBeStrictfp(ReferenceBinding type, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { new String(type.sourceName()), new String(methodDecl.selector) };
    handle(
      67109231, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void needImplementation() {
    abortDueToInternalError(Messages.abort_missingCode);
  }

  public void needToEmulateFieldAccess(FieldBinding field, ASTNode location, boolean isReadAccess) {
    int id = isReadAccess ? 
      33554622 : 
      33554623;
    int severity = computeSeverity(id);
    if (severity == -1) return;
    handle(
      id, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      severity, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }

  public void needToEmulateMethodAccess(MethodBinding method, ASTNode location)
  {
    if (method.isConstructor()) {
      int severity = computeSeverity(67109057);
      if (severity == -1) return;
      if (method.declaringClass.isEnum())
        return;
      handle(
        67109057, 
        new String[] { 
        new String(method.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false) }, 
        new String[] { 
        new String(method.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
      return;
    }
    int severity = computeSeverity(67109056);
    if (severity == -1) return;
    handle(
      67109056, 
      new String[] { 
      new String(method.declaringClass.readableName()), 
      new String(method.selector), 
      typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { 
      new String(method.declaringClass.shortReadableName()), 
      new String(method.selector), 
      typesAsString(method.isVarargs(), method.parameters, true) }, 
      severity, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void noAdditionalBoundAfterTypeVariable(TypeReference boundReference) {
    handle(
      16777789, 
      new String[] { new String(boundReference.resolvedType.readableName()) }, 
      new String[] { new String(boundReference.resolvedType.shortReadableName()) }, 
      boundReference.sourceStart, 
      boundReference.sourceEnd);
  }
  private int nodeSourceEnd(Binding field, ASTNode node) {
    return nodeSourceEnd(field, node, 0);
  }
  private int nodeSourceEnd(Binding field, ASTNode node, int index) {
    if ((node instanceof ArrayTypeReference))
      return ((ArrayTypeReference)node).originalSourceEnd;
    if ((node instanceof QualifiedNameReference)) {
      QualifiedNameReference ref = (QualifiedNameReference)node;
      if (ref.binding == field) {
        return (int)ref.sourcePositions[(ref.indexOfFirstFieldBinding - 1)];
      }
      FieldBinding[] otherFields = ref.otherBindings;
      if (otherFields != null) {
        int offset = ref.indexOfFirstFieldBinding;
        int i = 0; for (int length = otherFields.length; i < length; i++)
          if (otherFields[i] == field)
            return (int)ref.sourcePositions[(i + offset)];
      }
    }
    else if ((node instanceof ParameterizedQualifiedTypeReference)) {
      ParameterizedQualifiedTypeReference reference = (ParameterizedQualifiedTypeReference)node;
      if (index < reference.sourcePositions.length)
        return (int)reference.sourcePositions[index];
    }
    else if ((node instanceof ArrayQualifiedTypeReference)) {
      ArrayQualifiedTypeReference arrayQualifiedTypeReference = (ArrayQualifiedTypeReference)node;
      int length = arrayQualifiedTypeReference.sourcePositions.length;
      return (int)arrayQualifiedTypeReference.sourcePositions[(length - 1)];
    }
    return node.sourceEnd;
  }
  private int nodeSourceStart(Binding field, ASTNode node) {
    if ((node instanceof FieldReference)) {
      FieldReference fieldReference = (FieldReference)node;
      return (int)(fieldReference.nameSourcePosition >> 32);
    }if ((node instanceof QualifiedNameReference)) {
      QualifiedNameReference ref = (QualifiedNameReference)node;
      if (ref.binding == field) {
        return (int)(ref.sourcePositions[(ref.indexOfFirstFieldBinding - 1)] >> 32);
      }
      FieldBinding[] otherFields = ref.otherBindings;
      if (otherFields != null) {
        int offset = ref.indexOfFirstFieldBinding;
        int i = 0; for (int length = otherFields.length; i < length; i++)
          if (otherFields[i] == field)
            return (int)(ref.sourcePositions[(i + offset)] >> 32);
      }
    }
    else if ((node instanceof ParameterizedQualifiedTypeReference)) {
      ParameterizedQualifiedTypeReference reference = (ParameterizedQualifiedTypeReference)node;
      return (int)(reference.sourcePositions[0] >>> 32);
    }
    return node.sourceStart;
  }
  public void noMoreAvailableSpaceForArgument(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.name) };
    handle(
      (local instanceof SyntheticArgumentBinding) ? 
      536870979 : 
      536870977, 
      arguments, 
      arguments, 
      159, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void noMoreAvailableSpaceForConstant(TypeDeclaration typeDeclaration) {
    handle(
      536871343, 
      new String[] { new String(typeDeclaration.binding.readableName()) }, 
      new String[] { new String(typeDeclaration.binding.shortReadableName()) }, 
      159, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }

  public void noMoreAvailableSpaceForLocal(LocalVariableBinding local, ASTNode location) {
    String[] arguments = { new String(local.name) };
    handle(
      536870978, 
      arguments, 
      arguments, 
      159, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  public void noMoreAvailableSpaceInConstantPool(TypeDeclaration typeDeclaration) {
    handle(
      536871342, 
      new String[] { new String(typeDeclaration.binding.readableName()) }, 
      new String[] { new String(typeDeclaration.binding.shortReadableName()) }, 
      159, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }

  public void nonExternalizedStringLiteral(ASTNode location) {
    handle(
      536871173, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void nonGenericTypeCannotBeParameterized(ASTNode location, TypeBinding type, TypeBinding[] argumentTypes) {
    nonGenericTypeCannotBeParameterized(0, location, type, argumentTypes);
  }
  public void nonGenericTypeCannotBeParameterized(int index, ASTNode location, TypeBinding type, TypeBinding[] argumentTypes) {
    if (location == null) {
      handle(
        16777740, 
        new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false) }, 
        new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true) }, 
        131, 
        0, 
        0);
      return;
    }
    handle(
      16777740, 
      new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false) }, 
      new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true) }, 
      nodeSourceStart(null, location), 
      nodeSourceEnd(null, location, index));
  }
  public void nonStaticAccessToStaticField(ASTNode location, FieldBinding field) {
    int severity = computeSeverity(570425420);
    if (severity == -1) return;
    handle(
      570425420, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      severity, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void nonStaticAccessToStaticMethod(ASTNode location, MethodBinding method) {
    handle(
      603979893, 
      new String[] { new String(method.declaringClass.readableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { new String(method.declaringClass.shortReadableName()), new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void nonStaticContextForEnumMemberType(SourceTypeBinding type) {
    String[] arguments = { new String(type.sourceName()) };
    handle(
      536870944, 
      arguments, 
      arguments, 
      type.sourceStart(), 
      type.sourceEnd());
  }

  public void noSuchEnclosingInstance(TypeBinding targetType, ASTNode location, boolean isConstructorCall)
  {
    int id;
    int id;
    if (isConstructorCall)
    {
      id = 536870940;
    }
    else
    {
      int id;
      if (((location instanceof ExplicitConstructorCall)) && 
        (((ExplicitConstructorCall)location).accessMode == 1))
      {
        id = 16777236;
      }
      else
      {
        int id;
        if (((location instanceof AllocationExpression)) && (
          (((AllocationExpression)location).binding.declaringClass.isMemberType()) || (
          (((AllocationExpression)location).binding.declaringClass.isAnonymousType()) && 
          (((AllocationExpression)location).binding.declaringClass.superclass().isMemberType()))))
        {
          id = 16777237;
        }
        else
          id = 16777238;
      }
    }
    handle(
      id, 
      new String[] { new String(targetType.readableName()) }, 
      new String[] { new String(targetType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void notCompatibleTypesError(EqualExpression expression, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      16777231, 
      new String[] { leftName, rightName }, 
      new String[] { leftShortName, rightShortName }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void notCompatibleTypesError(InstanceOfExpression expression, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      16777232, 
      new String[] { leftName, rightName }, 
      new String[] { leftShortName, rightShortName }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void notCompatibleTypesErrorInForeach(Expression expression, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      16777796, 
      new String[] { leftName, rightName }, 
      new String[] { leftShortName, rightShortName }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void objectCannotBeGeneric(TypeDeclaration typeDecl) {
    handle(
      536871435, 
      NoArgument, 
      NoArgument, 
      typeDecl.typeParameters[0].sourceStart, 
      typeDecl.typeParameters[(typeDecl.typeParameters.length - 1)].sourceEnd);
  }
  public void objectCannotHaveSuperTypes(SourceTypeBinding type) {
    handle(
      536871241, 
      NoArgument, 
      NoArgument, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void objectMustBeClass(SourceTypeBinding type) {
    handle(
      536871242, 
      NoArgument, 
      NoArgument, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void operatorOnlyValidOnNumericType(CompoundAssignment assignment, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      16777233, 
      new String[] { leftName, rightName }, 
      new String[] { leftShortName, rightShortName }, 
      assignment.sourceStart, 
      assignment.sourceEnd);
  }
  public void overridesDeprecatedMethod(MethodBinding localMethod, MethodBinding inheritedMethod) {
    handle(
      67109276, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.readableName(), 
      localMethod.readableName(), 
      '.')), 
      new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.shortReadableName(), 
      localMethod.shortReadableName(), 
      '.')), 
      new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      localMethod.sourceStart(), 
      localMethod.sourceEnd());
  }
  public void overridesMethodWithoutSuperInvocation(MethodBinding localMethod) {
    handle(
      67109280, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.readableName(), 
      localMethod.readableName(), 
      '.')) }, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.shortReadableName(), 
      localMethod.shortReadableName(), 
      '.')) }, 
      localMethod.sourceStart(), 
      localMethod.sourceEnd());
  }
  public void overridesPackageDefaultMethod(MethodBinding localMethod, MethodBinding inheritedMethod) {
    handle(
      67109274, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.readableName(), 
      localMethod.readableName(), 
      '.')), 
      new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(
      CharOperation.concat(
      localMethod.declaringClass.shortReadableName(), 
      localMethod.shortReadableName(), 
      '.')), 
      new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      localMethod.sourceStart(), 
      localMethod.sourceEnd());
  }
  public void packageCollidesWithType(CompilationUnitDeclaration compUnitDecl) {
    String[] arguments = { CharOperation.toString(compUnitDecl.currentPackage.tokens) };
    handle(
      16777537, 
      arguments, 
      arguments, 
      compUnitDecl.currentPackage.sourceStart, 
      compUnitDecl.currentPackage.sourceEnd);
  }
  public void packageIsNotExpectedPackage(CompilationUnitDeclaration compUnitDecl) {
    String[] arguments = { 
      CharOperation.toString(compUnitDecl.compilationResult.compilationUnit.getPackageName()), 
      compUnitDecl.currentPackage == null ? "" : CharOperation.toString(compUnitDecl.currentPackage.tokens) };

    handle(
      536871240, 
      arguments, 
      arguments, 
      compUnitDecl.currentPackage == null ? 0 : compUnitDecl.currentPackage.sourceStart, 
      compUnitDecl.currentPackage == null ? 0 : compUnitDecl.currentPackage.sourceEnd);
  }
  public void parameterAssignment(LocalVariableBinding local, ASTNode location) {
    int severity = computeSeverity(536870971);
    if (severity == -1) return;
    String[] arguments = { new String(local.readableName()) };
    handle(
      536870971, 
      arguments, 
      arguments, 
      severity, 
      nodeSourceStart(local, location), 
      nodeSourceEnd(local, location));
  }
  private String parameterBoundAsString(TypeVariableBinding typeVariable, boolean makeShort) {
    StringBuffer nameBuffer = new StringBuffer(10);
    if (typeVariable.firstBound == typeVariable.superclass)
      nameBuffer.append(makeShort ? typeVariable.superclass.shortReadableName() : typeVariable.superclass.readableName());
    int length;
    if ((length = typeVariable.superInterfaces.length) > 0) {
      for (int i = 0; i < length; i++) {
        if ((i > 0) || (typeVariable.firstBound == typeVariable.superclass)) nameBuffer.append(" & ");
        nameBuffer.append(makeShort ? typeVariable.superInterfaces[i].shortReadableName() : typeVariable.superInterfaces[i].readableName());
      }
    }
    return nameBuffer.toString();
  }
  public void parameterizedMemberTypeMissingArguments(ASTNode location, TypeBinding type) {
    if (location == null) {
      handle(
        16777778, 
        new String[] { new String(type.readableName()) }, 
        new String[] { new String(type.shortReadableName()) }, 
        131, 
        0, 
        0);
      return;
    }
    handle(
      16777778, 
      new String[] { new String(type.readableName()) }, 
      new String[] { new String(type.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }

  public void parseError(int startPosition, int endPosition, int currentToken, char[] currentTokenSource, String errorTokenName, String[] possibleTokens)
  {
    if (possibleTokens.length == 0) {
      if (isKeyword(currentToken)) {
        String[] arguments = { new String(currentTokenSource) };
        handle(
          1610612946, 
          arguments, 
          arguments, 
          startPosition, 
          endPosition);
        return;
      }
      String[] arguments = { errorTokenName };
      handle(
        1610612941, 
        arguments, 
        arguments, 
        startPosition, 
        endPosition);
      return;
    }

    StringBuffer list = new StringBuffer(20);
    int i = 0; for (int max = possibleTokens.length; i < max; i++) {
      if (i > 0)
        list.append(", ");
      list.append('"');
      list.append(possibleTokens[i]);
      list.append('"');
    }

    if (isKeyword(currentToken)) {
      String[] arguments = { new String(currentTokenSource), list.toString() };
      handle(
        1610612945, 
        arguments, 
        arguments, 
        startPosition, 
        endPosition);
      return;
    }

    if ((isLiteral(currentToken)) || 
      (isIdentifier(currentToken))) {
      errorTokenName = new String(currentTokenSource);
    }

    String[] arguments = { errorTokenName, list.toString() };
    handle(
      1610612940, 
      arguments, 
      arguments, 
      startPosition, 
      endPosition);
  }

  public void parseErrorDeleteToken(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName)
  {
    syntaxError(
      1610612968, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      null);
  }

  public void parseErrorDeleteTokens(int start, int end)
  {
    handle(
      1610612969, 
      NoArgument, 
      NoArgument, 
      start, 
      end);
  }

  public void parseErrorInsertAfterToken(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName, String expectedToken)
  {
    syntaxError(
      1610612967, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      expectedToken);
  }

  public void parseErrorInsertBeforeToken(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName, String expectedToken)
  {
    syntaxError(
      1610612966, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      expectedToken);
  }

  public void parseErrorInsertToComplete(int start, int end, String inserted, String completed)
  {
    String[] arguments = { inserted, completed };
    handle(
      1610612976, 
      arguments, 
      arguments, 
      start, 
      end);
  }

  public void parseErrorInsertToCompletePhrase(int start, int end, String inserted)
  {
    String[] arguments = { inserted };
    handle(
      1610612978, 
      arguments, 
      arguments, 
      start, 
      end);
  }

  public void parseErrorInsertToCompleteScope(int start, int end, String inserted)
  {
    String[] arguments = { inserted };
    handle(
      1610612977, 
      arguments, 
      arguments, 
      start, 
      end);
  }

  public void parseErrorInvalidToken(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName, String expectedToken)
  {
    syntaxError(
      1610612971, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      expectedToken);
  }

  public void parseErrorMergeTokens(int start, int end, String expectedToken)
  {
    String[] arguments = { expectedToken };
    handle(
      1610612970, 
      arguments, 
      arguments, 
      start, 
      end);
  }

  public void parseErrorMisplacedConstruct(int start, int end)
  {
    handle(
      1610612972, 
      NoArgument, 
      NoArgument, 
      start, 
      end);
  }

  public void parseErrorNoSuggestion(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName)
  {
    syntaxError(
      1610612941, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      null);
  }

  public void parseErrorNoSuggestionForTokens(int start, int end)
  {
    handle(
      1610612974, 
      NoArgument, 
      NoArgument, 
      start, 
      end);
  }

  public void parseErrorReplaceToken(int start, int end, int currentKind, char[] errorTokenSource, String errorTokenName, String expectedToken)
  {
    syntaxError(
      1610612940, 
      start, 
      end, 
      currentKind, 
      errorTokenSource, 
      errorTokenName, 
      expectedToken);
  }

  public void parseErrorReplaceTokens(int start, int end, String expectedToken)
  {
    String[] arguments = { expectedToken };
    handle(
      1610612973, 
      arguments, 
      arguments, 
      start, 
      end);
  }

  public void parseErrorUnexpectedEnd(int start, int end)
  {
    String[] arguments;
    String[] arguments;
    if ((this.referenceContext instanceof ConstructorDeclaration)) {
      arguments = new String[] { Messages.parser_endOfConstructor };
    }
    else
    {
      String[] arguments;
      if ((this.referenceContext instanceof MethodDeclaration)) {
        arguments = new String[] { Messages.parser_endOfMethod };
      }
      else
      {
        String[] arguments;
        if ((this.referenceContext instanceof TypeDeclaration))
          arguments = new String[] { Messages.parser_endOfInitializer };
        else
          arguments = new String[] { Messages.parser_endOfFile }; 
      }
    }
    handle(
      1610612975, 
      arguments, 
      arguments, 
      start, 
      end);
  }
  public void possibleAccidentalBooleanAssignment(Assignment assignment) {
    handle(
      536871091, 
      NoArgument, 
      NoArgument, 
      assignment.sourceStart, 
      assignment.sourceEnd);
  }

  public void possibleFallThroughCase(CaseStatement caseStatement) {
    handle(
      536871106, 
      NoArgument, 
      NoArgument, 
      caseStatement.sourceStart, 
      caseStatement.sourceEnd);
  }
  public void publicClassMustMatchFileName(CompilationUnitDeclaration compUnitDecl, TypeDeclaration typeDecl) {
    this.referenceContext = typeDecl;
    String[] arguments = { new String(compUnitDecl.getFileName()), new String(typeDecl.name) };
    handle(
      16777541, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd, 
      compUnitDecl.compilationResult);
  }
  public void rawMemberTypeCannotBeParameterized(ASTNode location, ReferenceBinding type, TypeBinding[] argumentTypes) {
    if (location == null) {
      handle(
        16777777, 
        new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false), new String(type.enclosingType().readableName()) }, 
        new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true), new String(type.enclosingType().shortReadableName()) }, 
        131, 
        0, 
        0);
      return;
    }
    handle(
      16777777, 
      new String[] { new String(type.readableName()), typesAsString(false, argumentTypes, false), new String(type.enclosingType().readableName()) }, 
      new String[] { new String(type.shortReadableName()), typesAsString(false, argumentTypes, true), new String(type.enclosingType().shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void rawTypeReference(ASTNode location, TypeBinding type) {
    type = type.leafComponentType();
    handle(
      16777788, 
      new String[] { new String(type.readableName()), new String(type.erasure().readableName()) }, 
      new String[] { new String(type.shortReadableName()), new String(type.erasure().shortReadableName()) }, 
      location.sourceStart, 
      nodeSourceEnd(null, location));
  }
  public void recursiveConstructorInvocation(ExplicitConstructorCall constructorCall) {
    handle(
      134217865, 
      new String[] { 
      new String(constructorCall.binding.declaringClass.readableName()), 
      typesAsString(constructorCall.binding.isVarargs(), constructorCall.binding.parameters, false) }, 
      new String[] { 
      new String(constructorCall.binding.declaringClass.shortReadableName()), 
      typesAsString(constructorCall.binding.isVarargs(), constructorCall.binding.parameters, true) }, 
      constructorCall.sourceStart, 
      constructorCall.sourceEnd);
  }
  public void redefineArgument(Argument arg) {
    String[] arguments = { new String(arg.name) };
    handle(
      536870968, 
      arguments, 
      arguments, 
      arg.sourceStart, 
      arg.sourceEnd);
  }
  public void redefineLocal(LocalDeclaration localDecl) {
    String[] arguments = { new String(localDecl.name) };
    handle(
      536870967, 
      arguments, 
      arguments, 
      localDecl.sourceStart, 
      localDecl.sourceEnd);
  }

  public void referenceMustBeArrayTypeAt(TypeBinding arrayType, ArrayReference arrayRef) {
    handle(
      536871062, 
      new String[] { new String(arrayType.readableName()) }, 
      new String[] { new String(arrayType.shortReadableName()) }, 
      arrayRef.sourceStart, 
      arrayRef.sourceEnd);
  }
  public void reset() {
    this.positionScanner = null;
  }
  private int retrieveClosingAngleBracketPosition(int start) {
    if (this.referenceContext == null) return start;
    CompilationResult compilationResult = this.referenceContext.compilationResult();
    if (compilationResult == null) return start;
    ICompilationUnit compilationUnit = compilationResult.getCompilationUnit();
    if (compilationUnit == null) return start;
    char[] contents = compilationUnit.getContents();
    if (contents.length == 0) return start;
    if (this.positionScanner == null) {
      this.positionScanner = new Scanner(false, false, false, this.options.sourceLevel, this.options.complianceLevel, null, null, false);
      this.positionScanner.returnOnlyGreater = true;
    }
    this.positionScanner.setSource(contents);
    this.positionScanner.resetTo(start, contents.length);
    int end = start;
    int count = 0;
    try
    {
      int token;
      while ((token = this.positionScanner.getNextToken()) != 68)
      {
        int token;
        switch (token) {
        case 7:
          count++;
          break;
        case 12:
          count--;
          if (count == 0)
            end = this.positionScanner.currentPosition - 1;
          break;
        case 69:
          break label214;
        }
      }
    }
    catch (InvalidInputException localInvalidInputException) {
    }
    label214: return end;
  }
  private int retrieveEndingPositionAfterOpeningParenthesis(int sourceStart, int sourceEnd, int numberOfParen) {
    if (this.referenceContext == null) return sourceEnd;
    CompilationResult compilationResult = this.referenceContext.compilationResult();
    if (compilationResult == null) return sourceEnd;
    ICompilationUnit compilationUnit = compilationResult.getCompilationUnit();
    if (compilationUnit == null) return sourceEnd;
    char[] contents = compilationUnit.getContents();
    if (contents.length == 0) return sourceEnd;
    if (this.positionScanner == null) {
      this.positionScanner = new Scanner(false, false, false, this.options.sourceLevel, this.options.complianceLevel, null, null, false);
    }
    this.positionScanner.setSource(contents);
    this.positionScanner.resetTo(sourceStart, sourceEnd);
    try
    {
      int previousSourceEnd = sourceEnd;
      int token;
      while ((token = this.positionScanner.getNextToken()) != 68)
      {
        int token;
        switch (token) {
        case 29:
          return previousSourceEnd;
        }
        previousSourceEnd = this.positionScanner.currentPosition - 1;
      }
    }
    catch (InvalidInputException localInvalidInputException)
    {
    }
    return sourceEnd;
  }
  private int retrieveStartingPositionAfterOpeningParenthesis(int sourceStart, int sourceEnd, int numberOfParen) {
    if (this.referenceContext == null) return sourceStart;
    CompilationResult compilationResult = this.referenceContext.compilationResult();
    if (compilationResult == null) return sourceStart;
    ICompilationUnit compilationUnit = compilationResult.getCompilationUnit();
    if (compilationUnit == null) return sourceStart;
    char[] contents = compilationUnit.getContents();
    if (contents.length == 0) return sourceStart;
    if (this.positionScanner == null) {
      this.positionScanner = new Scanner(false, false, false, this.options.sourceLevel, this.options.complianceLevel, null, null, false);
    }
    this.positionScanner.setSource(contents);
    this.positionScanner.resetTo(sourceStart, sourceEnd);
    int count = 0;
    try
    {
      int token;
      while ((token = this.positionScanner.getNextToken()) != 68)
      {
        int token;
        switch (token) {
        case 28:
          count++;
          if (count == numberOfParen) {
            this.positionScanner.getNextToken();
            return this.positionScanner.startPosition;
          }
          break;
        }
      }
    } catch (InvalidInputException localInvalidInputException) {
    }
    return sourceStart;
  }
  public void returnTypeCannotBeVoidArray(MethodDeclaration methodDecl) {
    handle(
      536870966, 
      NoArgument, 
      NoArgument, 
      methodDecl.returnType.sourceStart, 
      methodDecl.returnType.sourceEnd);
  }
  public void scannerError(Parser parser, String errorTokenName) {
    Scanner scanner = parser.scanner;

    int flag = 1610612941;
    int startPos = scanner.startPosition;
    int endPos = scanner.currentPosition - 1;

    if (errorTokenName.equals("End_Of_Source")) {
      flag = 1610612986;
    } else if (errorTokenName.equals("Invalid_Hexa_Literal")) {
      flag = 1610612987;
    } else if (errorTokenName.equals("Invalid_Octal_Literal")) {
      flag = 1610612988;
    } else if (errorTokenName.equals("Invalid_Character_Constant")) {
      flag = 1610612989;
    } else if (errorTokenName.equals("Invalid_Escape")) {
      flag = 1610612990;
    } else if (errorTokenName.equals("Invalid_Unicode_Escape")) {
      flag = 1610612992;

      char[] source = scanner.source;
      int checkPos = scanner.currentPosition - 1;
      if (checkPos >= source.length) checkPos = source.length - 1;
      while (checkPos >= startPos) {
        if (source[checkPos] == '\\') break;
        checkPos--;
      }
      startPos = checkPos;
    } else if (errorTokenName.equals("Invalid_Low_Surrogate")) {
      flag = 1610612999;
    } else if (errorTokenName.equals("Invalid_High_Surrogate")) {
      flag = 1610613000;

      char[] source = scanner.source;
      int checkPos = scanner.startPosition + 1;
      while (checkPos <= endPos) {
        if (source[checkPos] == '\\') break;
        checkPos++;
      }
      endPos = checkPos - 1;
    } else if (errorTokenName.equals("Invalid_Float_Literal")) {
      flag = 1610612993;
    } else if (errorTokenName.equals("Unterminated_String")) {
      flag = 1610612995;
    } else if (errorTokenName.equals("Unterminated_Comment")) {
      flag = 1610612996;
    } else if (errorTokenName.equals("Invalid_Char_In_String")) {
      flag = 1610612995;
    } else if (errorTokenName.equals("Invalid_Digit")) {
      flag = 1610612998;
    }
    String[] arguments = flag == 1610612941 ? 
      new String[] { errorTokenName } : 
      NoArgument;
    handle(
      flag, 
      arguments, 
      arguments, 
      startPos, 
      endPos, 
      parser.compilationUnit.compilationResult);
  }
  public void shouldReturn(TypeBinding returnType, ASTNode location) {
    handle(
      603979884, 
      new String[] { new String(returnType.readableName()) }, 
      new String[] { new String(returnType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }

  public void signalNoImplicitStringConversionForCharArrayExpression(Expression expression) {
    handle(
      536871063, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void staticAndInstanceConflict(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    if (currentMethod.isStatic())
      handle(
        67109271, 
        new String[] { new String(inheritedMethod.declaringClass.readableName()) }, 
        new String[] { new String(inheritedMethod.declaringClass.shortReadableName()) }, 
        currentMethod.sourceStart(), 
        currentMethod.sourceEnd());
    else
      handle(
        67109270, 
        new String[] { new String(inheritedMethod.declaringClass.readableName()) }, 
        new String[] { new String(inheritedMethod.declaringClass.shortReadableName()) }, 
        currentMethod.sourceStart(), 
        currentMethod.sourceEnd()); 
  }

  public void staticFieldAccessToNonStaticVariable(ASTNode location, FieldBinding field) { String[] arguments = { new String(field.readableName()) };
    handle(
      33554506, 
      arguments, 
      arguments, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location)); }

  public void staticInheritedMethodConflicts(SourceTypeBinding type, MethodBinding concreteMethod, MethodBinding[] abstractMethods) {
    handle(
      67109272, 
      new String[] { 
      new String(concreteMethod.readableName()), 
      new String(abstractMethods[0].declaringClass.readableName()) }, 
      new String[] { 
      new String(concreteMethod.readableName()), 
      new String(abstractMethods[0].declaringClass.shortReadableName()) }, 
      type.sourceStart(), 
      type.sourceEnd());
  }
  public void staticMemberOfParameterizedType(ASTNode location, ReferenceBinding type) {
    if (location == null) {
      handle(
        16777779, 
        new String[] { new String(type.readableName()), new String(type.enclosingType().readableName()) }, 
        new String[] { new String(type.shortReadableName()), new String(type.enclosingType().shortReadableName()) }, 
        131, 
        0, 
        0);
      return;
    }
    int end = location.sourceEnd;

    handle(
      16777779, 
      new String[] { new String(type.readableName()), new String(type.enclosingType().readableName()) }, 
      new String[] { new String(type.shortReadableName()), new String(type.enclosingType().shortReadableName()) }, 
      location.sourceStart, 
      end);
  }
  public void stringConstantIsExceedingUtf8Limit(ASTNode location) {
    handle(
      536871064, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void superclassMustBeAClass(SourceTypeBinding type, TypeReference superclassRef, ReferenceBinding superType) {
    handle(
      16777528, 
      new String[] { new String(superType.readableName()), new String(type.sourceName()) }, 
      new String[] { new String(superType.shortReadableName()), new String(type.sourceName()) }, 
      superclassRef.sourceStart, 
      superclassRef.sourceEnd);
  }
  public void superfluousSemicolon(int sourceStart, int sourceEnd) {
    handle(
      536871092, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void superinterfaceMustBeAnInterface(SourceTypeBinding type, TypeReference superInterfaceRef, ReferenceBinding superType) {
    handle(
      16777531, 
      new String[] { new String(superType.readableName()), new String(type.sourceName()) }, 
      new String[] { new String(superType.shortReadableName()), new String(type.sourceName()) }, 
      superInterfaceRef.sourceStart, 
      superInterfaceRef.sourceEnd);
  }
  public void superinterfacesCollide(TypeBinding type, ASTNode decl, TypeBinding superType, TypeBinding inheritedSuperType) {
    handle(
      16777755, 
      new String[] { new String(superType.readableName()), new String(inheritedSuperType.readableName()), new String(type.sourceName()) }, 
      new String[] { new String(superType.shortReadableName()), new String(inheritedSuperType.shortReadableName()), new String(type.sourceName()) }, 
      decl.sourceStart, 
      decl.sourceEnd);
  }
  public void superTypeCannotUseWildcard(SourceTypeBinding type, TypeReference superclass, TypeBinding superTypeBinding) {
    String name = new String(type.sourceName());
    String superTypeFullName = new String(superTypeBinding.readableName());
    String superTypeShortName = new String(superTypeBinding.shortReadableName());
    if (superTypeShortName.equals(name)) superTypeShortName = superTypeFullName;
    handle(
      16777772, 
      new String[] { superTypeFullName, name }, 
      new String[] { superTypeShortName, name }, 
      superclass.sourceStart, 
      superclass.sourceEnd);
  }

  private void syntaxError(int id, int startPosition, int endPosition, int currentKind, char[] currentTokenSource, String errorTokenName, String expectedToken)
  {
    String eTokenName;
    String eTokenName;
    if ((isKeyword(currentKind)) || 
      (isLiteral(currentKind)) || 
      (isIdentifier(currentKind)))
      eTokenName = new String(currentTokenSource);
    else
      eTokenName = errorTokenName;
    String[] arguments;
    String[] arguments;
    if (expectedToken != null)
      arguments = new String[] { eTokenName, expectedToken };
    else {
      arguments = new String[] { eTokenName };
    }
    handle(
      id, 
      arguments, 
      arguments, 
      startPosition, 
      endPosition);
  }
  public void task(String tag, String message, String priority, int start, int end) {
    handle(
      536871362, 
      new String[] { tag, message, priority }, 
      new String[] { tag, message, priority }, 
      start, 
      end);
  }

  public void tooManyDimensions(ASTNode expression) {
    handle(
      536870980, 
      NoArgument, 
      NoArgument, 
      expression.sourceStart, 
      expression.sourceEnd);
  }

  public void tooManyFields(TypeDeclaration typeDeclaration) {
    handle(
      536871344, 
      new String[] { new String(typeDeclaration.binding.readableName()) }, 
      new String[] { new String(typeDeclaration.binding.shortReadableName()) }, 
      159, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }
  public void tooManyMethods(TypeDeclaration typeDeclaration) {
    handle(
      536871345, 
      new String[] { new String(typeDeclaration.binding.readableName()) }, 
      new String[] { new String(typeDeclaration.binding.shortReadableName()) }, 
      159, 
      typeDeclaration.sourceStart, 
      typeDeclaration.sourceEnd);
  }
  public void typeCastError(CastExpression expression, TypeBinding leftType, TypeBinding rightType) {
    String leftName = new String(leftType.readableName());
    String rightName = new String(rightType.readableName());
    String leftShortName = new String(leftType.shortReadableName());
    String rightShortName = new String(rightType.shortReadableName());
    if (leftShortName.equals(rightShortName)) {
      leftShortName = leftName;
      rightShortName = rightName;
    }
    handle(
      16777372, 
      new String[] { rightName, leftName }, 
      new String[] { rightShortName, leftShortName }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void typeCollidesWithEnclosingType(TypeDeclaration typeDecl) {
    String[] arguments = { new String(typeDecl.name) };
    handle(
      16777534, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd);
  }
  public void typeCollidesWithPackage(CompilationUnitDeclaration compUnitDecl, TypeDeclaration typeDecl) {
    this.referenceContext = typeDecl;
    String[] arguments = { new String(compUnitDecl.getFileName()), new String(typeDecl.name) };
    handle(
      16777538, 
      arguments, 
      arguments, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd, 
      compUnitDecl.compilationResult);
  }
  public void typeHiding(TypeDeclaration typeDecl, TypeVariableBinding hiddenTypeParameter) {
    int severity = computeSeverity(16777792);
    if (severity == -1) return;
    if ((hiddenTypeParameter.declaringElement instanceof TypeBinding)) {
      TypeBinding declaringType = (TypeBinding)hiddenTypeParameter.declaringElement;
      handle(
        16777792, 
        new String[] { new String(typeDecl.name), new String(hiddenTypeParameter.readableName()), new String(declaringType.readableName()) }, 
        new String[] { new String(typeDecl.name), new String(hiddenTypeParameter.shortReadableName()), new String(declaringType.shortReadableName()) }, 
        severity, 
        typeDecl.sourceStart, 
        typeDecl.sourceEnd);
    }
    else {
      MethodBinding declaringMethod = (MethodBinding)hiddenTypeParameter.declaringElement;
      handle(
        16777793, 
        new String[] { 
        new String(typeDecl.name), 
        new String(hiddenTypeParameter.readableName()), 
        new String(declaringMethod.selector), 
        typesAsString(declaringMethod.isVarargs(), declaringMethod.parameters, false), 
        new String(declaringMethod.declaringClass.readableName()) }, 
        new String[] { 
        new String(typeDecl.name), 
        new String(hiddenTypeParameter.shortReadableName()), 
        new String(declaringMethod.selector), 
        typesAsString(declaringMethod.isVarargs(), declaringMethod.parameters, true), 
        new String(declaringMethod.declaringClass.shortReadableName()) }, 
        severity, 
        typeDecl.sourceStart, 
        typeDecl.sourceEnd);
    }
  }

  public void typeHiding(TypeDeclaration typeDecl, TypeBinding hiddenType) { int severity = computeSeverity(16777249);
    if (severity == -1) return;
    handle(
      16777249, 
      new String[] { new String(typeDecl.name), new String(hiddenType.shortReadableName()) }, 
      new String[] { new String(typeDecl.name), new String(hiddenType.readableName()) }, 
      severity, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd); }

  public void typeHiding(TypeParameter typeParam, Binding hidden) {
    int severity = computeSeverity(16777787);
    if (severity == -1) return;
    TypeBinding hiddenType = (TypeBinding)hidden;
    handle(
      16777787, 
      new String[] { new String(typeParam.name), new String(hiddenType.readableName()) }, 
      new String[] { new String(typeParam.name), new String(hiddenType.shortReadableName()) }, 
      severity, 
      typeParam.sourceStart, 
      typeParam.sourceEnd);
  }
  public void typeMismatchError(TypeBinding actualType, TypeBinding expectedType, ASTNode location) {
    handle(
      16777233, 
      new String[] { new String(actualType.readableName()), new String(expectedType.readableName()) }, 
      new String[] { new String(actualType.shortReadableName()), new String(expectedType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void typeMismatchError(TypeBinding typeArgument, TypeVariableBinding typeParameter, ReferenceBinding genericType, ASTNode location) {
    if (location == null) {
      handle(
        16777742, 
        new String[] { new String(typeArgument.readableName()), new String(genericType.readableName()), new String(typeParameter.sourceName), parameterBoundAsString(typeParameter, false) }, 
        new String[] { new String(typeArgument.shortReadableName()), new String(genericType.shortReadableName()), new String(typeParameter.sourceName), parameterBoundAsString(typeParameter, true) }, 
        131, 
        0, 
        0);
      return;
    }
    handle(
      16777742, 
      new String[] { new String(typeArgument.readableName()), new String(genericType.readableName()), new String(typeParameter.sourceName), parameterBoundAsString(typeParameter, false) }, 
      new String[] { new String(typeArgument.shortReadableName()), new String(genericType.shortReadableName()), new String(typeParameter.sourceName), parameterBoundAsString(typeParameter, true) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  private String typesAsString(boolean isVarargs, TypeBinding[] types, boolean makeShort) {
    StringBuffer buffer = new StringBuffer(10);
    int i = 0; for (int length = types.length; i < length; i++) {
      if (i != 0)
        buffer.append(", ");
      TypeBinding type = types[i];
      boolean isVarargType = (isVarargs) && (i == length - 1);
      if (isVarargType) type = ((ArrayBinding)type).elementsType();
      buffer.append(new String(makeShort ? type.shortReadableName() : type.readableName()));
      if (isVarargType) buffer.append("...");
    }
    return buffer.toString();
  }
  public void undefinedAnnotationValue(TypeBinding annotationType, MemberValuePair memberValuePair) {
    if (isRecoveredName(memberValuePair.name)) return;
    String name = new String(memberValuePair.name);
    handle(
      67109475, 
      new String[] { name, new String(annotationType.readableName()) }, 
      new String[] { name, new String(annotationType.shortReadableName()) }, 
      memberValuePair.sourceStart, 
      memberValuePair.sourceEnd);
  }
  public void undefinedLabel(BranchStatement statement) {
    if (isRecoveredName(statement.label)) return;
    String[] arguments = { new String(statement.label) };
    handle(
      536871086, 
      arguments, 
      arguments, 
      statement.sourceStart, 
      statement.sourceEnd);
  }

  public void undefinedTypeVariableSignature(char[] variableName, ReferenceBinding binaryType) {
    handle(
      536871450, 
      new String[] { new String(variableName), new String(binaryType.readableName()) }, 
      new String[] { new String(variableName), new String(binaryType.shortReadableName()) }, 
      131, 
      0, 
      0);
  }
  public void undocumentedEmptyBlock(int blockStart, int blockEnd) {
    handle(
      536871372, 
      NoArgument, 
      NoArgument, 
      blockStart, 
      blockEnd);
  }
  public void unexpectedStaticModifierForField(SourceTypeBinding type, FieldDeclaration fieldDecl) {
    String[] arguments = { new String(fieldDecl.name) };
    handle(
      33554778, 
      arguments, 
      arguments, 
      fieldDecl.sourceStart, 
      fieldDecl.sourceEnd);
  }
  public void unexpectedStaticModifierForMethod(ReferenceBinding type, AbstractMethodDeclaration methodDecl) {
    String[] arguments = { new String(type.sourceName()), new String(methodDecl.selector) };
    handle(
      67109225, 
      arguments, 
      arguments, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }

  public void unhandledException(TypeBinding exceptionType, ASTNode location) {
    boolean insideDefaultConstructor = 
      ((this.referenceContext instanceof ConstructorDeclaration)) && 
      (((ConstructorDeclaration)this.referenceContext).isDefaultConstructor());
    boolean insideImplicitConstructorCall = 
      ((location instanceof ExplicitConstructorCall)) && 
      (((ExplicitConstructorCall)location).accessMode == 1);

    handle(
      insideImplicitConstructorCall ? 
      134217871 : insideDefaultConstructor ? 
      16777362 : 
      16777384, 
      new String[] { new String(exceptionType.readableName()) }, 
      new String[] { new String(exceptionType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void unhandledWarningToken(Expression token) {
    String[] arguments = { token.constant.stringValue() };
    handle(
      536871543, 
      arguments, 
      arguments, 
      token.sourceStart, 
      token.sourceEnd);
  }
  public void uninitializedBlankFinalField(FieldBinding field, ASTNode location) {
    String[] arguments = { new String(field.readableName()) };
    handle(
      33554513, 
      arguments, 
      arguments, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void uninitializedLocalVariable(LocalVariableBinding binding, ASTNode location) {
    String[] arguments = { new String(binding.readableName()) };
    handle(
      536870963, 
      arguments, 
      arguments, 
      nodeSourceStart(binding, location), 
      nodeSourceEnd(binding, location));
  }
  public void unmatchedBracket(int position, ReferenceContext context, CompilationResult compilationResult) {
    handle(
      1610612956, 
      NoArgument, 
      NoArgument, 
      position, 
      position, 
      context, 
      compilationResult);
  }
  public void unnecessaryCast(CastExpression castExpression) {
    int severity = computeSeverity(553648309);
    if (severity == -1) return;
    TypeBinding castedExpressionType = castExpression.expression.resolvedType;
    handle(
      553648309, 
      new String[] { new String(castedExpressionType.readableName()), new String(castExpression.type.resolvedType.readableName()) }, 
      new String[] { new String(castedExpressionType.shortReadableName()), new String(castExpression.type.resolvedType.shortReadableName()) }, 
      severity, 
      castExpression.sourceStart, 
      castExpression.sourceEnd);
  }
  public void unnecessaryElse(ASTNode location) {
    handle(
      536871101, 
      NoArgument, 
      NoArgument, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void unnecessaryEnclosingInstanceSpecification(Expression expression, ReferenceBinding targetType) {
    handle(
      16777239, 
      new String[] { new String(targetType.readableName()) }, 
      new String[] { new String(targetType.shortReadableName()) }, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void unnecessaryInstanceof(InstanceOfExpression instanceofExpression, TypeBinding checkType) {
    int severity = computeSeverity(553648311);
    if (severity == -1) return;
    TypeBinding expressionType = instanceofExpression.expression.resolvedType;
    handle(
      553648311, 
      new String[] { new String(expressionType.readableName()), new String(checkType.readableName()) }, 
      new String[] { new String(expressionType.shortReadableName()), new String(checkType.shortReadableName()) }, 
      severity, 
      instanceofExpression.sourceStart, 
      instanceofExpression.sourceEnd);
  }
  public void unnecessaryNLSTags(int sourceStart, int sourceEnd) {
    handle(
      536871177, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void unqualifiedFieldAccess(NameReference reference, FieldBinding field) {
    int sourceStart = reference.sourceStart;
    int sourceEnd = reference.sourceEnd;
    if ((reference instanceof SingleNameReference)) {
      int numberOfParens = (reference.bits & 0x1FE00000) >> 21;
      if (numberOfParens != 0) {
        sourceStart = retrieveStartingPositionAfterOpeningParenthesis(sourceStart, sourceEnd, numberOfParens);
        sourceEnd = retrieveEndingPositionAfterOpeningParenthesis(sourceStart, sourceEnd, numberOfParens);
      } else {
        sourceStart = nodeSourceStart(field, reference);
        sourceEnd = nodeSourceEnd(field, reference);
      }
    } else {
      sourceStart = nodeSourceStart(field, reference);
      sourceEnd = nodeSourceEnd(field, reference);
    }
    handle(
      570425423, 
      new String[] { new String(field.declaringClass.readableName()), new String(field.name) }, 
      new String[] { new String(field.declaringClass.shortReadableName()), new String(field.name) }, 
      sourceStart, 
      sourceEnd);
  }
  public void unreachableCatchBlock(ReferenceBinding exceptionType, ASTNode location) {
    handle(
      83886247, 
      new String[] { 
      new String(exceptionType.readableName()) }, 
      new String[] { 
      new String(exceptionType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void unreachableCode(Statement statement) {
    int sourceStart = statement.sourceStart;
    int sourceEnd = statement.sourceEnd;
    if ((statement instanceof LocalDeclaration)) {
      LocalDeclaration declaration = (LocalDeclaration)statement;
      sourceStart = declaration.declarationSourceStart;
      sourceEnd = declaration.declarationSourceEnd;
    } else if ((statement instanceof Expression)) {
      int statemendEnd = ((Expression)statement).statementEnd;
      if (statemendEnd != -1) sourceEnd = statemendEnd;
    }
    handle(
      536871073, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }

  public void unresolvableReference(NameReference nameRef, Binding binding)
  {
    String[] arguments = { new String(binding.readableName()) };
    int end = nameRef.sourceEnd;
    if ((nameRef instanceof QualifiedNameReference)) {
      QualifiedNameReference ref = (QualifiedNameReference)nameRef;
      if (isRecoveredName(ref.tokens)) return;
      if (ref.indexOfFirstFieldBinding >= 1)
        end = (int)ref.sourcePositions[(ref.indexOfFirstFieldBinding - 1)];
    } else {
      SingleNameReference ref = (SingleNameReference)nameRef;
      if (isRecoveredName(ref.token)) return;
    }
    handle(
      570425394, 
      arguments, 
      arguments, 
      nameRef.sourceStart, 
      end);
  }
  public void unsafeCast(CastExpression castExpression, Scope scope) {
    int severity = computeSeverity(16777761);
    if (severity == -1) return;
    TypeBinding castedExpressionType = castExpression.expression.resolvedType;
    TypeBinding castExpressionResolvedType = castExpression.resolvedType;
    handle(
      16777761, 
      new String[] { 
      new String(castedExpressionType.readableName()), 
      new String(castExpressionResolvedType.readableName()) }, 
      new String[] { 
      new String(castedExpressionType.shortReadableName()), 
      new String(castExpressionResolvedType.shortReadableName()) }, 
      severity, 
      castExpression.sourceStart, 
      castExpression.sourceEnd);
  }
  public void unsafeGenericArrayForVarargs(TypeBinding leafComponentType, ASTNode location) {
    int severity = computeSeverity(67109438);
    if (severity == -1) return;
    handle(
      67109438, 
      new String[] { new String(leafComponentType.readableName()) }, 
      new String[] { new String(leafComponentType.shortReadableName()) }, 
      severity, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void unsafeRawFieldAssignment(FieldBinding field, TypeBinding expressionType, ASTNode location) {
    int severity = computeSeverity(16777752);
    if (severity == -1) return;
    handle(
      16777752, 
      new String[] { 
      new String(expressionType.readableName()), new String(field.name), new String(field.declaringClass.readableName()), new String(field.declaringClass.erasure().readableName()) }, 
      new String[] { 
      new String(expressionType.shortReadableName()), new String(field.name), new String(field.declaringClass.shortReadableName()), new String(field.declaringClass.erasure().shortReadableName()) }, 
      severity, 
      nodeSourceStart(field, location), 
      nodeSourceEnd(field, location));
  }
  public void unsafeRawGenericMethodInvocation(ASTNode location, MethodBinding rawMethod) {
    boolean isConstructor = rawMethod.isConstructor();
    int severity = computeSeverity(isConstructor ? 16777785 : 16777786);
    if (severity == -1) return;
    if (isConstructor)
      handle(
        16777785, 
        new String[] { 
        new String(rawMethod.declaringClass.sourceName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.original().parameters, false), 
        new String(rawMethod.declaringClass.readableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, false) }, 
        new String[] { 
        new String(rawMethod.declaringClass.sourceName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.original().parameters, true), 
        new String(rawMethod.declaringClass.shortReadableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, true) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        16777786, 
        new String[] { 
        new String(rawMethod.selector), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.original().parameters, false), 
        new String(rawMethod.declaringClass.readableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, false) }, 
        new String[] { 
        new String(rawMethod.selector), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.original().parameters, true), 
        new String(rawMethod.declaringClass.shortReadableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, true) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void unsafeRawInvocation(ASTNode location, MethodBinding rawMethod) {
    boolean isConstructor = rawMethod.isConstructor();
    int severity = computeSeverity(isConstructor ? 16777746 : 16777747);
    if (severity == -1) return;
    if (isConstructor)
      handle(
        16777746, 
        new String[] { 
        new String(rawMethod.declaringClass.readableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, false), 
        new String(rawMethod.declaringClass.erasure().readableName()) }, 
        new String[] { 
        new String(rawMethod.declaringClass.shortReadableName()), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, true), 
        new String(rawMethod.declaringClass.erasure().shortReadableName()) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        16777747, 
        new String[] { 
        new String(rawMethod.selector), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, false), 
        new String(rawMethod.declaringClass.readableName()), 
        new String(rawMethod.declaringClass.erasure().readableName()) }, 
        new String[] { 
        new String(rawMethod.selector), 
        typesAsString(rawMethod.original().isVarargs(), rawMethod.parameters, true), 
        new String(rawMethod.declaringClass.shortReadableName()), 
        new String(rawMethod.declaringClass.erasure().shortReadableName()) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void unsafeReturnTypeOverride(MethodBinding currentMethod, MethodBinding inheritedMethod, SourceTypeBinding type) {
    int severity = computeSeverity(67109423);
    if (severity == -1) return;
    int start = type.sourceStart();
    int end = type.sourceEnd();
    if (currentMethod.declaringClass == type) {
      ASTNode location = ((MethodDeclaration)currentMethod.sourceMethod()).returnType;
      start = location.sourceStart();
      end = location.sourceEnd();
    }
    handle(
      67109423, 
      new String[] { 
      new String(currentMethod.returnType.readableName()), 
      new String(currentMethod.selector), 
      typesAsString(currentMethod.original().isVarargs(), currentMethod.original().parameters, false), 
      new String(currentMethod.declaringClass.readableName()), 
      new String(inheritedMethod.returnType.readableName()), 
      new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { 
      new String(currentMethod.returnType.shortReadableName()), 
      new String(currentMethod.selector), 
      typesAsString(currentMethod.original().isVarargs(), currentMethod.original().parameters, true), 
      new String(currentMethod.declaringClass.shortReadableName()), 
      new String(inheritedMethod.returnType.shortReadableName()), 
      new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      severity, 
      start, 
      end);
  }
  public void unsafeTypeConversion(Expression expression, TypeBinding expressionType, TypeBinding expectedType) {
    int severity = computeSeverity(16777748);
    if (severity == -1) return;
    handle(
      16777748, 
      new String[] { new String(expressionType.readableName()), new String(expectedType.readableName()), new String(expectedType.erasure().readableName()) }, 
      new String[] { new String(expressionType.shortReadableName()), new String(expectedType.shortReadableName()), new String(expectedType.erasure().shortReadableName()) }, 
      severity, 
      expression.sourceStart, 
      expression.sourceEnd);
  }
  public void unusedArgument(LocalDeclaration localDecl) {
    int severity = computeSeverity(536870974);
    if (severity == -1) return;
    String[] arguments = { new String(localDecl.name) };
    handle(
      536870974, 
      arguments, 
      arguments, 
      severity, 
      localDecl.sourceStart, 
      localDecl.sourceEnd);
  }
  public void unusedDeclaredThrownException(ReferenceBinding exceptionType, AbstractMethodDeclaration method, ASTNode location) {
    boolean isConstructor = method.isConstructor();
    int severity = computeSeverity(isConstructor ? 536871098 : 536871097);
    if (severity == -1) return;
    if (isConstructor)
      handle(
        536871098, 
        new String[] { 
        new String(method.binding.declaringClass.readableName()), 
        typesAsString(method.binding.isVarargs(), method.binding.parameters, false), 
        new String(exceptionType.readableName()) }, 
        new String[] { 
        new String(method.binding.declaringClass.shortReadableName()), 
        typesAsString(method.binding.isVarargs(), method.binding.parameters, true), 
        new String(exceptionType.shortReadableName()) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        536871097, 
        new String[] { 
        new String(method.binding.declaringClass.readableName()), 
        new String(method.selector), 
        typesAsString(method.binding.isVarargs(), method.binding.parameters, false), 
        new String(exceptionType.readableName()) }, 
        new String[] { 
        new String(method.binding.declaringClass.shortReadableName()), 
        new String(method.selector), 
        typesAsString(method.binding.isVarargs(), method.binding.parameters, true), 
        new String(exceptionType.shortReadableName()) }, 
        severity, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void unusedImport(ImportReference importRef) {
    int severity = computeSeverity(268435844);
    if (severity == -1) return;
    String[] arguments = { CharOperation.toString(importRef.tokens) };
    handle(
      268435844, 
      arguments, 
      arguments, 
      severity, 
      importRef.sourceStart, 
      importRef.sourceEnd);
  }
  public void unusedLabel(LabeledStatement statement) {
    int severity = computeSeverity(536871111);
    if (severity == -1) return;
    String[] arguments = { new String(statement.label) };
    handle(
      536871111, 
      arguments, 
      arguments, 
      severity, 
      statement.sourceStart, 
      statement.labelEnd);
  }
  public void unusedLocalVariable(LocalDeclaration localDecl) {
    int severity = computeSeverity(536870973);
    if (severity == -1) return;
    String[] arguments = { new String(localDecl.name) };
    handle(
      536870973, 
      arguments, 
      arguments, 
      severity, 
      localDecl.sourceStart, 
      localDecl.sourceEnd);
  }

  public void unusedPrivateConstructor(ConstructorDeclaration constructorDecl) {
    int severity = computeSeverity(603979910);
    if (severity == -1) return;

    MethodBinding constructor = constructorDecl.binding;
    handle(
      603979910, 
      new String[] { 
      new String(constructor.declaringClass.readableName()), 
      typesAsString(constructor.isVarargs(), constructor.parameters, false) }, 
      new String[] { 
      new String(constructor.declaringClass.shortReadableName()), 
      typesAsString(constructor.isVarargs(), constructor.parameters, true) }, 
      severity, 
      constructorDecl.sourceStart, 
      constructorDecl.sourceEnd);
  }

  public void unusedPrivateField(FieldDeclaration fieldDecl) {
    int severity = computeSeverity(570425421);
    if (severity == -1) return;

    FieldBinding field = fieldDecl.binding;

    if ((CharOperation.equals(TypeConstants.SERIALVERSIONUID, field.name)) && 
      (field.isStatic()) && 
      (field.isFinal()) && 
      (TypeBinding.LONG == field.type)) {
      return;
    }
    if ((CharOperation.equals(TypeConstants.SERIALPERSISTENTFIELDS, field.name)) && 
      (field.isStatic()) && 
      (field.isFinal()) && 
      (field.type.dimensions() == 1) && 
      (CharOperation.equals(TypeConstants.CharArray_JAVA_IO_OBJECTSTREAMFIELD, field.type.leafComponentType().readableName()))) {
      return;
    }
    handle(
      570425421, 
      new String[] { 
      new String(field.declaringClass.readableName()), 
      new String(field.name) }, 
      new String[] { 
      new String(field.declaringClass.shortReadableName()), 
      new String(field.name) }, 
      severity, 
      nodeSourceStart(field, fieldDecl), 
      nodeSourceEnd(field, fieldDecl));
  }

  public void unusedPrivateMethod(AbstractMethodDeclaration methodDecl) {
    int severity = computeSeverity(603979894);
    if (severity == -1) return;

    MethodBinding method = methodDecl.binding;

    if ((!method.isStatic()) && 
      (TypeBinding.VOID == method.returnType) && 
      (method.parameters.length == 1) && 
      (method.parameters[0].dimensions() == 0) && 
      (CharOperation.equals(method.selector, TypeConstants.READOBJECT)) && 
      (CharOperation.equals(TypeConstants.CharArray_JAVA_IO_OBJECTINPUTSTREAM, method.parameters[0].readableName()))) {
      return;
    }

    if ((!method.isStatic()) && 
      (TypeBinding.VOID == method.returnType) && 
      (method.parameters.length == 1) && 
      (method.parameters[0].dimensions() == 0) && 
      (CharOperation.equals(method.selector, TypeConstants.WRITEOBJECT)) && 
      (CharOperation.equals(TypeConstants.CharArray_JAVA_IO_OBJECTOUTPUTSTREAM, method.parameters[0].readableName()))) {
      return;
    }

    if ((!method.isStatic()) && 
      (1 == method.returnType.id) && 
      (method.parameters.length == 0) && 
      (CharOperation.equals(method.selector, TypeConstants.READRESOLVE))) {
      return;
    }

    if ((!method.isStatic()) && 
      (1 == method.returnType.id) && 
      (method.parameters.length == 0) && 
      (CharOperation.equals(method.selector, TypeConstants.WRITEREPLACE))) {
      return;
    }
    handle(
      603979894, 
      new String[] { 
      new String(method.declaringClass.readableName()), 
      new String(method.selector), 
      typesAsString(method.isVarargs(), method.parameters, false) }, 
      new String[] { 
      new String(method.declaringClass.shortReadableName()), 
      new String(method.selector), 
      typesAsString(method.isVarargs(), method.parameters, true) }, 
      severity, 
      methodDecl.sourceStart, 
      methodDecl.sourceEnd);
  }
  public void unusedPrivateType(TypeDeclaration typeDecl) {
    int severity = computeSeverity(553648135);
    if (severity == -1) return;

    ReferenceBinding type = typeDecl.binding;
    handle(
      553648135, 
      new String[] { 
      new String(type.readableName()) }, 
      new String[] { 
      new String(type.shortReadableName()) }, 
      severity, 
      typeDecl.sourceStart, 
      typeDecl.sourceEnd);
  }
  public void useAssertAsAnIdentifier(int sourceStart, int sourceEnd) {
    handle(
      536871352, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void useEnumAsAnIdentifier(int sourceStart, int sourceEnd) {
    handle(
      536871353, 
      NoArgument, 
      NoArgument, 
      sourceStart, 
      sourceEnd);
  }
  public void varargsArgumentNeedCast(MethodBinding method, TypeBinding argumentType, InvocationSite location) {
    int severity = this.options.getSeverity(34359738368L);
    if (severity == -1) return;
    ArrayBinding varargsType = (ArrayBinding)method.parameters[(method.parameters.length - 1)];
    if (method.isConstructor())
      handle(
        134218530, 
        new String[] { 
        new String(argumentType.readableName()), 
        new String(varargsType.readableName()), 
        new String(method.declaringClass.readableName()), 
        typesAsString(method.isVarargs(), method.parameters, false), 
        new String(varargsType.elementsType().readableName()) }, 
        new String[] { 
        new String(argumentType.shortReadableName()), 
        new String(varargsType.shortReadableName()), 
        new String(method.declaringClass.shortReadableName()), 
        typesAsString(method.isVarargs(), method.parameters, true), 
        new String(varargsType.elementsType().shortReadableName()) }, 
        severity, 
        location.sourceStart(), 
        location.sourceEnd());
    else
      handle(
        67109665, 
        new String[] { 
        new String(argumentType.readableName()), 
        new String(varargsType.readableName()), 
        new String(method.selector), 
        typesAsString(method.isVarargs(), method.parameters, false), 
        new String(method.declaringClass.readableName()), 
        new String(varargsType.elementsType().readableName()) }, 
        new String[] { 
        new String(argumentType.shortReadableName()), 
        new String(varargsType.shortReadableName()), 
        new String(method.selector), typesAsString(method.isVarargs(), method.parameters, true), 
        new String(method.declaringClass.shortReadableName()), 
        new String(varargsType.elementsType().shortReadableName()) }, 
        severity, 
        location.sourceStart(), 
        location.sourceEnd());
  }

  public void varargsConflict(MethodBinding method1, MethodBinding method2, SourceTypeBinding type) {
    handle(
      67109667, 
      new String[] { 
      new String(method1.selector), 
      typesAsString(method1.isVarargs(), method1.parameters, false), 
      new String(method1.declaringClass.readableName()), 
      typesAsString(method2.isVarargs(), method2.parameters, false), 
      new String(method2.declaringClass.readableName()) }, 
      new String[] { 
      new String(method1.selector), 
      typesAsString(method1.isVarargs(), method1.parameters, true), 
      new String(method1.declaringClass.shortReadableName()), 
      typesAsString(method2.isVarargs(), method2.parameters, true), 
      new String(method2.declaringClass.shortReadableName()) }, 
      method1.declaringClass == type ? method1.sourceStart() : type.sourceStart(), 
      method1.declaringClass == type ? method1.sourceEnd() : type.sourceEnd());
  }
  public void variableTypeCannotBeVoid(AbstractVariableDeclaration varDecl) {
    String[] arguments = { new String(varDecl.name) };
    handle(
      536870964, 
      arguments, 
      arguments, 
      varDecl.sourceStart, 
      varDecl.sourceEnd);
  }
  public void variableTypeCannotBeVoidArray(AbstractVariableDeclaration varDecl) {
    handle(
      536870966, 
      NoArgument, 
      NoArgument, 
      varDecl.type.sourceStart, 
      varDecl.type.sourceEnd);
  }
  public void visibilityConflict(MethodBinding currentMethod, MethodBinding inheritedMethod) {
    handle(
      67109273, 
      new String[] { new String(inheritedMethod.declaringClass.readableName()) }, 
      new String[] { new String(inheritedMethod.declaringClass.shortReadableName()) }, 
      currentMethod.sourceStart(), 
      currentMethod.sourceEnd());
  }
  public void wildcardAssignment(TypeBinding variableType, TypeBinding expressionType, ASTNode location) {
    handle(
      16777758, 
      new String[] { 
      new String(expressionType.readableName()), new String(variableType.readableName()) }, 
      new String[] { 
      new String(expressionType.shortReadableName()), new String(variableType.shortReadableName()) }, 
      location.sourceStart, 
      location.sourceEnd);
  }
  public void wildcardInvocation(ASTNode location, TypeBinding receiverType, MethodBinding method, TypeBinding[] arguments) {
    TypeBinding offendingArgument = null;
    TypeBinding offendingParameter = null;
    int i = 0; for (int length = method.parameters.length; i < length; i++) {
      TypeBinding parameter = method.parameters[i];
      if ((parameter.isWildcard()) && (((WildcardBinding)parameter).boundKind != 2)) {
        offendingParameter = parameter;
        offendingArgument = arguments[i];
        break;
      }
    }

    if (method.isConstructor())
      handle(
        16777756, 
        new String[] { 
        new String(receiverType.sourceName()), 
        typesAsString(method.isVarargs(), method.parameters, false), 
        new String(receiverType.readableName()), 
        typesAsString(false, arguments, false), 
        new String(offendingArgument.readableName()), 
        new String(offendingParameter.readableName()) }, 
        new String[] { 
        new String(receiverType.sourceName()), 
        typesAsString(method.isVarargs(), method.parameters, true), 
        new String(receiverType.shortReadableName()), 
        typesAsString(false, arguments, true), 
        new String(offendingArgument.shortReadableName()), 
        new String(offendingParameter.shortReadableName()) }, 
        location.sourceStart, 
        location.sourceEnd);
    else
      handle(
        16777757, 
        new String[] { 
        new String(method.selector), 
        typesAsString(method.isVarargs(), method.parameters, false), 
        new String(receiverType.readableName()), 
        typesAsString(false, arguments, false), 
        new String(offendingArgument.readableName()), 
        new String(offendingParameter.readableName()) }, 
        new String[] { 
        new String(method.selector), 
        typesAsString(method.isVarargs(), method.parameters, true), 
        new String(receiverType.shortReadableName()), 
        typesAsString(false, arguments, true), 
        new String(offendingArgument.shortReadableName()), 
        new String(offendingParameter.shortReadableName()) }, 
        location.sourceStart, 
        location.sourceEnd);
  }

  public void wrongSequenceOfExceptionTypesError(TryStatement statement, TypeBinding exceptionType, int under, TypeBinding hidingExceptionType)
  {
    TypeReference typeRef = statement.catchArguments[under].type;
    handle(
      553648315, 
      new String[] { 
      new String(exceptionType.readableName()), 
      new String(hidingExceptionType.readableName()) }, 
      new String[] { 
      new String(exceptionType.shortReadableName()), 
      new String(hidingExceptionType.shortReadableName()) }, 
      typeRef.sourceStart, 
      typeRef.sourceEnd);
  }
}