package org.quartz;

import java.io.Serializable;
import java.util.Comparator;
import java.util.Date;

public abstract interface Trigger extends Serializable, Cloneable, Comparable<Trigger>
{
  public static final long serialVersionUID = -3904243490805975570L;
  public static final int MISFIRE_INSTRUCTION_SMART_POLICY = 0;
  public static final int MISFIRE_INSTRUCTION_IGNORE_MISFIRE_POLICY = -1;
  public static final int DEFAULT_PRIORITY = 5;

  public abstract TriggerKey getKey();

  public abstract JobKey getJobKey();

  public abstract String getDescription();

  public abstract String getCalendarName();

  public abstract JobDataMap getJobDataMap();

  public abstract int getPriority();

  public abstract boolean mayFireAgain();

  public abstract Date getStartTime();

  public abstract Date getEndTime();

  public abstract Date getNextFireTime();

  public abstract Date getPreviousFireTime();

  public abstract Date getFireTimeAfter(Date paramDate);

  public abstract Date getFinalFireTime();

  public abstract int getMisfireInstruction();

  public abstract TriggerBuilder<? extends Trigger> getTriggerBuilder();

  public abstract ScheduleBuilder<? extends Trigger> getScheduleBuilder();

  public abstract boolean equals(Object paramObject);

  public abstract int compareTo(Trigger paramTrigger);

  public static class TriggerTimeComparator
    implements Comparator<Trigger>, Serializable
  {
    public int compare(Trigger trig1, Trigger trig2)
    {
      Date t1 = trig1.getNextFireTime();
      Date t2 = trig2.getNextFireTime();

      if ((t1 != null) || (t2 != null)) {
        if (t1 == null) {
          return 1;
        }

        if (t2 == null) {
          return -1;
        }

        if (t1.before(t2)) {
          return -1;
        }

        if (t1.after(t2)) {
          return 1;
        }
      }

      int comp = trig2.getPriority() - trig1.getPriority();
      if (comp != 0) {
        return comp;
      }

      return trig1.getKey().compareTo(trig2.getKey());
    }
  }

  public static enum CompletedExecutionInstruction
  {
    NOOP, RE_EXECUTE_JOB, SET_TRIGGER_COMPLETE, DELETE_TRIGGER, 
    SET_ALL_JOB_TRIGGERS_COMPLETE, SET_TRIGGER_ERROR, SET_ALL_JOB_TRIGGERS_ERROR;
  }

  public static enum TriggerState
  {
    NONE, NORMAL, PAUSED, COMPLETE, ERROR, BLOCKED;
  }
}