package org.quartz.core.jmx;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.management.openmbean.CompositeData;
import javax.management.openmbean.TabularData;

public abstract interface QuartzSchedulerMBean
{
  public static final String SCHEDULER_STARTED = "schedulerStarted";
  public static final String SCHEDULER_PAUSED = "schedulerPaused";
  public static final String SCHEDULER_SHUTDOWN = "schedulerShutdown";
  public static final String SCHEDULER_ERROR = "schedulerError";
  public static final String JOB_ADDED = "jobAdded";
  public static final String JOB_DELETED = "jobDeleted";
  public static final String JOB_SCHEDULED = "jobScheduled";
  public static final String JOB_UNSCHEDULED = "jobUnscheduled";
  public static final String JOBS_PAUSED = "jobsPaused";
  public static final String JOBS_RESUMED = "jobsResumed";
  public static final String JOB_EXECUTION_VETOED = "jobExecutionVetoed";
  public static final String JOB_TO_BE_EXECUTED = "jobToBeExecuted";
  public static final String JOB_WAS_EXECUTED = "jobWasExecuted";
  public static final String TRIGGER_FINALIZED = "triggerFinalized";
  public static final String TRIGGERS_PAUSED = "triggersPaused";
  public static final String TRIGGERS_RESUMED = "triggersResumed";
  public static final String SCHEDULING_DATA_CLEARED = "schedulingDataCleared";
  public static final String SAMPLED_STATISTICS_ENABLED = "sampledStatisticsEnabled";
  public static final String SAMPLED_STATISTICS_RESET = "sampledStatisticsReset";

  public abstract String getSchedulerName();

  public abstract String getSchedulerInstanceId();

  public abstract boolean isStandbyMode();

  public abstract boolean isShutdown();

  public abstract String getVersion();

  public abstract String getJobStoreClassName();

  public abstract String getThreadPoolClassName();

  public abstract int getThreadPoolSize();

  public abstract long getJobsScheduledMostRecentSample();

  public abstract long getJobsExecutedMostRecentSample();

  public abstract long getJobsCompletedMostRecentSample();

  public abstract Map<String, Long> getPerformanceMetrics();

  public abstract TabularData getCurrentlyExecutingJobs()
    throws Exception;

  public abstract TabularData getAllJobDetails()
    throws Exception;

  public abstract List<CompositeData> getAllTriggers()
    throws Exception;

  public abstract List<String> getJobGroupNames()
    throws Exception;

  public abstract List<String> getJobNames(String paramString)
    throws Exception;

  public abstract CompositeData getJobDetail(String paramString1, String paramString2)
    throws Exception;

  public abstract boolean isStarted();

  public abstract void start()
    throws Exception;

  public abstract void shutdown();

  public abstract void standby();

  public abstract void clear()
    throws Exception;

  public abstract Date scheduleJob(String paramString1, String paramString2, String paramString3, String paramString4)
    throws Exception;

  public abstract void scheduleBasicJob(Map<String, Object> paramMap1, Map<String, Object> paramMap2)
    throws Exception;

  public abstract void scheduleJob(Map<String, Object> paramMap1, Map<String, Object> paramMap2)
    throws Exception;

  public abstract void scheduleJob(String paramString1, String paramString2, Map<String, Object> paramMap)
    throws Exception;

  public abstract boolean unscheduleJob(String paramString1, String paramString2)
    throws Exception;

  public abstract boolean interruptJob(String paramString1, String paramString2)
    throws Exception;

  public abstract boolean interruptJob(String paramString)
    throws Exception;

  public abstract void triggerJob(String paramString1, String paramString2, Map<String, String> paramMap)
    throws Exception;

  public abstract boolean deleteJob(String paramString1, String paramString2)
    throws Exception;

  public abstract void addJob(CompositeData paramCompositeData, boolean paramBoolean)
    throws Exception;

  public abstract void addJob(Map<String, Object> paramMap, boolean paramBoolean)
    throws Exception;

  public abstract void pauseJobGroup(String paramString)
    throws Exception;

  public abstract void pauseJobsStartingWith(String paramString)
    throws Exception;

  public abstract void pauseJobsEndingWith(String paramString)
    throws Exception;

  public abstract void pauseJobsContaining(String paramString)
    throws Exception;

  public abstract void resumeJobGroup(String paramString)
    throws Exception;

  public abstract void resumeJobsStartingWith(String paramString)
    throws Exception;

  public abstract void resumeJobsEndingWith(String paramString)
    throws Exception;

  public abstract void resumeJobsContaining(String paramString)
    throws Exception;

  public abstract void pauseJob(String paramString1, String paramString2)
    throws Exception;

  public abstract void resumeJob(String paramString1, String paramString2)
    throws Exception;

  public abstract List<String> getTriggerGroupNames()
    throws Exception;

  public abstract List<String> getTriggerNames(String paramString)
    throws Exception;

  public abstract CompositeData getTrigger(String paramString1, String paramString2)
    throws Exception;

  public abstract String getTriggerState(String paramString1, String paramString2)
    throws Exception;

  public abstract List<CompositeData> getTriggersOfJob(String paramString1, String paramString2)
    throws Exception;

  public abstract Set<String> getPausedTriggerGroups()
    throws Exception;

  public abstract void pauseAllTriggers()
    throws Exception;

  public abstract void resumeAllTriggers()
    throws Exception;

  public abstract void pauseTriggerGroup(String paramString)
    throws Exception;

  public abstract void pauseTriggersStartingWith(String paramString)
    throws Exception;

  public abstract void pauseTriggersEndingWith(String paramString)
    throws Exception;

  public abstract void pauseTriggersContaining(String paramString)
    throws Exception;

  public abstract void resumeTriggerGroup(String paramString)
    throws Exception;

  public abstract void resumeTriggersStartingWith(String paramString)
    throws Exception;

  public abstract void resumeTriggersEndingWith(String paramString)
    throws Exception;

  public abstract void resumeTriggersContaining(String paramString)
    throws Exception;

  public abstract void pauseTrigger(String paramString1, String paramString2)
    throws Exception;

  public abstract void resumeTrigger(String paramString1, String paramString2)
    throws Exception;

  public abstract List<String> getCalendarNames()
    throws Exception;

  public abstract void deleteCalendar(String paramString)
    throws Exception;

  public abstract void setSampledStatisticsEnabled(boolean paramBoolean);

  public abstract boolean isSampledStatisticsEnabled();
}