package org.quartz.impl.matchers;

import org.quartz.Matcher;
import org.quartz.utils.Key;

public abstract class StringMatcher<T extends Key<?>>
  implements Matcher<T>
{
  protected String compareTo;
  protected StringOperatorName compareWith;

  protected StringMatcher(String compareTo, StringOperatorName compareWith)
  {
    if (compareTo == null)
      throw new IllegalArgumentException("CompareTo value cannot be null!");
    if (compareWith == null) {
      throw new IllegalArgumentException("CompareWith operator cannot be null!");
    }
    this.compareTo = compareTo;
    this.compareWith = compareWith;
  }

  protected abstract String getValue(T paramT);

  public boolean isMatch(T key)
  {
    return this.compareWith.evaluate(getValue(key), this.compareTo);
  }

  public int hashCode()
  {
    int prime = 31;
    int result = 1;
    result = 31 * result + (this.compareTo == null ? 0 : this.compareTo.hashCode());

    result = 31 * result + (this.compareWith == null ? 0 : this.compareWith.hashCode());

    return result;
  }

  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    StringMatcher other = (StringMatcher)obj;
    if (this.compareTo == null) {
      if (other.compareTo != null)
        return false;
    } else if (!this.compareTo.equals(other.compareTo))
      return false;
    if (this.compareWith == null) {
      if (other.compareWith != null)
        return false;
    } else if (!this.compareWith.equals(other.compareWith))
      return false;
    return true;
  }

  public String getCompareToValue() {
    return this.compareTo;
  }

  public StringOperatorName getCompareWithOperator() {
    return this.compareWith;
  }

  public static abstract enum StringOperatorName
  {
    EQUALS, 

    STARTS_WITH, 

    ENDS_WITH, 

    CONTAINS;

    public abstract boolean evaluate(String paramString1, String paramString2);
  }
}